####################
### AFCAD Reader ###
####################

$runway_designator = ["", "L", "R", "C", "W", "A", "B"]
$icao_format = ["", "", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"]
$default_afcad_folder = File.dirname(__FILE__)

def clear_icao
  $airport_icao=""
  status = $afcad_menu.set_validation_proc($clear_menu) { MF_GRAYED }
end

def read_afcad (afcad_file = "")
  tower_group = Sketchup.active_model.entities.add_group
  tower_group_ents = tower_group.entities
  tower_group.description = "AFCAD Tower"

  file_size = File.stat(afcad_file).size
  file_reader = File.new(afcad_file)
  file_id = ""

  ### Read file ###  
  file_content = []
  for step in 0..file_size - 1
    file_reader.seek (step)
    if $win32ole_ok == 1
      begin
        file_content[step] = file_reader.read(1)
      rescue
        file_content[step] = 26.chr
      end
    else
      begin
        file_content[step] = file_reader.read(1).unpack('C')
      rescue
        file_content[step] = [26]
      end
    end
  end
  
  for step in 0..7
    if $win32ole_ok == 1
      file_id += file_content[step].ord.to_s
    else
      file_id += file_content[step].to_s
    end
  end
  
  if file_id == "121462556000"
    sections = 0
    read_offset = 20
    for step in 0..3
      if $win32ole_ok == 1
        sections += file_content[step + read_offset].ord * 256 ** step
      else
  #      echo file_content[step + read_offset][0]
        sections += file_content[step + read_offset][0] * 256 ** step
      end
    end
    
    
    read_offset = 56
    section_type = []
    section_unknown = []
    section_subsections = []
    section_offset = []
    section_length = []

    ######################
    ### Section header ###
    ######################

    for step in 0..sections - 1
      section_type[step] = 0
      section_unknown[step] = 0
      section_subsections[step] = 0
      section_offset[step] = 0
      section_length[step] = 0
      
      for step2 in 0..3
        if $win32ole_ok == 1
          section_type[step] += file_content[step2 + (step * 20) + read_offset].ord * 256 ** step2
          section_unknown[step] += file_content[4 + step2 + (step * 20) + read_offset].ord * 256 ** step2
          section_subsections[step] += file_content[8 + step2 + (step * 20) + read_offset].ord * 256 ** step2
          section_offset[step] += file_content[12 + step2 + (step * 20) + read_offset].ord * 256 ** step2
          section_length[step] += file_content[16 + step2 + (step * 20) + read_offset].ord * 256 ** step2
        else
          section_type[step] += file_content[step2 + (step * 20) + read_offset][0] * 256 ** step2
          section_unknown[step] += file_content[4 + step2 + (step * 20) + read_offset][0] * 256 ** step2
          section_subsections[step] += file_content[8 + step2 + (step * 20) + read_offset][0] * 256 ** step2
          section_offset[step] += file_content[12 + step2 + (step * 20) + read_offset][0] * 256 ** step2
          section_length[step] += file_content[16 + step2 + (step * 20) + read_offset][0] * 256 ** step2
        end
        
      end
 #     echo section_type[step].to_s + " - " + section_offset[step].to_s + " - " + section_length[step].to_s
    end
    for step in 0..sections - 1
      if section_type[step] == 3 ### Airport data ###
        
        section_bytes_to_read = section_length[step]
         
        ###### Go to airport header ######
        read_offset = section_offset[step]
        subsection_id = 0
        subsection_record_count = 0
        subsection_offset = 0
        subsection_length = 0
        airport_current_apron = 0
        airport_current_apron2 = 0
        airport_current_helipad = 0
        
        for step2 in 0..3
          if $win32ole_ok == 1
            subsection_id += file_content[step2 + (step * 16) + read_offset].ord * 256 ** step2
            subsection_record_count += file_content[4 + step2 + (step * 16) + read_offset].ord * 256 ** step2
            subsection_offset += file_content[8 + step2 + (step * 16) + read_offset].ord * 256 ** step2
            subsection_length += file_content[12 + step2 + (step * 16) + read_offset].ord * 256 ** step2
          else
            subsection_id += file_content[step2 + (step * 16) + read_offset][0] * 256 ** step2
            subsection_record_count += file_content[4 + step2 + (step * 16) + read_offset][0] * 256 ** step2
            subsection_offset += file_content[8 + step2 + (step * 16) + read_offset][0] * 256 ** step2
            subsection_length += file_content[12 + step2 + (step * 16) + read_offset][0] * 256 ** step2
          end
        end
        
        section_bytes_to_read = subsection_length
           
        ##### Go to airport sub section #####
        
        read_offset = subsection_offset
##        file_reader.seek(subsection_offset)
        
        
        while section_bytes_to_read > 0
          
          temp_id = 0
          for step2 in 0..1
            if $win32ole_ok == 1
              temp_id += file_content[step2 + read_offset].ord * 256 ** step2
            else
              temp_id += file_content[step2 + read_offset][0] * 256 ** step2
            end
          end

   #       echo temp_id
          if temp_id == 60 or temp_id == 3 #### Main airport record, 60 for FSX, 3 for FS9 ####
            airport_record_id = temp_id
            airport_record_length = 0
            airport_longitude = 0.0
            airport_latitude = 0.0
            airport_altitude = 0.0
            airport_tower_longitude = 0.0
            airport_tower_latitude = 0.0
            airport_tower_altitude = 0.0
            airport_mag_var = 0.0
            airport_icao_calc = 0
            airport_icao = ""
            airport_unknown1 = 0
            airport_unknown2 = 0
            airport_unknown3 = 0
            airport_traffic_scalar = 0
            airport_unknown4 = 0
            airport_mag_var_calc = ""

            for step2 in 0..3
              if $win32ole_ok == 1
                airport_record_length += file_content[2 + step2 + read_offset].ord * 256 ** step2
              else
                airport_record_length += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            if $win32ole_ok == 1
              airport_runway_records = file_content[6 + read_offset].ord
              airport_com_records = file_content[7 + read_offset].ord
              airport_start_records = file_content[8 + read_offset].ord
              airport_approach_records = file_content[9 + read_offset].ord
              airport_apron_records_delete_flag = file_content[10 + read_offset].ord
              #### Code to split the above (not done) ####
              airport_helipad_records = file_content[11 + read_offset].ord
            else
              airport_runway_records = file_content[6 + read_offset][0]
              airport_com_records = file_content[7 + read_offset][0]
              airport_start_records = file_content[8 + read_offset][0]
              airport_approach_records = file_content[9 + read_offset][0]
              airport_apron_records_delete_flag = file_content[10 + read_offset][0]
              #### Code to split the above (not done) ####
              airport_helipad_records = file_content[11 + read_offset][0]
            end
            for step2 in 0..3
              if $win32ole_ok == 1
                airport_longitude += file_content[12 + step2 + read_offset].ord * 256 ** step2
                airport_latitude += file_content[16 + step2 + read_offset].ord * 256 ** step2
                airport_altitude += file_content[20 + step2 + read_offset].ord * 256 ** step2
                airport_tower_longitude += file_content[24 + step2 + read_offset].ord * 256 ** step2
                airport_tower_latitude += file_content[28 + step2 + read_offset].ord * 256 ** step2
                airport_tower_altitude += file_content[32 + step2 + read_offset].ord * 256 ** step2
                airport_mag_var = file_content[36 + step2 + read_offset] + airport_mag_var_calc
                airport_icao_calc += file_content[40 + step2 + read_offset].ord * 256 ** step2
                airport_unknown1 += file_content[44 + step2 + read_offset].ord * 256 ** step2
                airport_unknown2 += file_content[48 + step2 + read_offset].ord * 256 ** step2
              else
                airport_longitude += file_content[12 + step2 + read_offset][0] * 256 ** step2
                airport_latitude += file_content[16 + step2 + read_offset][0] * 256 ** step2
                airport_altitude += file_content[20 + step2 + read_offset][0] * 256 ** step2
                airport_tower_longitude += file_content[24 + step2 + read_offset][0] * 256 ** step2
                airport_tower_latitude += file_content[28 + step2 + read_offset][0] * 256 ** step2
                airport_tower_altitude += file_content[32 + step2 + read_offset][0] * 256 ** step2
                airport_mag_var = file_content[36 + step2 + read_offset][0].chr + airport_mag_var_calc
                airport_icao_calc += file_content[40 + step2 + read_offset][0] * 256 ** step2
                airport_unknown1 += file_content[44 + step2 + read_offset][0] * 256 ** step2
                airport_unknown2 += file_content[48 + step2 + read_offset][0] * 256 ** step2
              end
            end
            if $win32ole_ok == 1
              airport_unknown3 = file_content[52 + read_offset].ord
              airport_traffic_scalar = file_content[53 + read_offset].ord
            else
              airport_unknown3 = file_content[52 + read_offset][0]
              airport_traffic_scalar = file_content[53 + read_offset][0]
            end
            for step2 in 0..1
              if $win32ole_ok == 1
                airport_unknown4 += file_content[54 + step2 + read_offset].ord * 256 ** step2
              else
                airport_unknown4 += file_content[54 + step2 + read_offset][0] * 256 ** step2
              end
            end
            airport_mag_var = airport_mag_var_calc.unpack('g')[0]
            airport_longitude = airport_longitude * 360 / (3 * 268435456) - 180
            airport_latitude = 90 - (airport_latitude * 180 / (2 * 268435456))
            airport_altitude = airport_altitude / 1000
            airport_tower_longitude = airport_tower_longitude * 360 / (3 * 268435456) - 180
            airport_tower_latitude = 90 - (airport_tower_latitude * 180 / (2 * 268435456))
            airport_tower_altitude = airport_tower_altitude / 1000

            if airport_longitude > 180 or airport_longitude < -180 or airport_latitude > 90 or airport_latitude < -90
              echo "error: " + temp_id.to_s
            end

            if temp_id == 60
              section_bytes_to_read -= 56
              read_offset += 56
            else
              section_bytes_to_read -= 52
              read_offset += 52
            end
          elsif temp_id == 51 #### Delete Airport data ####
            
            delete_airport_id = temp_id
            delete_airport_size = 0
            delete_airport_flags = 0
            delete_airport_runway_count = 0
            delete_airport_start_count = 0
            delete_airport_frequency_count = 0
            delete_airport_unused = 0
            delete_airport_runway = []
            delete_airport_start = []
            delete_airport_frequency = []
            if $win32ole_ok == 1
              for step2 in 0..3
                delete_airport_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                delete_airport_flags += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
              delete_airport_runway_count = file_content[8 + read_offset].ord
              delete_airport_start_count = file_content[9 + read_offset].ord
              delete_airport_frequency_count = file_content[10 + read_offset].ord
              delete_airport_unused = file_content[11 + read_offset].ord
            else
              for step2 in 0..3
                delete_airport_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                delete_airport_flags += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
              delete_airport_runway_count = file_content[8 + read_offset][0]
              delete_airport_start_count = file_content[9 + read_offset][0]
              delete_airport_frequency_count = file_content[10 + read_offset][0]
              delete_airport_unused = file_content[11 + read_offset][0]
            end
            
            for step2 in 0..delete_airport_runway_count - 1
          #    echo "XXXX"
            end
            for step2 in 0..delete_airport_start_count - 1
         #     echo "XXXX"
            end
            for step2 in 0..delete_airport_frequency_count - 1
           #   echo "XXXX"
            end


            
            section_bytes_to_read -= delete_airport_size
            read_offset += delete_airport_size
            
          elsif temp_id == 25 #### Airport name ####
            airport_name_id = temp_id
            airport_name_size = 0
            airport_name = ""
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_name_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..airport_name_size - 7
                airport_name += file_content[6 + step2 + read_offset]
              end
            else
              for step2 in 0..3
                airport_name_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..airport_name_size - 7
                airport_name += file_content[6 + step2 + read_offset][0].chr
              end
            end

            section_bytes_to_read -= airport_name_size
            read_offset += airport_name_size
          elsif temp_id == 4 ### Runway ###

            #############################
            ### Read airport sections ###
            #############################
        
            current_subrecord_id = 0
       
            airport_runway_id = []
            airport_runway_subrecord_size = []
            airport_runway_surface_type = []
            airport_runway_primary_runway_number = []
            airport_runway_primary_runway_designator = []
            airport_runway_secondary_runway_number = []
            airport_runway_secondary_runway_designator = []
            airport_runway_primary_ils_icao = []
            airport_runway_secondary_ils_icao = []
            airport_runway_longitude = []
            airport_runway_latitude = []
            airport_runway_elevation = []
            airport_runway_length = []
            airport_runway_width = []
            airport_runway_heading = []
            airport_runway_pattern_altitude = []
            airport_runway_marking_flags = []
            airport_runway_light_flags = []
            airport_runway_pattern_flags = []
            
            ### Sub-sub-records ###
            airport_runway_offset_threshold_primary_id = []
            airport_runway_offset_threshold_primary_size = []
            airport_runway_offset_threshold_primary_surface = []
            airport_runway_offset_threshold_primary_length = []
            airport_runway_offset_threshold_primary_width = []
            airport_runway_offset_threshold_secondary_id = []
            airport_runway_offset_threshold_secondary_size = []
            airport_runway_offset_threshold_secondary_surface = []
            airport_runway_offset_threshold_secondary_length = []
            airport_runway_offset_threshold_secondary_width = []
            airport_runway_blastpad_primary_id = []
            airport_runway_blastpad_primary_size = []
            airport_runway_blastpad_primary_surface = []
            airport_runway_blastpad_primary_length = []
            airport_runway_blastpad_primary_width = []
            airport_runway_blastpad_secondary_id = []
            airport_runway_blastpad_secondary_size = []
            airport_runway_blastpad_secondary_surface = []
            airport_runway_blastpad_secondary_length = []
            airport_runway_blastpad_secondary_width = []
            airport_runway_overrun_primary_id = []
            airport_runway_overrun_primary_size = []
            airport_runway_overrun_primary_surface = []
            airport_runway_overrun_primary_length = []
            airport_runway_overrun_primary_width = []
            airport_runway_overrun_secondary_id = []
            airport_runway_overrun_secondary_size = []
            airport_runway_overrun_secondary_surface = []
            airport_runway_overrun_secondary_length = []
            airport_runway_overrun_secondary_width = []
            airport_runway_vasi_primary_left_id = []
            airport_runway_vasi_primary_left_size = []
            airport_runway_vasi_primary_left_type = []
            airport_runway_vasi_primary_left_biasx = []
            airport_runway_vasi_primary_left_biasz = []
            airport_runway_vasi_primary_left_spacing = []
            airport_runway_vasi_primary_left_pitch = []
            airport_runway_vasi_primary_right_id = []
            airport_runway_vasi_primary_right_size = []
            airport_runway_vasi_primary_right_type = []
            airport_runway_vasi_primary_right_biasx = []
            airport_runway_vasi_primary_right_biasz = []
            airport_runway_vasi_primary_right_spacing = []
            airport_runway_vasi_primary_right_pitch = []
            airport_runway_vasi_secondary_left_id = []
            airport_runway_vasi_secondary_left_size = []
            airport_runway_vasi_secondary_left_type = []
            airport_runway_vasi_secondary_left_biasx = []
            airport_runway_vasi_secondary_left_biasz = []
            airport_runway_vasi_secondary_left_spacing = []
            airport_runway_vasi_secondary_left_pitch = []
            airport_runway_vasi_secondary_right_id = []
            airport_runway_vasi_secondary_right_size = []
            airport_runway_vasi_secondary_right_type = []
            airport_runway_vasi_secondary_right_biasx = []
            airport_runway_vasi_secondary_right_biasz = []
            airport_runway_vasi_secondary_right_spacing = []
            airport_runway_vasi_secondary_right_pitch = []
            airport_runway_approach_lights_primary_id = []
            airport_runway_approach_lights_primary_size = []
            airport_runway_approach_lights_primary_system = []
            airport_runway_approach_lights_primary_strobes = []
            airport_runway_approach_lights_secondary_id = []
            airport_runway_approach_lights_secondary_size = []
            airport_runway_approach_lights_secondary_system = []
            airport_runway_approach_lights_secondary_strobes = []
                  
            airport_runway_group = []
            airport_runway_group_ents = []
            airport_runway_x_pos = []
            airport_runway_y_pos = []
    
            ###############################
            ### Read runway sub records ###
            ###############################

            for step2 in 0..airport_runway_records - 1
              airport_runway_group[step2] = Sketchup.active_model.entities.add_group
              airport_runway_group_ents[step2] = airport_runway_group[step2].entities
              airport_runway_id[step2] = 0
              airport_runway_subrecord_size[step2] = 0
              airport_runway_surface_type[step2] = 0
              airport_runway_primary_runway_number[step2] = 0
              airport_runway_primary_runway_designator[step2] = 0
              airport_runway_secondary_runway_number[step2] = 0
              airport_runway_secondary_runway_designator[step2] = 0
              airport_runway_primary_ils_icao_calc = 0
              airport_runway_secondary_ils_icao_calc = 0
              airport_runway_primary_ils_icao[step2] = ""
              airport_runway_secondary_ils_icao[step2] = ""
              airport_runway_longitude[step2] = 0.0
              airport_runway_latitude[step2] = 0.0
              airport_runway_elevation[step2] = 0.0
              airport_runway_length[step2] = 0.0
              airport_runway_width[step2] = 0.0
              airport_runway_heading[step2] = 0.0
              airport_runway_pattern_altitude[step2] = 0.0
              airport_runway_marking_flags[step2] = 0
              airport_runway_light_flags[step2] = 0
              airport_runway_pattern_flags[step2] = 0
          
              airport_runway_id[step2] = temp_id
              airport_runway_length_calc = ""
              airport_runway_width_calc = ""
              airport_runway_heading_calc = ""
              airport_runway_pattern_altitude_calc = ""


              if $win32ole_ok == 1
                for step3 in 0..3
                  airport_runway_subrecord_size[step2] += file_content[2 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_primary_ils_icao_calc += file_content[12 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_secondary_ils_icao_calc += file_content[16 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_longitude[step2] += file_content[20 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_latitude[step2] += file_content[24 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_elevation[step2] += file_content[28 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_length_calc = file_content[32 + step3 + read_offset] + airport_runway_length_calc
                  airport_runway_width_calc = file_content[36 + step3 + read_offset] + airport_runway_width_calc
                  airport_runway_heading_calc = file_content[40 + step3 + read_offset] + airport_runway_heading_calc
                  airport_runway_pattern_altitude_calc = file_content[44 + step3 + read_offset] + airport_runway_pattern_altitude_calc
                end
                for step3 in 0..1
                  airport_runway_surface_type[step2] += file_content[6 + step3 + read_offset].ord * 256 ** step3
                  airport_runway_marking_flags[step2] += file_content[48 + step3 + read_offset].ord * 256 ** step3
                end
                airport_runway_primary_runway_number[step2] = file_content[8 + read_offset].ord
                airport_runway_primary_runway_designator[step2] = file_content[9 + read_offset].ord
                airport_runway_secondary_runway_number[step2] = file_content[10 + read_offset].ord
                airport_runway_secondary_runway_designator[step2] = file_content[11 + read_offset].ord
                airport_runway_light_flags[step2] = file_content[50 + read_offset].ord
                airport_runway_pattern_flags[step2] = file_content[51 + read_offset].ord
              else
                for step3 in 0..3
                  airport_runway_subrecord_size[step2] += file_content[2 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_primary_ils_icao_calc += file_content[12 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_secondary_ils_icao_calc += file_content[16 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_longitude[step2] += file_content[20 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_latitude[step2] += file_content[24 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_elevation[step2] += file_content[28 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_length_calc = file_content[32 + step3 + read_offset][0].chr + airport_runway_length_calc
                  airport_runway_width_calc = file_content[36 + step3 + read_offset][0].chr + airport_runway_width_calc
                  airport_runway_heading_calc = file_content[40 + step3 + read_offset][0].chr + airport_runway_heading_calc
                  airport_runway_pattern_altitude_calc = file_content[44 + step3 + read_offset][0].chr + airport_runway_pattern_altitude_calc
                end
                for step3 in 0..1
                  airport_runway_surface_type[step2] += file_content[6 + step3 + read_offset][0] * 256 ** step3
                  airport_runway_marking_flags[step2] += file_content[48 + step3 + read_offset][0] * 256 ** step3
                end
                airport_runway_primary_runway_number[step2] = file_content[8 + read_offset][0]
                airport_runway_primary_runway_designator[step2] = file_content[9 + read_offset][0]
                airport_runway_secondary_runway_number[step2] = file_content[10 + read_offset][0]
                airport_runway_secondary_runway_designator[step2] = file_content[11 + read_offset][0]
                airport_runway_light_flags[step2] = file_content[50 + read_offset][0]
                airport_runway_pattern_flags[step2] = file_content[51 + read_offset][0]
              end

              ### Calculate ILS ICAO ###
              airport_runway_primary_ils_icao_calc = ((airport_runway_primary_ils_icao_calc - 1) / 32).floor
              airport_runway_secondary_ils_icao_calc = ((airport_runway_secondary_ils_icao_calc - 1) / 32).floor
              for step3 in 0..4
                if airport_runway_primary_ils_icao_calc > 0
                  airport_runway_primary_ils_icao[step2] = $icao_format[airport_runway_primary_ils_icao_calc - ((airport_runway_primary_ils_icao_calc / 38).floor * 38)] + airport_runway_primary_ils_icao[step2]
                  airport_runway_primary_ils_icao_calc = (airport_runway_primary_ils_icao_calc / 38).floor
                end
                if airport_runway_secondary_ils_icao_calc > 0
                  airport_runway_secondary_ils_icao[step2] = $icao_format[airport_runway_secondary_ils_icao_calc - ((airport_runway_secondary_ils_icao_calc / 38).floor * 38)] + airport_runway_secondary_ils_icao[step2]
                  airport_runway_secondary_ils_icao_calc = (airport_runway_secondary_ils_icao_calc / 38).floor
                end
              end

              airport_runway_longitude[step2] = airport_runway_longitude[step2] * 360 / (3 * 268435456) - 180
              airport_runway_latitude[step2] = 90 - (airport_runway_latitude[step2] * 180 / (2 * 268435456))
              airport_runway_elevation[step2] = airport_runway_elevation[step2] / 1000
              airport_runway_length[step2] = airport_runway_length_calc.unpack('g')[0]
              airport_runway_width[step2] = airport_runway_width_calc.unpack('g')[0]
              airport_runway_heading[step2] = airport_runway_heading_calc.unpack('g')[0]
              airport_runway_pattern_altitude[step2] = airport_runway_pattern_altitude_calc.unpack('g')[0]

              if airport_runway_longitude[step2] > 180 or airport_runway_longitude[step2] < -180 or airport_runway_latitude[step2] > 90 or airport_runway_latitude[step2] < -90
                echo "error: " + temp_id.to_s
              end

              
              if airport_runway_longitude[step2] > airport_longitude
                airport_runway_x_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_runway_longitude[step2]) / $inch
              else
                airport_runway_x_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_runway_longitude[step2]) / $inch
              end
              if airport_runway_latitude[step2] > airport_latitude
                airport_runway_y_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_runway_latitude[step2], airport_longitude) / $inch
              else
                airport_runway_y_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_runway_latitude[step2], airport_longitude) / $inch
              end

              airport_runway_group[step2].description = "Runway|" + airport_runway_primary_runway_number[step2].to_s + $runway_designator[airport_runway_primary_runway_designator[step2]].to_s + "|" + airport_runway_length[step2].to_s + "|" + airport_runway_width[step2].to_s + "|" + airport_runway_heading[step2].to_s + "|" + airport_runway_x_pos[step2].to_s + "|" + airport_runway_y_pos[step2].to_s

              ############################################
              ### Check for additional sub-sub-records ###
              ############################################

              runway_byte_offset = 52
              if airport_runway_subrecord_size[step2] > 52
                bytes_to_read = airport_runway_subrecord_size[step2] - 52 
            
                while bytes_to_read > 0
                  airport_runway_subsubrecord_id_calc = 0
                  for step3 in 0..1
                    if $win32ole_ok == 1
                      airport_runway_subsubrecord_id_calc += file_content[runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                    else
                      airport_runway_subsubrecord_id_calc += file_content[runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                    end
                  end
              
                  if airport_runway_subsubrecord_id_calc == 5
                    airport_runway_offset_threshold_primary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_offset_threshold_primary_size[step2] = 0
                    airport_runway_offset_threshold_primary_surface[step2] = 0
                    airport_runway_offset_threshold_primary_length[step2] = 0.0
                    airport_runway_offset_threshold_primary_width[step2] = 0.0
                    airport_runway_offset_threshold_primary_length_calc = ""
                    airport_runway_offset_threshold_primary_width_calc = ""
                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_offset_threshold_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_offset_threshold_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_offset_threshold_primary_length_calc
                        airport_runway_offset_threshold_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_offset_threshold_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_offset_threshold_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_offset_threshold_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_offset_threshold_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_offset_threshold_primary_length_calc
                        airport_runway_offset_threshold_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_offset_threshold_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_offset_threshold_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end
                    airport_runway_offset_threshold_primary_length[step2] = airport_runway_offset_threshold_primary_length_calc.unpack('g')[0]
                    airport_runway_offset_threshold_primary_width[step2] = airport_runway_offset_threshold_primary_width_calc.unpack('g')[0]
                  
                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 6
                    airport_runway_offset_threshold_secondary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_offset_threshold_secondary_size[step2] = 0
                    airport_runway_offset_threshold_secondary_surface[step2] = 0
                    airport_runway_offset_threshold_secondary_length[step2] = 0.0
                    airport_runway_offset_threshold_secondary_width[step2] = 0.0
                    airport_runway_offset_threshold_secondary_length_calc = ""
                    airport_runway_offset_threshold_secondary_width_calc = ""
                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_offset_threshold_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_offset_threshold_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_offset_threshold_secondary_length_calc
                        airport_runway_offset_threshold_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_offset_threshold_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_offset_threshold_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_offset_threshold_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_offset_threshold_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_offset_threshold_secondary_length_calc
                        airport_runway_offset_threshold_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_offset_threshold_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_offset_threshold_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_offset_threshold_secondary_length[step2] = airport_runway_offset_threshold_secondary_length_calc.unpack('g')[0]
                    airport_runway_offset_threshold_secondary_width[step2] = airport_runway_offset_threshold_secondary_width_calc.unpack('g')[0]
                
                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 7
                    airport_runway_blastpad_primary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_blastpad_primary_size[step2] = 0
                    airport_runway_blastpad_primary_surface[step2] = 0
                    airport_runway_blastpad_primary_length[step2] = 0.0
                    airport_runway_blastpad_primary_width[step2] = 0.0
                    airport_runway_blastpad_primary_length_calc = ""
                    airport_runway_blastpad_primary_width_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_blastpad_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_blastpad_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_blastpad_primary_length_calc
                        airport_runway_blastpad_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_blastpad_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_blastpad_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_blastpad_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_blastpad_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_blastpad_primary_length_calc
                        airport_runway_blastpad_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_blastpad_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_blastpad_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end
                    airport_runway_blastpad_primary_length[step2] = airport_runway_blastpad_primary_length_calc.unpack('g')[0]
                    airport_runway_blastpad_primary_width[step2] = airport_runway_blastpad_primary_width_calc.unpack('g')[0]
                
                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 8
                    airport_runway_blastpad_secondary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_blastpad_secondary_size[step2] = 0
                    airport_runway_blastpad_secondary_surface[step2] = 0
                    airport_runway_blastpad_secondary_length[step2] = 0.0
                    airport_runway_blastpad_secondary_width[step2] = 0.0
                    airport_runway_blastpad_secondary_length_calc = ""
                    airport_runway_blastpad_secondary_width_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_blastpad_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_blastpad_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_blastpad_secondary_length_calc
                        airport_runway_blastpad_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_blastpad_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_blastpad_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_blastpad_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_blastpad_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_blastpad_secondary_length_calc
                        airport_runway_blastpad_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_blastpad_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_blastpad_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end
                   
                    airport_runway_blastpad_secondary_length[step2] = airport_runway_blastpad_secondary_length_calc.unpack('g')[0]
                    airport_runway_blastpad_secondary_width[step2] = airport_runway_blastpad_secondary_width_calc.unpack('g')[0]
                    
                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 9
                    airport_runway_overrun_primary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_overrun_primary_size[step2] = 0
                    airport_runway_overrun_primary_surface[step2] = 0
                    airport_runway_overrun_primary_length[step2] = 0.0
                    airport_runway_overrun_primary_width[step2] = 0.0
                    airport_runway_overrun_primary_length_calc = ""
                    airport_runway_overrun_primary_width_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_overrun_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_overrun_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_overrun_primary_length_calc
                        airport_runway_overrun_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_overrun_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_overrun_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_overrun_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_overrun_primary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_overrun_primary_length_calc
                        airport_runway_overrun_primary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_overrun_primary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_overrun_primary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_overrun_primary_length[step2] = airport_runway_overrun_primary_length_calc.unpack('g')[0]
                    airport_runway_overrun_primary_width[step2] = airport_runway_overrun_primary_width_calc.unpack('g')[0]

                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 10
                    airport_runway_overrun_secondary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_overrun_secondary_size[step2] = 0
                    airport_runway_overrun_secondary_surface[step2] = 0
                    airport_runway_overrun_secondary_length[step2] = 0.0
                    airport_runway_overrun_secondary_width[step2] = 0.0
                    airport_runway_overrun_secondary_length_calc = ""
                    airport_runway_overrun_secondary_width_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_overrun_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_overrun_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_overrun_secondary_length_calc
                        airport_runway_overrun_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_overrun_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_overrun_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_overrun_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_overrun_secondary_length_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_overrun_secondary_length_calc
                        airport_runway_overrun_secondary_width_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_overrun_secondary_width_calc
                      end
                      for step3 in 0..1
                        airport_runway_overrun_secondary_surface[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_overrun_secondary_length[step2] = airport_runway_overrun_secondary_length_calc.unpack('g')[0]
                    airport_runway_overrun_secondary_width[step2] = airport_runway_overrun_secondary_width_calc.unpack('g')[0]
                
                    bytes_to_read -= 16
                    runway_byte_offset += 16
                  elsif airport_runway_subsubrecord_id_calc == 11
                    airport_runway_vasi_primary_left_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_vasi_primary_left_size[step2] = 0
                    airport_runway_vasi_primary_left_type[step2] = 0
                    airport_runway_vasi_primary_left_biasx[step2] = 0.0
                    airport_runway_vasi_primary_left_biasz[step2] = 0.0
                    airport_runway_vasi_primary_left_spacing[step2] = 0.0
                    airport_runway_vasi_primary_left_pitch[step2] = 0.0
                    airport_runway_vasi_primary_left_biasx_calc = ""
                    airport_runway_vasi_primary_left_biasz_calc = ""
                    airport_runway_vasi_primary_left_spacing_calc = ""
                    airport_runway_vasi_primary_left_pitch_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_vasi_primary_left_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_vasi_primary_left_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_left_biasx_calc
                        airport_runway_vasi_primary_left_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_left_biasz_calc
                        airport_runway_vasi_primary_left_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_left_spacing_calc
                        airport_runway_vasi_primary_left_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_left_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_primary_left_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_vasi_primary_left_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_vasi_primary_left_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_left_biasx_calc
                        airport_runway_vasi_primary_left_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_left_biasz_calc
                        airport_runway_vasi_primary_left_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_left_spacing_calc
                        airport_runway_vasi_primary_left_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_left_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_primary_left_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_vasi_primary_left_biasx[step2] = airport_runway_vasi_primary_left_biasx_calc.unpack('g')[0]
                    airport_runway_vasi_primary_left_biasz[step2] = airport_runway_vasi_primary_left_biasz_calc.unpack('g')[0]
                    airport_runway_vasi_primary_left_spacing[step2] = airport_runway_vasi_primary_left_spacing_calc.unpack('g')[0]
                    airport_runway_vasi_primary_left_pitch[step2] = airport_runway_vasi_primary_left_pitch_calc.unpack('g')[0]

                    bytes_to_read -= 24
                    runway_byte_offset += 24
                  elsif airport_runway_subsubrecord_id_calc == 12
                    airport_runway_vasi_primary_right_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_vasi_primary_right_size[step2] = 0
                    airport_runway_vasi_primary_right_type[step2] = 0
                    airport_runway_vasi_primary_right_biasx[step2] = 0.0
                    airport_runway_vasi_primary_right_biasz[step2] = 0.0
                    airport_runway_vasi_primary_right_spacing[step2] = 0.0
                    airport_runway_vasi_primary_right_pitch[step2] = 0.0
                    airport_runway_vasi_primary_right_biasx_calc = ""
                    airport_runway_vasi_primary_right_biasz_calc = ""
                    airport_runway_vasi_primary_right_spacing_calc = ""
                    airport_runway_vasi_primary_right_pitch_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_vasi_primary_right_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_vasi_primary_right_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_right_biasx_calc
                        airport_runway_vasi_primary_right_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_right_biasz_calc
                        airport_runway_vasi_primary_right_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_right_spacing_calc
                        airport_runway_vasi_primary_right_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_primary_right_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_primary_right_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_vasi_primary_right_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_vasi_primary_right_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_right_biasx_calc
                        airport_runway_vasi_primary_right_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_right_biasz_calc
                        airport_runway_vasi_primary_right_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_right_spacing_calc
                        airport_runway_vasi_primary_right_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_primary_right_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_primary_right_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_vasi_primary_right_biasx[step2] = airport_runway_vasi_primary_right_biasx_calc.unpack('g')[0]
                    airport_runway_vasi_primary_right_biasz[step2] = airport_runway_vasi_primary_right_biasz_calc.unpack('g')[0]
                    airport_runway_vasi_primary_right_spacing[step2] = airport_runway_vasi_primary_right_spacing_calc.unpack('g')[0]
                    airport_runway_vasi_primary_right_pitch[step2] = airport_runway_vasi_primary_right_pitch_calc.unpack('g')[0]
    
                    bytes_to_read -= 24
                    runway_byte_offset += 24
                  elsif airport_runway_subsubrecord_id_calc == 13
                    airport_runway_vasi_secondary_left_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_vasi_secondary_left_size[step2] = 0
                    airport_runway_vasi_secondary_left_type[step2] = 0
                    airport_runway_vasi_secondary_left_biasx[step2] = 0.0
                    airport_runway_vasi_secondary_left_biasz[step2] = 0.0
                    airport_runway_vasi_secondary_left_spacing[step2] = 0.0
                    airport_runway_vasi_secondary_left_pitch[step2] = 0.0
                    airport_runway_vasi_secondary_left_biasx_calc = ""
                    airport_runway_vasi_secondary_left_biasz_calc = ""
                    airport_runway_vasi_secondary_left_spacing_calc = ""
                    airport_runway_vasi_secondary_left_pitch_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_vasi_secondary_left_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_vasi_secondary_left_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_left_biasx_calc
                        airport_runway_vasi_secondary_left_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_left_biasz_calc
                        airport_runway_vasi_secondary_left_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_left_spacing_calc
                        airport_runway_vasi_secondary_left_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_left_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_secondary_left_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_vasi_secondary_left_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_vasi_secondary_left_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_left_biasx_calc
                        airport_runway_vasi_secondary_left_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_left_biasz_calc
                        airport_runway_vasi_secondary_left_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_left_spacing_calc
                        airport_runway_vasi_secondary_left_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_left_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_secondary_left_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_vasi_secondary_left_biasx[step2] = airport_runway_vasi_secondary_left_biasx_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_left_biasz[step2] = airport_runway_vasi_secondary_left_biasz_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_left_spacing[step2] = airport_runway_vasi_secondary_left_spacing_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_left_pitch[step2] = airport_runway_vasi_secondary_left_pitch_calc.unpack('g')[0]
    
                    bytes_to_read -= 24
                    runway_byte_offset += 24
                  elsif airport_runway_subsubrecord_id_calc == 14
                    airport_runway_vasi_secondary_right_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_vasi_secondary_right_size[step2] = 0
                    airport_runway_vasi_secondary_right_type[step2] = 0
                    airport_runway_vasi_secondary_right_biasx[step2] = 0.0
                    airport_runway_vasi_secondary_right_biasz[step2] = 0.0
                    airport_runway_vasi_secondary_right_spacing[step2] = 0.0
                    airport_runway_vasi_secondary_right_pitch[step2] = 0.0
                    airport_runway_vasi_secondary_right_biasx_calc = ""
                    airport_runway_vasi_secondary_right_biasz_calc = ""
                    airport_runway_vasi_secondary_right_spacing_calc = ""
                    airport_runway_vasi_secondary_right_pitch_calc = ""

                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_vasi_secondary_right_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                        airport_runway_vasi_secondary_right_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_right_biasx_calc
                        airport_runway_vasi_secondary_right_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_right_biasz_calc
                        airport_runway_vasi_secondary_right_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_right_spacing_calc
                        airport_runway_vasi_secondary_right_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset] + airport_runway_vasi_secondary_right_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_secondary_right_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                    else
                      for step3 in 0..3
                        airport_runway_vasi_secondary_right_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                        airport_runway_vasi_secondary_right_biasx_calc = file_content[8 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_right_biasx_calc
                        airport_runway_vasi_secondary_right_biasz_calc = file_content[12 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_right_biasz_calc
                        airport_runway_vasi_secondary_right_spacing_calc = file_content[16 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_right_spacing_calc
                        airport_runway_vasi_secondary_right_pitch_calc = file_content[20 + runway_byte_offset + step3 + read_offset][0].chr + airport_runway_vasi_secondary_right_pitch_calc
                      end
                      for step3 in 0..1
                        airport_runway_vasi_secondary_right_type[step2] += file_content[6 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                    end

                    airport_runway_vasi_secondary_right_biasx[step2] = airport_runway_vasi_secondary_right_biasx_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_right_biasz[step2] = airport_runway_vasi_secondary_right_biasz_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_right_spacing[step2] = airport_runway_vasi_secondary_right_spacing_calc.unpack('g')[0]
                    airport_runway_vasi_secondary_right_pitch[step2] = airport_runway_vasi_secondary_right_pitch_calc.unpack('g')[0]

                    bytes_to_read -= 24
                    runway_byte_offset += 24
                  elsif airport_runway_subsubrecord_id_calc == 15
                    airport_runway_approach_lights_primary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_approach_lights_primary_size[step2] = 0
                    airport_runway_approach_lights_primary_system[step2] = 0
                    airport_runway_approach_lights_primary_strobes[step2] = 0
                    
                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_approach_lights_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                      airport_runway_approach_lights_primary_system[step2] = file_content[6 + runway_byte_offset + read_offset].ord
                      airport_runway_approach_lights_primary_strobes[step2] = file_content[7 + runway_byte_offset + read_offset].ord
                    else
                      for step3 in 0..3
                        airport_runway_approach_lights_primary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                      airport_runway_approach_lights_primary_system[step2] = file_content[6 + runway_byte_offset + read_offset][0]
                      airport_runway_approach_lights_primary_strobes[step2] = file_content[7 + runway_byte_offset + read_offset][0]
                    end

                    bytes_to_read -= 8
                    runway_byte_offset += 8
                  elsif airport_runway_subsubrecord_id_calc == 16
                    airport_runway_approach_lights_secondary_id[step2] = airport_runway_subsubrecord_id_calc
                    airport_runway_approach_lights_secondary_size[step2] = 0
                    airport_runway_approach_lights_secondary_system[step2] = 0
                    airport_runway_approach_lights_secondary_strobes[step2] = 0
                    
                    if $win32ole_ok == 1
                      for step3 in 0..3
                        airport_runway_approach_lights_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset].ord * 256 ** step3
                      end
                      airport_runway_approach_lights_secondary_system[step2] = file_content[6 + runway_byte_offset + read_offset].ord
                      airport_runway_approach_lights_secondary_strobes[step2] = file_content[7 + runway_byte_offset + read_offset].ord
                    else
                      for step3 in 0..3
                        airport_runway_approach_lights_secondary_size[step2] += file_content[2 + runway_byte_offset + step3 + read_offset][0] * 256 ** step3
                      end
                      airport_runway_approach_lights_secondary_system[step2] = file_content[6 + runway_byte_offset + read_offset][0]
                      airport_runway_approach_lights_secondary_strobes[step2] = file_content[7 + runway_byte_offset + read_offset][0]
                    end

                    bytes_to_read -= 8
                    runway_byte_offset += 8
                  else
                    echo "Unknown runway sub sub record." + airport_runway_subsubrecord_id_calc.to_s
                    bytes_to_read = 0  
                  end
             
                end
              end

          
              ################################
              ### Draw and relocate runway ###
              ################################

              pts = []
              pts[0] = [0, 0, 0]
              pts[1] = [airport_runway_length[step2] * 100 / $inch, 0, 0]
              pts[2] = [airport_runway_length[step2] * 100 / $inch, airport_runway_width[step2] * 100 / $inch, 0]
              pts[3] = [0, airport_runway_width[step2] * 100 / $inch, 0]
              face = airport_runway_group_ents[step2].add_face pts
              status = face.reverse!
          
              trans = airport_runway_group[step2].transformation
              runway_new_x_pos = airport_runway_x_pos[step2] + trans.origin[0] - (airport_runway_length[step2] * 100 / $inch) / 2
              runway_new_y_pos = airport_runway_y_pos[step2] + trans.origin[1] - (airport_runway_width[step2] * 100 / $inch) / 2
              point = Geom::Point3d.new(runway_new_x_pos, runway_new_y_pos, trans.origin[2])
              t = Geom::Transformation.new point
              airport_runway_group[step2].move! t
              runway_angle = (450 - airport_runway_heading[step2]).modulo(360).degrees
              t = Geom::Transformation.rotation [airport_runway_x_pos[step2], airport_runway_y_pos[step2], 0], Z_AXIS, runway_angle
              airport_runway_group[step2].transform! t

              section_bytes_to_read -= airport_runway_subrecord_size[step2] 
              read_offset += airport_runway_subrecord_size[step2] 
            end     

          elsif temp_id == 17 #### Starts ####
            airport_start_id = []
            airport_start_size = []
            airport_start_runway = []
            airport_start_designator_type = []
            airport_start_longitude = []
            airport_start_latitude = []
            airport_start_elevation = []
            airport_start_heading = []
            airport_start_x_pos =[]
            airport_start_y_pos =[]
            airport_start_group = []
            airport_start_group_ents = []

            for step2 in 0..airport_start_records - 1
              airport_start_group[step2] = Sketchup.active_model.entities.add_group
              airport_start_group_ents[step2] = airport_start_group[step2].entities
              airport_start_id[step2] = 0
              airport_start_size[step2] = 0
              airport_start_runway[step2] = 0
              airport_start_designator_type[step2] = 0
              airport_start_longitude[step2] = 0.0
              airport_start_latitude[step2] = 0.0
              airport_start_elevation[step2] = 0.0
              airport_start_heading[step2] = 0.0
              airport_start_x_pos[step2] = 0.0
              airport_start_y_pos[step2] = 0.0
              airport_start_heading_calc = ""
              airport_start_id[step2] = temp_id
              
              if $win32ole_ok == 1
                airport_start_runway[step2] = file_content[6 + read_offset].ord
                airport_start_designator_type[step2] = file_content[7 + read_offset].ord
                for step3 in 0..3
                  airport_start_size[step2] += file_content[2 + step3 + read_offset].ord * 256 ** step3
                  airport_start_longitude[step2] += file_content[8 + step3 + read_offset].ord * 256 ** step3
                  airport_start_latitude[step2] += file_content[12 + step3 + read_offset].ord * 256 ** step3
                  airport_start_elevation[step2] += file_content[16 + step3 + read_offset].ord * 256 ** step3
                  airport_start_heading_calc = file_content[20 + step3 + read_offset] + airport_start_heading_calc
                end
              else   
                airport_start_runway[step2] = file_content[6 + read_offset]
                airport_start_designator_type[step2] = file_content[7 + read_offset]
                for step3 in 0..3
                  airport_start_size[step2] += file_content[2 + step3 + read_offset][0] * 256 ** step3
                  airport_start_longitude[step2] += file_content[8 + step3 + read_offset][0] * 256 ** step3
                  airport_start_latitude[step2] += file_content[12 + step3 + read_offset][0] * 256 ** step3
                  airport_start_elevation[step2] += file_content[16 + step3 + read_offset][0] * 256 ** step3
                  airport_start_heading_calc = file_content[20 + step3 + read_offset][0].chr + airport_start_heading_calc
                end
              end
 
              airport_start_longitude[step2] = airport_start_longitude[step2] * 360 / (3 * 268435456) - 180
              airport_start_latitude[step2] = 90 - (airport_start_latitude[step2] * 180 / (2 * 268435456))
              airport_start_elevation[step2] = airport_start_elevation[step2] / 1000
              airport_start_heading[step2] = airport_start_heading_calc.unpack('g')[0]
  
              if airport_start_longitude[step2] > airport_longitude
                airport_start_x_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_start_longitude[step2]) / $inch
              else
                airport_start_x_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_start_longitude[step2]) / $inch
              end
              if airport_start_latitude[step2] > airport_latitude
                airport_start_y_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_start_latitude[step2], airport_longitude) / $inch
              else
                airport_start_y_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_start_latitude[step2], airport_longitude) / $inch
              end
              
              centerpoint = Geom::Point3d.new(airport_start_x_pos[step2], airport_start_y_pos[step2], 0)
              pts = []
              pts[0] = [airport_start_x_pos[step2], airport_start_y_pos[step2], 0]
              pts[1] = [airport_start_x_pos[step2] + 1200 / $inch, airport_start_y_pos[step2], 0]
              vector = Geom::Vector3d.new 0,0,-1
              vector2 = vector.normalize!
              model = Sketchup.active_model
              circle = airport_start_group_ents[step2].add_circle centerpoint, vector2, 1200 / $inch
              edge = airport_start_group_ents[step2].add_edges pts[0], pts[1]
              start_angle = (450 - airport_start_heading[step2]).modulo(360).degrees
              t = Geom::Transformation.rotation [airport_start_x_pos[step2], airport_start_y_pos[step2], 0], Z_AXIS, start_angle
              airport_start_group[step2].transform! t
        
              section_bytes_to_read -= 24
              read_offset += 24
            end
          elsif temp_id == 18 ### COM ###
            airport_com_id = []
            airport_com_size = []
            airport_com_type = []
            airport_com_frequency = []
            airport_com_name = []
            
            for step2 in 0..airport_com_records - 1
              airport_com_id[step2] = 0
              airport_com_size[step2] = 0
              airport_com_type[step2] = 0
              airport_com_frequency[step2] = 0
              airport_com_name[step2] = ""
              airport_com_id[step2] = temp_id

              if $win32ole_ok == 1
                for step3 in 0..3
                  airport_com_size[step2] += file_content[2 + step3 + read_offset].ord * 256 ** step3
                  airport_com_frequency[step2] += file_content[8 + step3 + read_offset].ord * 256 ** step3
                end
                for step3 in 0..1
                  airport_com_type[step2] += file_content[6 + step3 + read_offset].ord * 256 ** step3
                end
                for step3 in 0..airport_com_size[step2] - 13
                  airport_com_name[step2] += file_content[12 + step3 + read_offset]
                end
              else
                for step3 in 0..3
                  airport_com_size[step2] += file_content[2 + step3 + read_offset][0] * 256 ** step3
                  airport_com_frequency[step2] += file_content[8 + step3 + read_offset][0] * 256 ** step3
                end
                for step3 in 0..1
                  airport_com_type[step2] += file_content[6 + step3 + read_offset][0] * 256 ** step3
                end
                for step3 in 0..airport_com_size[step2] - 13
                  airport_com_name[step2] += file_content[12 + step3 + read_offset][0].chr
                end
              end
              
      #        echo airport_com_size[step2]
              section_bytes_to_read -= airport_com_size[step2]
              read_offset += airport_com_size[step2]
              
            end

          elsif temp_id == 26 #### Taxiway points ####
            airport_taxiway_point_id = temp_id
            airport_taxiway_point_size = 0
            airport_taxiway_point_count = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_taxiway_point_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiway_point_count += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_taxiway_point_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiway_point_count += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            airport_taxiway_point_type = []
            airport_taxiway_point_flag = []
            airport_taxiway_point_unknown = []
            airport_taxiway_point_longitude = []
            airport_taxiway_point_latitude = []
            airport_taxiway_point_x_pos = []
            airport_taxiway_point_y_pos = []
            airport_taxiway_point_group = []
            airport_taxiway_point_group_ents = []

            for step2 in 0..airport_taxiway_point_count - 1
              airport_taxiway_point_group[step2] = Sketchup.active_model.entities.add_group
              airport_taxiway_point_group_ents[step2] = airport_taxiway_point_group[step2].entities
              airport_taxiway_point_type[step2] = 0
              airport_taxiway_point_flag[step2] = 0
              airport_taxiway_point_unknown[step2] = 0
              airport_taxiway_point_longitude[step2] = 0.0
              airport_taxiway_point_latitude[step2] = 0.0
              
              if $win32ole_ok == 1
                airport_taxiway_point_type[step2] = file_content[8 + (step2 * 12) + read_offset].ord
                airport_taxiway_point_flag[step2] = file_content[9 + (step2 * 12) + read_offset].ord
                for step3 in 0..1
                  airport_taxiway_point_unknown[step2] += file_content[10 + step3 + (step2 * 12) + read_offset].ord * 256 ** step3
                end
                for step3 in 0..3
                  airport_taxiway_point_longitude[step2] += file_content[12 + step3 + (step2 * 12) + read_offset].ord * 256 ** step3
                  airport_taxiway_point_latitude[step2] += file_content[16 + step3 + (step2 * 12) + read_offset].ord * 256 ** step3
                end
              else
                airport_taxiway_point_type[step2] = file_content[8 + (step2 * 12) + read_offset][0]
                airport_taxiway_point_flag[step2] = file_content[9 + (step2 * 12) + read_offset][0]
                for step3 in 0..1
                  airport_taxiway_point_unknown[step2] += file_content[10 + step3 + (step2 * 12) + read_offset][0] * 256 ** step3
                end
                for step3 in 0..3
                  airport_taxiway_point_longitude[step2] += file_content[12 + step3 + (step2 * 12) + read_offset][0] * 256 ** step3
                  airport_taxiway_point_latitude[step2] += file_content[16 + step3 + (step2 * 12) + read_offset][0] * 256 ** step3
                end
              end
              
              airport_taxiway_point_longitude[step2] = airport_taxiway_point_longitude[step2] * 360 / (3 * 268435456) - 180
              airport_taxiway_point_latitude[step2] = 90 - (airport_taxiway_point_latitude[step2] * 180 / (2 * 268435456))
              if airport_taxiway_point_longitude[step2] > airport_longitude
                airport_taxiway_point_x_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_taxiway_point_longitude[step2]) / $inch
              else
                airport_taxiway_point_x_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_taxiway_point_longitude[step2]) / $inch
              end
              if airport_taxiway_point_latitude[step2] > airport_latitude
                airport_taxiway_point_y_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_taxiway_point_latitude[step2], airport_longitude) / $inch
              else
                airport_taxiway_point_y_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_taxiway_point_latitude[step2], airport_longitude) / $inch
              end
              
              if airport_taxiway_point_x_pos[step2] > 200000 or airport_taxiway_point_x_pos[step2] < -200000
                echo "ERROR:" + step2.to_s + "  " + airport_taxiway_point_latitude[step2].to_s
                break
              end
              centerpoint = Geom::Point3d.new(airport_taxiway_point_x_pos[step2], airport_taxiway_point_y_pos[step2], 0)
              vector = Geom::Vector3d.new 0,0,-1
              vector2 = vector.normalize!
              model = Sketchup.active_model
              circle = airport_taxiway_point_group_ents[step2].add_circle centerpoint, vector2, 100 / $inch
            end
            
            section_bytes_to_read -= airport_taxiway_point_size
            read_offset += airport_taxiway_point_size
          elsif temp_id == 61 #### Parking spots, 61 for FSX, 27 for FS9 ####
            airport_parking_id = temp_id
            airport_parking_size = 0
            airport_parking_count = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_parking_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                airport_parking_count += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_parking_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                airport_parking_count += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            airport_parking_temp_values = []
            airport_parking_airline_codes = []
            airport_parking_radius = []
            airport_parking_heading = []
            airport_parking_tee_offset1 = []
            airport_parking_tee_offset2 = []
            airport_parking_tee_offset3 = []
            airport_parking_tee_offset4 = []
            airport_parking_longitude = []
            airport_parking_latitude = []
            airport_parking_airline_designator = []
            airport_parking_x_pos = []
            airport_parking_y_pos = []
            airport_parking_group = []
            airport_parking_group_ents = []
            parking_read_offset = 8
            for step2 in 0..airport_parking_count - 1
              airport_parking_temp_values[step2] = 0
              airport_parking_airline_codes[step2] = 0
              airport_parking_radius[step2] = 0.0
              airport_parking_heading[step2] = 0.0
              airport_parking_tee_offset1[step2] = 0.0
              airport_parking_tee_offset2[step2] = 0.0
              airport_parking_tee_offset3[step2] = 0.0
              airport_parking_tee_offset4[step2] = 0.0
              airport_parking_longitude[step2] = 0.0
              airport_parking_latitude[step2] = 0.0
              airport_parking_airline_designator[step2] = []
              airport_parking_group[step2] = Sketchup.active_model.entities.add_group
              airport_parking_group_ents[step2] = airport_parking_group[step2].entities
              airport_parking_radius_calc = ""
              airport_parking_heading_calc = ""
              airport_parking_tee_offset1_calc = ""
              airport_parking_tee_offset2_calc = ""
              airport_parking_tee_offset3_calc = ""
              airport_parking_tee_offset4_calc = ""
              
              if $win32ole_ok == 1
                for step3 in 0..2 ###### Temporary values to calculate parking number, type, pushback and name
                  airport_parking_temp_values[step2] += file_content[parking_read_offset + step3 + read_offset].ord * 256 ** step3
                end
                airport_parking_airline_codes[step2] = file_content[3 + parking_read_offset + read_offset].ord
                for step3 in 0..3
                  airport_parking_radius_calc = file_content[4 + parking_read_offset + step3 + read_offset] + airport_parking_radius_calc
                  airport_parking_heading_calc = file_content[8 + parking_read_offset + step3 + read_offset] + airport_parking_heading_calc
                  airport_parking_tee_offset1_calc = file_content[12 + parking_read_offset + step3 + read_offset] + airport_parking_tee_offset1_calc
                  airport_parking_tee_offset2_calc = file_content[16 + parking_read_offset + step3 + read_offset] + airport_parking_tee_offset2_calc
                  airport_parking_tee_offset3_calc = file_content[20 + parking_read_offset + step3 + read_offset] + airport_parking_tee_offset3_calc
                  airport_parking_tee_offset4_calc = file_content[24 + parking_read_offset + step3 + read_offset] + airport_parking_tee_offset4_calc
                  airport_parking_longitude[step2] += file_content[28 + parking_read_offset + step3 + read_offset].ord * 256 ** step3
                  airport_parking_latitude[step2] += file_content[32 + parking_read_offset + step3 + read_offset].ord * 256 ** step3
                end
                for step3 in 0..airport_parking_airline_codes[step2] - 1
                  airport_parking_airline_designator[step2][step3] = ""
                  for step4 in 0..3
                    airport_parking_airline_designator[step2][step3] += file_content[36 + (step3 * 4) + parking_read_offset + step3 + read_offset]
                  end
                end
              else
                for step3 in 0..2 ###### Temporary values to calculate parking number, type, pushback and name
                  airport_parking_temp_values[step2] += file_content[parking_read_offset + step3 + read_offset][0] * 256 ** step3
                end
                airport_parking_airline_codes[step2] = file_content[3 + parking_read_offset + read_offset][0]
                for step3 in 0..3
                  airport_parking_radius_calc = file_content[4 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_radius_calc
                  airport_parking_heading_calc = file_content[8 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_heading_calc
                  airport_parking_tee_offset1_calc = file_content[12 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_tee_offset1_calc
                  airport_parking_tee_offset2_calc = file_content[16 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_tee_offset2_calc
                  airport_parking_tee_offset3_calc = file_content[20 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_tee_offset3_calc
                  airport_parking_tee_offset4_calc = file_content[24 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_tee_offset4_calc
                  airport_parking_longitude[step2] += file_content[28 + parking_read_offset + step3 + read_offset][0] * 256 ** step3
                  airport_parking_latitude[step2] += file_content[32 + parking_read_offset + step3 + read_offset][0] * 256 ** step3
                end
                for step3 in 0..airport_parking_airline_codes[step2] - 1
                  airport_parking_airline_designator[step2][step3] = ""
                  for step4 in 0..3
                    airport_parking_airline_designator[step2][step3] += file_content[36 + (step3 * 4) + parking_read_offset + step3 + read_offset].chr
                  end
                end
              end
              
              airport_parking_radius[step2] = airport_parking_radius_calc.unpack('g')[0]
              airport_parking_heading[step2] = airport_parking_heading_calc.unpack('g')[0]
              airport_parking_tee_offset1[step2] = airport_parking_tee_offset1_calc.unpack('g')[0]
              airport_parking_tee_offset2[step2] = airport_parking_tee_offset2_calc.unpack('g')[0]
              airport_parking_tee_offset3[step2] = airport_parking_tee_offset3_calc.unpack('g')[0]
              airport_parking_tee_offset4[step2] = airport_parking_tee_offset4_calc.unpack('g')[0]
              airport_parking_longitude[step2] = airport_parking_longitude[step2] * 360 / (3 * 268435456) - 180
              airport_parking_latitude[step2] = 90 - (airport_parking_latitude[step2] * 180 / (2 * 268435456))
              if airport_parking_longitude[step2] > airport_longitude
                airport_parking_x_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_parking_longitude[step2]) / $inch
              else
                airport_parking_x_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_parking_longitude[step2]) / $inch
              end
              if airport_parking_latitude[step2] > airport_latitude
                airport_parking_y_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_parking_latitude[step2], airport_longitude) / $inch
              else
                airport_parking_y_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_parking_latitude[step2], airport_longitude) / $inch
              end
              centerpoint = Geom::Point3d.new(airport_parking_x_pos[step2], airport_parking_y_pos[step2], 0)
              vector = Geom::Vector3d.new 0,0,-1
              vector2 = vector.normalize!
              model = Sketchup.active_model
              circle = airport_parking_group_ents[step2].add_circle centerpoint, vector2, airport_parking_radius[step2] * 100 / $inch
              airport_parking_group[step2].description = "Parking|" + airport_parking_radius[step2].to_s + "|" + airport_parking_x_pos[step2].to_s + "|" + airport_parking_y_pos[step2].to_s + "|" + airport_parking_heading[step2].to_s
              parking_read_offset += 36 + (airport_parking_airline_codes[step2] * 4)
            end
            
            section_bytes_to_read -= airport_parking_size
            read_offset += airport_parking_size
          elsif temp_id == 27 #### Parking spots, 61 for FSX, 27 for FS9 ####
            airport_parking_id = temp_id
            airport_parking_size = 0
            airport_parking_count = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_parking_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                airport_parking_count += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_parking_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                airport_parking_count += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            airport_parking_temp_values = []
            airport_parking_airline_codes = []
            airport_parking_radius = []
            airport_parking_heading = []
            airport_parking_longitude = []
            airport_parking_latitude = []
            airport_parking_airline_designator = []
            airport_parking_x_pos = []
            airport_parking_y_pos = []
            airport_parking_group = []
            airport_parking_group_ents = []
            parking_read_offset = 8
            for step2 in 0..airport_parking_count - 1
              airport_parking_temp_values[step2] = 0
              airport_parking_airline_codes[step2] = 0
              airport_parking_radius[step2] = 0.0
              airport_parking_heading[step2] = 0.0
              airport_parking_longitude[step2] = 0.0
              airport_parking_latitude[step2] = 0.0
              airport_parking_airline_designator[step2] = []
              airport_parking_group[step2] = Sketchup.active_model.entities.add_group
              airport_parking_group_ents[step2] = airport_parking_group[step2].entities
              airport_parking_radius_calc = ""
              airport_parking_heading_calc = ""
              
              if $win32ole_ok == 1
                for step3 in 0..2 ###### Temporary values to calculate parking number, type, pushback and name
                  airport_parking_temp_values[step2] += file_content[parking_read_offset + step3 + read_offset].ord * 256 ** step3
                end
                airport_parking_airline_codes[step2] = file_content[3 + parking_read_offset + read_offset].ord
                for step3 in 0..3
                  airport_parking_radius_calc = file_content[4 + parking_read_offset + step3 + read_offset] + airport_parking_radius_calc
                  airport_parking_heading_calc = file_content[8 + parking_read_offset + step3 + read_offset] + airport_parking_heading_calc
                  airport_parking_longitude[step2] += file_content[12 + parking_read_offset + step3 + read_offset].ord * 256 ** step3
                  airport_parking_latitude[step2] += file_content[16 + parking_read_offset + step3 + read_offset].ord * 256 ** step3
                end
                for step3 in 0..airport_parking_airline_codes[step2] - 1
                  airport_parking_airline_designator[step2][step3] = ""
                  for step4 in 0..3
                    airport_parking_airline_designator[step2][step3] += file_content[20 + (step3 * 4) + parking_read_offset + step3 + read_offset]
                  end
                end
              else
                for step3 in 0..2 ###### Temporary values to calculate parking number, type, pushback and name
                  airport_parking_temp_values[step2] += file_content[parking_read_offset + step3 + read_offset][0] * 256 ** step3
                end
                airport_parking_airline_codes[step2] = file_content[3 + parking_read_offset + read_offset][0]
                for step3 in 0..3
                  airport_parking_radius_calc = file_content[4 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_radius_calc
                  airport_parking_heading_calc = file_content[8 + parking_read_offset + step3 + read_offset][0].chr + airport_parking_heading_calc
                  airport_parking_longitude[step2] += file_content[12 + parking_read_offset + step3 + read_offset][0] * 256 ** step3
                  airport_parking_latitude[step2] += file_content[16 + parking_read_offset + step3 + read_offset][0] * 256 ** step3
                end
                for step3 in 0..airport_parking_airline_codes[step2] - 1
                  airport_parking_airline_designator[step2][step3] = ""
                  for step4 in 0..3
                    airport_parking_airline_designator[step2][step3] += file_content[20 + (step3 * 4) + parking_read_offset + step3 + read_offset][0].chr
                  end
                end
              end
              
              airport_parking_radius[step2] = airport_parking_radius_calc.unpack('g')[0]
              airport_parking_heading[step2] = airport_parking_heading_calc.unpack('g')[0]
              airport_parking_longitude[step2] = airport_parking_longitude[step2] * 360 / (3 * 268435456) - 180
              airport_parking_latitude[step2] = 90 - (airport_parking_latitude[step2] * 180 / (2 * 268435456))
              if airport_parking_longitude[step2] > airport_longitude
                airport_parking_x_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_parking_longitude[step2]) / $inch
              else
                airport_parking_x_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_parking_longitude[step2]) / $inch
              end
              if airport_parking_latitude[step2] > airport_latitude
                airport_parking_y_pos[step2] = calculate_distance(airport_latitude, airport_longitude, airport_parking_latitude[step2], airport_longitude) / $inch
              else
                airport_parking_y_pos[step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_parking_latitude[step2], airport_longitude) / $inch
              end
              centerpoint = Geom::Point3d.new(airport_parking_x_pos[step2], airport_parking_y_pos[step2], 0)
              vector = Geom::Vector3d.new 0,0,-1
              vector2 = vector.normalize!
              model = Sketchup.active_model
              circle = airport_parking_group_ents[step2].add_circle centerpoint, vector2, airport_parking_radius[step2] * 100 / $inch
              airport_parking_group[step2].description = "Parking|" + airport_parking_radius[step2].to_s + "|" + airport_parking_x_pos[step2].to_s + "|" + airport_parking_y_pos[step2].to_s + "|" + airport_parking_heading[step2].to_s
              parking_read_offset += 20 + (airport_parking_airline_codes[step2] * 4)
            end
            
            section_bytes_to_read -= airport_parking_size
            read_offset += airport_parking_size
          elsif temp_id == 28 #### Taxiway path ####
            airport_taxiway_path_id = temp_id
            airport_taxiway_path_size = 0
            airport_taxiway_path_count = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_taxiway_path_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiway_path_count += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_taxiway_path_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiway_path_count += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            airport_taxiway_path_start_index = []
            airport_taxiway_path_end_index = []
            airport_taxiway_path_runway_designator = []
            airport_taxiway_path_type = []
            airport_taxiway_path_runway_number = []
            airport_taxiway_path_bitfield = []
            airport_taxiway_path_surface = []
            airport_taxiway_path_width = []
            airport_taxiway_path_weight_limit = []
            airport_taxiway_path_unknown = []
            airport_taxiway_path_length = []
            airport_taxiway_path_group = []
            airport_taxiway_path_group_ents = []
            
            for step2 in 0..airport_taxiway_path_count - 1
              airport_taxiway_path_group[step2] = Sketchup.active_model.entities.add_group
              airport_taxiway_path_group_ents[step2] = airport_taxiway_path_group[step2].entities
              airport_taxiway_path_start_index[step2] = 0
              airport_taxiway_path_end_index[step2] = 0
              airport_taxiway_path_runway_designator[step2] = 0
              airport_taxiway_path_type[step2] = 0
              airport_taxiway_path_runway_number[step2] = 0
              airport_taxiway_path_bitfield[step2] = 0
              airport_taxiway_path_surface[step2] = 0
              airport_taxiway_path_width[step2] = 0.0
              airport_taxiway_path_weight_limit[step2] = 0
              airport_taxiway_path_unknown[step2] = 0
              airport_taxiway_path_end_runway_calc = 0
              airport_taxiway_path_width_calc = ""

              if $win32ole_ok == 1
                for step3 in 0..1
                  airport_taxiway_path_start_index[step2] += file_content[8 + step3 + (step2 * 20) + read_offset].ord * 256 ** step3
                  airport_taxiway_path_end_runway_calc += file_content[10 + step3 + (step2 * 20) + read_offset].ord * 256 ** step3
                end
                #### Path type is one byte in FSX AFCADs, but only a number of bits in FS9 AFCADs ####
                airport_taxiway_path_type[step2] = file_content[12 + (step2 * 20) + read_offset].ord
                airport_taxiway_path_runway_number[step2] = file_content[13 + (step2 * 20) + read_offset].ord
                airport_taxiway_path_bitfield[step2] = file_content[14 + (step2 * 20) + read_offset].ord
                airport_taxiway_path_surface[step2] = file_content[15 + (step2 * 20) + read_offset].ord
                for step3 in 0..3
                  airport_taxiway_path_width_calc = file_content[16 + step3 + (step2 * 20) + read_offset] + airport_taxiway_path_width_calc
                  airport_taxiway_path_weight_limit[step2] += file_content[20 + step3 + (step2 * 20) + read_offset].ord * 256 ** step3
                  airport_taxiway_path_unknown[step2] += file_content[24 + step3 + (step2 * 20) + read_offset].ord * 256 ** step3
                end
              else
                for step3 in 0..1
                  airport_taxiway_path_start_index[step2] += file_content[8 + step3 + (step2 * 20) + read_offset][0] * 256 ** step3
                  airport_taxiway_path_end_runway_calc += file_content[10 + step3 + (step2 * 20) + read_offset][0] * 256 ** step3
                end
                airport_taxiway_path_type[step2] = file_content[12 + (step2 * 20) + read_offset][0]
                airport_taxiway_path_runway_number[step2] = file_content[13 + (step2 * 20) + read_offset][0]
                airport_taxiway_path_bitfield[step2] = file_content[14 + (step2 * 20) + read_offset][0]
                airport_taxiway_path_surface[step2] = file_content[15 + (step2 * 20) + read_offset][0]
                for step3 in 0..3
                  airport_taxiway_path_width_calc = file_content[16 + step3 + (step2 * 20) + read_offset][0].chr + airport_taxiway_path_width_calc
                  airport_taxiway_path_weight_limit[step2] += file_content[20 + step3 + (step2 * 20) + read_offset][0] * 256 ** step3
                  airport_taxiway_path_unknown[step2] += file_content[24 + step3 + (step2 * 20) + read_offset][0] * 256 ** step3
                end
              end
              
              airport_taxiway_path_end_index[step2] = airport_taxiway_path_end_runway_calc.modulo(4096)
              airport_taxiway_path_runway_designator[step2] = (airport_taxiway_path_end_runway_calc - airport_taxiway_path_end_index[step2]) / 4096
              airport_taxiway_path_width[step2] = airport_taxiway_path_width_calc.unpack('g')[0] * 100 / $inch
              
           #   airport_taxiway_path_group[step2].description = "Apron" + airport_taxiway_path_type[step2].to_s
              if airport_taxiway_path_type[step2] == 1 or airport_taxiway_path_type[step2] == 97 ### Taxiway, 1=FSX, 97=FS9 ###
                pts = []
                x_difference = airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]] - airport_taxiway_point_x_pos[airport_taxiway_path_end_index[step2]]
                y_difference = airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]] - airport_taxiway_point_y_pos[airport_taxiway_path_end_index[step2]]
                airport_taxiway_path_length[step2] = Math.sqrt((x_difference * x_difference) + (y_difference * y_difference))
                airport_taxiway_path_angle = Math.atan((y_difference.abs) / (x_difference.abs)).radians
                pts = []
                pts[0] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]] - airport_taxiway_path_width[step2] / 2, 0]
                pts[1] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]] + airport_taxiway_path_width[step2] / 2, 0]
                pts[2] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]] + airport_taxiway_path_length[step2], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]] + airport_taxiway_path_width[step2] / 2, 0]
                pts[3] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]] + airport_taxiway_path_length[step2], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]] - airport_taxiway_path_width[step2] / 2, 0]
                face = airport_taxiway_path_group_ents[step2].add_face pts
                status = face.reverse!
                if x_difference < 0 and y_difference > 0
                  rotate_angle = (0 -  airport_taxiway_path_angle).degrees
                elsif x_difference > 0 and y_difference < 0
                  rotate_angle = (180 - airport_taxiway_path_angle).degrees
                elsif x_difference > 0 and y_difference > 0
                  rotate_angle = (180 + airport_taxiway_path_angle).degrees
                else
                  rotate_angle = airport_taxiway_path_angle.degrees
                end
                t = Geom::Transformation.rotation [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]], 0], Z_AXIS, rotate_angle
                airport_taxiway_path_group[step2].transform! t
              elsif airport_taxiway_path_type[step2] == 2 or airport_taxiway_path_type[step2] == 4 or airport_taxiway_path_type[step2] == 98 or airport_taxiway_path_type[step2] == 100 ### Taxiway, apron and runway links ###
                pts = []
                pts[0] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]], 0]
                pts[1] = [airport_taxiway_point_x_pos[airport_taxiway_path_end_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_end_index[step2]], 0]
                line = airport_taxiway_path_group_ents[step2].add_edges pts[0], pts[1]
              elsif airport_taxiway_path_type[step2] == 3 or airport_taxiway_path_type[step2] == 99 ### Parking links ###
                pts = []
                pts[0] = [airport_taxiway_point_x_pos[airport_taxiway_path_start_index[step2]], airport_taxiway_point_y_pos[airport_taxiway_path_start_index[step2]], 0]
                pts[1] = [airport_parking_x_pos[airport_taxiway_path_end_index[step2]], airport_parking_y_pos[airport_taxiway_path_end_index[step2]], 0]
                line = airport_taxiway_path_group_ents[step2].add_edges pts[0], pts[1]
              else
  #              echo airport_taxiway_path_type[step2]
    #            echo airport_taxiway_path_start_index[step2].to_s + " - " + airport_taxiway_path_end_index[step2].to_s
              end
            end

            section_bytes_to_read -= airport_taxiway_path_size
            read_offset += airport_taxiway_path_size
          elsif temp_id == 29 ### Taxiname ###
            airport_taxiname_id = temp_id
            airport_taxiname_size = 0
            airport_taxiname_entries = 0
            airport_taxiname = []
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_taxiname_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiname_entries += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_taxiname_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..1
                airport_taxiname_entries += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end

            for step2 in 0..airport_taxiname_entries - 1
              airport_taxiname[step2] = ""
              for step3 in 0..7
                if $win32ole_ok == 1
                  airport_taxiname[step2] += file_content[8 + (step2 * 8) + step3 + read_offset]
                else
                  airport_taxiname[step2] += file_content[8 + (step2 * 8) + step3 + read_offset][0].chr
                end
              end
            end
            section_bytes_to_read -= airport_taxiname_size
            read_offset += airport_taxiname_size
          elsif temp_id == 55 ### Aprons ###
            if airport_current_apron === 0
              airport_apron_id = []
              airport_apron_size = []
              airport_apron_surface = []
              airport_apron_vertice_count = []
              airport_apron_longitude = []
              airport_apron_latitude = []
              airport_apron_x_pos = []
              airport_apron_y_pos = []
              airport_apron_pts = []
              airport_apron_group = []
              airport_apron_group_ents = []
            end
            airport_apron_id[airport_current_apron] = temp_id
            airport_apron_size[airport_current_apron] = 0
            airport_apron_surface[airport_current_apron] = 0
            airport_apron_vertice_count[airport_current_apron] = 0
            airport_apron_longitude[airport_current_apron] = []
            airport_apron_latitude[airport_current_apron] = []
            airport_apron_x_pos[airport_current_apron] = []
            airport_apron_y_pos[airport_current_apron] = []
            airport_apron_pts[airport_current_apron] = []
            airport_apron_group[airport_current_apron] = Sketchup.active_model.entities.add_group
            airport_apron_group_ents[airport_current_apron] = airport_apron_group[airport_current_apron].entities
            airport_apron_group[airport_current_apron].description = "Apron" + read_offset.to_s
            
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_apron_size[airport_current_apron] += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
              airport_apron_surface[airport_current_apron] = file_content[6 + read_offset].ord
              for step2 in 0..1
                airport_apron_vertice_count[airport_current_apron] += file_content[7 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_apron_size[airport_current_apron] += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
              airport_apron_surface[airport_current_apron] = file_content[6 + read_offset][0]
              for step2 in 0..1
                airport_apron_vertice_count[airport_current_apron] += file_content[7 + step2 + read_offset][0] * 256 ** step2
              end
            end

            for step2 in 0..airport_apron_vertice_count[airport_current_apron] - 1
              airport_apron_longitude[airport_current_apron][step2] = 0.0
              airport_apron_latitude[airport_current_apron][step2] = 0.0
              if $win32ole_ok == 1
                for step3 in 0..3
                  airport_apron_longitude[airport_current_apron][step2] += file_content[9 + step3 + (step2 * 8) + read_offset].ord * 256 ** step3
                  airport_apron_latitude[airport_current_apron][step2] += file_content[13 + step3 + (step2 * 8) + read_offset].ord * 256 ** step3
                end
              else
                for step3 in 0..3
                  airport_apron_longitude[airport_current_apron][step2] += file_content[9 + step3 + (step2 * 8) + read_offset][0] * 256 ** step3
                  airport_apron_latitude[airport_current_apron][step2] += file_content[13 + step3 + (step2 * 8) + read_offset][0] * 256 ** step3
                end
              end
              
              airport_apron_longitude[airport_current_apron][step2] = airport_apron_longitude[airport_current_apron][step2] * 360 / (3 * 268435456) - 180
              airport_apron_latitude[airport_current_apron][step2] = 90 - (airport_apron_latitude[airport_current_apron][step2] * 180 / (2 * 268435456))
              
              if airport_apron_longitude[airport_current_apron][step2] > airport_longitude
                airport_apron_x_pos[airport_current_apron][step2] = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_apron_longitude[airport_current_apron][step2]) / $inch
              else
                airport_apron_x_pos[airport_current_apron][step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_apron_longitude[airport_current_apron][step2]) / $inch
              end
              if airport_apron_latitude[airport_current_apron][step2] > airport_latitude
                airport_apron_y_pos[airport_current_apron][step2] = calculate_distance(airport_latitude, airport_longitude, airport_apron_latitude[airport_current_apron][step2], airport_longitude) / $inch
              else
                airport_apron_y_pos[airport_current_apron][step2] = 0 - calculate_distance(airport_latitude, airport_longitude, airport_apron_latitude[airport_current_apron][step2], airport_longitude) / $inch
              end
              airport_apron_pts[airport_current_apron][step2] = [airport_apron_x_pos[airport_current_apron][step2], airport_apron_y_pos[airport_current_apron][step2], 0]
        #      echo read_offset.to_s + " " + airport_apron_pts[airport_current_apron][step2].to_s
            end


            face = airport_apron_group_ents[airport_current_apron].add_face airport_apron_pts[airport_current_apron]
            status = face.reverse!
            section_bytes_to_read -= airport_apron_size[airport_current_apron]
            read_offset += airport_apron_size[airport_current_apron]
            airport_current_apron += 1
          elsif temp_id == 48 ### Apron 2 ###
            if airport_current_apron2 === 0
              airport_apron2_id = []
              airport_apron2_size = []
              airport_apron2_surface = []
              airport_apron2_flag = []
              airport_apron2_vertice_count = []
              airport_apron2_triangle_count = []
            end
            airport_apron2_id[airport_current_apron2] = temp_id
            airport_apron2_size[airport_current_apron2] = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_apron2_size[airport_current_apron2] += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_apron2_size[airport_current_apron2] += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            section_bytes_to_read -= airport_apron2_size[airport_current_apron2]
            read_offset += airport_apron2_size[airport_current_apron2]
            airport_current_apron2 += 1
          elsif temp_id == 38 ### Helipad ###
            if airport_current_helipad === 0
              airport_helipad_id = []
              airport_helipad_size = []
              airport_helipad_surface = []
              airport_helipad_type = [] ### bit 0-3 ###
              airport_helipad_transparent = [] ### bit 4 ###
              airport_helipad_closed = [] ### bit 5, bit 6-7 are unused ###
              airport_helipad_colour = []
              airport_helipad_longitude = []
              airport_helipad_latitude = []
              airport_helipad_altitude = []
              airport_helipad_length = []
              airport_helipad_width = []
              airport_helipad_heading = []
            end
            section_bytes_to_read -= 36
            read_offset += 36
            airport_current_helipad += 1
          elsif temp_id == 36 ### Approach, not to be drawn ###
            airport_approach_size = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_approach_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_approach_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            section_bytes_to_read -= airport_approach_size
            read_offset += airport_approach_size
          elsif temp_id == 57 ### Fence, not to be drawn ###
            airport_fence_size = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_fence_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_fence_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            section_bytes_to_read -= airport_fence_size
            read_offset += airport_fence_size
          elsif temp_id == 49 ### Apron edge lights, not to be drawn ###
            airport_apron_edge_lights_size = 0
            if $win32ole_ok == 1
              for step2 in 0..3
                airport_apron_edge_lights_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..3
                airport_apron_edge_lights_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            
            section_bytes_to_read -= airport_apron_edge_lights_size
            read_offset += airport_apron_edge_lights_size
          else
            echo "Bytes left (Section 3): " + section_bytes_to_read.to_s + ". ID: " + temp_id.to_s
            section_bytes_to_read = 0
          end
        end
 
        
        if airport_tower_longitude > airport_longitude
          $airport_tower_x_pos = calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_tower_longitude) / $inch
        else
          $airport_tower_x_pos = 0 - calculate_distance(airport_latitude, airport_longitude, airport_latitude, airport_tower_longitude) / $inch
        end
        if airport_tower_latitude > airport_latitude
          $airport_tower_y_pos = calculate_distance(airport_latitude, airport_longitude, airport_tower_latitude, airport_longitude) / $inch
        else
          $airport_tower_y_pos = 0 - calculate_distance(airport_latitude, airport_longitude, airport_tower_latitude, airport_longitude) / $inch
        end

        centerpoint = Geom::Point3d.new($airport_tower_x_pos, $airport_tower_y_pos, 0)
        vector = Geom::Vector3d.new 0,0,-1
        vector2 = vector.normalize!
        model = Sketchup.active_model
        circle = tower_group_ents.add_circle centerpoint, vector2, 300 / $inch
        face = tower_group_ents.add_face circle



        ### Calculate ICAO ###
        airport_icao_calc = ((airport_icao_calc - 1) / 32).floor
        
        for step2 in 0..4
          if airport_icao_calc > 0
            airport_icao = $icao_format[airport_icao_calc - ((airport_icao_calc / 38).floor * 38)] + airport_icao
            airport_icao_calc = (airport_icao_calc / 38).floor
          end
        end
        
        $airport_icao = airport_icao
        status = $afcad_menu.set_validation_proc($clear_menu) { MF_ENABLED }
      elsif section_type[step] == 37 ### Scenery objects ###
        section_bytes_to_read = section_length[step]
         
        ###### Go to airport header ######
        
        read_offset = section_offset[step]
        subsection_id = 0
        subsection_record_count = 0
        subsection_offset = 0
        subsection_length = 0
        
        for step2 in 0..3
          if $win32ole_ok == 1
            subsection_id += file_content[step2 + read_offset].ord * 256 ** step2
            subsection_record_count += file_content[4 + step2 + read_offset].ord * 256 ** step2
            subsection_offset += file_content[8 + step2 + read_offset].ord * 256 ** step2
            subsection_length += file_content[12 + step2 + read_offset].ord * 256 ** step2
          else
            subsection_id += file_content[step2 + read_offset][0] * 256 ** step2
            subsection_record_count += file_content[4 + step2 + read_offset][0] * 256 ** step2
            subsection_offset += file_content[8 + step2 + read_offset][0] * 256 ** step2
            subsection_length += file_content[12 + step2 + read_offset][0] * 256 ** step2
          end
        end
        
        section_bytes_to_read = subsection_length
        read_offset = subsection_offset
        while section_bytes_to_read > 0
          
          temp_id = 0
          for step2 in 0..1
            if $win32ole_ok == 1
              temp_id += file_content[step2 + read_offset].ord * 256 ** step2
            else
              temp_id += file_content[step2 + read_offset][0] * 256 ** step2
            end
          end
          if temp_id == 10 #### Generic building ####
            generic_building_size = 0
            if $win32ole_ok == 1
              for step2 in 0..1
                generic_building_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..1
                generic_building_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            section_bytes_to_read -= generic_building_size
            read_offset += generic_building_size
          elsif temp_id == 12 #### Windsock ####
            section_bytes_to_read -= 62
            read_offset += 62
          elsif temp_id == 11 #### Library object ####
            section_bytes_to_read -= 64
            read_offset += 64
          elsif temp_id == 14 #### Effect ####
            effect_size = 0
            if $win32ole_ok == 1
              for step2 in 0..1
                effect_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..1
                effect_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            section_bytes_to_read -= effect_size
            read_offset += effect_size
          elsif temp_id == 16 #### Trigger ####
            trigger_size = 0
            if $win32ole_ok == 1
              for step2 in 0..1
                trigger_size += file_content[2 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..1
                trigger_size += file_content[2 + step2 + read_offset][0] * 256 ** step2
              end
            end
            section_bytes_to_read -= trigger_size
            read_offset += trigger_size
          elsif temp_id == 4098 #### Attached object ####
            attached_object_size = 0
            if $win32ole_ok == 1
              for step2 in 0..1
                attached_object_size += file_content[6 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..1
                attached_object_size += file_content[6 + step2 + read_offset][0] * 256 ** step2
              end
            end
            section_bytes_to_read -= attached_object_size
            read_offset += attached_object_size
          else
       #     echo "Bytes left (Section 37): " + section_bytes_to_read.to_s + ". ID: " + temp_id.to_s
            section_bytes_to_read = 0
          end
        end
      elsif section_type[step] == 39 ### Namelist ###
        section_bytes_to_read = section_length[step]

        ###### Go to airport header ######
        
        read_offset = section_offset[step]
        subsection_id = 0
        subsection_record_count = 0
        subsection_offset = 0
        subsection_length = 0
        for step2 in 0..3
          if $win32ole_ok == 1
            subsection_id += file_content[step2 + read_offset].ord * 256 ** step2
            subsection_record_count += file_content[4 + step2 + read_offset].ord * 256 ** step2
            subsection_offset += file_content[8 + step2 + read_offset].ord * 256 ** step2
            subsection_length += file_content[12 + step2 + read_offset].ord * 256 ** step2
          else
            subsection_id += file_content[step2 + read_offset][0] * 256 ** step2
            subsection_record_count += file_content[4 + step2 + read_offset][0] * 256 ** step2
            subsection_offset += file_content[8 + step2 + read_offset][0] * 256 ** step2
            subsection_length += file_content[12 + step2 + read_offset][0] * 256 ** step2
          end
        end
        
        section_bytes_to_read = subsection_length
        read_offset = subsection_offset
        while section_bytes_to_read > 0
          
          temp_id = 0
          for step2 in 0..1
            if $win32ole_ok == 1
              temp_id += file_content[step2 + read_offset].ord * 256 ** step2
            else
              temp_id += file_content[step2 + read_offset][0] * 256 ** step2
            end
          end
          if temp_id == 39 #### Namelist ####
            namelist_id = temp_id
            namelist_size = 0
            namelist_region_names = 0
            namelist_country_names = 0
            namelist_state_names = 0
            namelist_city_names = 0
            namelist_airport_names = 0
            namelist_icao_ident = 0
            namelist_region_offset = 0
            namelist_country_offset = 0
            namelist_state_offset = 0
            namelist_city_offset = 0
            namelist_airport_offset = 0
            namelist_icao_offset = 0
            namelist_region = ""
            namelist_country = ""
            namelist_state = ""
            namelist_city = ""
            namelist_airport = ""
            if $win32ole_ok == 1
              for step2 in 0..1
                namelist_region_names += file_content[6 + step2 + read_offset].ord * 256 ** step2
                namelist_country_names += file_content[8 + step2 + read_offset].ord * 256 ** step2
                namelist_state_names += file_content[10 + step2 + read_offset].ord * 256 ** step2
                namelist_city_names += file_content[12 + step2 + read_offset].ord * 256 ** step2
                namelist_airport_names += file_content[14 + step2 + read_offset].ord * 256 ** step2
                namelist_icao_ident += file_content[16 + step2 + read_offset].ord * 256 ** step2
              end
              for step2 in 0..3
                namelist_region_offset += file_content[18 + step2 + read_offset].ord * 256 ** step2
                namelist_country_offset += file_content[22 + step2 + read_offset].ord * 256 ** step2
                namelist_state_offset += file_content[26 + step2 + read_offset].ord * 256 ** step2
                namelist_city_offset += file_content[30 + step2 + read_offset].ord * 256 ** step2
                namelist_airport_offset += file_content[34 + step2 + read_offset].ord * 256 ** step2
                namelist_icao_offset += file_content[38 + step2 + read_offset].ord * 256 ** step2
              end
            else
              for step2 in 0..1
                namelist_region_names += file_content[6 + step2 + read_offset][0] * 256 ** step2
                namelist_country_names += file_content[8 + step2 + read_offset][0] * 256 ** step2
                namelist_state_names += file_content[10 + step2 + read_offset][0] * 256 ** step2
                namelist_city_names += file_content[12 + step2 + read_offset][0] * 256 ** step2
                namelist_airport_names += file_content[14 + step2 + read_offset][0] * 256 ** step2
                namelist_icao_ident += file_content[16 + step2 + read_offset][0] * 256 ** step2
              end
              for step2 in 0..3
                namelist_region_offset += file_content[18 + step2 + read_offset][0] * 256 ** step2
                namelist_country_offset += file_content[22 + step2 + read_offset][0] * 256 ** step2
                namelist_state_offset += file_content[26 + step2 + read_offset][0] * 256 ** step2
                namelist_city_offset += file_content[30 + step2 + read_offset][0] * 256 ** step2
                namelist_airport_offset += file_content[34 + step2 + read_offset][0] * 256 ** step2
                namelist_icao_offset += file_content[38 + step2 + read_offset][0] * 256 ** step2
              end
            end
            #### Find country ####
            namelist_country_length = namelist_state_offset - namelist_country_offset - 5
            namelist_state_length = namelist_city_offset - namelist_state_offset - 5
            namelist_city_length = namelist_airport_offset - namelist_city_offset - 5
            if $win32ole_ok == 1
              for step2 in 0..namelist_country_length - 1
                namelist_country += file_content[namelist_country_offset + 4 + step2 + read_offset]
              end
              for step2 in 0..namelist_state_length - 1
                namelist_state += file_content[namelist_state_offset + 4 + step2 + read_offset]
              end
              for step2 in 0..namelist_city_length - 1
                namelist_city += file_content[namelist_city_offset + 4 + step2 + read_offset]
              end
            else
              for step2 in 0..namelist_country_length - 1
                namelist_country += file_content[namelist_country_offset + 4 + step2 + read_offset][0].chr
              end
              for step2 in 0..namelist_state_length - 1
                namelist_state += file_content[namelist_state_offset + 4 + step2 + read_offset][0].chr
              end
              for step2 in 0..namelist_city_length - 1
                namelist_city += file_content[namelist_city_offset + 4 + step2 + read_offset][0].chr
              end
            end
            
        #    echo namelist_city

            section_bytes_to_read -= namelist_icao_offset + 20
            read_offset += namelist_icao_offset + 20
          else
            
            echo "Bytes left (Section 39): " + section_bytes_to_read.to_s + ". ID: " + temp_id.to_s
            section_bytes_to_read = 0
          end
        end
      end
    end    


    ##############################
    ### Set model geo-location ###
    ##############################
    
    if namelist_state == ""
      geolocate_model(airport_latitude, airport_longitude, namelist_city, namelist_country)
    else
      geolocate_model(airport_latitude, airport_longitude, namelist_city.to_s + " (" + namelist_state.to_s + ")", namelist_country)
    end

   
   else
    echo "Not a valid AFCAD file"
  end
  
  
  
end

    ##############################
    ### Texture Runway ###
    ##############################
 

def texture_runway (runway_name = "", runway_length = 0.0, runway_width = 0.0, runway_heading = 0.0, runway_x_pos = 0.0, runway_y_pos = 0.0)
 
  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials


  default_length = (runway_length.to_f / 10).floor
  default_width = (runway_width.to_f / 10).floor
  prompts = ["Runway length segments?", "Runway width segments?", "Texture file?", "File type?"]
  defaults = [default_length, default_width, "", "jpg"]
  list = ["", "", "", "jpg|png"]
  input = UI.inputbox prompts, defaults, list, "Texture Runway. Length: " + runway_length.to_s + " metres. Width: " + runway_width.to_s + " metres."

  length_sections = input[0].to_i
  width_sections = input[1].to_i
  texture_file_name = input[2]
  texture_file_type = input[3]
  
  runway_section_length = (runway_length.to_f * 100 / $inch) / length_sections
  runway_section_width = (runway_width.to_f * 100 / $inch) / width_sections
  runway_midpoint_length = (runway_length.to_f * 50 / $inch)
  runway_midpoint_width = (runway_width.to_f * 50 / $inch)
  runway_angle = (90 - runway_heading.to_f).modulo(360).degrees
  
  for step in 0..width_sections - 1
    for step2 in 0..length_sections - 1
      texture = materials.add runway_name.to_s + " " + step.to_s + "x" + step2.to_s
      texture.texture = $texture_files + texture_file_name + "_" + step.to_s + "_" + step2.to_s + "." + texture_file_type
      texture.texture.size = [runway_section_length, runway_section_width]
      pts = []
      pts[0] = [runway_section_length * step2, runway_section_width * step, 0]
      pts[1] = [runway_section_length * (step2 + 1), runway_section_width * step, 0]
      pts[2] = [runway_section_length * (step2 + 1), runway_section_width * (step + 1), 0]
      pts[3] = [runway_section_length * step2, runway_section_width * (step + 1), 0]
      face = group_ents.add_face pts
      if step == 0 and step2 == 0
        status = face.reverse!
      end
      status = face.material = runway_name.to_s + " " + step.to_s + "x" + step2.to_s
    end
  end


  trans = group.transformation
  point = Geom::Point3d.new(runway_x_pos.to_f + trans.origin[0] - runway_midpoint_length, runway_y_pos.to_f + trans.origin[1] - runway_midpoint_width, trans.origin[2])
  t = Geom::Transformation.new point
  group.move! t
  t = Geom::Transformation.rotation [runway_x_pos.to_f, runway_y_pos.to_f, 0], Z_AXIS, runway_angle
  group.transform! t
  
end


def texture_apron(selection = [])
 
  face_id = -1
  
  for step in 0..selection.entities.count - 1
    if selection.entities[step].typename == "Face"
      face_id = step
    end
  end
  
  x_pos_offset = 0.0
  y_pos_offset = 0.0
  x_pos_max = 0.0
  y_pos_max = 0.0
  
  for step in 0..selection.entities[face_id].vertices.count - 1
    if step == 0
      x_pos_offset = selection.entities[face_id].vertices[step].position[0].to_f
      y_pos_offset = selection.entities[face_id].vertices[step].position[1].to_f
      x_pos_max = selection.entities[face_id].vertices[step].position[0].to_f
      y_pos_max = selection.entities[face_id].vertices[step].position[1].to_f
    else
      if x_pos_offset > selection.entities[face_id].vertices[step].position[0].to_f
        x_pos_offset = selection.entities[face_id].vertices[step].position[0].to_f
      end
      if y_pos_offset > selection.entities[face_id].vertices[step].position[1].to_f
        y_pos_offset = selection.entities[face_id].vertices[step].position[1].to_f
      end
      if x_pos_max < selection.entities[face_id].vertices[step].position[0].to_f
        x_pos_max = selection.entities[face_id].vertices[step].position[0].to_f
      end
      if y_pos_max < selection.entities[face_id].vertices[step].position[1].to_f
        y_pos_max = selection.entities[face_id].vertices[step].position[1].to_f
      end
    end
  end
  
  x_pos_difference = x_pos_max - x_pos_offset
  y_pos_difference = y_pos_max - y_pos_offset

  prompts = ["Segments X(" + (x_pos_difference / 100 * $inch).to_s + " metres)?", "Segments Y(" + (y_pos_difference / 100 * $inch).to_s + " metres)?", "Texture file?", "File type?"]
  defaults = ["3", "3", "", "jpg"]
  list = ["", "", "", "jpg|png"]
  input = UI.inputbox prompts, defaults, list, "Texture Apron" #. X: " + x_pos_difference.to_s + " metres. Y: " + y_pos_difference.to_s + " metres."

  x_division = input[0].to_i
  y_division = input[1].to_i
  
  x_division_size = x_pos_difference / x_division
  y_division_size = y_pos_difference / y_division
  
  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials

  pts = []
  for step in 0..selection.entities[face_id].vertices.count - 1
    pts[step] = [selection.entities[face_id].vertices[step].position[0].to_f - x_pos_offset, selection.entities[face_id].vertices[step].position[1].to_f - y_pos_offset, 0]
  end
  face = group_ents.add_face pts
  status = face.reverse!

  for step in 1..x_division - 1
      ptsx = []
      ptsx[0] = [step * x_division_size, 0, 0]
      ptsx[1] = [step * x_division_size, y_pos_difference, 0]
      face = group_ents.add_edges ptsx
  end
  for step in 1..y_division - 1
      ptsx = []
      ptsx[0] = [0, step * y_division_size, 0]
      ptsx[1] = [x_pos_difference, step * y_division_size, 0]
      face = group_ents.add_edges ptsx
  end

  group_array = group.explode
  texture_file_name = input[2]
  texture_file_type = input[3]

  for step in 0..x_division - 1
    for step2 in 0..y_division - 1
      texture = materials.add "Apron " + step.to_s + "x" + step2.to_s
      texture.texture = $texture_files + texture_file_name + "_" + step.to_s + "_" + step2.to_s + "." + texture_file_type
      texture.texture.size = [x_division_size, y_division_size]
    end
  end
  
  entities = Sketchup.active_model.active_entities  

  new_group = Sketchup.active_model.entities.add_group
  new_group_ents = new_group.entities
  
  for step in 0..entities.count - 1
    if entities[step].typename == "Face"
      pts = []
      for step2 in 0..entities[step].vertices.count - 1
        if step2 == 0
          min_x_face = entities[step].vertices[step2].position[0].to_f
          min_y_face = entities[step].vertices[step2].position[1].to_f
        else
          if min_x_face > entities[step].vertices[step2].position[0].to_f
            min_x_face = entities[step].vertices[step2].position[0].to_f
          end
          if min_y_face > entities[step].vertices[step2].position[1].to_f
            min_y_face = entities[step].vertices[step2].position[1].to_f
          end
        end
        pts[step2] = entities[step].vertices[step2].position
      end
      x_face = ((min_x_face + 1) / x_division_size).floor
      y_face = ((min_y_face + 1) / y_division_size).floor
      status = entities[step].material = "Apron " + x_face.to_s + "x" + y_face.to_s

      face = new_group_ents.add_face pts
      if face.normal[2].to_i == -1
        status = face.reverse!
      end
      status = face.material = "Apron " + x_face.to_s + "x" + y_face.to_s
   #   echo entities[step].vertices[0].position.to_s + " - " + ((min_x_face + 1) / x_division_size).to_s + " , " + ((min_y_face + 1) / y_division_size).to_s
    end
  end

  trans = new_group.transformation
  point = Geom::Point3d.new(trans.origin[0] + x_pos_offset, trans.origin[1] + y_pos_offset, trans.origin[2])
  t = Geom::Transformation.new point
  new_group.move! t
  new_group.description = "Apron"
  begin ### Workaround for deleting missing faces ###
    entities = Sketchup.active_model.active_entities 
    ent_count = 0 
    for step in 0..entities.count - 1
      begin
        if entities[step].typename != "Group"
          ent_count += 1
          entities[step].erase!
        end
      rescue
      end
    end
  end until ent_count < 1
end