###########################
### Beams, beams, beams ###
###########################


################################
### Code for duo pitch truss ###
################################

def duo_pitch_truss (hangar_type = "", roof_type = "", beam_texture = "", x_offset = 0.0, y_offset = 0.0, main_beam_width = 0.0, main_beam_depth = 0.0, main_beam_height = 0.0, main_beam_top = 0.0, total_width = 0.0, internal_length = 0.0, arc_height = 0.0, beam_thickness = 0.0, main_beams = 0, diagonal_beams = 0)

  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials
  
  diagonal_beam_distance = (total_width - main_beam_width * 2) / diagonal_beams
  beam_distance = (internal_length - main_beam_width) / (main_beams - 1)

  for step in 0..$texture_name.length - 1
    if beam_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      texture.texture = $texture_files + $texture_filename[step]
      texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
    end
  end
  
  if beam_texture == "Untextured"
    texture = materials.add "Untextured"
    texture.color = "White"
  end
  
  for beam_step in 0..main_beams - 1
    main_beam_start_point = beam_step * beam_distance + x_offset
    main_beam_end_point = main_beam_start_point + main_beam_width + x_offset
    beam_start_point = main_beam_start_point + main_beam_width / 2 - beam_thickness / 2 + x_offset
    beam_end_point = beam_start_point + beam_thickness + x_offset
    
    pts = []
    pts[7] = [main_beam_end_point, y_offset,  0]
    pts[6] = [main_beam_end_point, y_offset,  main_beam_height + main_beam_top]
    pts[5] = [main_beam_end_point, y_offset + main_beam_width,  main_beam_height + main_beam_top]
    pts[4] = [main_beam_end_point, y_offset + main_beam_width,  0]
    pts[3] = [main_beam_start_point, y_offset + main_beam_width,  0]
    pts[2] = [main_beam_start_point, y_offset + main_beam_width,  main_beam_height + main_beam_top]
    pts[1] = [main_beam_start_point, y_offset,  main_beam_height + main_beam_top]
    pts[0] = [main_beam_start_point, y_offset,  0]
    for step in 0..3
      face = group_ents.add_face pts[(step * 2).modulo(8)], pts[(step * 2 + 1).modulo(8)], pts[(step * 2 + 2).modulo(8)], pts[(step * 2 + 3).modulo(8)]
      status = face.material = beam_texture
    end
    face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
    status = face.material = beam_texture

    pts = []
    pts[0] = [main_beam_start_point, y_offset + total_width,  main_beam_height + main_beam_top]
    pts[1] = [main_beam_start_point, y_offset + total_width,  0]
    pts[2] = [main_beam_start_point, y_offset + total_width - main_beam_width,  0]
    pts[3] = [main_beam_start_point, y_offset + total_width - main_beam_width,  main_beam_height + main_beam_top]
    pts[4] = [main_beam_end_point, y_offset + total_width - main_beam_width,  main_beam_height + main_beam_top]
    pts[5] = [main_beam_end_point, y_offset + total_width - main_beam_width,  0]
    pts[6] = [main_beam_end_point, y_offset + total_width,  0]
    pts[7] = [main_beam_end_point, y_offset + total_width,  main_beam_height + main_beam_top]
    for step in 0..3
      face = group_ents.add_face pts[(step * 2).modulo(8)], pts[(step * 2 + 1).modulo(8)], pts[(step * 2 + 2).modulo(8)], pts[(step * 2 + 3).modulo(8)]
      status = face.material = beam_texture
    end
    face = group_ents.add_face pts[0], pts[3], pts[4], pts[7]
    status = face.material = beam_texture

    pts = []
    pts[7] = [beam_start_point, y_offset + main_beam_width,  main_beam_height]
    pts[6] = [beam_start_point, y_offset + total_width - main_beam_width,  main_beam_height]
    pts[5] = [beam_start_point, y_offset + total_width - main_beam_width,  main_beam_height + beam_thickness]
    pts[4] = [beam_start_point, y_offset + main_beam_width,  main_beam_height + beam_thickness]
    pts[3] = [beam_end_point, y_offset + main_beam_width,  main_beam_height + beam_thickness]
    pts[2] = [beam_end_point, y_offset + total_width - main_beam_width,  main_beam_height + beam_thickness]
    pts[1] = [beam_end_point, y_offset + total_width - main_beam_width,  main_beam_height]
    pts[0] = [beam_end_point, y_offset + main_beam_width,  main_beam_height]
    for step in 0..3
      face = group_ents.add_face pts[(step * 2).modulo(8)], pts[(step * 2 + 1).modulo(8)], pts[(step * 2 + 2).modulo(8)], pts[(step * 2 + 3).modulo(8)]
      status = face.material = beam_texture
    end

    pts = []
    pts[0] = [beam_start_point, y_offset,  main_beam_height + main_beam_top]
    pts[1] = [beam_start_point, y_offset + total_width / 2,  main_beam_height + main_beam_top + arc_height]
    pts[2] = [beam_start_point, y_offset + total_width,  main_beam_height + main_beam_top]
    pts[3] = [beam_start_point, y_offset + total_width,  main_beam_height + main_beam_top + beam_thickness]
    pts[4] = [beam_start_point, y_offset + total_width / 2,  main_beam_height + main_beam_top + arc_height + beam_thickness]
    pts[5] = [beam_start_point, y_offset,  main_beam_height + main_beam_top + beam_thickness]
    pts[6] = [beam_end_point, y_offset,  main_beam_height + main_beam_top + beam_thickness]
    pts[7] = [beam_end_point, y_offset + total_width / 2,  main_beam_height + main_beam_top + arc_height + beam_thickness]
    pts[8] = [beam_end_point, y_offset + total_width,  main_beam_height + main_beam_top + beam_thickness]
    pts[9] = [beam_end_point, y_offset + total_width,  main_beam_height + main_beam_top]
    pts[10] = [beam_end_point, y_offset + total_width / 2,  main_beam_height + main_beam_top + arc_height]
    pts[11] = [beam_end_point, y_offset,  main_beam_height + main_beam_top]
    face = group_ents.add_face pts[5], pts[6], pts[7], pts[4] ### Top ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[8], pts[7], pts[4], pts[3] ### Top ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[6], pts[7], pts[8], pts[9], pts[10], pts[11] ### Side ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[0], pts[1], pts[2], pts[3], pts[4], pts[5] ### Side ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[1], pts[0], pts[11], pts[10] ### Bottom ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[9], pts[10], pts[1], pts[2] ### Bottom ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[2], pts[3], pts[8], pts[9] ### End ###
    status = face.material = beam_texture
    face = group_ents.add_face pts[0], pts[5], pts[6], pts[11] ### End ###
    status = face.material = beam_texture

    for step in 0..diagonal_beams - 1
      beam_start_front_edge = y_offset + main_beam_width + diagonal_beam_distance * step - beam_thickness / 2
      beam_start_back_edge = y_offset + main_beam_width + diagonal_beam_distance * step + beam_thickness / 2
      beam_end_front_edge = y_offset + main_beam_width + diagonal_beam_distance * (step + 1) - beam_thickness / 2
      beam_end_back_edge = y_offset + main_beam_width + diagonal_beam_distance * (step + 1) + beam_thickness / 2
      pts = []
      if step >= diagonal_beams / 2
        if step.modulo(2) == 1
          pts[0] = [beam_start_point, beam_end_front_edge,  main_beam_height + beam_thickness]
          pts[1] = [beam_start_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_front_edge) / (total_width / 2)))]
          pts[2] = [beam_start_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_back_edge) / (total_width / 2)))]
          pts[3] = [beam_start_point, beam_end_back_edge,  main_beam_height + beam_thickness]
          pts[4] = [beam_end_point, beam_end_back_edge,  main_beam_height + beam_thickness]
          pts[5] = [beam_end_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_back_edge) / (total_width / 2)))]
          pts[6] = [beam_end_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_front_edge) / (total_width / 2)))]
          pts[7] = [beam_end_point, beam_end_front_edge,  main_beam_height + beam_thickness]
       else
          pts[0] = [beam_start_point, beam_start_front_edge,  main_beam_height + beam_thickness]
          pts[1] = [beam_start_point, beam_end_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_end_front_edge) / (total_width / 2)))]
          pts[2] = [beam_start_point, beam_end_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_end_back_edge) / (total_width / 2)))]
          pts[3] = [beam_start_point, beam_start_back_edge,  main_beam_height + beam_thickness]
          pts[4] = [beam_end_point, beam_start_back_edge,  main_beam_height + beam_thickness]
          pts[5] = [beam_end_point, beam_end_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_end_back_edge) / (total_width / 2)))]
          pts[6] = [beam_end_point, beam_end_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_end_front_edge) / (total_width / 2)))]
          pts[7] = [beam_end_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        end
      else
        if step.modulo(2) == 0
          pts[0] = [beam_start_point, beam_start_front_edge,  main_beam_height + beam_thickness]
          pts[1] = [beam_start_point, beam_end_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_end_front_edge / (total_width / 2)))]
          pts[2] = [beam_start_point, beam_end_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_end_back_edge / (total_width / 2)))]
          pts[3] = [beam_start_point, beam_start_back_edge,  main_beam_height + beam_thickness]
          pts[4] = [beam_end_point, beam_start_back_edge,  main_beam_height + beam_thickness]
          pts[5] = [beam_end_point, beam_end_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_end_back_edge / (total_width / 2)))]
          pts[6] = [beam_end_point, beam_end_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_end_front_edge / (total_width / 2)))]
          pts[7] = [beam_end_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        else
          pts[0] = [beam_start_point, beam_end_front_edge,  main_beam_height + beam_thickness]
          pts[1] = [beam_start_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_front_edge / (total_width / 2)))]
          pts[2] = [beam_start_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_back_edge / (total_width / 2)))]
          pts[3] = [beam_start_point, beam_end_back_edge,  main_beam_height + beam_thickness]
          pts[4] = [beam_end_point, beam_end_back_edge,  main_beam_height + beam_thickness]
          pts[5] = [beam_end_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_back_edge / (total_width / 2)))]
          pts[6] = [beam_end_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_front_edge / (total_width / 2)))]
          pts[7] = [beam_end_point, beam_end_front_edge,  main_beam_height + beam_thickness]
        end
      end
  
      for step2 in 0..3
        face = group_ents.add_face pts[(step2 * 2).modulo(8)], pts[(step2 * 2 + 1).modulo(8)], pts[(step2 * 2 + 2).modulo(8)], pts[(step2 * 2 + 3).modulo(8)]
        status = face.material = beam_texture
      end

      if step == 1 or step == diagonal_beams / 2
        pts = []
        pts[0] = [beam_start_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        pts[1] = [beam_start_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_front_edge / (total_width / 2)))]
        pts[2] = [beam_start_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_back_edge / (total_width / 2)))]
        pts[3] = [beam_start_point, beam_start_back_edge,  main_beam_height + beam_thickness]
        pts[4] = [beam_end_point, beam_start_back_edge,  main_beam_height + beam_thickness]
        pts[5] = [beam_end_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_back_edge / (total_width / 2)))]
        pts[6] = [beam_end_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * (beam_start_front_edge / (total_width / 2)))]
        pts[7] = [beam_end_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        for step2 in 0..3
          face = group_ents.add_face pts[(step2 * 2).modulo(8)], pts[(step2 * 2 + 1).modulo(8)], pts[(step2 * 2 + 2).modulo(8)], pts[(step2 * 2 + 3).modulo(8)]
          status = face.material = beam_texture
        end
      elsif step == diagonal_beams - 1
        pts = []
        pts[0] = [beam_start_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        pts[1] = [beam_start_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_front_edge) / (total_width / 2)))]
        pts[2] = [beam_start_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_back_edge) / (total_width / 2)))]
        pts[3] = [beam_start_point, beam_start_back_edge,  main_beam_height + beam_thickness]
        pts[4] = [beam_end_point, beam_start_back_edge,  main_beam_height + beam_thickness]
        pts[5] = [beam_end_point, beam_start_back_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_back_edge) / (total_width / 2)))]
        pts[6] = [beam_end_point, beam_start_front_edge,  main_beam_height + main_beam_top + (arc_height * ((total_width - beam_start_front_edge) / (total_width / 2)))]
        pts[7] = [beam_end_point, beam_start_front_edge,  main_beam_height + beam_thickness]
        for step2 in 0..3
          face = group_ents.add_face pts[(step2 * 2).modulo(8)], pts[(step2 * 2 + 1).modulo(8)], pts[(step2 * 2 + 2).modulo(8)], pts[(step2 * 2 + 3).modulo(8)]
          status = face.material = beam_texture
        end
      end
    end
  end
  
  return group
  
end

##############################
### Code for simple T beam ###
##############################

def t_beam (hangar_type = "", roof_type = "", beam_texture = "", x_offset = 0.0, y_offset = 0.0, beam_width = 0.0, beam_depth = 0.0, beam_thickness = 0.0, beam_height = 0.0, arc_height = 0.0, internal_length = 0.0, internal_width = 0.0, beam_rows = 0)
  
  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials

  for step in 0..$texture_name.length - 1
    if beam_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      texture.texture = $texture_files + $texture_filename[step]
      texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
    end
  end

  if beam_texture == "Untextured"
    texture = materials.add "Untextured"
    texture.color = "White"
  end

  if beam_rows < 3
    beam_rows = 3
  end
  ### End beams to be L-shaped. All beams have a foundation.
  
  beam_footprint = ((beam_width + beam_thickness * 2) * (beam_rows - 2)) + ((beam_depth + beam_thickness * 2) * 2)
  beam_distance = (internal_length - beam_footprint) / (beam_rows - 1)

  echo (beam_distance * $inch / 100).to_s

  for step in 0..beam_rows - 1
    if step == 0
      beam_start_offset = x_offset
    else
      beam_start_offset = x_offset + (beam_depth + beam_thickness * 2) + (beam_distance * step) + ((beam_thickness * 2 + beam_width) * (step - 1))
    end
    if step == 0 or step == beam_rows - 1
      beam_profile_pts = set_beam_profile("Square", (beam_thickness * 2 + beam_depth).to_f, (beam_thickness * 2 + beam_width).to_f, beam_thickness.to_f)
    else
      beam_profile_pts = set_beam_profile("Square", (beam_thickness * 2 + beam_width).to_f, (beam_thickness * 2 + beam_depth).to_f, beam_thickness.to_f)
    end
    pts = []
    ptsx = []
    for step2 in 0..beam_profile_pts[0].length - 1
      pts[step2] = [beam_start_offset + beam_profile_pts[0][step2], y_offset + beam_profile_pts[1][step2], 0]
      pts[step2 + beam_profile_pts[0].length] = [beam_start_offset + beam_profile_pts[0][step2], y_offset + beam_profile_pts[1][step2], beam_thickness]
      ptsx[step2] = [beam_start_offset + beam_profile_pts[0][step2], y_offset + beam_profile_pts[1][step2], beam_thickness]
    end

    for step2 in 0..beam_profile_pts[0].length - 1
      face = group_ents.add_face pts[(step2).modulo(beam_profile_pts[0].length)], pts[(step2 + 1).modulo(beam_profile_pts[0].length)], pts[(step2 + 1).modulo(beam_profile_pts[0].length) + beam_profile_pts[0].length], pts[(step2).modulo(beam_profile_pts[0].length) + beam_profile_pts[0].length]
      status = face.material = beam_texture
    end
    face = group_ents.add_face ptsx
    status = face.material = beam_texture
    
    if step == 0 or step == beam_rows - 1
      beam_profile_pts = set_beam_profile("L", beam_depth.to_f, beam_width.to_f, beam_thickness.to_f)
    else
      beam_profile_pts = set_beam_profile("T", beam_width.to_f, beam_depth.to_f, beam_thickness.to_f)
    end
    pts = []
    for step2 in 0..beam_profile_pts[0].length - 1
      if step == 0
        pts[step2] = [beam_start_offset + beam_thickness + beam_profile_pts[0][step2], y_offset + beam_thickness + beam_width - beam_profile_pts[1][step2], beam_thickness]
        pts[step2 + beam_profile_pts[0].length] = [beam_start_offset + beam_thickness + beam_profile_pts[0][step2], y_offset + beam_thickness + beam_width - beam_profile_pts[1][step2], beam_height]
      elsif step == beam_rows - 1
        pts[step2] = [beam_start_offset + beam_thickness + beam_depth - beam_profile_pts[0][step2], y_offset + beam_thickness + beam_width - beam_profile_pts[1][step2], beam_thickness]
        pts[step2 + beam_profile_pts[0].length] = [beam_start_offset + beam_thickness + beam_depth - beam_profile_pts[0][step2], y_offset + beam_thickness + beam_width - beam_profile_pts[1][step2], beam_height]
      else
        pts[step2] = [beam_start_offset + beam_thickness + beam_profile_pts[0][step2], y_offset + beam_thickness + beam_profile_pts[1][step2], beam_thickness]
        pts[step2 + beam_profile_pts[0].length] = [beam_start_offset + beam_thickness + beam_profile_pts[0][step2], y_offset + beam_thickness + beam_profile_pts[1][step2], beam_height]
      end
    end

    for step2 in 0..beam_profile_pts[0].length - 1
      face = group_ents.add_face pts[(step2).modulo(beam_profile_pts[0].length)], pts[(step2 + 1).modulo(beam_profile_pts[0].length)], pts[(step2 + 1).modulo(beam_profile_pts[0].length) + beam_profile_pts[0].length], pts[(step2).modulo(beam_profile_pts[0].length) + beam_profile_pts[0].length]
      if step2 == 0 and step == 0
        status = face.reverse!
      end
      status = face.material = beam_texture
    end
    
    
    
  end
  
  
  return group
  
end



########################################
### Function to create beam profiles ###
########################################

def set_beam_profile (beam_profile_name = "", beam_width = 0.0, beam_depth = 0.0, beam_thickness = 0.0)
  
  
  if beam_profile_name == "H"
    beam_profile = []
    beam_profile[0] = [0, beam_thickness, beam_thickness, beam_width - beam_thickness, beam_width - beam_thickness, beam_width, beam_width, beam_width - beam_thickness, beam_width - beam_thickness, beam_thickness, beam_thickness, 0]
    beam_profile[1] = [0, 0, beam_depth / 2 - beam_thickness / 2, beam_depth / 2 - beam_thickness / 2, 0, 0, beam_depth, beam_depth, beam_depth / 2 + beam_thickness / 2, beam_depth / 2 + beam_thickness / 2, beam_depth, beam_depth]
  elsif beam_profile_name == "T"
    beam_profile = []
    beam_profile[0] = [beam_width, beam_width, beam_width / 2 + beam_thickness / 2, beam_width / 2 + beam_thickness / 2, beam_width / 2 - beam_thickness / 2, beam_width / 2 - beam_thickness / 2, 0, 0]
    beam_profile[1] = [0, beam_thickness, beam_thickness, beam_depth, beam_depth, beam_thickness, beam_thickness, 0]
  elsif beam_profile_name == "L"
    beam_profile = []
    beam_profile[0] = [0, beam_thickness, beam_thickness, beam_width, beam_width, 0]
    beam_profile[1] = [0, 0, beam_depth - beam_thickness, beam_depth - beam_thickness, beam_depth, beam_depth]
  elsif beam_profile_name == "U"
    beam_profile = []
    beam_profile[0] = [0, beam_thickness, beam_thickness, beam_width - beam_thickness, beam_width - beam_thickness, beam_width, beam_width, 0]
    beam_profile[1] = [0, 0, beam_depth - beam_thickness, beam_depth - beam_thickness, 0, 0, beam_depth, beam_depth]
  elsif beam_profile_name == "O"
    beam_profile = []
    beam_profile[0] = []
    beam_profile[1] = []
    beam_angle = 360 / beam_thickness
    for step in 0..beam_thickness - 1
      beam_profile[0][step] = Math.cos((beam_angle * step).degrees) * beam_width
      beam_profile[1][step] = Math.sin((beam_angle * step).degrees) * beam_depth
    end
  elsif beam_profile_name == "Square"
    beam_profile = []
    beam_profile[0] = [0, beam_width, beam_width, 0]
    beam_profile[1] = [0, 0, beam_depth, beam_depth]
  end
 
  return beam_profile
end
