########################
### One Click Bridge ###
###   01-APR-2014    ###
########################


##############################
##############################
### Code for HTML bridge   ###
##############################
##############################

def new_bridge
  
  dlg = UI::WebDialog.new("One Click Bridge", true, "NewBridge", 720, 400, 150, 150, true)
  dlg.set_url $ochtml_folder + "bridge_menu.html"
  js_script = "dataFromSketchup(#{name.inspect})"
  
  dlg.add_action_callback("help_link") do |web_dialog,action_name|

    js_script = "dataFromSketchup(#{name.inspect})"
    dlg.execute_script(js_script) 
  
  end

  dlg.show {
    dlg.execute_script(js_script) 
  }
  
  
end


##############################
##############################
### Code for arched bridge ###
##############################
##############################

def arched_bridge
  
  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials
  
  prompts = ["Arch type", "Bridge length", "Bridge width", "Bridge height", "Arches", "Arch height", "Arch length", "Arch degrees", "Foundation height", "Foundation length", "Foundation width", "Angle", "X-pos", "Y-pos", "Extensions"]
  defaults = ["Solid", "4000", "600", "400", "5", "200", "600", "60", "300", "400", "700", "0", "0", "0", ""]
  list = ["Solid|Open spandrel", "", "", "", "", "", "", "", "", "", "", "", "", "", ""]
  input = UI.inputbox prompts, defaults, list, "Arch bridge info"

  arch_type = input[0]
  bridge_length = input[1].to_f / $inch
  bridge_width = input[2].to_f / $inch
  bridge_height = input[3].to_f / $inch
  arches = input[4].to_i
  arch_height = input[5].to_f / $inch
  arch_length = input[6].to_f / $inch
  arch_start_degrees = input[7].to_f
  foundation_height = input[8].to_f / $inch
  foundation_length = input[9].to_f / $inch
  foundation_width = input[10].to_f / $inch
  
  arch_division = (bridge_length - arch_length * arches) / (arches + 1)  
  
  arch_level = 30
  arch_degrees = 180 / arch_level
  start_degrees = 90 - arch_start_degrees
  end_degrees = 90 + arch_start_degrees
  z_radius = arch_height / (1 - Math.sin(start_degrees.degrees))
  x_radius = arch_length / Math.cos(start_degrees.degrees)
  height_offset = z_radius - arch_height
  section_degrees = (end_degrees - start_degrees) / arch_level
  
  if foundation_length < arch_division
    foundation_length = arch_division
  end
  if foundation_width < bridge_width
    foundation_width = bridge_width
  end
  
  ###################
  ### Draw bridge ###
  ###################

  if arch_type == "Solid"
    steps = arches * (arch_level + 1) + 4
    pts = []
    ptsx = []
    pts[0] = [0, 0, 0 + foundation_height]
    ptsx[0] = [0, bridge_width, 0 + foundation_height]
    for step in 0..arches - 1
      for step2 in 0..arch_level
        x_coord = Math.cos((step2 * section_degrees + start_degrees).degrees) * x_radius / 2
        z_coord = Math.sin((step2 * section_degrees + start_degrees).degrees) * z_radius
        pts[step + 1 + step * arch_level + step2] = [(arch_length + arch_division) * (step + 1) - x_coord - arch_length / 2, 0, z_coord - height_offset + foundation_height]
        ptsx[step + 1 + step * arch_level + step2] = [(arch_length + arch_division) * (step + 1) - x_coord - arch_length / 2, bridge_width, z_coord - height_offset + foundation_height]
      end
    end
    pts[1 + arches * (arch_level + 1)] = [bridge_length, 0, 0 + foundation_height]
    pts[2 + arches * (arch_level + 1)] = [bridge_length, 0, bridge_height + foundation_height]
    pts[3 + arches * (arch_level + 1)] = [0, 0, bridge_height + foundation_height]
    ptsx[1 + arches * (arch_level + 1)] = [bridge_length, bridge_width, 0 + foundation_height]
    ptsx[2 + arches * (arch_level + 1)] = [bridge_length, bridge_width, bridge_height + foundation_height]
    ptsx[3 + arches * (arch_level + 1)] = [0, bridge_width, bridge_height + foundation_height]
  
    face = group_ents.add_face pts
  
    for step in 0..steps - 1
      face = group_ents.add_face pts[step.modulo(steps)], pts[(step + 1).modulo(steps)], ptsx[(step + 1).modulo(steps)], ptsx[step.modulo(steps)]
    end

    face = group_ents.add_face ptsx
  elsif arch_type == "Open spandrel"
    ### Additional prompts ###
    prompts = ["Arch thickness",  "Middle column thickness", "Arch columns", "Arch column thickness", "Deck thickness", "Deck width", "Deck height"]
    defaults = ["50", "70", "4", "70", "50", "700", "0"]
    list = ["", "", "", "", ""]
    input = UI.inputbox prompts, defaults, list, "Arch bridge additional info"
    arch_thickness = input[0].to_f / $inch
    middle_column_thickness = input[1].to_f / $inch
    arch_columns = input[2].to_i
    arch_column_thickness = input[3].to_f / $inch
    deck_thickness = input[4].to_f / $inch
    deck_width = input[5].to_f / $inch
    deck_height = input[6].to_f / $inch
    if arch_height + arch_thickness < bridge_height - deck_thickness and arch_columns == 0
      arch_colums = 1
    end  
    ### Draw arches ###
    pts = []
    ptsx = []
    steps = (arches * arch_level * 2) + (arches * 2) - 1
    for step in 0..arches - 1
      for step2 in 0..arch_level
        xi_coord = Math.cos((step2 * section_degrees + start_degrees).degrees) * x_radius / 2
        zi_coord = Math.sin((step2 * section_degrees + start_degrees).degrees) * z_radius
        xo_coord = Math.cos((step2 * section_degrees + start_degrees).degrees) * (x_radius / 2 + arch_thickness)
        zo_coord = Math.sin((step2 * section_degrees + start_degrees).degrees) * (z_radius + arch_thickness)
        ### Inner arch ###
        pts[step + step * arch_level + step2] = [(arch_length + arch_division) * (step + 1) - xi_coord - arch_length / 2, 0, zi_coord - height_offset + foundation_height]
        ptsx[step + step * arch_level + step2] = [(arch_length + arch_division) * (step + 1) - xi_coord - arch_length / 2, bridge_width, zi_coord - height_offset + foundation_height]
        ### Outer arch ###
        pts[steps - (step + step * arch_level + step2)] = [(arch_length + arch_division) * (step + 1) - xo_coord - arch_length / 2, 0, zo_coord - height_offset + foundation_height]
        ptsx[steps - (step + step * arch_level + step2)] = [(arch_length + arch_division) * (step + 1) - xo_coord - arch_length / 2, bridge_width, zo_coord - height_offset + foundation_height]
      end
    end
    face = group_ents.add_face pts
    steps = steps + 1
    for step in 0..steps - 1
      face = group_ents.add_face pts[step.modulo(steps)], pts[(step + 1).modulo(steps)], ptsx[(step + 1).modulo(steps)], ptsx[step.modulo(steps)]
    end
    face = group_ents.add_face ptsx
    
    ## Draw columns ###
    if middle_column_thickness > 0
      for step in 0..arches
        start_x_point = arch_division / 2 + (arch_division + arch_length) * step - middle_column_thickness / 2
        end_x_point = start_x_point + middle_column_thickness
       
        pts [0] = [start_x_point, 0, foundation_height]
        pts [1] = [end_x_point, 0, foundation_height]
        pts [2] = [end_x_point, bridge_width, foundation_height]
        pts [3] = [start_x_point, bridge_width, foundation_height]
        pts [4] = [start_x_point, 0, bridge_height - deck_thickness + foundation_height]
        pts [5] = [end_x_point, 0, bridge_height - deck_thickness + foundation_height]
        pts [6] = [end_x_point, bridge_width, bridge_height - deck_thickness + foundation_height]
        pts [7] = [start_x_point, bridge_width, bridge_height - deck_thickness + foundation_height]
        for step2 in 0..3
          face = group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
        end
        ### Draw arch colums ###
        if arch_columns > 0
          if step < arches
            arch_column_total_distance = arch_length + arch_division - middle_column_thickness
            arch_column_distance = (arch_column_total_distance - arch_column_thickness * arch_columns) / (arch_columns + 1)
            start_arch_columns_point = end_x_point
            for step2 in 0..arch_columns - 1
              start_x_point = start_arch_columns_point + arch_column_distance + (arch_column_distance + arch_column_thickness) * step2
              end_x_point = start_x_point + arch_column_thickness
              pts = []
              pts[0] = [start_x_point, 0, bridge_height - deck_thickness + foundation_height]
              pts[1] = [end_x_point, 0, bridge_height - deck_thickness + foundation_height]
              pts[2] = [end_x_point, bridge_width, bridge_height - deck_thickness + foundation_height]
              pts[3] = [start_x_point, bridge_width, bridge_height - deck_thickness + foundation_height]
              face = group_ents.add_face pts
            end
          end
        end
      end
    end
    
    ### Draw deck ###
    start_y_point = (bridge_width - deck_width) / 2
    end_y_point = start_y_point + deck_width
    pts = []
    pts [0] = [0, start_y_point, bridge_height - deck_thickness + foundation_height]
    pts [1] = [bridge_length, start_y_point, bridge_height - deck_thickness + foundation_height]
    pts [2] = [bridge_length, end_y_point, bridge_height - deck_thickness + foundation_height]
    pts [3] = [0, end_y_point, bridge_height - deck_thickness + foundation_height]
    pts [4] = [0, start_y_point, bridge_height + foundation_height]
    pts [5] = [bridge_length, start_y_point, bridge_height + foundation_height]
    pts [6] = [bridge_length, end_y_point, bridge_height + foundation_height]
    pts [7] = [0, end_y_point, bridge_height + foundation_height]
    for step2 in 0..3
      face = group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
    end
    face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
    face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
  end




  ########################
  ### Draw foundations ###
  ########################
 
  if foundation_height > 0
    foundation_group = Sketchup.active_model.entities.add_group
    foundation_group_ents = foundation_group.entities
    for step in 0..arches
      start_x_point = arch_division / 2 + (arch_division + arch_length) * step - foundation_length / 2
      end_x_point = start_x_point + foundation_length
      start_y_point = (bridge_width - foundation_width) / 2
      end_y_point = start_y_point + foundation_width
      pts = []
      pts [0] = [start_x_point, start_y_point, 0]
      pts [1] = [end_x_point, start_y_point, 0]
      pts [2] = [end_x_point, end_y_point, 0]
      pts [3] = [start_x_point, end_y_point, 0]
      pts [4] = [start_x_point, start_y_point, foundation_height]
      pts [5] = [end_x_point, start_y_point, foundation_height]
      pts [6] = [end_x_point, end_y_point, foundation_height]
      pts [7] = [start_x_point, end_y_point, foundation_height]
      for step2 in 0..3
        face = foundation_group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
      end
      face = foundation_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
    end
   
  end
 
  #####################
  ### Draw railings ###
  #####################
 
end


##################################
##################################
### Code for box girder bridge ###
##################################
##################################
   
def box_girder_bridge

  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials
  prompts = ["Bridge length", "Bridge height", "Deck width", "Deck thickness", "Box width", "Box height", "Pillars", "Pillar width", "Pillar thickness", "Pillar height", "Angle", "X-pos", "Y-pos", "Extensions"]
  defaults = ["10000", "2000", "700", "100", "300", "400", "6", "300", "100", "1500", "0", "0", "0", ""]
  list = ["", "", "", "", "", "", "", "", "", "", "", "", "", ""]
  input = UI.inputbox prompts, defaults, list, "Box girder bridge info"

  bridge_length = input[0].to_f / $inch
  bridge_height = input[1].to_f / $inch
  deck_width = input[2].to_f / $inch
  deck_thickness = input[3].to_f / $inch
  box_width = input[4].to_f / $inch
  box_height = input[5].to_f / $inch
  pillars = input[6].to_i
  pillar_width = input[7].to_f / $inch
  pillar_thickness = input[8].to_f / $inch
  pillar_height = input[9].to_f / $inch

  ### Calculate distance between pillars, leaving same distance at both ends ###
  pillar_distance = (bridge_length - pillar_thickness * pillars) / (pillars + 1)

  ### Draw pillars ###
  for step in 0.. pillars - 1
    start_x_position = pillar_distance + (pillar_distance + pillar_thickness) * step
    end_x_position = start_x_position + pillar_thickness
    start_y_position = deck_width / 2 - pillar_width / 2
    end_y_position = start_y_position + pillar_width
    pts = []
    pts[0] = [start_x_position, start_y_position, 0]
    pts[1] = [end_x_position, start_y_position, 0]
    pts[2] = [end_x_position, end_y_position, 0]
    pts[3] = [start_x_position, end_y_position, 0]
    pts[4] = [start_x_position, start_y_position, pillar_height]
    pts[5] = [end_x_position, start_y_position, pillar_height]
    pts[6] = [end_x_position, end_y_position, pillar_height]
    pts[7] = [start_x_position, end_y_position, pillar_height]
    for step2 in 0..3
      face = group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
    end
  end
  
  ### Draw box girder ###
  start_y_position = deck_width / 2 - box_width / 2
  end_y_position = start_y_position + box_width
  pts = []
  pts[0] = [0, start_y_position, pillar_height]
  pts[1] = [bridge_length, start_y_position, pillar_height]
  pts[2] = [bridge_length, end_y_position, pillar_height]
  pts[3] = [0, end_y_position, pillar_height]
  pts[4] = [0, start_y_position, pillar_height + box_height]
  pts[5] = [bridge_length, start_y_position, pillar_height + box_height]
  pts[6] = [bridge_length, end_y_position, pillar_height + box_height]
  pts[7] = [0, end_y_position, pillar_height + box_height]
  for step2 in 0..3
    face = group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
  end
  face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
  
  ### Draw deck ###
  pts = []
  pts[0] = [0, 0, pillar_height + box_height]
  pts[1] = [bridge_length, 0, pillar_height + box_height]
  pts[2] = [bridge_length, deck_width, pillar_height + box_height]
  pts[3] = [0, deck_width, pillar_height + box_height]
  pts[4] = [0, 0, bridge_height]
  pts[5] = [bridge_length, 0, bridge_height]
  pts[6] = [bridge_length, deck_width, bridge_height]
  pts[7] = [0, deck_width, bridge_height]
  for step2 in 0..3
    face = group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
  end
  face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
  face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]

end 




def deck_test

  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials

  profile_data = [1000 / $inch, 100 / $inch, 50 / $inch, 250 / $inch, 100 / $inch]
  profile = deck_profile("Wing", profile_data)
  
  pts = []
  
  for step in 0..profile.count - 1
    pts[step] = [0, profile[step][0], profile[step][1]]
  end
 
  face = group_ents.add_face pts
end   




def deck_profile (profile_type = "", profile_data = [])
  
##  profile_type = "Wing"
##  profile_data = [1000 / $inch, 100 / $inch, 50 / $inch, 250 / $inch, 100 / $inch]
  profile = []
  
  if profile_type == "Wing"
    profile[0] = [profile_data[1], 0]
    profile[1] = [0, 0 - profile_data[2]]
    profile[2] = [profile_data[3], 0 - profile_data[2] - profile_data[4]]
    profile[3] = [profile_data[0] + (profile_data[1] * 2) - profile_data[3], 0 - profile_data[2] - profile_data[4]]
    profile[4] = [profile_data[0] + (profile_data[1] * 2), 0 - profile_data[2]]
    profile[5] = [profile_data[0] + profile_data[1], 0]
    
  end  


  return profile
end



def check_line
  
  selection = Sketchup.active_model.selection
  items = selection.count
  item_count = 0
  line_pts = []
  
  for step in 0..items - 1
    if selection[step].typename == "Edge"
      
      echo step
      
    end
    
    
  end  
  
end