############################
#### Drawing procedures ####
############################

$drawing_on = 0
$fence_drawing_on = 0
$gate_drawing_on = 0

$fence_settings = 0
$fence_posts = []
$fence_post_count = 0

$taxiway_width = 800 / $inch
$taxiline_width = 20 / $inch

$draw_count = 0

$fence_height = 180.0 / $inch
$fence_thickness = 0 / $inch
$fence_type = "Panel"
$post_height = 200.0 / $inch
$post_thickness = 5.0 / $inch
$post_type = "Square"
$post_distance = 300.0 / $inch
$post_drawn_at_end = "Calculated"
$post_force_post_angle = 15
$fence_follow_terrain = "Yes"
$fence_texture = "Untextured"
$post_texture = "Untextured"
$post_remaining_distance = 0 / $inch

$gate_type = "Farm"
$gate_height = 120 / $inch
$gate_ground_offset = 10 / $inch
$gate_thickness = 5 / $inch
$gate_frame_thickness = 10 / $inch
$gate_horizontal_beams = 3
$gate_vertical_beams = 1
$gate_level = 45 ### Between -90 and 90 degrees, with 0 being closed
$gate_back_level = 45 ### Between -90 and 90 degrees, with 0 being closed
$gate_split = "Yes"
$gate_connect_to_post = "Yes"
$gate_fence_at_start = 50 / $inch
$gate_fence_at_end = 50 / $inch
$gate_post_type = "Square"
$gate_post_height = 140 / $inch
$gate_post_thickness = 20 / $inch
$gate_draw_filler = "Yes"
$gate_texture = "Untextured"
$gate_frame_texture = "Untextured"
$gate_post_texture = "Untextured"

$current_entity_id = 0
$previous_entityid = 0

$previous_angle = 0


########################################
### 

class MyEntitiesObserver < Sketchup::EntitiesObserver
  def onElementAdded(entities, entity)
    
    if $drawing_on == 1 and entity.typename == "Edge"
      
      group = Sketchup.active_model.entities.add_group
      group_ents = group.entities

      start_x_pos = entity.start.position[0].to_f
      start_y_pos = entity.start.position[1].to_f
      end_x_pos = entity.end.position[0].to_f
      end_y_pos = entity.end.position[1].to_f
        
      x_length = (end_x_pos - start_x_pos)
      y_length = (end_y_pos - start_y_pos)

      taxiway_angle = Math.atan((y_length / x_length)).radians.modulo(360)
        
      if x_length < 0
        taxiway_angle = (taxiway_angle - 180).modulo(360)
      end
      
      taxiway_angle = taxiway_angle.degrees
      taxiway_radius = $taxiway_width / 2
        
      start_right_x_pos = start_x_pos + taxiway_radius * Math.sin(taxiway_angle)
      start_left_x_pos = start_x_pos - taxiway_radius * Math.sin(taxiway_angle)
      end_right_x_pos = end_x_pos + taxiway_radius * Math.sin(taxiway_angle)
      end_left_x_pos = end_x_pos - taxiway_radius * Math.sin(taxiway_angle)
      
      start_right_y_pos = start_y_pos - taxiway_radius * Math.cos(taxiway_angle)
      start_left_y_pos = start_y_pos + taxiway_radius * Math.cos(taxiway_angle)
      end_right_y_pos = end_y_pos - taxiway_radius * Math.cos(taxiway_angle)
      end_left_y_pos = end_y_pos + taxiway_radius * Math.cos(taxiway_angle)
        
      pts = []        
      pts[0] = [start_right_x_pos, start_right_y_pos, 0]
      pts[1] = [start_left_x_pos, start_left_y_pos, 0]
      pts[2] = [end_left_x_pos, end_left_y_pos, 0]
      pts[3] = [end_right_x_pos, end_right_y_pos, 0]
      face = group_ents.add_face pts
      if face.normal[2].to_i == -1
        status = face.reverse!
      end
      delete_guide(entity.entityID)
    elsif $fence_drawing_on == 1 and entity.typename == "Edge"
      group = Sketchup.active_model.entities.add_group
      group_ents = group.entities
      materials = Sketchup.active_model.materials
      
      start_x_pos = entity.start.position[0].to_f
      start_y_pos = entity.start.position[1].to_f
      end_x_pos = entity.end.position[0].to_f
      end_y_pos = entity.end.position[1].to_f
      
      if $fence_follow_terrain == "Yes"
        start_z_pos = entity.start.position[2].to_f
        end_z_pos = entity.end.position[2].to_f
      else
        start_z_pos = 0
        end_z_pos = 0
      end

      x_length = (end_x_pos - start_x_pos)
      y_length = (end_y_pos - start_y_pos)
      z_length = (end_z_pos - start_z_pos)
      
      fence_angle = Math.atan((y_length / x_length)).radians.modulo(360)
        
      if x_length < 0
        fence_angle = (fence_angle + 90).modulo(360)
      else
        fence_angle = (fence_angle - 90).modulo(360)
      end

      fence_angle_difference = (fence_angle - $previous_angle.modulo(360)).abs
      $previous_angle = fence_angle
      fence_angle = fence_angle.degrees

      fence_length = Math.sqrt(x_length * x_length + y_length * y_length).to_f

      for step in 0..$texture_name.length - 1
        if $fence_texture == $texture_name[step]
          texture = materials.add $texture_name[step]
          if $texture_filename[step].count(":") === 0
            texture.texture = $texture_files + $texture_filename[step]
          else
            texture.texture = $texture_filename[step]
          end
          texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
        end
      end

      if $fence_texture == "Untextured"
          texture = materials.add "Untextured"
          texture.color = "White"
      end

      ########################
      ### Draw fence panel ###
      ########################
      
      if $fence_type == "Panel"
        if $fence_thickness === 0
          pts = []        
          pts[0] = [start_x_pos, start_y_pos, start_z_pos]
          pts[1] = [start_x_pos, start_y_pos, start_z_pos + $fence_height]
          pts[2] = [end_x_pos, end_y_pos, end_z_pos + $fence_height]
          pts[3] = [end_x_pos, end_y_pos, end_z_pos]
          face = group_ents.add_face pts
          status = face.material = $fence_texture
          status = face.back_material = $fence_texture
        else
          pts = []
          pts[0] = [start_x_pos - $fence_thickness / 2, start_y_pos, start_z_pos]
          pts[1] = [start_x_pos - $fence_thickness / 2, start_y_pos + fence_length, end_z_pos]
          pts[2] = [start_x_pos + $fence_thickness / 2, start_y_pos + fence_length, end_z_pos]
          pts[3] = [start_x_pos + $fence_thickness / 2, start_y_pos, start_z_pos]
          pts[4] = [start_x_pos - $fence_thickness / 2, start_y_pos, start_z_pos + $fence_height]
          pts[5] = [start_x_pos - $fence_thickness / 2, start_y_pos + fence_length, end_z_pos + $fence_height]
          pts[6] = [start_x_pos + $fence_thickness / 2, start_y_pos + fence_length, end_z_pos + $fence_height]
          pts[7] = [start_x_pos + $fence_thickness / 2, start_y_pos, start_z_pos + $fence_height]
          for step in 0..3
            face = group_ents.add_face pts[(step + 1).modulo(4)], pts[(step).modulo(4)], pts[(step).modulo(4) + 4], pts[(step + 1).modulo(4) + 4]
            status = face.material = $fence_texture
          end
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          status = face.material = $fence_texture
          t = Geom::Transformation.rotation [start_x_pos,start_y_pos,start_z_pos], Z_AXIS, fence_angle
          group.transform! t
        end
      elsif $fence_type == "Picket"
        beam_thickness = $post_thickness - $fence_thickness
        picket_width = $post_thickness * 2
        bottom_beam_offset = beam_thickness * 2
        top_beam_offset = $fence_height - beam_thickness * 2
          pts = []
          pts[0] = [start_x_pos + $post_thickness / 2, start_y_pos, bottom_beam_offset]
          pts[1] = [start_x_pos + $post_thickness / 2, start_y_pos + fence_length, bottom_beam_offset]
          pts[2] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos + fence_length, bottom_beam_offset]
          pts[3] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos, bottom_beam_offset]
          pts[4] = [start_x_pos + $post_thickness / 2, start_y_pos, bottom_beam_offset + beam_thickness]
          pts[5] = [start_x_pos + $post_thickness / 2, start_y_pos + fence_length, bottom_beam_offset + beam_thickness]
          pts[6] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos + fence_length, bottom_beam_offset + beam_thickness]
          pts[7] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos, bottom_beam_offset + beam_thickness]
          for step in 0..3
            face = group_ents.add_face pts[(step).modulo(4)], pts[(step + 1).modulo(4)], pts[(step + 1).modulo(4) + 4], pts[(step).modulo(4) + 4]
          end
          face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          pts = []
          pts[0] = [start_x_pos + $post_thickness / 2, start_y_pos, top_beam_offset]
          pts[1] = [start_x_pos + $post_thickness / 2, start_y_pos + fence_length, top_beam_offset]
          pts[2] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos + fence_length, top_beam_offset]
          pts[3] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos, top_beam_offset]
          pts[4] = [start_x_pos + $post_thickness / 2, start_y_pos, top_beam_offset + beam_thickness]
          pts[5] = [start_x_pos + $post_thickness / 2, start_y_pos + fence_length, top_beam_offset + beam_thickness]
          pts[6] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos + fence_length, top_beam_offset + beam_thickness]
          pts[7] = [start_x_pos + $post_thickness / 2 - beam_thickness, start_y_pos, top_beam_offset + beam_thickness]
          for step in 0..3
            face = group_ents.add_face pts[(step).modulo(4)], pts[(step + 1).modulo(4)], pts[(step + 1).modulo(4) + 4], pts[(step).modulo(4) + 4]
          end
          face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          # Draw pickets
          pickets = (($post_distance - $fence_thickness) / (picket_width + $fence_thickness)).floor
          
          for step2 in 0..pickets - 1
            current_y_pos = start_y_pos + $fence_thickness + $post_thickness / 2 + (picket_width + $fence_thickness) * step2
            pts = []
            pts[0] = [start_x_pos + $post_thickness / 2 - beam_thickness - $fence_thickness, current_y_pos, bottom_beam_offset]
            pts[1] = [start_x_pos + $post_thickness / 2 - beam_thickness - $fence_thickness, current_y_pos + picket_width, bottom_beam_offset]
            pts[2] = [start_x_pos + $post_thickness / 2 - beam_thickness, current_y_pos + picket_width, bottom_beam_offset]
            pts[3] = [start_x_pos + $post_thickness / 2 - beam_thickness, current_y_pos, bottom_beam_offset]
            pts[4] = [start_x_pos + $post_thickness / 2 - beam_thickness - $fence_thickness, current_y_pos, $fence_height]
            pts[5] = [start_x_pos + $post_thickness / 2 - beam_thickness - $fence_thickness, current_y_pos + picket_width, $fence_height]
            pts[6] = [start_x_pos + $post_thickness / 2 - beam_thickness, current_y_pos + picket_width, $fence_height]
            pts[7] = [start_x_pos + $post_thickness / 2 - beam_thickness, current_y_pos, $fence_height]
            for step in 0..3
            face = group_ents.add_face pts[(step + 1).modulo(4)], pts[(step).modulo(4)], pts[(step).modulo(4) + 4], pts[(step + 1).modulo(4) + 4]
            end
            face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
            face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          end
          
          
          t = Geom::Transformation.rotation [start_x_pos,start_y_pos,0], Z_AXIS, fence_angle
          group.transform! t
      end
      
      ######################
      ### Draw end post ###
      ######################

      if $post_drawn_at_end == "Yes" and valid_post(end_x_pos, end_y_pos) == true
        logstring("Draw end post", 3)
        post_group = draw_post(end_x_pos, end_y_pos, end_z_pos)
        t = Geom::Transformation.rotation [end_x_pos,end_y_pos,end_z_pos], Z_AXIS, fence_angle
        post_group.transform! t
        group = group.parent.entities.add_group([group, post_group])
        $fence_posts[$fence_post_count] = [end_x_pos, end_y_pos]
        $fence_post_count += 1
      end
        
      ### Draw remaining posts ###
      if fence_angle_difference > $post_force_post_angle
        $post_remaining_distance = 0.0 / $inch
      end
      logstring("Check force angle " + fence_angle_difference.to_s + " - " + $post_force_post_angle.to_s, 3)
      logstring("Distance from previous part " + $post_remaining_distance.to_s, 3)
      
      post_count = ((fence_length + $post_remaining_distance) / $post_distance).floor
      post_rate = (fence_length + $post_remaining_distance) / $post_distance
      post_rate = fence_length / $post_distance
      post_length_offset_rate = 1 - $post_remaining_distance / $post_distance


      for step2 in 0..post_count
        logstring("Set post " + step2.to_s + " of " + post_count.to_s, 3)
        if $post_remaining_distance > 0
          current_post_rate = (step2 + post_length_offset_rate) / post_rate
        else
          current_post_rate = step2 / post_rate
        end
        mid_x_pos = start_x_pos + x_length * current_post_rate
        mid_y_pos = start_y_pos + y_length * current_post_rate
        mid_z_pos = start_z_pos + z_length * current_post_rate
        logstring("Post position " + mid_x_pos.to_s + " - " + mid_y_pos.to_s + " - " + mid_z_pos.to_s, 3)
        if valid_post(mid_x_pos, mid_y_pos) == true and current_post_rate <= 1
          post_group = draw_post(mid_x_pos, mid_y_pos, mid_z_pos)
          t = Geom::Transformation.rotation [mid_x_pos,mid_y_pos,mid_z_pos], Z_AXIS, fence_angle
          post_group.transform! t
          group = group.parent.entities.add_group([group, post_group])
          $fence_posts[$fence_post_count] = [mid_x_pos, mid_y_pos]
          $fence_post_count += 1
        end
      end
 
      if $post_drawn_at_end == "Calculated" or $post_drawn_at_end == "No"
        $post_remaining_distance = fence_length + $post_remaining_distance - $post_distance * post_count
        logstring("Distance remaining " + $post_remaining_distance.to_s, 3)
      end
      logstring("Deleting guide line " + entity.entityID.to_s, 3)
      delete_guide(entity.entityID)
    elsif $gate_drawing_on == 1 and entity.typename == "Edge"
      group = Sketchup.active_model.entities.add_group
      group_ents = group.entities
      materials = Sketchup.active_model.materials
      
      start_x_pos = entity.start.position[0].to_f
      start_y_pos = entity.start.position[1].to_f
      end_x_pos = entity.end.position[0].to_f
      end_y_pos = entity.end.position[1].to_f
      
      if $fence_follow_terrain == "Yes"
        start_z_pos = entity.start.position[2].to_f
        end_z_pos = entity.end.position[2].to_f
      else
        start_z_pos = 0
        end_z_pos = 0
      end
      x_length = (end_x_pos - start_x_pos)
      y_length = (end_y_pos - start_y_pos)
      z_length = (end_z_pos - start_z_pos)
      
      gate_angle = Math.atan((y_length / x_length)).radians.modulo(360)
        
      if x_length < 0
        gate_angle = (gate_angle + 90).modulo(360)
      else
        gate_angle = (gate_angle - 90).modulo(360)
      end
      
      gate_angle = gate_angle.degrees
      gate_length = Math.sqrt(x_length * x_length + y_length * y_length).to_f

      for step in 0..$texture_name.length - 1
        if $gate_texture == $texture_name[step] or $fence_texture == $texture_name[step]
          texture = materials.add $texture_name[step]
          if $texture_filename[step].count(":") === 0
            texture.texture = $texture_files + $texture_filename[step]
          else
            texture.texture = $texture_filename[step]
          end
          texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
        end
      end

      if $gate_texture == "Untextured" or $fence_texture == "Untextured"
          texture = materials.add "Untextured"
          texture.color = "White"
      end
      
      #### Draw gate start ####
      
      gate_middle_length = gate_length - $gate_fence_at_start - $gate_fence_at_end - $gate_post_thickness * 2
      start_post_rates = [($gate_fence_at_start + $gate_post_thickness / 2) / gate_length, (gate_length - $gate_fence_at_end - $gate_post_thickness / 2) / gate_length]
      gate_pivot_point = $gate_fence_at_start + $gate_post_thickness

      if $gate_connect_to_post == "Yes"
        if ($fence_thickness == 0)
          pts = []        
          pts[0] = [start_x_pos, start_y_pos, start_z_pos]
          pts[1] = [start_x_pos, start_y_pos, start_z_pos + $fence_height]
          pts[2] = [start_x_pos, start_y_pos + $gate_fence_at_start, start_z_pos + $gate_post_height]
          pts[3] = [start_x_pos, start_y_pos + $gate_fence_at_start, start_z_pos]
          face = group_ents.add_face pts
          status = face.material = $fence_texture
          status = face.back_material = $fence_texture
          pts = []        
          pts[0] = [start_x_pos, start_y_pos + gate_length, start_z_pos]
          pts[1] = [start_x_pos, start_y_pos + gate_length, start_z_pos + $fence_height]
          pts[2] = [start_x_pos, start_y_pos + gate_length - $gate_fence_at_end, start_z_pos + $gate_post_height]
          pts[3] = [start_x_pos, start_y_pos + gate_length - $gate_fence_at_end, start_z_pos]
          face = group_ents.add_face pts
          status = face.material = $fence_texture
          status = face.back_material = $fence_texture
        else
          pts = []
          pts[0] = [start_x_pos - $fence_thickness / 2, start_y_pos, start_z_pos]
          pts[1] = [start_x_pos - $fence_thickness / 2, start_y_pos + fence_length, end_z_pos]
          pts[2] = [start_x_pos + $fence_thickness / 2, start_y_pos + fence_length, end_z_pos]
          pts[3] = [start_x_pos + $fence_thickness / 2, start_y_pos, start_z_pos]
          pts[4] = [start_x_pos - $fence_thickness / 2, start_y_pos, start_z_pos + $fence_height]
          pts[5] = [start_x_pos - $fence_thickness / 2, start_y_pos + fence_length, end_z_pos + $fence_height]
          pts[6] = [start_x_pos + $fence_thickness / 2, start_y_pos + fence_length, end_z_pos + $fence_height]
          pts[7] = [start_x_pos + $fence_thickness / 2, start_y_pos, start_z_pos + $fence_height]
          for step2 in 0..3
            face = group_ents.add_face pts[(step2 + 1).modulo(4)], pts[(step2).modulo(4)], pts[(step2).modulo(4) + 4], pts[(step2 + 1).modulo(4) + 4]
            status = face.material = $fence_texture
          end
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          status = face.material = $fence_texture
          t = Geom::Transformation.rotation [start_x_pos,start_y_pos,start_z_pos], Z_AXIS, gate_angle
          group.transform! t
        end
      end

      #### Draw gate ####

      if $gate_split == "No"
        gate_group = draw_gate(start_x_pos, start_y_pos + gate_pivot_point, start_z_pos, gate_middle_length)
        t = Geom::Transformation.rotation [start_x_pos,start_y_pos + gate_pivot_point,start_z_pos], Z_AXIS, $gate_level.degrees
        gate_group.transform! t
        group = group.parent.entities.add_group([group, gate_group])
      else
        gate_group = draw_gate(start_x_pos, start_y_pos + gate_pivot_point, start_z_pos, gate_middle_length / 2)
        t = Geom::Transformation.rotation [start_x_pos,start_y_pos + gate_pivot_point,start_z_pos], Z_AXIS, $gate_level.degrees
        gate_group.transform! t
        gate_group2 = draw_gate(start_x_pos, start_y_pos + gate_pivot_point + gate_middle_length, start_z_pos, 0 - gate_middle_length / 2)
        t = Geom::Transformation.rotation [start_x_pos,start_y_pos + gate_pivot_point + gate_middle_length,start_z_pos], Z_AXIS, (0 - $gate_back_level).degrees
        gate_group2.transform! t
        group = group.parent.entities.add_group([group, gate_group, gate_group2])
      end
      
      t = Geom::Transformation.rotation [start_x_pos,start_y_pos,start_z_pos], Z_AXIS, gate_angle
      group.transform! t

           
      for step in 0..1
        mid_x_pos = start_x_pos + x_length * start_post_rates[step]
        mid_y_pos = start_y_pos + y_length * start_post_rates[step]
        mid_z_pos = start_z_pos + z_length * start_post_rates[step]
        if valid_post(mid_x_pos, mid_y_pos) == true
          post_group = draw_post(mid_x_pos, mid_y_pos, mid_z_pos)
          t = Geom::Transformation.rotation [mid_x_pos,mid_y_pos,mid_z_pos], Z_AXIS, gate_angle
          post_group.transform! t
          group = group.parent.entities.add_group([group, post_group])
          $fence_posts[$fence_post_count] = [mid_x_pos, mid_y_pos]
          $fence_post_count += 1
        end
      end

      delete_guide(entity.entityID)
    end
  end
end

Sketchup.active_model.entities.add_observer(MyEntitiesObserver.new)






#################################################
#### Function to check for overlapping posts ####
#################################################

def valid_post (x_pos = 0.0, y_pos = 0.0)
  for step in 0..$fence_post_count - 1
    if (x_pos - $fence_posts[step][0]).abs < $post_thickness and (y_pos - $fence_posts[step][1]).abs < $post_thickness
      logstring("Post at coordinates " + x_pos.to_s + ", " + y_pos.to_s + " skipped", 2)
      return false
    end
  end
  return true
end
 

################################
#### Function to draw gates ####
################################

def draw_gate (x_pos = 0.0, y_pos = 0.0, z_pos = 0.0, gate_length = 0.0)
  gate_group = Sketchup.active_model.entities.add_group
  gate_group_ents = gate_group.entities
  materials = Sketchup.active_model.materials 

  for step in 0..$texture_name.length - 1
    if $gate_texture == $texture_name[step] or $gate_frame_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
      texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
    end
  end
  
  if $gate_texture == "Untextured" or $gate_frame_texture == "Untextured"
    texture = materials.add "Untextured"
    texture.color = "White"
  end

  if gate_length < 0
    gate_side = -1
  else
    gate_side = 1
  end

  if $gate_thickness === 0 and $gate_frame_thickness === 0
    pts = []        
    pts[0] = [x_pos, y_pos, z_pos + $gate_ground_offset]
    pts[1] = [x_pos, y_pos, z_pos + $gate_height]
    pts[2] = [x_pos, y_pos + gate_length, z_pos + $gate_height]
    pts[3] = [x_pos, y_pos + gate_length, z_pos + $gate_ground_offset]
    face = gate_group_ents.add_face pts
    status = face.material = $gate_texture
    status = face.back_material = $gate_texture
  elsif $gate_thickness === 0
    pts = []        
    pts[0] = [x_pos, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    pts[1] = [x_pos, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    pts[2] = [x_pos, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    pts[3] = [x_pos, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    face = gate_group_ents.add_face pts
    status = face.material = $gate_texture
    status = face.back_material = $gate_texture
  else
    ### Horizontal beams ###
    vertical_beam_distance = (gate_length - $gate_frame_thickness * 2 * gate_side - $gate_thickness * $gate_vertical_beams * gate_side) / ($gate_vertical_beams + 1)
    horizontal_beam_distance = ($gate_height - $gate_ground_offset - $gate_frame_thickness * 2 - $gate_thickness * $gate_horizontal_beams) / ($gate_horizontal_beams + 1)
    
    for step in 0..$gate_horizontal_beams - 1
      start_z_pos = $gate_ground_offset + $gate_frame_thickness + (horizontal_beam_distance + $gate_thickness) * step + horizontal_beam_distance
      end_z_pos = start_z_pos + $gate_thickness
      pts = []
      pts[0] = [x_pos + $gate_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + start_z_pos]
      pts[1] = [x_pos + $gate_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + start_z_pos]
      pts[2] = [x_pos + $gate_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + end_z_pos]
      pts[3] = [x_pos + $gate_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + end_z_pos]
      pts[4] = [x_pos - $gate_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + start_z_pos]
      pts[5] = [x_pos - $gate_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + start_z_pos]
      pts[6] = [x_pos - $gate_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + end_z_pos]
      pts[7] = [x_pos - $gate_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + end_z_pos]
      face = gate_group_ents.add_face pts[0], pts[1], pts[2], pts[3]
      status = face.material = $gate_frame_texture
      if gate_side == -1
        status = face.reverse!
      end
      face = gate_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
      status = face.material = $gate_frame_texture
      face = gate_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
      status = face.material = $gate_frame_texture
      face = gate_group_ents.add_face pts[7], pts[6], pts[2], pts[3]
      status = face.material = $gate_frame_texture
    end

    for step in 0..$gate_vertical_beams - 1
      start_y_pos = (vertical_beam_distance + $gate_thickness * gate_side) * step + $gate_frame_thickness * gate_side + vertical_beam_distance
      end_y_pos = start_y_pos + $gate_thickness * gate_side
      pts = []
      pts[0] = [x_pos + $gate_thickness / 2, y_pos + start_y_pos, z_pos + $gate_ground_offset + $gate_frame_thickness]
      pts[1] = [x_pos + $gate_thickness / 2, y_pos + end_y_pos, z_pos + $gate_ground_offset + $gate_frame_thickness]
      pts[2] = [x_pos + $gate_thickness / 2, y_pos + end_y_pos, z_pos + $gate_height - $gate_frame_thickness]
      pts[3] = [x_pos + $gate_thickness / 2, y_pos + start_y_pos, z_pos + $gate_height - $gate_frame_thickness]
      pts[4] = [x_pos - $gate_thickness / 2, y_pos + start_y_pos, z_pos + $gate_ground_offset + $gate_frame_thickness]
      pts[5] = [x_pos - $gate_thickness / 2, y_pos + end_y_pos, z_pos + $gate_ground_offset + $gate_frame_thickness]
      pts[6] = [x_pos - $gate_thickness / 2, y_pos + end_y_pos, z_pos + $gate_height - $gate_frame_thickness]
      pts[7] = [x_pos - $gate_thickness / 2, y_pos + start_y_pos, z_pos + $gate_height - $gate_frame_thickness]
      face = gate_group_ents.add_face pts[0], pts[1], pts[2], pts[3]
      status = face.material = $gate_frame_texture
      if gate_side == -1
        status = face.reverse!
      end
      face = gate_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
      status = face.material = $gate_frame_texture
      face = gate_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
      status = face.material = $gate_frame_texture
      face = gate_group_ents.add_face pts[0], pts[3], pts[7], pts[4]
      status = face.material = $gate_frame_texture
    end

    if $gate_draw_filler == "Yes"
      pts = []        
      pts[0] = [x_pos, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
      pts[1] = [x_pos, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
      pts[2] = [x_pos, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
      pts[3] = [x_pos, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
      face = gate_group_ents.add_face pts
      status = face.material = $gate_texture
      status = face.back_material = $gate_texture
    end
  end
  if $gate_frame_thickness > 0
    # Draw frame
    pts = []
    pts[0] = [x_pos + $gate_frame_thickness / 2, y_pos, z_pos + $gate_ground_offset]
    pts[1] = [x_pos + $gate_frame_thickness / 2, y_pos + gate_length, z_pos + $gate_ground_offset]
    pts[2] = [x_pos + $gate_frame_thickness / 2, y_pos + gate_length, z_pos + $gate_height]
    pts[3] = [x_pos + $gate_frame_thickness / 2, y_pos, z_pos + $gate_height]
    pts[4] = [x_pos + $gate_frame_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    pts[5] = [x_pos + $gate_frame_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    pts[6] = [x_pos + $gate_frame_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    pts[7] = [x_pos + $gate_frame_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    pts[8] = [x_pos - $gate_frame_thickness / 2, y_pos, z_pos + $gate_ground_offset]
    pts[9] = [x_pos - $gate_frame_thickness / 2, y_pos + gate_length, z_pos + $gate_ground_offset]
    pts[10] = [x_pos - $gate_frame_thickness / 2, y_pos + gate_length, z_pos + $gate_height]
    pts[11] = [x_pos - $gate_frame_thickness / 2, y_pos, z_pos + $gate_height]
    pts[12] = [x_pos - $gate_frame_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    pts[13] = [x_pos - $gate_frame_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_ground_offset + $gate_frame_thickness]
    pts[14] = [x_pos - $gate_frame_thickness / 2, y_pos + gate_length - $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    pts[15] = [x_pos - $gate_frame_thickness / 2, y_pos + $gate_frame_thickness * gate_side, z_pos + $gate_height - $gate_frame_thickness]
    face = gate_group_ents.add_face pts[0], pts[1], pts[2], pts[3]
    status = face.material = $gate_frame_texture
    if gate_side == -1
      status = face.reverse!
    end
    face = gate_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
    status = face.erase!
    face = gate_group_ents.add_face pts[11], pts[10], pts[9], pts[8]
    status = face.material = $gate_frame_texture
    if gate_side == -1
      status = face.reverse!
    end
    face = gate_group_ents.add_face pts[15], pts[14], pts[13], pts[12]
    status = face.erase!
    
    face = gate_group_ents.add_face pts[0], pts[1], pts[9], pts[8]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[1], pts[2], pts[10], pts[9]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[2], pts[3], pts[11], pts[10]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[3], pts[0], pts[8], pts[11]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[4], pts[5], pts[13], pts[12]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[5], pts[6], pts[14], pts[13]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[6], pts[7], pts[15], pts[14]
    status = face.material = $gate_frame_texture
    face = gate_group_ents.add_face pts[7], pts[4], pts[12], pts[15]
    status = face.material = $gate_frame_texture
    
  end
  return gate_group
end


################################
#### Function to draw posts ####
################################

def draw_post (x_pos = 0.0, y_pos = 0.0, z_pos = 0.0)
  post_group = Sketchup.active_model.entities.add_group
  post_group_ents = post_group.entities
  materials = Sketchup.active_model.materials


  if $gate_drawing_on == 1
    post_type = $gate_post_type
    post_texture = $gate_post_texture
    post_thickness = $gate_post_thickness
    post_height = $gate_post_height   
  elsif $fence_drawing_on == 1
    post_type = $post_type
    post_texture = $post_texture
    post_thickness = $post_thickness
    post_height = $post_height
  end
  
  for step in 0..$texture_name.length - 1
    if post_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
      texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
    end
  end

  if post_texture == "Untextured"
    texture = materials.add "Untextured"
    texture.color = "White"
  end
  
  if post_type == "Square"          
    pts = []
    pts[0] = [x_pos - post_thickness / 2, y_pos - post_thickness / 2, z_pos]
    pts[1] = [x_pos - post_thickness / 2, y_pos + post_thickness / 2, z_pos]
    pts[2] = [x_pos + post_thickness / 2, y_pos + post_thickness / 2, z_pos]
    pts[3] = [x_pos + post_thickness / 2, y_pos - post_thickness / 2, z_pos]
    pts[4] = [x_pos - post_thickness / 2, y_pos - post_thickness / 2, z_pos + post_height]
    pts[5] = [x_pos - post_thickness / 2, y_pos + post_thickness / 2, z_pos + post_height]
    pts[6] = [x_pos + post_thickness / 2, y_pos + post_thickness / 2, z_pos + post_height]
    pts[7] = [x_pos + post_thickness / 2, y_pos - post_thickness / 2, z_pos + post_height]
    for step in 0..3
      face = post_group_ents.add_face pts[(step + 1).modulo(4)], pts[(step).modulo(4)], pts[(step).modulo(4) + 4], pts[(step + 1).modulo(4) + 4]
      status = face.material = post_texture
    end
    face = post_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
    status = face.material = post_texture
  elsif post_type == "T post"
    metal_thickness = post_thickness / 8
    pts = []
    pts[0] = [x_pos - post_thickness / 2, y_pos - post_thickness / 2, z_pos]
    pts[1] = [x_pos - post_thickness / 2, y_pos + post_thickness / 2, z_pos]
    pts[2] = [x_pos - post_thickness / 2 + metal_thickness, y_pos + post_thickness / 2, z_pos]
    pts[3] = [x_pos - post_thickness / 2 + metal_thickness, y_pos + metal_thickness / 2, z_pos]
    pts[4] = [x_pos + post_thickness / 2, y_pos + metal_thickness / 2, z_pos]
    pts[5] = [x_pos + post_thickness / 2, y_pos - metal_thickness / 2, z_pos]
    pts[6] = [x_pos - post_thickness / 2 + metal_thickness, y_pos - metal_thickness / 2, z_pos]
    pts[7] = [x_pos - post_thickness / 2 + metal_thickness, y_pos - post_thickness / 2, z_pos]
    pts[8] = [x_pos - post_thickness / 2, y_pos - post_thickness / 2, z_pos + post_height]
    pts[9] = [x_pos - post_thickness / 2, y_pos + post_thickness / 2, z_pos + post_height]
    pts[10] = [x_pos - post_thickness / 2 + metal_thickness, y_pos + post_thickness / 2, z_pos + post_height]
    pts[11] = [x_pos - post_thickness / 2 + metal_thickness, y_pos + metal_thickness / 2, z_pos + post_height]
    pts[12] = [x_pos + post_thickness / 2, y_pos + metal_thickness / 2, z_pos + post_height]
    pts[13] = [x_pos + post_thickness / 2, y_pos - metal_thickness / 2, z_pos + post_height]
    pts[14] = [x_pos - post_thickness / 2 + metal_thickness, y_pos - metal_thickness / 2, z_pos + post_height]
    pts[15] = [x_pos - post_thickness / 2 + metal_thickness, y_pos - post_thickness / 2, z_pos + post_height]
    for step in 0..7
      face = post_group_ents.add_face pts[(step + 1).modulo(8)], pts[(step).modulo(8)], pts[(step).modulo(8) + 8], pts[(step + 1).modulo(8) + 8]
      status = face.material = post_texture
    end
    face = post_group_ents.add_face pts[8], pts[9], pts[10], pts[11], pts[12], pts[13], pts[14], pts[15]
    status = face.material = post_texture
  end
  return post_group
end


def delete_guide (entityID)
  $current_entity_id = entityID
  model = Sketchup.active_model
  entities = model.active_entities
  for step in 0..entities.count - 1
    if $previous_entityid == entities[step].entityID
      entities.erase_entities entities[step]
      break
    end
  end
  $previous_entityid = $current_entity_id

end


def add_texture (texture_name = "")
  
end


### Code for fence settings ###

def fence_settings
  prompts = ["Fence height", "Fence thickness", "Fence type", "Post height", "Post thickness", "Post type", "Post distance", "Post drawn at end", "Force post at x degrees", "Follow terrain", "Fence Texture", "Post Texture"]
  lists = ["", "", "Panel|Picket", "", "", "Square|T post", "", $yn + "|Calculated", "", $yn, $fence_list, $metal_list]
  defaults = [$fence_height * $inch, $fence_thickness * $inch, $fence_type, $post_height * $inch, $post_thickness * $inch, $post_type, $post_distance * $inch, $post_drawn_at_end, $post_force_post_angle, $fence_follow_terrain, $fence_texture, $post_texture]
  input = UI.inputbox prompts, defaults, lists, "Fence settings"
  
  $fence_height = input[0].to_f / $inch
  $fence_thickness = input[1].to_f / $inch
  $fence_type = input[2]
  $post_height = input[3].to_f / $inch
  $post_thickness = input[4].to_f / $inch
  $post_type = input[5]
  $post_distance = input[6] / $inch
  $post_drawn_at_end = input[7]
  $post_force_post_angle = input[8].to_i
  $fence_follow_terrain = input[9]
  $fence_texture = input[10]
  $post_texture = input[11]
  if ($fence_height < 1 / $inch)
    $fence_height = 1 / $inch
  end
  if ($fence_thickness < 0)
    $fence_thickness = 0
  end
  if ($post_thickness < $fence_thickness)
    $post_thickness = $fence_thickness
  end
  if ($post_height < $fence_height)
    $post_height = $fence_height
  end
  if ($post_distance < 0)
    $post_distance = 0
  end
end


### Code for gate settings ###

def gate_settings
  prompts = ["Gate type", "Gate height", "Gate ground offset", "Gate frame thickness", "Gate thickness", "Horizontal beams", "Vertical beams", "Draw filler", "Post type", "Post height", "Post thickness", "Fence offset at start", "Fence offset at end", "Connect to fence", "Gate open level", "Gate back open level", "Gate split", "Gate texture", "Frame texture", "Post texture"]
  lists = ["Farm", "", "", "", "", "", "", $yn, "Square", "", "", "", "", $yn, "", "", $yn, $fence_list, $metal_list, $wall_list]
  defaults = [$gate_type, $gate_height * $inch, $gate_ground_offset * $inch, $gate_frame_thickness * $inch, $gate_thickness * $inch, $gate_horizontal_beams, $gate_vertical_beams, $gate_draw_filler, $gate_post_type, $gate_post_height * $inch, $gate_post_thickness * $inch, $gate_fence_at_start * $inch, $gate_fence_at_end * $inch, $gate_connect_to_post, $gate_level, $gate_back_level, $gate_split, $gate_texture, $gate_frame_texture, $gate_post_texture]
  input = UI.inputbox prompts, defaults, lists, "Gate settings"

  $gate_type = input[0]
  $gate_height = input[1].to_f / $inch
  $gate_ground_offset = input[2].to_f / $inch
  $gate_frame_thickness = input[3].to_f / $inch
  $gate_thickness = input[4].to_f / $inch
  $gate_horizontal_beams = input[5].to_i
  $gate_vertical_beams = input[6].to_i
  $gate_draw_filler = input[7]
  $gate_post_type = input[8]
  $gate_post_height = input[9].to_f / $inch
  $gate_post_thickness = input[10].to_f / $inch
  $gate_fence_at_start = input[11].to_f / $inch
  $gate_fence_at_end = input[12].to_f / $inch
  $gate_connect_to_post = input[13]
  $gate_level = input[14].to_i ### Between -90 and 90 degrees, with 0 being closed
  $gate_back_level = input[15].to_i ### Between -90 and 90 degrees, with 0 being closed
  $gate_split = input[16]
  $gate_texture = input[17]
  $gate_frame_texture = input[18]
  $gate_post_texture = input[19]
  
  ### Validation ###
  $gate_height = $gate_post_height if $gate_height > $gate_post_height
  $gate_horizontal_beams = 0 if $gate_horizontal_beams < 0
  $gate_vertical_beams = 0 if $gate_vertical_beams < 0
  $gate_level = 90 if $gate_level > 90
  $gate_level = -90 if $gate_level < -90
  $gate_back_level = 90 if $gate_back_level > 90
  $gate_back_level = -90 if $gate_back_level < -90
  $gate_ground_offset = $gate_height - $gate_frame_thickness * 2 - $gate_thickness * $gate_horizontal_beams if $gate_ground_offset > $gate_height
  $gate_ground_offset = 0 if $gate_ground_offset < 0
end
