##################################
####      Extensions for      ####
#### One Click Software Suite ####
####  ©2014 Morten Blindheim  ####
##################################

###########################################################
###### Below is the code to add extensions to hangar ######
###########################################################

def add_extensions(extension_code = "", hangar_type = "", hangar_length = 0.0, hangar_width = 0.0, hangar_height = 0.0, hangar_arc = 0.0, wall_thickness = 0.0, door_type = "", wall_texture = "", roof_texture = "", internal_wall_texture = "")

  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  group.description = "Extension:" + extension_code
  materials = Sketchup.active_model.materials

  extension = []

  if extension_code.include? ","
    extension = extension_code.split(",")
  else
    extension[0] = extension_code
  end

  # Default values
  roof_only = 0
  wall_only = 0
  front_left_cut = [0.0, 0.0] # first for cut at wall, second for cut at end
  back_right_cut = [0.0, 0.0]
  height = hangar_height
  arc = 0.0
  mid_arc = 0.0
  arc_type = ""
  arc_direction = ""
  length = 0.0
  extension_offset = 0.0
  posts = 0
  post_offset = 0.0
  floor_height = 0.0
  gate_count = 0
  gate_info = []
  no_posts = 0
  rotate_extension = 0
  height_repeat = 0
  height_gap = 0
  extension_attribute_type = []
  extension_attribute_string = []
  extension_attribute_value = []
  door_x_position = []
  door_y_position = []
  door_x_size = []
  door_y_size = []
  door_texture = []
  door_frame_texture = []
  door_frame_thickness = []
  door_frame_width = []
  door_wall = []
  door_number = 0
  texture_added = []
  window_count = 0
  window_x_pos = []
  window_y_pos = []
  window_x_size = []
  window_y_size = []
  window_texture = []
  window_transparency = []
  window_wall = []
  window_repeat = []
  window_gap = [] 
  
  extension_type = extension[0][0..extension[0].index("(") - 1]
  extension_details =  extension[0][extension[0].index("(") + 1..extension[0].index(")") - 1]
  
  if extension_details.include? "x"
    extension_values = extension_details.split("x")
    extension_offset = extension_values[1].to_f / $inch
    length = extension_values[0].to_f / $inch
  else
    length = extension_details.to_f / $inch
  end
  
  if extension_offset < 0
    extension_offset = 0 - extension_offset
  end
  
  if extension.length > 1
  
    for step in 1..extension.length - 1
      if extension[step].include? "(" and extension[step].include? ")"
        extension_attribute_type[step] = extension[step][0..extension[step].index("(") - 1]
        extension_attribute_string[step] = extension[step][extension[step].index("(") + 1..extension[step].index(")") - 1]
        
        #Cut extension
        if extension_attribute_type[step] == "C"
          if extension_attribute_string[step].count(";") > 0
            cut_ext = extension_attribute_string[step].split(";")
            for cut_step in 0..1
              if cut_ext[cut_step].count("x") > 0
                cut_ext2 = cut_ext[cut_step].split("x")
                front_left_cut[cut_step] = cut_ext2[0].to_f / $inch
                back_right_cut[cut_step] = cut_ext2[1].to_f / $inch
              else
                front_left_cut[cut_step] = cut_ext[0].to_f / $inch
              end
            end
          else
            if extension_attribute_string[step].count("x") > 0
              cut_ext = extension_attribute_string[step].split("x")
              front_left_cut[0] = cut_ext[0].to_f / $inch
              front_left_cut[1] = cut_ext[0].to_f / $inch
              back_right_cut[0] = cut_ext[1].to_f / $inch
              back_right_cut[1] = cut_ext[1].to_f / $inch
            else
              front_left_cut[0] = extension_attribute_string[step].to_f / $inch
              front_left_cut[1] = extension_attribute_string[step].to_f / $inch
            end
            
          end
        #Height
        elsif extension_attribute_type[step] == "H" and extension_type != "RF" and extension_type != "LF"
          if extension_attribute_string[step].count('x') > 0
            height_info = extension_attribute_string[step].split('x')
            height = height_info[0].to_f / $inch
            height_repeat = height_info[1].to_i
            height_gap = height_info[2].to_f / $inch
          else
            height = extension_attribute_string[step].to_f / $inch
          end
        elsif extension_attribute_type[step] == "A" and extension_type != "RF" and extension_type != "LF"
          if extension_attribute_string[step].count("x") > 0
            arc_ext = extension_attribute_string[step].split("x") 
            arc = arc_ext[0].to_f / $inch
            mid_arc = arc_ext[1].to_f / $inch
            if extension_attribute_string[step].count("x") > 2
              arc_type = arc_ext[2]
              arc_direction = arc_ext[3]
            elsif extension_attribute_string[step].count("x") == 2
              arc_type = arc_ext[2]
            end
          else
            arc = extension_attribute_string[step].to_f / $inch
          end
        #Posts
        elsif extension_attribute_type[step] == "P"
          if extension_attribute_string[step].count('x') > 0
            post_offset =(extension_attribute_string[step][extension_attribute_string[step].index("x") + 1..extension_attribute_string[step].length].to_f) / $inch
            posts = extension_attribute_string[step][0..extension_attribute_string[step].index("x") - 1].to_i
          else
            posts = extension_attribute_string[step].to_i
            post_offset = 0
          end
          
          if posts < 2
            posts = 2
          end
        #Floor
        elsif extension_attribute_type[step] == "F"
          if extension_attribute_string[step].count(';') > 0
            floor_height =(extension_attribute_string[step][0..extension_attribute_string[step].index(";") - 1].to_f) / $inch
            ext_floor_texture = extension_attribute_string[step][extension_attribute_string[step].index(";") + 1..extension_attribute_string[step].length]
          else
            floor_height = extension_attribute_string[step].to_f / $inch
            ext_floor_texture = "W"
          end
          if floor_height < 1 / $inch
            floor_height = 1 / $inch
          end
          if ext_floor_texture != "W" and ext_floor_texture != "C"
            ext_floor_texture = "W"
          end

        #Gates
        elsif extension_attribute_type[step] == "G"
          if extension_attribute_string[step].count('x') > 0
          else
            gate_info[gate_count] = extension_attribute_string[step].to_i
            gate_count = gate_count + 1
          end
        #Extension rotation(angle x x_coord x y_coord
        elsif extension_attribute_type[step] == "N"
          if extension_attribute_string[step].count('x') == 2
            rotation_info = extension_attribute_string[step].split("x")
            rotate_extension = 1
          end

        #Custom textures        
        elsif extension_attribute_type[step] == "T"
          if extension_attribute_string[step].count(';') > 0
            texture_info = extension_attribute_string[step].split(';')
            texture_found = 0
            for step in 0..$texture_name.length
              if texture_info[1] == $texture_name[step]
                texture_found = 1
              end
            end
      #     UI.messagebox texture_info[0]
            if texture_found == 1
              if texture_info[0] == "W"
                wall_texture = texture_info[1]
              elsif texture_info[0] == "R"
                roof_texture = texture_info[1]
              elsif texture_info[0] == "F"
                floor_texture = texture_info[1]
              end
            end
          end
        #Doors
        elsif extension_attribute_type[step] == "D"
          door_failed = 0
          if extension_attribute_string[step].count(';') > 0
            door_info = extension_attribute_string[step].split(';')
            if door_info.length >= 4
              if door_info[0].count('x') == 1
                temp_door_info = door_info[0].split('x')
                door_x_position[door_number] = temp_door_info[0].to_f / $inch
                door_y_position[door_number] = temp_door_info[1].to_f / $inch
              else
                door_failed = 1
              end
              if door_info[1].count('x') == 1
                temp_door_info = door_info[1].split('x')
                door_x_size[door_number] = temp_door_info[0].to_f / $inch
                door_y_size[door_number] = temp_door_info[1].to_f / $inch
              else
                door_failed = 1
              end
              door_wall[door_number] = door_info[2]
              door_texture[door_number] = door_info[3]
              if door_info.length == 7
                door_frame_width[door_number] = door_info[4].to_f / $inch
                door_frame_thickness[door_number] = door_info[5].to_f / $inch
                door_frame_texture[door_number] = door_info[6]
              else
                door_frame_width[door_number] = 10 / $inch
                door_frame_thickness[door_number] = 2 / $inch
                door_frame_texture[door_number] = "Wooden planks"
              end
            else
              door_failed = 1
            end
          end
          if door_failed == 0
            door_number = door_number + 1
          end
        end
        # Windows
        if extension_attribute_type[step] == "W"
          if extension_attribute_string[step].count(';')>= 4
            window_failed = 0
            window_items = extension_attribute_string[step].split(";")
            if window_items[0].count("x") == 1
              window_details = window_items[0].split("x")
              window_x_pos[window_count] = window_details[0].to_f / $inch
              window_y_pos[window_count] = window_details[1].to_f / $inch
            else
            window_failed = 1
            end
            if window_items[1].count("x") == 1
              window_details = window_items[1].split("x")
              window_x_size[window_count] = window_details[0].to_f / $inch
              window_y_size[window_count] = window_details[1].to_f / $inch
            else
              window_failed = 1
            end
            window_texture[window_count] = window_items[3]
            window_transparency[window_count] = window_items[4].to_i
            if window_transparency[window_count] < 0
              window_transparency[window_count] = 0
            elsif window_transparency[window_count] > 100
              window_transparency[window_count] = 100
            end
            window_wall[window_count] = window_items[2]
            if window_wall[window_count] != "F" and window_wall[window_count] != "B" and window_wall[window_count] != "R" and window_wall[window_count] != "L"
              window_failed = 1
            end
            if extension_attribute_string[step].count(";") == 6
              window_repeat[window_count] = window_items[5].to_i
              window_gap[window_count] = window_items[6].to_f / $inch
            else
              window_repeat[window_count] = 1
              window_gap[window_count] = 0
            end
            if window_failed == 0
              window_count = window_count + 1
            end
          end
        end
          
        
      else
        if extension[step] == "R" and wall_only == 0
          roof_only = 1
          extension_attribute_type[step] = ""
        elsif extension[step] == "W" and roof_only == 0
          wall_only = 1
          extension_attribute_type[step] = ""
        elsif extension[step] == "NP"
          no_posts = 1
          extension_attribute_type[step] = ""
        end
      end
    
    end
  end
  



  # Load textures

  for step in 0..$texture_name.length - 1
    texture_added[step] = 0
  end
  
  for step in 0..$texture_name.length - 1
    if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or floor_texture == $texture_name[step] or internal_wall_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      texture.texture = $texture_files + $texture_filename[step]
      texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch] 
      texture_added[step] = 1
    end 
    if door_frame_texture.length > 0
      for texture_step in 0..door_frame_texture.length - 1
        if door_frame_texture[texture_step] == $texture_name[step] and texture_added[step] == 0
          texture = materials.add $texture_name[step]
          texture.texture = $texture_files + $texture_filename[step]
          texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch] 
          texture_added[step] = 1
        end 
      end
    end
    if door_texture.length > 0
      for texture_step in 0..door_texture.length - 1
        if door_texture[texture_step] == $texture_name[step] and texture_added[step] == 0
          texture = materials.add $texture_name[step]
          texture.texture = $texture_files + $texture_filename[step]
          texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch] 
          texture_added[step] = 1
        end 
      end   
    end
    if window_texture.length > 0
      for texture_step in 0..window_texture.length - 1
        if window_texture[texture_step] == $texture_name[step] and texture_added[step] == 0
          texture = materials.add $texture_name[step]
          texture.texture = $texture_files + $texture_filename[step]
          $texture_x_size[step] = $texture_x_size[step].to_s
          $texture_y_size[step] = $texture_y_size[step].to_s
          texture.texture.size = [$texture_x_size[step].to_f / $inch, $texture_y_size[step].to_f / $inch]
          texture_added[step] = 1
        end 
      end   
    end
  end 

  
  if(extension_type == "RF" or extension_type == "LF") and hangar_type == "Standard"
    height = hangar_height - (hangar_arc *((length + extension_offset) /(hangar_width / 2)))
    if height < 0
      max_length = hangar_height /(hangar_arc /(hangar_width / 2))
      length = max_length - extension_offset
      height = 0
    end
    arc = hangar_height - (hangar_arc *(extension_offset /(hangar_width / 2))) - height
  end



# Draw extensions

  if extension_type == "B" or extension_type == "L" or extension_type == "R" or extension_type == "LF" or extension_type == "RF"
    if extension_type == "B"
      extension_width = hangar_width
    else
      extension_width = hangar_length
    end
    
    
    extension_offset = 0 - extension_offset
    
    if roof_only == 0 and wall_only == 0
      ext_x_start = extension_offset - length
      ext_x_end = extension_offset
      ext_y_start = front_left_cut[0]
      ext_y_end = extension_width - back_right_cut[0]

      pts = []
      pts[0] = [extension_offset, front_left_cut[0], 0]
      pts[1] = [extension_offset - length, front_left_cut[1], 0]
      pts[2] = [extension_offset - length, extension_width - back_right_cut[1], 0]
      pts[3] = [extension_offset, extension_width - back_right_cut[0], 0]
      
      pts[4] = [extension_offset, front_left_cut[0], height + arc]
      pts[5] = [extension_offset - length, front_left_cut[1], height]
      pts[6] = [extension_offset - length, extension_width - back_right_cut[1], height]
      pts[7] = [extension_offset, extension_width - back_right_cut[0], height + arc]
      if height > 0
        face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
        if face.normal[1] > 0
          status = face.reverse!
        end
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
        face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
        face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
      else
        face = group_ents.add_face pts[0], pts[1], pts[4]
        if face.normal[1] > 0
          status = face.reverse!
        end
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
        face = group_ents.add_face pts[2], pts[3], pts[7]
        if face.normal[1] < 0
          status = face.reverse!
        end
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
      end
      if height > hangar_height or hangar_type == "Arched" or extension_offset < 0
        face = group_ents.add_face pts[0], pts[3], pts[7], pts[4]
        status = face.material = wall_texture
        status = face.back_material = internal_wall_texture
      end
      
      #Draw doors
      if door_number > 0
        for step in 0..door_number - 1
          if door_wall[step] == "F"
            pts = []
            pts[0] = [ext_x_start - 1 / $inch, ext_y_start + door_x_position[step], door_y_position[step]]
            pts[1] = [ext_x_start - 1 / $inch, ext_y_start + door_x_position[step], door_y_position[step] + door_y_size[step]]
            pts[2] = [ext_x_start - 1 / $inch, ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step] + door_y_size[step]]
            pts[3] = [ext_x_start - 1 / $inch, ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step]]
            face = group_ents.add_face pts
            status = face.material = door_texture[step]

            pts[4] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step], door_y_position[step]]
            pts[5] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step], door_y_position[step] + door_y_size[step]]
            pts[6] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step] + door_y_size[step]]
            pts[7] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step]]

            pts[8] = [ext_x_start, ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step]]
            pts[9] = [ext_x_start, ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[10] = [ext_x_start, ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[11] = [ext_x_start, ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step]]
                        
            pts[12] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step]]
            pts[13] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[14] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[15] = [ext_x_start - 1 / $inch - door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step]]
            
            face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[4], pts[5], pts[6], pts[7], pts[15], pts[14], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[8], pts[9], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[9], pts[10], pts[14], pts[13]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[10], pts[11], pts[15], pts[14]
            status = face.material = door_frame_texture[step]
          elsif  door_wall[step] == "L"
            pts = []
            pts[0] = [ext_x_start + door_x_position[step], ext_y_end + 1 / $inch, door_y_position[step]]
            pts[1] = [ext_x_start + door_x_position[step], ext_y_end + 1 / $inch, door_y_position[step] + door_y_size[step]]
            pts[2] = [ext_x_start + door_x_position[step] + door_x_size[step] + 1 / $inch, ext_y_end, door_y_position[step] + door_y_size[step]]
            pts[3] = [ext_x_start + door_x_position[step] + door_x_size[step] + 1 / $inch, ext_y_end, door_y_position[step]]
            face = group_ents.add_face pts
            status = face.material = door_texture[step]

            pts[4] = [ext_x_start + door_x_position[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step]]
            pts[5] = [ext_x_start + door_x_position[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step] + door_y_size[step]]
            pts[6] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step] + door_y_size[step]]
            pts[7] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step]]

            pts[8] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_end, door_y_position[step]]
            pts[9] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_end, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[10] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_end, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[11] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_end, door_y_position[step]]
                        
            pts[12] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step]]
            pts[13] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[14] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[15] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_end + 1 / $inch + door_frame_thickness[step], door_y_position[step]]
            
            face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[4], pts[5], pts[6], pts[7], pts[15], pts[14], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[8], pts[9], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[9], pts[10], pts[14], pts[13]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[10], pts[11], pts[15], pts[14]
            status = face.material = door_frame_texture[step]
          elsif  door_wall[step] == "R"
            pts = []
            pts[0] = [ext_x_start + door_x_position[step], ext_y_start - 1 / $inch, door_y_position[step]]
            pts[1] = [ext_x_start + door_x_position[step], ext_y_start - 1 / $inch, door_y_position[step] + door_y_size[step]]
            pts[2] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_start - 1 / $inch, door_y_position[step] + door_y_size[step]]
            pts[3] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_start - 1 / $inch, door_y_position[step]]
            face = group_ents.add_face pts
            status = face.reverse!
            status = face.material = door_texture[step]

            pts[4] = [ext_x_start + door_x_position[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step]]
            pts[5] = [ext_x_start + door_x_position[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step] + door_y_size[step]]
            pts[6] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step] + door_y_size[step]]
            pts[7] = [ext_x_start + door_x_position[step] + door_x_size[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step]]

            pts[8] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_start, door_y_position[step]]
            pts[9] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_start, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[10] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_start, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[11] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_start, door_y_position[step]]
                        
            pts[12] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step]]
            pts[13] = [ext_x_start + door_x_position[step] - door_frame_width[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[14] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[15] = [ext_x_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], ext_y_start - door_frame_thickness[step] - 1 / $inch, door_y_position[step]]
            
            face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[4], pts[5], pts[6], pts[7], pts[15], pts[14], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[8], pts[9], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[9], pts[10], pts[14], pts[13]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[10], pts[11], pts[15], pts[14]
            status = face.material = door_frame_texture[step]
          elsif  door_wall[step] == "B"
            pts = []
            pts[0] = [ext_x_end + 1 / $inch, ext_y_start + door_x_position[step], door_y_position[step]]
            pts[1] = [ext_x_end + 1 / $inch, ext_y_start + door_x_position[step], door_y_position[step] + door_y_size[step]]
            pts[2] = [ext_x_end + 1 / $inch, ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step] + door_y_size[step]]
            pts[3] = [ext_x_end + 1 / $inch, ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step]]
            face = group_ents.add_face pts
            status = face.reverse!
            status = face.material = door_texture[step]

            pts[4] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step], door_y_position[step]]
            pts[5] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step], door_y_position[step] + door_y_size[step]]
            pts[6] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step] + door_y_size[step]]
            pts[7] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step], door_y_position[step]]

            pts[8] = [ext_x_end, ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step]]
            pts[9] = [ext_x_end, ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[10] = [ext_x_end, ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[11] = [ext_x_end, ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step]]
                        
            pts[12] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step]]
            pts[13] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] - door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[14] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step] + door_y_size[step] + door_frame_width[step]]
            pts[15] = [ext_x_end + 1 / $inch + door_frame_thickness[step], ext_y_start + door_x_position[step] + door_x_size[step] + door_frame_width[step], door_y_position[step]]
            
            face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[4], pts[5], pts[6], pts[7], pts[15], pts[14], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[8], pts[9], pts[13], pts[12]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[9], pts[10], pts[14], pts[13]
            status = face.material = door_frame_texture[step]
            face = group_ents.add_face pts[10], pts[11], pts[15], pts[14]
            status = face.material = door_frame_texture[step]
                        
          end
        end
      end

      #Draw windows

      if window_count > 0
        for step in 0..window_count - 1
          if window_wall[step] == "L"
            for window_step in 0..window_repeat[step] - 1
              pts = []
              pts[0] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end, window_y_pos[step]]
              pts[1] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end, window_y_pos[step]]
              pts[2] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end, window_y_pos[step] + window_y_size[step]]
              pts[3] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end, window_y_pos[step] + window_y_size[step]]
              face = group_ents.add_face pts
              status = face.erase!
      
              window_group = Sketchup.active_model.entities.add_group
              window_group_ents = window_group.entities
              face = window_group_ents.add_face pts
              status = face.reverse!
              status = face.material = window_texture[step]
              status = face.back_material = window_texture[step]
              status = face.material.alpha = window_transparency[step].to_f / 100
            
              pts[4] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end - 1 / $inch, window_y_pos[step]]
              pts[5] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end - 1 / $inch, window_y_pos[step]]
              pts[6] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end - 1 / $inch, window_y_pos[step] + window_y_size[step]]
              pts[7] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_end - 1 / $inch, window_y_pos[step] + window_y_size[step]]
              face = window_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
              status = face.reverse!
              status = face.material = window_texture[step]
              face = window_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[2], pts[3], pts[7], pts[6]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[3], pts[0], pts[4], pts[7]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
              status = face.material = window_texture[step]
              group = group.parent.entities.add_group([group, window_group])
            end
          elsif window_wall[step] == "R"
            for window_step in 0..window_repeat[step] - 1
              pts = []
              pts[0] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start, window_y_pos[step]]
              pts[1] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start, window_y_pos[step]]
              pts[2] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start, window_y_pos[step] + window_y_size[step]]
              pts[3] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start, window_y_pos[step] + window_y_size[step]]
              face = group_ents.add_face pts
              status = face.erase!
    
              window_group = Sketchup.active_model.entities.add_group
              window_group_ents = window_group.entities
              face = window_group_ents.add_face pts
              status = face.reverse!
              status = face.material = window_texture[step]
              status = face.back_material = window_texture[step]
              status = face.material.alpha = window_transparency[step].to_f / 100
            
              pts[0] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start + 1 / $inch, window_y_pos[step]]
              pts[1] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start + 1 / $inch, window_y_pos[step]]
              pts[2] = [ext_x_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start + 1 / $inch, window_y_pos[step] + window_y_size[step]]
              pts[3] = [ext_x_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), ext_y_start + 1 / $inch, window_y_pos[step] + window_y_size[step]]
              face = window_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
              status = face.reverse!
              status = face.material = window_texture[step]
              face = window_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[2], pts[3], pts[7], pts[6]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[3], pts[0], pts[4], pts[7]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
              status = face.material = window_texture[step]
              group = group.parent.entities.add_group([group, window_group])
            end
          elsif window_wall[step] == "B"
            for window_step in 0..window_repeat[step] - 1
              pts = []
              pts[0] = [ext_x_start, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[1] = [ext_x_start, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[2] = [ext_x_start, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              pts[3] = [ext_x_start, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              face = group_ents.add_face pts
              status = face.erase!

              window_group = Sketchup.active_model.entities.add_group
              window_group_ents = window_group.entities
              face = window_group_ents.add_face pts
              status = face.reverse!
              status = face.material = window_texture[step]
              status = face.back_material = window_texture[step]
              status = face.material.alpha = window_transparency[step].to_f / 100

              pts[4] = [ext_x_start + 1 / $inch, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[5] = [ext_x_start + 1 / $inch, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[6] = [ext_x_start + 1 / $inch, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              pts[7] = [ext_x_start + 1 / $inch, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
        
              face = window_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
              status = face.reverse!
              status = face.material = window_texture[step]
              face = window_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[2], pts[3], pts[7], pts[6]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[3], pts[0], pts[4], pts[7]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
              status = face.material = window_texture[step]
              group = group.parent.entities.add_group([group, window_group])
            end
          elsif window_wall[step] == "F"
            for window_step in 0..window_repeat[step] - 1
              pts = []
              pts[0] = [ext_x_end, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[1] = [ext_x_end, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[2] = [ext_x_end, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              pts[3] = [ext_x_end, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              face = group_ents.add_face pts
              status = face.erase!

              window_group = Sketchup.active_model.entities.add_group
              window_group_ents = window_group.entities
              face = window_group_ents.add_face pts
              status = face.reverse!
              status = face.material = window_texture[step]
              status = face.back_material = window_texture[step]
              status = face.material.alpha = window_transparency[step].to_f / 100

              pts[4] = [ext_x_end - 1 / $inch, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[5] = [ext_x_end - 1 / $inch, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
              pts[6] = [ext_x_end - 1 / $inch, ext_y_start + window_x_pos[step] + window_x_size[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
              pts[7] = [ext_x_end - 1 / $inch, ext_y_start + window_x_pos[step] + ((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
        
              face = window_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
              status = face.reverse!
              status = face.material = window_texture[step]
              face = window_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[2], pts[3], pts[7], pts[6]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[3], pts[0], pts[4], pts[7]
              status = face.material = window_texture[step]     
              face = window_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
              status = face.material = window_texture[step]
              group = group.parent.entities.add_group([group, window_group])
            end
      
          end
        end
      end
    end
    
    #Draw roof 
    if wall_only == 0
      cut_ends = 0
      if mid_arc == 0 #Check if mid_arc has been set
        pts = []
        pts[0] = [extension_offset, 0 + front_left_cut[0], height + arc]
        pts[1] = [extension_offset - length, 0 + front_left_cut[1], height]
        pts[2] = [extension_offset - length, extension_width - back_right_cut[1], height]
        pts[3] = [extension_offset, extension_width - back_right_cut[0], height + arc]

        face = group_ents.add_face pts
        if face.normal[2] < 0
          status = face.reverse!
        end
        status = face.material = roof_texture
        status = face.back_material = roof_texture
        if roof_only == 1
          for roof_step in 0..3
            pts[roof_step + 4] = pts[roof_step]
            pts[roof_step + 4][2] = pts[roof_step][2] - wall_thickness
          end
          post_type = 0
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          if face.normal[2] > 0
            status = face.reverse!
          end
          status = face.material = roof_texture
          status = face.back_material = roof_texture
        end
        
      else
        if arc_type == "N" or arc_type == "" or front_left_cut[0] != front_left_cut[1] or back_right_cut[0] != back_right_cut[1]  #Normal arc
          if arc_direction == "" or arc_direction == "W" or front_left_cut[0] != front_left_cut[1] or back_right_cut[0] != back_right_cut[1] #Arc running length-wise on extension
            # Draw end walls
            pts = []
            pts[0] = [extension_offset - length, 0 + front_left_cut[1], height]
            pts[1] = [extension_offset - length / 2,(front_left_cut[0] + front_left_cut[1]) / 2, height + mid_arc]
            pts[2] = [extension_offset, 0 + front_left_cut[0], height + arc]
            if roof_only == 0
              face = group_ents.add_face pts
              status = face.material = wall_texture
              status = face.back_material = internal_wall_texture
            end
            pts[3] = [extension_offset, extension_width - back_right_cut[0], height + arc]
            pts[4] = [extension_offset - length / 2,((extension_width - back_right_cut[0]) + (extension_width - back_right_cut[1])) / 2, height + mid_arc]
            pts[5] = [extension_offset - length, extension_width - back_right_cut[1], height]
            if roof_only == 0
              face = group_ents.add_face pts[3], pts[4], pts[5]
              status = face.material = wall_texture
              status = face.back_material = internal_wall_texture
            end
            # Draw roof
            face = group_ents.add_face pts[0], pts[1], pts[4], pts[5]
            if face.normal[2] < 0
              status = face.reverse!
            end
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            face = group_ents.add_face pts[2], pts[1], pts[4], pts[3]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            if roof_only == 1
              for roof_step in 0..5
                pts[roof_step][2] = pts[roof_step][2] - wall_thickness
              end
              face = group_ents.add_face pts[0], pts[1], pts[4], pts[5]
              if face.normal[2] > 0
                status = face.reverse!
              end
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[2], pts[1], pts[4], pts[3]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              cut_ends = 1
              post_type = 1
            end
          else
            # Draw end walls
            pts = []
            pts[0] = [extension_offset - length, 0 + front_left_cut[1], height]
            pts[1] = [extension_offset - length,(front_left_cut[1] + extension_width + back_right_cut[1]) / 2, height + mid_arc]
            pts[2] = [extension_offset - length, extension_width - back_right_cut[1], height + arc]
            if roof_only == 0
              face = group_ents.add_face pts
              status = face.material = wall_texture
              status = face.back_material = internal_wall_texture
            end
            pts[3] = [extension_offset, 0 + front_left_cut[0], height]
            pts[4] = [extension_offset,(front_left_cut[0] + extension_width + back_right_cut[0]) / 2, height + mid_arc]
            pts[5] = [extension_offset, extension_width - back_right_cut[0], height + arc]
            if roof_only == 0
              face = group_ents.add_face pts[3], pts[4], pts[5]
              status = face.material = wall_texture
              status = face.back_material = internal_wall_texture
            end
            face = group_ents.add_face pts[0], pts[1], pts[4], pts[3]
              if face.normal[2] < 0
                status = face.reverse!
              end
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            face = group_ents.add_face pts[2], pts[1], pts[4], pts[5]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
        
            if roof_only == 1
              for roof_step in 0..5
                pts[roof_step][2] = pts[roof_step][2] - wall_thickness
              end
              face = group_ents.add_face pts[0], pts[1], pts[4], pts[3]
              if face.normal[2] > 0
                status = face.reverse!
              end
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[2], pts[1], pts[4], pts[5]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              cut_ends = 2
              post_type = 2
            end
      
          end
          
        elsif arc_type == "H" or arc_type == "HW" or arc_type == "HP" #Hipped roof
          if(extension_width - (front_left_cut[1] + back_right_cut[1])) / length > 3 or arc_type == "HW" and arc_type != "HP"
            hip_roof_type =((front_left_cut[1] + extension_width + back_right_cut[1]) / length).to_i #Standard hipped roof
          else
            hip_roof_type = 2 #Square hipped roof
          end
          pts = []
          pts[0] = [extension_offset - length, 0 + front_left_cut[1], height]
          pts[1] = [extension_offset, front_left_cut[0], height + arc]
          pts[2] = [extension_offset - (length / 2), front_left_cut[0] + (extension_width - (front_left_cut[1] + back_right_cut[1])) / hip_roof_type, height + mid_arc]
          face = group_ents.add_face pts
          if face.normal[2] < 0
            status = face.reverse!
          end
          status = face.material = roof_texture
          status = face.back_material = roof_texture
          pts[3] = [extension_offset - length, extension_width - back_right_cut[1], height]
          pts[4] = [extension_offset, extension_width - back_right_cut[0], height + arc]
          pts[5] = [extension_offset - (length / 2),(extension_width - back_right_cut[0]) - (extension_width - (front_left_cut[1] + back_right_cut[1])) / hip_roof_type, height + mid_arc]
          face = group_ents.add_face pts[3], pts[4], pts[5]
          if face.normal[2] < 0
            status = face.reverse!
          end
          status = face.material = roof_texture
          status = face.back_material = roof_texture
          if hip_roof_type == 2
            face = group_ents.add_face pts[0], pts[3], pts[2]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            face = group_ents.add_face pts[1], pts[4], pts[2]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            if roof_only == 1
              for roof_step in 0..5
                pts[roof_step + 6] = pts[roof_step]
                pts[roof_step + 6][2] = pts[roof_step][2] - wall_thickness
              end
              face = group_ents.add_face pts[6], pts[7], pts[8]
              if face.normal[2] > 0
                status = face.reverse!
              end
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[9], pts[10], pts[11]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[6], pts[9], pts[8]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[7], pts[10], pts[8]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              post_type = 3
            end
          else
            face = group_ents.add_face pts[0], pts[3], pts[5], pts[2]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            face = group_ents.add_face pts[1], pts[4], pts[5], pts[2]
            status = face.material = roof_texture
            status = face.back_material = roof_texture
            if roof_only == 1
              for roof_step in 0..5
                pts[roof_step + 6] = pts[roof_step]
                pts[roof_step + 6][2] = pts[roof_step][2] - wall_thickness
              end
              face = group_ents.add_face pts[6], pts[7], pts[8]
              if face.normal[2] > 0
                status = face.reverse!
              end
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[9], pts[10], pts[11]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[6], pts[9], pts[11], pts[8]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              face = group_ents.add_face pts[7], pts[10], pts[11], pts[8]
              status = face.material = roof_texture
              status = face.back_material = roof_texture
              post_type = 3
            end
          end
        end
      end
      
      if roof_only == 1
        #Draw roof edges
        pts = []
        pts[0] = [extension_offset, 0 + front_left_cut[0], height + arc]
        pts[1] = [extension_offset - length, 0 + front_left_cut[1], height]
        pts[2] = [extension_offset - length, extension_width - back_right_cut[1], height]
        pts[3] = [extension_offset, extension_width - back_right_cut[0], height + arc]
        pts[4] = [extension_offset, extension_width - back_right_cut[0], height + arc - wall_thickness]
        pts[5] = [extension_offset - length, extension_width - back_right_cut[1], height - wall_thickness]
        pts[6] = [extension_offset - length, 0 + front_left_cut[1], height - wall_thickness]
        pts[7] = [extension_offset, 0 + front_left_cut[0], height + arc - wall_thickness]
        if cut_ends != 1
          face = group_ents.add_face pts[0], pts[1], pts[6], pts[7]
          status = face.material = roof_texture
          status = face.back_material = roof_texture
          face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
          status = face.material = roof_texture
          status = face.back_material = roof_texture
        end
        if cut_ends != 2
          face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
          status = face.material = roof_texture
              status = face.back_material = roof_texture
          face = group_ents.add_face pts[3], pts[0], pts[7], pts[4]
          status = face.material = roof_texture
          status = face.back_material = roof_texture
        end
        if cut_ends == 1
          pts[8] = [extension_offset - length / 2,(front_left_cut[0] + front_left_cut[1]) / 2, height + mid_arc]
          pts[9] = [extension_offset - length / 2,((extension_width - back_right_cut[0]) + (extension_width - back_right_cut[1])) / 2, height + mid_arc]
          pts[10] = [extension_offset - length / 2,(front_left_cut[0] + front_left_cut[1]) / 2, height + mid_arc - wall_thickness]
          pts[11] = [extension_offset - length / 2,((extension_width - back_right_cut[0]) + (extension_width - back_right_cut[1])) / 2, height + mid_arc - wall_thickness]
          face = group_ents.add_face pts[0], pts[8], pts[1], pts[6], pts[10], pts[7]
          status = face.material = roof_texture
              status = face.back_material = roof_texture
          face = group_ents.add_face pts[2], pts[9], pts[3], pts[4], pts[11], pts[5]
          status = face.material = roof_texture
          status = face.back_material = roof_texture
        elsif cut_ends == 2
          pts[8] = [extension_offset - length,(front_left_cut[1] + extension_width + back_right_cut[1]) / 2, height + mid_arc]
          pts[9] = [extension_offset,(front_left_cut[0] + extension_width + back_right_cut[0]) / 2, height + mid_arc]
          pts[10] = [extension_offset - length,(front_left_cut[1] + extension_width + back_right_cut[1]) / 2, height + mid_arc - wall_thickness]
          pts[11] = [extension_offset,(front_left_cut[0] + extension_width + back_right_cut[0]) / 2, height + mid_arc - wall_thickness]
          face = group_ents.add_face pts[1], pts[8], pts[2], pts[5], pts[10], pts[6]
          status = face.material = roof_texture
              status = face.back_material = roof_texture
          face = group_ents.add_face pts[3], pts[9],pts[0], pts[7], pts[11], pts[4]
          status = face.material = roof_texture
          status = face.back_material = roof_texture
        
        end
        #Draw posts
        extension_width_end = extension_width - (front_left_cut[1] + back_right_cut[1]) - (post_offset * 2) - wall_thickness
        extension_width_wall = extension_width - (front_left_cut[0] + back_right_cut[0]) - (post_offset * 2) - wall_thickness
        
        post_distance_end = extension_width_end /(posts - 1)
        post_distance_wall = extension_width_wall /(posts - 1)
        
        for step in 0..posts - 1
          
          start_point_y = front_left_cut[1] + post_offset
          start_point_x = extension_offset - length + post_offset
          wall_point_x = extension_offset - post_offset
          
          if post_type == 0
            post_height_outer1 = height + (arc *(post_offset / length)) - wall_thickness
            post_height_outer2 = height + (arc *(post_offset / length)) - wall_thickness
            post_height_inner1 = height + (arc *((post_offset + wall_thickness) / length)) - wall_thickness
            post_height_inner2 = height + (arc *((post_offset + wall_thickness) / length)) - wall_thickness
          elsif post_type == 1
            post_height_outer1 = height + (mid_arc *(post_offset /(length / 2))) - wall_thickness
            post_height_outer2 = height + (mid_arc *(post_offset /(length / 2))) - wall_thickness
            post_height_inner1 = height + (mid_arc *((post_offset + wall_thickness) /(length / 2))) - wall_thickness
            post_height_inner2 = height + (mid_arc *((post_offset + wall_thickness) /(length / 2))) - wall_thickness
          end         
          
          
          pts = []
          pts[0] = [start_point_x, start_point_y + (post_distance_end * step), floor_height]
          pts[1] = [start_point_x + wall_thickness, start_point_y + (post_distance_end * step), floor_height]
          pts[2] = [start_point_x + wall_thickness, start_point_y + (post_distance_end * step) + wall_thickness, floor_height]
          pts[3] = [start_point_x, start_point_y + (post_distance_end * step) + wall_thickness, floor_height]
          pts[4] = [start_point_x, start_point_y + (post_distance_end * step) + wall_thickness, post_height_outer2]
          pts[5] = [start_point_x + wall_thickness, start_point_y + (post_distance_end * step) + wall_thickness, post_height_inner2]
          pts[6] = [start_point_x + wall_thickness, start_point_y + (post_distance_end * step), post_height_inner1]
          pts[7] = [start_point_x, start_point_y + (post_distance_end * step), post_height_outer1]

          face = group_ents.add_face pts[0], pts[1], pts[6], pts[7]
          status = face.material = wall_texture
          face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
          status = face.material = wall_texture
          face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
          status = face.material = wall_texture
          face = group_ents.add_face pts[3], pts[0], pts[7], pts[4]
          status = face.material = wall_texture
          
          
          # Back posts
          if post_type != 0 or height + arc > hangar_height or hangar_type == "Arched"

            if post_type == 0
              post_height_outer1 = height + arc - (arc *(post_offset / length)) - wall_thickness
              post_height_outer2 = height + arc - (arc *(post_offset / length)) - wall_thickness
              post_height_inner1 = height + arc - (arc *((post_offset + wall_thickness) / length)) - wall_thickness
              post_height_inner2 = height + arc - (arc *((post_offset + wall_thickness) / length)) - wall_thickness
            elsif post_type == 1
              post_height_outer1 = height + arc + (mid_arc *(post_offset /(length / 2))) - wall_thickness
              post_height_outer2 = height + arc + (mid_arc *(post_offset /(length / 2))) - wall_thickness
              post_height_inner1 = height + arc + (mid_arc *((post_offset + wall_thickness) /(length / 2))) - wall_thickness
              post_height_inner2 = height + arc + (mid_arc *((post_offset + wall_thickness) /(length / 2))) - wall_thickness
            end         

            pts = []
            pts[0] = [wall_point_x, start_point_y + (post_distance_end * step), floor_height]
            pts[1] = [wall_point_x - wall_thickness, start_point_y + (post_distance_end * step), floor_height]
            pts[2] = [wall_point_x - wall_thickness, start_point_y + (post_distance_end * step) + wall_thickness, floor_height]
            pts[3] = [wall_point_x, start_point_y + (post_distance_end * step) + wall_thickness, floor_height]
            pts[4] = [wall_point_x, start_point_y + (post_distance_end * step) + wall_thickness, post_height_outer2]
            pts[5] = [wall_point_x - wall_thickness, start_point_y + (post_distance_end * step) + wall_thickness, post_height_inner2]
            pts[6] = [wall_point_x - wall_thickness, start_point_y + (post_distance_end * step), post_height_inner1]
            pts[7] = [wall_point_x, start_point_y + (post_distance_end * step), post_height_outer1]
  
            face = group_ents.add_face pts[0], pts[1], pts[6], pts[7]
            if face.normal[1] > 0
              status = face.reverse!
            end
            status = face.material = wall_texture
            face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
            status = face.material = wall_texture
            face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
            status = face.material = wall_texture
            face = group_ents.add_face pts[3], pts[0], pts[7], pts[4]
            status = face.material = wall_texture
        
          end
        end
        
        #Draw floor
        
        if floor_height > 0
          pts = []
          if ext_floor_texture == "W"
            texture = materials.add "W"
            texture.texture = $texture_files + "suht_wood_planks_brown.jpg"
            texture.texture.size = 80 / $inch
          else
            texture = materials.add "C"
            texture.texture = $texture_files + "suht_concrete_flat.jpg"
            texture.texture.size = 100 / $inch
          end
          
          pts[0] = [extension_offset, 0 + front_left_cut[0], 0]
          pts[1] = [extension_offset - length, 0 + front_left_cut[1], 0]
          pts[2] = [extension_offset - length, extension_width - back_right_cut[1], 0]
          pts[3] = [extension_offset, extension_width - back_right_cut[0], 0]
          pts[4] = [extension_offset, extension_width - back_right_cut[0], floor_height]
          pts[5] = [extension_offset - length, extension_width - back_right_cut[1], floor_height]
          pts[6] = [extension_offset - length, 0 + front_left_cut[1], floor_height]
          pts[7] = [extension_offset, 0 + front_left_cut[0], floor_height]
          face = group_ents.add_face pts[0], pts[1], pts[6], pts[7]
          if face.normal[1] > 0
            status = face.reverse!
          end
          status = face.material = ext_floor_texture
          face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
          status = face.material = ext_floor_texture
          face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
          status = face.material = ext_floor_texture
          face = group_ents.add_face pts[3], pts[0], pts[7], pts[4]
          status = face.material = ext_floor_texture
          face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
          status = face.material = ext_floor_texture
        
        
        
        end
        
      end
      
      
      
    end

    if wall_only == 1

      # Check if wall is sectioned

      if posts > 0
        # Draw posts
        
        if no_posts == 0
        


        end
        
        # Draw wall sections
        extension_width_end = extension_width - (front_left_cut[1] + back_right_cut[1]) - wall_thickness
        extension_width_wall = extension_width - (front_left_cut[0] + back_right_cut[0]) - wall_thickness
        
        post_distance_end = extension_width_end /(posts - 1)
        post_distance_wall = extension_width_wall /(posts - 1)
        
        for step in 0..posts - 2
        
          no_draw = 0
          
          for gate_step in 0..gate_count - 1
            if gate_info[gate_step] == step
              no_draw = 1
            end
          
          end
          
          if no_draw == 0
          
            for height_step in 0..height_repeat
        
        
              if height_repeat == 0
                bottom_end = floor_height
                top_end = height
              else
                bottom_end = floor_height + ((height + height_gap) * height_step)
                top_end = bottom_end + height
              end
              pts = []
              pts[0] = [extension_offset - length + wall_thickness / 4, front_left_cut[1] + (post_distance_end * step) + wall_thickness, bottom_end]
              pts[1] = [extension_offset - length + wall_thickness * 3 / 4, front_left_cut[1] + (post_distance_end * step) + wall_thickness, bottom_end]
              pts[2] = [extension_offset - length + wall_thickness * 3 / 4, front_left_cut[1] + (post_distance_end *(step + 1)), bottom_end]
              pts[3] = [extension_offset - length + wall_thickness / 4, front_left_cut[1] + (post_distance_end *(step + 1)) , bottom_end]
              pts[4] = [extension_offset - length + wall_thickness / 4, front_left_cut[1] + (post_distance_end *(step + 1)), top_end]
              pts[5] = [extension_offset - length + wall_thickness * 3 / 4, front_left_cut[1] + (post_distance_end *(step + 1)), top_end]
              pts[6] = [extension_offset - length + wall_thickness * 3 / 4, front_left_cut[1] + (post_distance_end * step) + wall_thickness, top_end]
              pts[7] = [extension_offset - length + wall_thickness / 4, front_left_cut[1] + (post_distance_end * step) + wall_thickness, top_end]
              
              face = group_ents.add_face pts[0], pts[1], pts[2], pts[3]
              if face.normal[2] > 0
                status = face.reverse!
              end
              status = face.material = wall_texture
              face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
              status = face.material = wall_texture
              face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
              status = face.material = wall_texture
              face = group_ents.add_face pts[0], pts[3], pts[4], pts[7]
              status = face.material = wall_texture
            end
          
          end
        end
        
      end
    end

    pts = []
    pts[0] = [0, extension_width, 0]
    pts[1] = [-10, extension_width, 0]
    edges = group_ents.add_edges pts
    
    if extension_type == "L" or extension_type == "LF"
      point = Geom::Point3d.new(hangar_length, 0, 0)
      t = Geom::Transformation.new point
      group.move! t

      radians = 90.degrees
      t = Geom::Transformation.rotation [hangar_length, 0, 0], Z_AXIS, radians
      group.transform! t
    elsif extension_type == "R" or extension_type == "RF"
      point = Geom::Point3d.new(0, hangar_width, 0)
      t = Geom::Transformation.new point
      group.move! t

      radians = -90.degrees
      t = Geom::Transformation.rotation [0, hangar_width, 0], Z_AXIS, radians
      group.transform! t
    end

  end
  
  if rotate_extension == 1 and extension_type != "LF" and extension_type != "RF"
    radians = rotation_info[0].to_f.degrees
    hangar_x = rotation_info[1].to_f / $inch
    hangar_y = rotation_info[2].to_f / $inch
    if extension_type == "B"
      hangar_x = 0 - hangar_x
      hangar_y = 0 - hangar_y
    end
    t = Geom::Transformation.rotation [hangar_x,hangar_y,0], Z_AXIS, radians
    group.transform! t
  end
  
  return group
end


######################################################
###### Below is the code to add signs to hangar ######
######################################################

def draw_signs(sign_type = "", sign_texture = "", sign_frame_texture = "", sign_wall = "", sign_x_pos = 0.0, sign_y_pos = 0.0, sign_x_size = 0.0, sign_y_size = 0.0, sign_thickness = 0.0, sign_filled = 0, sign_double_sided = 0, external_front_position = 0.0, external_back_position = 0.0, external_left_position = 0.0, external_right_position = 0.0)

  sign_group = Sketchup.active_model.entities.add_group
  sign_group_ents = sign_group.entities
  materials = Sketchup.active_model.materials

  if sign_type == "L"
        
    #Add logo texture
        
    texture_id = sign_group.to_s
        
    for texture_step in 0..$texture_name.length - 1
      if sign_texture == $texture_name[texture_step]
        texture = materials.add $texture_name[texture_step] + " sign " + texture_id
        texture.texture = $texture_files + $texture_filename[texture_step]
        if sign_filled == 0
          $texture_x_size[texture_step] = $texture_x_size[texture_step].to_s
          $texture_y_size[texture_step] = $texture_y_size[texture_step].to_s
          texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
        else
          texture.texture.size = [sign_x_size, sign_y_size]
        end
      end
    end

        
    #Draw sign
    pts = []
    pts[0] = [0, 0, 0]
    pts[1] = [0, 0, sign_y_size]
    pts[2] = [0, sign_x_size, sign_y_size]
    pts[3] = [0, sign_x_size, 0]
    face = sign_group_ents.add_face pts
    if sign_double_sided == 0
      status = face.material = sign_frame_texture
    else
      status = face.material = sign_texture + " sign " + texture_id
    end
    
    if sign_thickness > 0
      pts[4] = [0 + sign_thickness, 0, 0]
      pts[5] = [0 + sign_thickness, 0, sign_y_size]
      pts[6] = [0 + sign_thickness, sign_x_size, sign_y_size]
      pts[7] = [0 + sign_thickness, sign_x_size, 0]
      face = sign_group_ents.add_face pts[0], pts[1], pts[5], pts[4]
      status = face.material = sign_frame_texture
      face = sign_group_ents.add_face pts[1], pts[2], pts[6], pts[5]
      status = face.material = sign_frame_texture
      face = sign_group_ents.add_face pts[2], pts[3], pts[7], pts[6]
      status = face.material = sign_frame_texture
      face = sign_group_ents.add_face pts[3], pts[0], pts[4], pts[7]
      status = face.material = sign_frame_texture
      face = sign_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
      status = face.material = sign_texture + " sign " + texture_id
    else
      if sign_double_sided == 0
        status = face.back_material = sign_frame_texture
      else
        status = face.back_material = sign_texture + " sign " + texture_id
      end
    end
    
    if sign_wall == "F"
      point = Geom::Point3d.new(external_front_position, external_left_position + sign_x_pos, sign_y_pos)
      t = Geom::Transformation.new point
      sign_group.move! t
    elsif sign_wall == "B"
      point = Geom::Point3d.new(external_back_position, external_left_position + sign_x_pos, sign_y_pos)
      t = Geom::Transformation.new point
      sign_group.move! t
      sign_radians = 180.degrees
      t = Geom::Transformation.rotation [external_back_position, external_left_position + sign_x_pos + sign_x_size / 2, sign_y_pos], Z_AXIS, sign_radians
      sign_group.transform! t
    elsif sign_wall == "L"
      point = Geom::Point3d.new(external_front_position - sign_x_pos - sign_x_size, external_left_position, sign_y_pos)
      t = Geom::Transformation.new point
      sign_group.move! t
      sign_radians = -90.degrees
      t = Geom::Transformation.rotation [external_front_position - sign_x_pos - sign_x_size, external_left_position, sign_y_pos], Z_AXIS, sign_radians
      sign_group.transform! t
    elsif sign_wall == "R"
      point = Geom::Point3d.new(external_front_position - sign_x_pos, external_right_position, sign_y_pos)
      t = Geom::Transformation.new point
      sign_group.move! t
      sign_radians = 90.degrees
      t = Geom::Transformation.rotation [external_front_position - sign_x_pos, external_right_position, sign_y_pos], Z_AXIS, sign_radians
      sign_group.transform! t
    end
      
    return sign_group
  end
end


###############################################################
###### Below is the code to add entrance doors to hangar ######
###############################################################

def draw_entrance_door(entrance_door_x_pos = 0.0, entrance_door_y_pos = 0.0, entrance_door_x_size = 0.0, entrance_door_y_size = 0.0, entrance_door_wall = "", entrance_door_type = "", entrance_door_texture = "", entrance_door_glass_texture = "", entrance_door_glass_back_texture = "", wall_thickness = 0.0, external_left_position = 0.0, external_right_position = 0.0, external_front_position = 0.0, external_back_position = 0.0)

  entrance_door_group = Sketchup.active_model.entities.add_group
  entrance_door_group_ents = entrance_door_group.entities
  materials = Sketchup.active_model.materials

  entrance_door_width = 8 / $inch
  entrance_door_thickness = 1 / $inch

  entrance_door_type_extra = ""

  if entrance_door_type.count(':') == 1
    entrance_door_type_temp = entrance_door_type.split(':')
    entrance_door_type = entrance_door_type_temp[0]
    entrance_door_type_extra = entrance_door_type_temp[1]
  end
  
  # Add textures

  texture_id = entrance_door_group.to_s
  for texture_step in 0..$texture_name.length - 1
    if entrance_door_texture == $texture_name[texture_step] or entrance_door_glass_texture == $texture_name[texture_step] or entrance_door_glass_back_texture == $texture_name[texture_step] 
      texture = materials.add $texture_name[texture_step] + " door " + texture_id
      texture.texture = $texture_files + $texture_filename[texture_step]
      texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
    end
  end
  

  # Draw door
  pts = []
  pts[0] = [0, 0, 0]
  pts[1] = [entrance_door_x_size, 0, 0]
  pts[2] = [entrance_door_x_size, 0, entrance_door_y_size]
  pts[3] = [0, 0, entrance_door_y_size]
  pts[4] = [0, wall_thickness, 0]
  pts[5] = [entrance_door_x_size, wall_thickness, 0]
  pts[6] = [entrance_door_x_size, wall_thickness, entrance_door_y_size]
  pts[7] = [0, wall_thickness, entrance_door_y_size]

  if entrance_door_type == "Plain"
    face = entrance_door_group_ents.add_face pts[0], pts[1], pts[2], pts[3]
    status = face.material = entrance_door_texture + " door " + texture_id

    if wall_thickness > 0
      face = entrance_door_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
      status = face.reverse!
      status = face.material = entrance_door_texture + " door " + texture_id
    else
      status = face.back_material = entrance_door_texture + " door " + texture_id
    end
    
  elsif entrance_door_type == "Glass" or entrance_door_type == "Half glass"

    pts[8] = [entrance_door_width, 0, entrance_door_width]
    pts[9] = [entrance_door_x_size - entrance_door_width, 0, entrance_door_width]
    pts[10] = [entrance_door_x_size - entrance_door_width, 0, entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[11] = [entrance_door_width, 0, entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[12] = [entrance_door_width, 0, entrance_door_y_size - entrance_door_width]
    pts[13] = [entrance_door_x_size - entrance_door_width, 0, entrance_door_y_size - entrance_door_width]
    pts[14] = [entrance_door_x_size - entrance_door_width, 0, entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[15] = [entrance_door_width, 0, entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[16] = [entrance_door_width, entrance_door_thickness , entrance_door_width]
    pts[17] = [entrance_door_x_size - entrance_door_width, entrance_door_thickness , entrance_door_width]
    pts[18] = [entrance_door_x_size - entrance_door_width, entrance_door_thickness , entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[19] = [entrance_door_width, entrance_door_thickness , entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[20] = [entrance_door_width, entrance_door_thickness , entrance_door_y_size - entrance_door_width]
    pts[21] = [entrance_door_x_size - entrance_door_width, entrance_door_thickness , entrance_door_y_size - entrance_door_width]
    pts[22] = [entrance_door_x_size - entrance_door_width, entrance_door_thickness , entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[23] = [entrance_door_width, entrance_door_thickness , entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[24] = [entrance_door_width, wall_thickness, entrance_door_width]
    pts[25] = [entrance_door_x_size - entrance_door_width, wall_thickness, entrance_door_width]
    pts[26] = [entrance_door_x_size - entrance_door_width, wall_thickness, entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[27] = [entrance_door_width, wall_thickness, entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[28] = [entrance_door_width, wall_thickness, entrance_door_y_size - entrance_door_width]
    pts[29] = [entrance_door_x_size - entrance_door_width, wall_thickness, entrance_door_y_size - entrance_door_width]
    pts[30] = [entrance_door_x_size - entrance_door_width, wall_thickness, entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[31] = [entrance_door_width, wall_thickness, entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[32] = [entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_width]
    pts[33] = [entrance_door_x_size - entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_width]
    pts[34] = [entrance_door_x_size - entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[35] = [entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size / 2 - entrance_door_width / 2]
    pts[36] = [entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size - entrance_door_width]
    pts[37] = [entrance_door_x_size - entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size - entrance_door_width]
    pts[38] = [entrance_door_x_size - entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size / 2 + entrance_door_width / 2]
    pts[39] = [entrance_door_width, wall_thickness - entrance_door_thickness , entrance_door_y_size / 2 + entrance_door_width / 2]

    face = entrance_door_group_ents.add_face pts[0], pts[1], pts[2], pts[3]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
    status = face.reverse!
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[8], pts[9], pts[10], pts[11]
    status = face.erase!
    face = entrance_door_group_ents.add_face pts[12], pts[13], pts[14], pts[15]
    status = face.erase!
    face = entrance_door_group_ents.add_face pts[24], pts[25], pts[26], pts[27]
    status = face.erase!
    face = entrance_door_group_ents.add_face pts[28], pts[29], pts[30], pts[31]
    status = face.erase!
    
    face = entrance_door_group_ents.add_face pts[8], pts[9], pts[25], pts[24]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[9], pts[10], pts[26], pts[25]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[10], pts[11], pts[27], pts[26]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[11], pts[8], pts[24], pts[27]
    status = face.material = entrance_door_texture + " door " + texture_id
      
    face = entrance_door_group_ents.add_face pts[12], pts[13], pts[29], pts[28]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[13], pts[14], pts[30], pts[29]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[14], pts[15], pts[31], pts[30]
    status = face.material = entrance_door_texture + " door " + texture_id
    face = entrance_door_group_ents.add_face pts[15], pts[12], pts[28], pts[31]
    status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[16], pts[17], pts[18], pts[19]
    if entrance_door_type == "Glass"
      status = face.material = entrance_door_glass_texture + " door " + texture_id
      status = face.back_material = entrance_door_glass_texture + " door " + texture_id
      if entrance_door_glass_texture != entrance_door_texture 
        status = face.material.alpha = 0.3
      end
    else
      status = face.material = entrance_door_texture + " door " + texture_id
    end
    face = entrance_door_group_ents.add_face pts[20], pts[21], pts[22], pts[23]
    status = face.material = entrance_door_glass_texture + " door " + texture_id
    status = face.back_material = entrance_door_glass_texture + " door " + texture_id
    if entrance_door_glass_texture != entrance_door_texture 
      status = face.material.alpha = 0.3
    end
    face = entrance_door_group_ents.add_face pts[32], pts[33], pts[34], pts[35]
    if entrance_door_type == "Glass"
      status = face.material = entrance_door_glass_texture + " door " + texture_id
      status = face.back_material = entrance_door_glass_texture + " door " + texture_id
    else
      status = face.reverse!
      status = face.material = entrance_door_texture + " door " + texture_id
    end
    face = entrance_door_group_ents.add_face pts[36], pts[37], pts[38], pts[39]
    status = face.material = entrance_door_glass_texture + " door " + texture_id
    status = face.back_material = entrance_door_glass_texture + " door " + texture_id
    
  end



  # Handles
  if entrance_door_type_extra != "NoHandle"
    for handle_step in 0..1
      handle_step_index = 1 - handle_step * 2
      if handle_step == 0
        handle_position = 0
      else
        handle_position = wall_thickness
      end
      pts = []
      pts[0] = [entrance_door_width / 2 - 1 / $inch, handle_position, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[1] = [entrance_door_width / 2 + 1 / $inch, handle_position, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[2] = [entrance_door_width / 2 + 1 / $inch, handle_position, entrance_door_y_size / 2 + entrance_door_width / 2]
      pts[3] = [entrance_door_width / 2 - 1 / $inch, handle_position, entrance_door_y_size / 2 + entrance_door_width / 2]
      pts[4] = [entrance_door_width / 2 - 1 / $inch, handle_position - entrance_door_width * handle_step_index, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[5] = [entrance_door_width / 2 + 1 / $inch, handle_position - (entrance_door_width - 2 / $inch) * handle_step_index, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[6] = [entrance_door_width / 2 + 1 / $inch, handle_position - (entrance_door_width - 2 / $inch) * handle_step_index, entrance_door_y_size / 2 + entrance_door_width / 2]
      pts[7] = [entrance_door_width / 2 - 1 / $inch, handle_position - entrance_door_width * handle_step_index, entrance_door_y_size / 2 + entrance_door_width / 2]
      pts[8] = [entrance_door_width * 1.5, handle_position - entrance_door_width * handle_step_index, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[9] = [entrance_door_width * 1.5, handle_position - (entrance_door_width - 2 / $inch) * handle_step_index, entrance_door_y_size / 2 - entrance_door_width / 2]
      pts[10] = [entrance_door_width * 1.5, handle_position - (entrance_door_width - 2 / $inch) * handle_step_index, entrance_door_y_size / 2 + entrance_door_width / 2]
      pts[11] = [entrance_door_width * 1.5, handle_position - entrance_door_width * handle_step_index, entrance_door_y_size / 2 + entrance_door_width / 2]

      face = entrance_door_group_ents.add_face pts[0], pts[3], pts[7], pts[4]
      if face.normal[0] > 0
        status = face.reverse!
      end
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[7], pts[4], pts[8], pts[11]
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[8], pts[9], pts[10], pts[11]
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[9], pts[10], pts[6], pts[5]
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[6], pts[5], pts[1], pts[2]
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[0], pts[1], pts[5], pts[9], pts[8], pts[4]
      status = face.material = entrance_door_texture + " door " + texture_id
      face = entrance_door_group_ents.add_face pts[2], pts[3], pts[7], pts[11], pts[10], pts[6]
      status = face.material = entrance_door_texture + " door " + texture_id
    end
  end

  if entrance_door_wall == "L"
    point = Geom::Point3d.new(external_front_position - entrance_door_x_pos - entrance_door_x_size, external_left_position, entrance_door_y_pos)
    t = Geom::Transformation.new point
    entrance_door_group.move! t
    if entrance_door_type_extra == "ReverseHandle"
      entrance_door_radians = 180.degrees
      t = Geom::Transformation.rotation [external_front_position - entrance_door_x_pos - entrance_door_x_size / 2, external_left_position + wall_thickness / 2, entrance_door_y_pos], Z_AXIS, entrance_door_radians
      entrance_door_group.transform! t
    end
  elsif entrance_door_wall == "R"
    point = Geom::Point3d.new(external_front_position - entrance_door_x_pos - entrance_door_x_size, external_right_position - wall_thickness, entrance_door_y_pos)
    t = Geom::Transformation.new point
    entrance_door_group.move! t
    if entrance_door_type_extra == "ReverseHandle"
      entrance_door_radians = 180.degrees
      t = Geom::Transformation.rotation [external_front_position - entrance_door_x_pos - entrance_door_x_size / 2, external_right_position - wall_thickness / 2, entrance_door_y_pos], Z_AXIS, entrance_door_radians
      entrance_door_group.transform! t
    end
  elsif entrance_door_wall == "B"
    point = Geom::Point3d.new(external_back_position + wall_thickness, external_left_position + entrance_door_x_pos, entrance_door_y_pos)
    t = Geom::Transformation.new point
    entrance_door_group.move! t
    entrance_door_radians = 90.degrees
    t = Geom::Transformation.rotation [external_back_position + wall_thickness, external_left_position + entrance_door_x_pos, entrance_door_y_pos], Z_AXIS, entrance_door_radians
    entrance_door_group.transform! t
    if entrance_door_type_extra == "ReverseHandle"
      entrance_door_radians = 180.degrees
      t = Geom::Transformation.rotation [external_back_position + wall_thickness / 2, external_left_position + entrance_door_x_pos + entrance_door_x_size / 2, entrance_door_y_pos], Z_AXIS, entrance_door_radians
      entrance_door_group.transform! t
    end
  elsif entrance_door_wall == "F"
    point = Geom::Point3d.new(external_front_position, external_left_position + entrance_door_x_pos, entrance_door_y_pos)
    t = Geom::Transformation.new point
    entrance_door_group.move! t
    entrance_door_radians = 90.degrees
    t = Geom::Transformation.rotation [external_front_position, external_left_position + entrance_door_x_pos, entrance_door_y_pos], Z_AXIS, entrance_door_radians
    entrance_door_group.transform! t
    if entrance_door_type_extra == "ReverseHandle"
      entrance_door_radians = 180.degrees
      t = Geom::Transformation.rotation [external_front_position - wall_thickness / 2, external_left_position + entrance_door_x_pos + entrance_door_x_size / 2, entrance_door_y_pos], Z_AXIS, entrance_door_radians
      entrance_door_group.transform! t
    end
  end

  return entrance_door_group

end


########################################################
###### Below is the code to add windows to hangar ######
########################################################

def draw_window(window_x_pos = 0.0, window_y_pos = 0.0, window_x_size = 0.0, window_y_size = 0.0, window_wall = "", window_texture = "", window_back_texture = "", window_transparency = 0, wall_thickness = 0.0, external_left_position = 0.0, external_right_position = 0.0, external_front_position = 0.0, external_back_position = 0.0)

  window_group = Sketchup.active_model.entities.add_group
  window_group_ents = window_group.entities
  materials = Sketchup.active_model.materials

  texture_id = window_group.to_s
  for texture_step in 0..$texture_name.length - 1
    if window_texture == $texture_name[texture_step] or window_back_texture == $texture_name[texture_step] 
      texture = materials.add $texture_name[texture_step] + " window " + texture_id
      texture.texture = $texture_files + $texture_filename[texture_step]
      if window_texture != window_back_texture and window_texture == $texture_name[texture_step]
        texture.texture.size = [window_x_size, window_y_size]
      else
        texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
      end
    end
  end
  
  pts = []
  pts[0] = [0, 0, 0]
  pts[1] = [window_x_size, 0, 0]
  pts[2] = [window_x_size, 0, window_y_size]
  pts[3] = [0, 0, window_y_size]
  face = window_group_ents.add_face pts
  if face.normal[1] > 0
    status = face.reverse!
  end
  status = face.material = window_texture + " window " + texture_id
  status = face.back_material = window_back_texture + " window " + texture_id
  if window_texture == window_back_texture
    status = face.material.alpha = window_transparency.to_f / 100
  else
    status = face.back_material.alpha = window_transparency.to_f / 100
  end
          
  if wall_thickness > 0
    pts[4] = [0, wall_thickness, 0]
    pts[5] = [window_x_size, wall_thickness, 0]
    pts[6] = [window_x_size, wall_thickness, window_y_size]
    pts[7] = [0, wall_thickness, window_y_size]
    face = window_group_ents.add_face pts[4], pts[5], pts[6], pts[7]
    if face.normal[1] < 0
      status = face.reverse!
    end
    status = face.material = window_texture + " window " + texture_id 
    status = face.back_material = window_back_texture + " window " + texture_id
    if window_texture == window_back_texture
      status = face.material.alpha = window_transparency.to_f / 100
    else
      status = face.back_material.alpha = window_transparency.to_f / 100
    end
  end

  if window_wall == "L"
    point = Geom::Point3d.new(external_front_position - window_x_pos - window_x_size, external_left_position, window_y_pos)
    t = Geom::Transformation.new point
    window_group.move! t
  elsif window_wall == "R"
    point = Geom::Point3d.new(external_front_position - window_x_pos - window_x_size, external_right_position - wall_thickness, window_y_pos)
    t = Geom::Transformation.new point
    window_group.move! t
  elsif window_wall == "B"
    point = Geom::Point3d.new(external_back_position + wall_thickness, external_left_position + window_x_pos, window_y_pos)
    t = Geom::Transformation.new point
    window_group.move! t
    window_radians = 90.degrees
    t = Geom::Transformation.rotation [external_back_position + wall_thickness, external_left_position + window_x_pos, window_y_pos], Z_AXIS, window_radians
    window_group.transform! t
  elsif window_wall == "F"
    point = Geom::Point3d.new(external_front_position, external_left_position + window_x_pos, window_y_pos)
    t = Geom::Transformation.new point
    window_group.move! t
    window_radians = 90.degrees
    t = Geom::Transformation.rotation [external_front_position, external_left_position + window_x_pos, window_y_pos], Z_AXIS, window_radians
    window_group.transform! t
  end

  return window_group

end


#############################################################
###### Below is the code to add roof windows to hangar ######
#############################################################

def draw_roof_window(roof_window_x_pos = 0.0, roof_window_y_pos = 0.0, roof_window_x_size = 0.0, roof_window_y_size = 0.0, roof_window_side = "", roof_window_texture = "", roof_window_back_texture = "", roof_window_transparency = 0, roof_window_repeat = 0, roof_window_gap = 0.0, wall_thickness = 0.0, external_left_position = 0.0, external_right_position = 0.0, external_front_position = 0.0, roof_slope = 0.0)




end