##################################################
##################################################
###### Below is the code for drawing walls  ######
##################################################
##################################################


def draw_wall(wall_point_count = 0, wall_side = "", wall_offset = 0.0, wall_points = [], wall_texture = "", wall_cuts = [])
  
  ### wall_point_count  - The number of corners on each wall. A normal wall will have four corners. A wall with gables will typically have five corners. This may be redundant.
  ### wall_side         - The sides will either run via the X or Y axis, determined by the wall_offset. If the side is followed by a dash, the wall will face the opposite way.
  ### wall_offset       - The offset in cm from the X or Y axis.
  ### wall_points       - A set of points for each corner of the wall. The first number will be X or Y and the second number will be Z.
  ### wall_texture      - The name of the wall texture.
  ### wall_cuts         - A set of points to mark windows and doors. The first number will be X or Y and the second number will be Z. This is optional.
  
  
  #### Check if wall is textured in some fashion ####
  
  if wall_texture == ""
    logstring("Unable to draw wall. Error in syntax.", 1)
    return false
  end
  
  
    
  
  group = Sketchup.active_model.entities.add_group
  group_ents = group.entities
  materials = Sketchup.active_model.materials  
  
  #### Load texture ####
  
  for step in 0..$texture_name.length - 1
    if wall_texture == $texture_name[step]
      texture = materials.add $texture_name[step]
      if 
        $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
      texture.texture.size = [$texture_x_size[step] / $inch * $unit_multiplier, $texture_y_size[step] / $inch * $unit_multiplier]
    end
  end

  if wall_texture == "Untextured"
    texture = materials.add "Untextured"
    texture.color = "White"
  end
  
  
  #### Create wall points ####
  
  pts = []
  if wall_side == "Y" or wall_side == "Y-"
    for step in 0..wall_point_count - 1
      pts[step] = [wall_points[step][0], wall_offset, wall_points[step][1]]
    end
    wc = wall_cuts.length
    if wc > 0
      for step in 0..wc - 1
        pts[wall_point_count + step * 4] = [wall_cuts[step][2], wall_offset, wall_cuts[step][3]]
        pts[wall_point_count + step * 4 + 1] = [wall_cuts[step][2], wall_offset, wall_cuts[step][1]]
        pts[wall_point_count + step * 4 + 2] = [wall_cuts[step][0], wall_offset, wall_cuts[step][1]]
        pts[wall_point_count + step * 4 + 3] = [wall_cuts[step][0], wall_offset, wall_cuts[step][3]]
      end
    end
  else
    for step in 0..wall_point_count - 1
      pts[step] = [wall_offset, wall_points[step][0], wall_points[step][1]]
    end
  end
  face = group_ents.add_face pts
  status = face.material = wall_texture
  
  if wall_side == "Y-" or wall_side == "X-"
    status = face.reverse!
  end
  

  
  
  logstring("Wall drawn successfully", 1)
  return group
  
end 
