##################################
#### One Click Software Suite ####
####  ©2014 Morten Blindheim  ####
##################################

require 'sketchup'

#######################################
### Check if win32ole can be loaded ###
#######################################

$win32ole_ok = 1
begin
  require 'win32ole'
rescue LoadError
  $win32ole_ok = 0
end

###############################
#### Load global variables ####
###############################

$inch = 2.54
$unit_multiplier = 1
$unit_type = ""
$yn = "Yes|No"


$local_folder = File.dirname(__FILE__)
$och_folder = $local_folder + "/OCH"
$texture_files = $och_folder + "/TextureFiles/"
$html_files = $och_folder + "/HangarForms/"
$ochtml_folder = $och_folder + "/ocHTML/"
$log_file = $och_folder + "/log.och"
$metal_list = "Untextured"
$wall_list = "Untextured"
$roof_list = "Untextured"
$door_list = "Untextured"
$floor_list = "No floor|Untextured"
$internal_wall_list = "Same as external|Untextured"
$wood_list = "Untextured"
$fence_list = "Untextured"
$extension_list = ""

$texture_name = []
$texture_filename = []
$texture_x_size = []
$texture_y_size = []

$airport_icao = ""
$airport_tower_x_pos = 0.0
$airport_tower_y_pos = 0.0

### Settings ###
$debug_on = 0
$log_level = 0
$log_separator = ""
$shortcuts_enabled = 0
$development_enabled

$version_number = "01-05-2015"
$file_name = File.basename(__FILE__)

$help_file_imported = 0
$help_item = []
$operating_system = "Windows"
$textures_imported = 0 ## Possibly obsolete

###########################
### Load required files ###
###########################

$oscc_functions_rb = $och_folder + "/oscc_functions.rb"
$oscc_functions_rb_loaded = 0;
$scripts_loaded = 0;
if File.exist?($oscc_functions_rb)
  require $oscc_functions_rb
  $oscc_functions_rb_loaded = 1;
else
  UI.messagebox "Core functions missing. Please check installation manual.\n\nTips:\nAll .rb files should be located in the OCH sub-folder.\nDownload manual from: http://www.fsdeveloper.com/forum/showthread.php?t=424466\nSupport queries can be emailed to: fpsplitter@gmail.com",MB_MULTILINE,"One Click Hangar - Load Error"
end

if $oscc_functions_rb_loaded == 1
  ### Import all settings ###
  import_settings
  logstring("Settings imported", 1)
  
  ### Import all textures ###
  logstring("Start import of textures", 1)
  import_textures

  if $debug_on == 1
    logstring("Display SketchUp Ruby console", 2)
    SKETCHUP_CONSOLE.show
  end

  ### Check for old rb files ###
  $old_files = ["afcad.rb", "beams.rb", "bridge.rb", "building.rb", "class.rb", "development.rb", "drawing_development.rb", "furniture.rb", "hangar.rb", "hangar_doors.rb", "ocss_extensions.rb", "ocss_functions.rb"]
  for step in 0..$old_files.length - 1
    filename_check = $local_folder + "/" + $old_files[step]
    if File.exist?(filename_check)
      logstring("Old file found: " + filename_check, 0)
    end
  end

    
  $hangar_rb = $och_folder + "/hangar.rb"
  $standalone_hangar_rb = $och_folder + "/standalone_hangar.rb"
  $hangar_rb_loaded = 0
  $hangar_doors_rb = $och_folder + "/hangar_doors.rb"
  $hangar_doors_rb_loaded = 0
  $walls_rb = $och_folder + "/walls.rb"
  $walls_rb_loaded = 0
  $beams_rb = $och_folder + "/beams.rb"
  $beams_rb_loaded = 0
  $ocss_extensions_rb = $och_folder + "/ocss_extensions.rb"
  $ocss_extensions_rb_loaded = 0
  $drawing_rb = $och_folder + "/drawing.rb"
  $drawing_rb_loaded = 0
  $development_rb = $och_folder + "/development.rb"
  $development_rb_loaded = 0  
  $afcad_rb = $och_folder + "/afcad.rb"
  $afcad_rb_loaded = 0
  $building_rb = $och_folder + "/building.rb"
  $building_rb_loaded = 0
  $bridge_rb = $och_folder + "/bridge.rb"
  $bridge_rb_loaded = 0
  $furniture_rb = $och_folder + "/furniture.rb"
  $furniture_rb_loaded = 0
  logstring("Checking for hangar.rb", 1)
  if File.exist?($hangar_rb) and File.exist?($standalone_hangar_rb)
    if File.exist?($hangar_doors_rb) and File.exist?($walls_rb)
      require $hangar_rb
      require $standalone_hangar_rb
      require $hangar_doors_rb
      require $walls_rb
      logstring("Hangar.rb loaded", 1)
      logstring("Standalone_hangar.rb loaded", 1)
      logstring("Hangar_doors.rb loaded", 1)
      logstring("Walls.rb loaded", 1)
      $hangar_rb_loaded = 1
      $hangar_doors_rb_loaded = 1
      $walls_rb_loaded = 1
      $scripts_loaded = 1
    elsif File.exist?($hangar_doors_rb)
      logstring("Walls.rb missing - this is required for hangar.rb", 1)
    else
      logstring("Hangar_doors.rb missing - this is required for hangar.rb", 1)
    end
    if File.exist?($ocss_extensions_rb)
      require $ocss_extensions_rb
      logstring("OCSS_extensions.rb loaded", 1)
      $ocss_extensions_rb_loaded = 1
    else
      logstring("OCSS_extensions.rb missing. Please note this file is required to draw hangar extensions", 1)
    end
    if File.exist?($beams_rb)
      require $beams_rb
      logstring("Beams.rb loaded", 1)
      $beams_rb_loaded = 1
    else
      logstring("Beams.rb missing. Please note this file is required to draw some internal beams", 1)
    end
    
  else
    logstring("Hangar.rb missing", 1)
  end

  if File.exist?($drawing_rb)
    require $drawing_rb
    logstring("Drawing.rb loaded", 1)
    $drawing_rb_loaded = 1
    $scripts_loaded = 1
  end

  if File.exist?($development_rb)
    require $development_rb
    logstring("Development.rb loaded", 1)
    $development_rb_loaded = 1
    $scripts_loaded = 1
  end
  
  if File.exist?($afcad_rb)
    require $afcad_rb
    logstring("Afcad.rb loaded", 1)
    $afcad_rb_loaded = 1
    $scripts_loaded = 1
  end
  
  if File.exist?($building_rb)
    require $building_rb
    logstring("Building.rb loaded", 1)
    $building_rb_loaded = 1
    $scripts_loaded = 1
  end  
  
  if File.exist?($bridge_rb)
    require $bridge_rb
    logstring("Bridge.rb loaded", 1)
    $bridge_rb_loaded = 1
    $scripts_loaded = 1
  end
  
  if File.exist?($furniture_rb)
    require $furniture_rb
    logstring("Furniture.rb loaded", 1)
    $furniture_rb_loaded = 1
    $scripts_loaded = 1
  end
  
end

##################
### Load menus ###
##################

if $scripts_loaded == 1
  $main_menu = UI.menu("PlugIns")
  $help_menu = UI.menu("Help")
  $och_menu = $main_menu.add_submenu("One Click Hangar")
  $help_menu.add_item("One Click Hangar help") { help_file }
  if $hangar_rb_loaded == 1
    logstring("Creating 'Create hangar' menu", 1)
    $hangar_menu = $och_menu.add_submenu("Create hangar")
    $building_menu = $och_menu.add_submenu("Create airport structure")
    $hangar_menu.add_item("Standard hangar") { standalone_hangar }
    $hangar_menu.add_item("T-hangar") { detailed_t_hangar }
    $hangar_menu.add_item("Arched hangar") { arched_hangar }
    $hangar_menu.add_item("Circular hangar") { circular_hangar }
    $hangar_menu.add_item("Hangar row") { hangar_row }
    $hangar_menu.add_item("T-hangar row") { t_hangar_row }
    $hangar_menu.add_item("Double nested T-hangar row") { d_nested_t_hangar_row }
    $hangar_menu.add_item("Ramada shelter") { ramada_shelter }
    $building_menu.add_item("Airport Tower") { atc_tower }
    $hangar_menu.add_item("Create hangar (other)") {
      prompts = ["Please select hangar type: "]
      defaults = ["Import parking"]
      list = ["Import parking|Change hangar door|Draw door|Units"] #|Find end point"]
      input = UI.inputbox prompts, defaults, list, "Hangar type"
      if input[0] == "Find end point"
        find_endpoints
      elsif input[0] == "Standard hangar - test"
        standard_hangar_test
      elsif input[0] == "Import parking"
        import_parking
      elsif input[0] == "Change hangar door"
        hangar_door_changer
      elsif input[0] == "Units"
        change_units
      elsif input[0] == "Draw door"
        draw_door
      end
      }

    ### Add context menu ###
    UI.add_context_menu_handler do |context_menu|
      context_menu.add_separator
      convert_menu = context_menu.add_submenu("Convert to hangar") 
      convert_menu.add_item ("Standard hangar") { convert_to_hangar "Standard" }
      convert_menu.add_item ("Double nested T-hangar row") { convert_to_hangar "Double nested T-hangar row" }
      convert_menu.add_item ("T-hangar row") { convert_to_hangar "T-hangar row" }
      convert_menu.add_item ("Hangar row") { convert_to_hangar "Hangar row" }
      context_menu.add_separator
      context_menu.add_item("Change hangar door") { hangar_door_changer }
      context_menu.add_item("Show hangar info") { hangar_info }
    end
  end

  ### Add preference menu ###
  
  if $drawing_rb_loaded == 1
    logstring("Creating 'Instant drawing' menu", 1)
    $drawing_menu = $och_menu.add_submenu("Instant drawing")
    $taxiway_menu = $drawing_menu.add_item("Draw Taxiways") {
      if $drawing_on == 0
        logstring("Draw Taxiways switched on", 1)
        $drawing_on = 1
        $fence_drawing_on = 0
        $gate_drawing_on = 0
        status = $drawing_menu.set_validation_proc($taxiway_menu) { MF_CHECKED }
        status = $drawing_menu.set_validation_proc($gate_menu) { MF_UNCHECKED }
        status = $drawing_menu.set_validation_proc($fence_menu) { MF_UNCHECKED }
        $draw_count = 0
        $current_entity_id = 0
      else
        logstring("Draw Taxiways switched off", 1)
        $drawing_on = 0
        status = $drawing_menu.set_validation_proc($taxiway_menu) { MF_UNCHECKED }
      end
      }
 
    $fence_menu = $drawing_menu.add_item("Draw Fence") {
      if $fence_drawing_on == 0
        logstring("Draw Fence switched on", 1)
        $drawing_on = 0
        $gate_drawing_on = 0
        $fence_drawing_on = 1
        status = $drawing_menu.set_validation_proc($taxiway_menu) { MF_UNCHECKED }
        status = $drawing_menu.set_validation_proc($gate_menu) { MF_UNCHECKED }
        status = $drawing_menu.set_validation_proc($fence_menu) { MF_CHECKED }
        $draw_count = 0
        $current_entity_id = 0
      else
        logstring("Draw Fence switched off", 1)
        $fence_drawing_on = 0
        status = $drawing_menu.set_validation_proc($fence_menu) { MF_UNCHECKED }
      end
      }

    $gate_menu = $drawing_menu.add_item("Draw Gate") {
      if $gate_drawing_on == 0
        logstring("Draw Gate switched on", 1)
        $drawing_on = 0
        $fence_drawing_on = 0
        $gate_drawing_on = 1
        status = $drawing_menu.set_validation_proc($taxiway_menu) { MF_UNCHECKED }
        status = $drawing_menu.set_validation_proc($fence_menu) { MF_UNCHECKED }
        status = $drawing_menu.set_validation_proc($gate_menu) { MF_CHECKED }
        $draw_count = 0
        $current_entity_id = 0
      else
        logstring("Draw Gate switched off", 1)
        $gate_drawing_on = 0
        status = $drawing_menu.set_validation_proc($gate_menu) { MF_UNCHECKED }
      end
      }
  end

  if $afcad_rb_loaded == 1
    $afcad_menu = $och_menu.add_submenu("AFCADs")
    $afcad_menu.add_item("Read AFCAD") {
      if $airport_icao == ""
        if $win32ole_ok == 1
          file = UI.openpanel("Open AFCAD File", $default_afcad_folder, "All Files(*.*)|*.*|AFCAD Files(*.bgl)|*.bgl||")      
        else
          file = UI.openpanel("Open AFCAD File", $default_afcad_folder, "")      
        end
        if not file or file == ""
          puts "No filename entered."
        else
          $default_afcad_folder = file.rpartition("\\").first
          read_afcad (file)
        end 
      else
        reset_afcad = UI.messagebox "Unable to load another AFCAD. Airport already set to " + $airport_icao + ". Do you wish to reset location?", MB_YESNO
        if reset_afcad == IDYES
          clear_icao
        end
      end
      }

    $clear_menu = $afcad_menu.add_item("Clear AFCAD location") { clear_icao }
    status = $afcad_menu.set_validation_proc($clear_menu) { MF_GRAYED }
  end

  if $building_rb_loaded == 1 or $bridge_rb_loaded == 1 or $furniture_rb_loaded == 1 or $development_rb_loaded == 1
    $dev_feature_menu = $och_menu.add_submenu("Development features")
  end

  if $building_rb_loaded == 1
    $dev_feature_menu.add_item("Create building") {
      prompts = ["Please select building type: "]
      defaults = ["House"]
      list = ["House"]
      input = UI.inputbox prompts, defaults, list, "Building type"
      if input[0] == "House"
        house
      end
      }

    UI.add_context_menu_handler do |context_menu|
      context_menu.add_item("Create house") { create_house }
      context_menu.add_item("Hide item") { standalone_hangar }
    end    
  end

  if $bridge_rb_loaded == 1
    $dev_feature_menu.add_item("Create bridge") {
      prompts = ["Please select bridge type: "]
      defaults = ["Arched bridge"]
      list = ["Arched bridge|Box girder bridge|New bridge|Deck test|Check line"]
      input = UI.inputbox prompts, defaults, list, "Bridge type"
  
      if input[0] == "Arched bridge"
        arched_bridge
      elsif input[0] == "Box girder bridge"
        box_girder_bridge
      elsif input[0] == "New bridge"
        new_bridge
      elsif input[0] == "Deck test"
        deck_test
      elsif input[0] == "Check line"
        check_line
      end
      }    
  end

  if $furniture_rb_loaded == 1
    $dev_feature_menu.add_item("Create furniture") {
      prompts = ["Please select furniture type: "]
      defaults = ["Computer desk"]
      list = ["Computer desk|Cabinet|Create office"]
      input = UI.inputbox prompts, defaults, list, "Furniture"
  
      if input[0] == "Computer desk"
        computer_desk
      elsif input[0] == "Cabinet"
        cabinet
      elsif input[0] == "Create office"
        create_office
      end
      }    
  end

  if $development_rb_loaded == 1
    $dev_feature_menu.add_item("Beam truss") { beam_truss }
    $dev_feature_menu.add_item("Beam joints") { beam_joints }
    $dev_feature_menu.add_item("Extensions") { extension_import }
    $dev_feature_menu.add_item("Arched beam") { arched_beam }
    $dev_feature_menu.add_item("Window test") { window_test }
    $dev_feature_menu.add_item("Custom hangar form") { custom_hangar_form }
    UI.add_context_menu_handler do |context_menu|
      context_menu.add_item("Fill grass") { fill_grass }
    end    
  end

  $preferences_menu = $och_menu.add_submenu("Preferences")
  if $drawing_rb_loaded == 1
    $preferences_menu.add_item("Fence settings") { fence_settings }
    $preferences_menu.add_item("Gate settings") { gate_settings }
  end
  $preferences_menu.add_item("Texture manager") { texture_manager }
  $preferences_menu.add_item("About One Click Hangar") { about_plugin }


  ###########################
  #### Load context menu ####
  ###########################

  UI.add_context_menu_handler do |context_menu|
    context_menu.add_item("Show item info") { 
      show_item_info
    }
  end
  logstring("Creating context menu", 1)

end

