#################################
#####												#####
#####	One Click Hangar			#####
#####												#####
#####	Version - 19-03-2015	#####
#####												#####
#################################


######################################################
###### Below is the code to display hangar info ######
######################################################

def hangar_info
	model = Sketchup.active_model
 	selection = model.selection
 
	if selection.length == 1
		if selection[0].typename == "Group"
			group = selection[0]
      logstring("Displaying hangar info", 2)
			dlg = UI::WebDialog.new("Hangar info", true, "HangarInfo", 720, 400, 150, 150, true)
			dlg.set_url $html_files + "hangar_info.html"
			name = group.name
			js_script = "dataFromSketchup(#{name.inspect})"
		
			dlg.show {
				dlg.execute_script(js_script) 
			}
			logstring("Hangar info dialogue box closed", 2)
    else
      logstring("Unable to display hangar info - selected item is not a group", 2)
		end
	else
	  logstring("Unable to display hangar info - more than one item selected", 2)
	end
end


#####################################################
###### Below is the code to change hangar door ######
#####################################################

def hangar_door_changer

	model = Sketchup.active_model
 	selection = model.selection
 
	if selection.length == 1
		if selection[0].typename == "Group"
			group = selection[0]
			description = group.description
			logstring("Grouped item selected - " + description, 1)
			if group.entities.count == 2
				status = group.explode
				logstring("Dividing group into main structure and door", 2)
				for step in 0..1
					if status[step].description.include? "Sliding doors" or status[step].description.include? "Detailed door" or status[step].description.include? "Encased sliding doors" or status[step].description.include? "Bi-folding doors" or status[step].description.include? "Hydraulic swing door" or status[step].description.include? "Encased sliding doors - single sided"
						door_group_id = step.to_i
						hangar_group_id = (step.to_i - 1).abs
						logstring("Door group is " + status[step].description, 2)
					end
				end
				door_details = status[door_group_id].description.split(":")
				hangar_details = status[hangar_group_id].description.split(":")
				logstring("Displaying door prompts", 2)
				prompts = ["Door type?", "Door folding level?", "Door sections?", "Door frame thickness?", "Door skin thickness?", "Door frame texture?", "Door texture?", "Beam texture?", "Extensions?"]
				defaults = [door_details[0], door_details[2], door_details[3], door_details[8].to_f * $inch, door_details[9].to_f * $inch, door_details[12], door_details[13], door_details[16], door_details[21]]
				list = ["Sliding doors|Bi-folding doors|Hydraulic swing door|Encased sliding doors|Encased sliding doors - single sided", "", "", "", "", $metal_list, $door_list, $metal_list, ""]
				input = UI.inputbox prompts, defaults, list, "Door changer"
				
				if input == false
				  logstring("Cancel button clicked - keeping existing door", 1)
					group = status[hangar_group_id].parent.entities.add_group([status[hangar_group_id], status[door_group_id]])
				else
					if input[0] != door_details[0] or input[1].to_i != door_details[2].to_i or input[2].to_i != door_details[3].to_i or input[3].to_f != door_details[8].to_f or input[4].to_f != door_details[9].to_f or input[5] != door_details[12] or input[6] != door_details[13] or input[7] != door_details[16] or input[8] != door_details[21]
						door_group = Sketchup.active_model.entities.add_group
						door_group_ents = door_group.entities
						door_group = insert_door(input[0], door_details[1].to_f, input[1].to_i, input[2].to_i, door_details[4].to_f, door_details[5].to_f, door_details[6].to_f, door_details[7].to_f, input[3].to_f / $inch, input[4].to_f / $inch, door_details[10], door_details[11], input[5], input[6], door_details[14], door_details[15], input[7], door_details[17], door_details[18], door_details[19], door_details[20].to_f, input[8])
						new_door_group_info = door_group.description.split(":")
						point = Geom::Point3d.new(hangar_details[1].to_f, hangar_details[2].to_f, 0)
						t = Geom::Transformation.new point
						door_group.move! t
						radians = hangar_details[3].to_f.degrees
						if hangar_details.length == 4
							t = Geom::Transformation.rotation [hangar_details[1].to_f,hangar_details[2].to_f,0], Z_AXIS, radians
						else
							t = Geom::Transformation.rotation [hangar_details[4].to_f,hangar_details[5].to_f,0], Z_AXIS, radians
						end
						door_group.transform! t
	
						group_x = status[hangar_group_id]
						group = group_x.parent.entities.add_group([group_x, door_group])
						status[door_group_id].erase!
					else
					  logstring("No changes made - keeping existing door", 1)
						group = status[hangar_group_id].parent.entities.add_group([status[hangar_group_id], status[door_group_id]])
					end
				end
			end
		end
	end
end


########################################################################
###### Below is the code to import parking spots from AFCAD files	######
########################################################################

def import_parking

	max_latitude = 999.to_f
	min_latitude = -999.to_f
	max_longitude = 999.to_f
	min_longitude = -999.to_f
	hangar_row = []
	hangar_row_type = []
	hangar_row_id = []
	hangar_row_id_count = []
	hangar_row_count = 0
	hangar_parking_id = []
	hangar_parking_side = []
	hangar_row_front_count = []
	hangar_row_back_count = []
	hangar_row_front_total_angle = []
	hangar_row_front_average_angle = []
	hangar_row_front_min_angle = []
	hangar_row_front_max_angle = []
	hangar_row_back_total_angle = []
	hangar_row_back_average_angle = []
	hangar_row_back_min_angle = []
	hangar_row_back_max_angle = []

  prompts = ["Airport ICAO: ", "Automatically export:", "Display parking spots?"]
	defaults = ["", ".kmz(Google Earth)", "Yes"]
	lists = ["", ".kmz(Google Earth)|.dae(COLLADA)|No export", $yn]
	input = UI.inputbox prompts, defaults, lists, "Import airport data"
	csv_filename = File.dirname(__FILE__). + "/" + input[0] + "_Parking.och"
	parking_array = IO.readlines(csv_filename)
	display_parking = input[2]

	## Analyse parking file
	for step in 0..parking_array.length - 1
		parking_array[step] = parking_array[step].split(";")
		if parking_array[step][6] == ""
			parking_array[step][6] = 0.to_f
		end
		if max_latitude > parking_array[step][4].to_f
			max_latitude = parking_array[step][4].to_f
		end
		if min_latitude < parking_array[step][4].to_f
			min_latitude = parking_array[step][4].to_f
		end
		if max_longitude > parking_array[step][5].to_f
			max_longitude = parking_array[step][5].to_f
		end
		if min_longitude < parking_array[step][5].to_f
			min_longitude = parking_array[step][5].to_f
		end
		if parking_array[step].length > 8
			temp_string = parking_array[step][7]
			for temp_step in 8..parking_array[step].length - 1
				temp_string = temp_string + ";" + parking_array[step][temp_step]
			end
			parking_array[step][7] = temp_string
		end
		
		## Check for grouped parking spots
		if parking_array[step][7].count("(") > 0
			if parking_array[step][7][0..parking_array[step][7].index("(") - 1] == "TRow" or parking_array[step][7][0..parking_array[step][7].index("(") - 1] == "TDRow"
				hangar_row_info = parking_array[step][7][parking_array[step][7].index("(") + 1..parking_array[step][7].index(")") - 1].split("x")

				if hangar_row_count == 0
					hangar_row_type[0] = parking_array[step][7][0..parking_array[step][7].index("(") - 1]
					hangar_row_front_count[0] = 0
					hangar_row_back_count[0] = 0
					hangar_row_id[0] = hangar_row_info[1]
					hangar_parking_id[step] = 0
					hangar_row_id_count[0] = 1
					hangar_row_front_total_angle[0] = 0
					hangar_row_front_min_angle[0] = 360
					hangar_row_front_max_angle[0] = 0
					hangar_row_back_total_angle[0] = 0
					hangar_row_back_min_angle[0] = 360
					hangar_row_back_max_angle[0] = 0

					if hangar_row_info[0] == "F"
						hangar_row_front_count[0] = hangar_row_front_count[0] + 1
						hangar_row_front_total_angle[0] = hangar_row_front_total_angle[0] + parking_array[step][6].to_f
						hangar_parking_side[step] = "F"
						if parking_array[step][6].to_f < hangar_row_front_min_angle[0]
							hangar_row_front_min_angle[0] = parking_array[step][6].to_f
						end
						if parking_array[step][6].to_f > hangar_row_front_max_angle[0]
							hangar_row_front_max_angle[0] = parking_array[step][6].to_f
						end
					else
						hangar_row_back_count[0] = hangar_row_back_count[0] + 1
						hangar_row_back_total_angle[0] = hangar_row_back_total_angle[0] + parking_array[step][6].to_f
						if parking_array[step][6].to_f < hangar_row_back_min_angle[0]
							hangar_row_back_min_angle[0] = parking_array[step][6].to_f
						end
						if parking_array[step][6].to_f > hangar_row_back_max_angle[0]
							hangar_row_back_max_angle[0] = parking_array[step][6].to_f
						end
						hangar_parking_side[step] = "B"
					end

					hangar_row_count = hangar_row_count + 1

				else
					new_hangar = 1
					for hangar_step in 0..hangar_row_count - 1
						if hangar_row_info[1] == hangar_row_id[hangar_step]
							new_hangar = 0
							hangar_parking_id[step] = hangar_step
							hangar_row_id_count[hangar_step] = hangar_row_id_count[hangar_step] + 1
							if hangar_row_info[0] == "F"
								hangar_row_front_count[hangar_step] = hangar_row_front_count[hangar_step] + 1
								hangar_row_front_total_angle[hangar_step] = hangar_row_front_total_angle[hangar_step] + parking_array[step][6].to_f
								if parking_array[step][6].to_f < hangar_row_front_min_angle[hangar_step]
									hangar_row_front_min_angle[hangar_step] = parking_array[step][6].to_f
								end
								if parking_array[step][6].to_f > hangar_row_front_max_angle[hangar_step]
									hangar_row_front_max_angle[hangar_step] = parking_array[step][6].to_f
								end
								hangar_parking_side[step] = "F"
							else
								hangar_row_back_count[hangar_step] = hangar_row_back_count[hangar_step] + 1
								hangar_row_back_total_angle[hangar_step] = hangar_row_back_total_angle[hangar_step] + parking_array[step][6].to_f
								if parking_array[step][6].to_f < hangar_row_back_min_angle[hangar_step]
									hangar_row_back_min_angle[hangar_step] = parking_array[step][6].to_f
								end
								if parking_array[step][6].to_f > hangar_row_back_max_angle[hangar_step]
									hangar_row_back_max_angle[hangar_step] = parking_array[step][6].to_f
								end
								hangar_parking_side[step] = "B"
							end

						end
					end
					
					
					if new_hangar == 1
						hangar_row_type[hangar_row_count] = parking_array[step][7][0..parking_array[step][7].index("(") - 1]
						hangar_row_front_count[hangar_row_count] = 0
						hangar_row_back_count[hangar_row_count] = 0
						hangar_row_id[hangar_row_count] = hangar_row_info[1]
						hangar_parking_id[step] = hangar_row_count
						hangar_row_id_count[hangar_row_count] = 1
						hangar_row_front_total_angle[hangar_row_count] = 0
						hangar_row_front_min_angle[hangar_row_count] = 360
						hangar_row_front_max_angle[hangar_row_count] = 0
						hangar_row_back_total_angle[hangar_row_count] = 0
						hangar_row_back_min_angle[hangar_row_count] = 360
						hangar_row_back_max_angle[hangar_row_count] = 0

						if hangar_row_info[0] == "F"
							hangar_row_front_count[hangar_row_count] = hangar_row_front_count[hangar_row_count] + 1
							hangar_row_front_total_angle[hangar_row_count] = hangar_row_front_total_angle[hangar_row_count] + parking_array[step][6].to_f
							hangar_parking_side[step] = "F"
							if parking_array[step][6].to_f < hangar_row_front_min_angle[hangar_row_count]
								hangar_row_front_min_angle[hangar_row_count] = parking_array[step][6].to_f
							end
							if parking_array[step][6].to_f > hangar_row_front_max_angle[hangar_row_count]
								hangar_row_front_max_angle[hangar_row_count] = parking_array[step][6].to_f
							end
						else
							hangar_row_back_count[hangar_row_count] = hangar_row_back_count[hangar_row_count] + 1
							hangar_row_back_total_angle[hangar_row_count] = hangar_row_back_total_angle[hangar_row_count] + parking_array[step][6].to_f
							if parking_array[step][6].to_f < hangar_row_back_min_angle[hangar_row_count]
								hangar_row_back_min_angle[hangar_row_count] = parking_array[step][6].to_f
							end
							if parking_array[step][6].to_f > hangar_row_back_max_angle[hangar_row_count]
								hangar_row_back_max_angle[hangar_row_count] = parking_array[step][6].to_f
							end
							hangar_parking_side[step] = "B"
						end
	
						hangar_row_count = hangar_row_count + 1
					
					end
				end
			end


			if parking_array[step][7][0..parking_array[step][7].index("(") - 1] == "HRow"
				hangar_row_info = parking_array[step][7][parking_array[step][7].index("(") + 1..parking_array[step][7].index(")") - 1] #.split("x")

				if hangar_row_count == 0
					hangar_row_type[0] = "HRow"
					hangar_row_front_count[0] = 0
					hangar_row_back_count[0] = 0
					hangar_row_id[0] = hangar_row_info
					hangar_parking_id[step] = 0
					hangar_row_id_count[0] = 1
					hangar_row_front_total_angle[0] = 0
					hangar_row_front_min_angle[0] = 360
					hangar_row_front_max_angle[0] = 0
					hangar_row_back_total_angle[0] = 0
					hangar_row_back_min_angle[0] = 360
					hangar_row_back_max_angle[0] = 0

					hangar_row_front_count[0] = hangar_row_front_count[0] + 1
					hangar_row_front_total_angle[0] = hangar_row_front_total_angle[0] + parking_array[step][6].to_f
					hangar_parking_side[step] = "F"
					if parking_array[step][6].to_f < hangar_row_front_min_angle[0]
						hangar_row_front_min_angle[0] = parking_array[step][6].to_f
					end
					if parking_array[step][6].to_f > hangar_row_front_max_angle[0]
						hangar_row_front_max_angle[0] = parking_array[step][6].to_f
					end

					hangar_row_count = hangar_row_count + 1

				else
					new_hangar = 1
					for hangar_step in 0..hangar_row_count - 1
						if hangar_row_info == hangar_row_id[hangar_step]
							new_hangar = 0
							hangar_parking_id[step] = hangar_step
							hangar_row_id_count[hangar_step] = hangar_row_id_count[hangar_step] + 1
							hangar_row_front_count[hangar_step] = hangar_row_front_count[hangar_step] + 1
							hangar_row_front_total_angle[hangar_step] = hangar_row_front_total_angle[hangar_step] + parking_array[step][6].to_f
							if parking_array[step][6].to_f < hangar_row_front_min_angle[hangar_step]
								hangar_row_front_min_angle[hangar_step] = parking_array[step][6].to_f
							end
							if parking_array[step][6].to_f > hangar_row_front_max_angle[hangar_step]
								hangar_row_front_max_angle[hangar_step] = parking_array[step][6].to_f
							end
							hangar_parking_side[step] = "F"
						end
					end
					
					
					if new_hangar == 1
						hangar_row_type[hangar_row_count] = "HRow"
						hangar_row_front_count[hangar_row_count] = 0
						hangar_row_back_count[hangar_row_count] = 0
						hangar_row_id[hangar_row_count] = hangar_row_info
						hangar_parking_id[step] = hangar_row_count
						hangar_row_id_count[hangar_row_count] = 1
						hangar_row_front_total_angle[hangar_row_count] = 0
						hangar_row_front_min_angle[hangar_row_count] = 360
						hangar_row_front_max_angle[hangar_row_count] = 0
						hangar_row_back_total_angle[hangar_row_count] = 0
						hangar_row_back_min_angle[hangar_row_count] = 360
						hangar_row_back_max_angle[hangar_row_count] = 0

						hangar_row_front_count[hangar_row_count] = hangar_row_front_count[hangar_row_count] + 1
						hangar_row_front_total_angle[hangar_row_count] = hangar_row_front_total_angle[hangar_row_count] + parking_array[step][6].to_f
						hangar_parking_side[step] = "F"
						if parking_array[step][6].to_f < hangar_row_front_min_angle[hangar_row_count]
							hangar_row_front_min_angle[hangar_row_count] = parking_array[step][6].to_f
						end
						if parking_array[step][6].to_f > hangar_row_front_max_angle[hangar_row_count]
							hangar_row_front_max_angle[hangar_row_count] = parking_array[step][6].to_f
						end
	
						hangar_row_count = hangar_row_count + 1
					
					end
				end
			end
			
		end
		
	end

	## Geolocate in SketchUp
	
	geolocate_model((max_latitude + min_latitude) / 2, (max_longitude + min_longitude) / 2)

	## Calculate and draw hangar rows
	
	for step in 0..hangar_row_count - 1
	
		if hangar_row_type[step] == "TRow" or hangar_row_type[step] == "TDRow"
			hangar_row_front_average_angle[step] = hangar_row_front_total_angle[step] / hangar_row_front_count[step]
			hangar_row_back_average_angle[step] = hangar_row_back_total_angle[step] / hangar_row_back_count[step]
		
			if(hangar_row_front_average_angle[step] - hangar_row_back_average_angle[step]).abs != 180
				UI.messagebox "Warning - parking spots in hangar " + hangar_row_id[step] + " are not perfectly aligned"
			end
		
			max_distance = 0
			max_distance_pair = ""
			end_coordinate = 0
			comparison_made = 0
			first_hangar = 0
			
			for hangar_x_step in 0..parking_array.length - 1
				for hangar_y_step in 0..parking_array.length - 1
					if hangar_x_step != hangar_y_step and hangar_parking_id[hangar_x_step] == step and hangar_parking_id[hangar_y_step] == step and hangar_parking_side[hangar_y_step] == "F" and hangar_parking_side[hangar_x_step] == "F"
						if comparison_made == 0
							if hangar_row_front_average_angle[step] >= 45 and hangar_row_front_average_angle[step] < 135
								end_coordinate = parking_array[hangar_x_step][4].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] >= 225 and hangar_row_front_average_angle[step] < 315
								end_coordinate = parking_array[hangar_x_step][4].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] >= 135 and hangar_row_front_average_angle[step] < 225
								end_coordinate = parking_array[hangar_x_step][5].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] < 45 and hangar_row_front_average_angle[step] >= 315
								end_coordinate = parking_array[hangar_x_step][5].to_f
							first_hangar = hangar_x_step
								end
							comparison_made = 1
						else
							if hangar_row_front_average_angle[step] >= 45 and hangar_row_front_average_angle[step] < 135
								if end_coordinate < parking_array[hangar_x_step][4].to_f
									end_coordinate = parking_array[hangar_x_step][4].to_f
									first_hangar = hangar_x_step
								end
							end
							if hangar_row_front_average_angle[step] >= 225 and hangar_row_front_average_angle[step] < 315
								if end_coordinate > parking_array[hangar_x_step][4].to_f
									end_coordinate = parking_array[hangar_x_step][4].to_f
									first_hangar = hangar_x_step
								end
							end
								if hangar_row_front_average_angle[step] >= 135 and hangar_row_front_average_angle[step] < 225
								if end_coordinate > parking_array[hangar_x_step][5].to_f
									end_coordinate = parking_array[hangar_x_step][5].to_f
									first_hangar = hangar_x_step
								end
							end
							if hangar_row_front_average_angle[step] < 45 and hangar_row_front_average_angle[step] >= 315
								if end_coordinate < parking_array[hangar_x_step][5].to_f
									end_coordinate = parking_array[hangar_x_step][5].to_f
									first_hangar = hangar_x_step
								end
							end

						end
						earth_radius = 6371
						lat1 = parking_array[hangar_x_step][4].to_f
						lat2 = parking_array[hangar_y_step][4].to_f
						long1 = parking_array[hangar_x_step][5].to_f
						long2 = parking_array[hangar_y_step][5].to_f
						dlat =(lat2-lat1).degrees
						dlong =(long2-long1).degrees
						rlat1 = lat1.degrees
						rlat2 = lat2.degrees
			
						calc_a = Math.sin(dlat/2) * Math.sin(dlat/2) + Math.sin(dlong/2) * Math.sin(dlong/2) * Math.cos(rlat1) * Math.cos(rlat2)
						calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
						distance =(earth_radius * calc_c * 100000)
	
						if distance > max_distance
							max_distance = distance
							max_distance_pair = hangar_y_step.to_s + ":" + hangar_x_step.to_s
						end
	
					
					end
				end
			end
			
			if hangar_row_type[step] == "TRow"
			
				hangar_section = max_distance / (hangar_row_front_count[step] - 1) * 3 / 4
				hangar_length =(hangar_section * hangar_row_front_count[step]) +((hangar_section / 2) *(hangar_row_front_count[step] - 1)) +(hangar_section * 0.75)
				hangar_string = parking_array[first_hangar][7] + ":" + parking_array[first_hangar][4] + ":" + parking_array[first_hangar][5] + ":" + hangar_row_front_average_angle[step].to_s + ":" + hangar_section.to_s + ":" + hangar_row_front_count[step].to_s + ":" + hangar_row_back_count[step].to_s
				t_hangar_row(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)

			else
				hangar_section = max_distance / (hangar_row_front_count[step] - 1)
				hangar_string = parking_array[first_hangar][7] + ":" + parking_array[first_hangar][4] + ":" + parking_array[first_hangar][5] + ":" + hangar_row_front_average_angle[step].to_s + ":" + hangar_section.to_s + ":" + hangar_row_front_count[step].to_s + ":" + hangar_row_back_count[step].to_s
				d_nested_t_hangar_row(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
			end
		end

		if hangar_row_type[step] == "HRow"
			hangar_row_front_average_angle[step] = hangar_row_front_total_angle[step] / hangar_row_front_count[step]
		
			max_distance = 0
			max_distance_pair = ""
			end_coordinate = 0
			comparison_made = 0
			first_hangar = 0
			
			for hangar_x_step in 0..parking_array.length - 1
				for hangar_y_step in 0..parking_array.length - 1
					if hangar_x_step != hangar_y_step and hangar_parking_id[hangar_x_step] == step and hangar_parking_id[hangar_y_step] == step and hangar_parking_side[hangar_y_step] == "F" and hangar_parking_side[hangar_x_step] == "F"
						if comparison_made == 0
							if hangar_row_front_average_angle[step] >= 45 and hangar_row_front_average_angle[step] < 135
								end_coordinate = parking_array[hangar_x_step][4].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] >= 225 and hangar_row_front_average_angle[step] < 315
								end_coordinate = parking_array[hangar_x_step][4].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] >= 135 and hangar_row_front_average_angle[step] < 225
								end_coordinate = parking_array[hangar_x_step][5].to_f
								first_hangar = hangar_x_step
							end
							if hangar_row_front_average_angle[step] < 45 and hangar_row_front_average_angle[step] >= 315
								end_coordinate = parking_array[hangar_x_step][5].to_f
							first_hangar = hangar_x_step
								end
							comparison_made = 1
						else
							if hangar_row_front_average_angle[step] >= 45 and hangar_row_front_average_angle[step] < 135
								if end_coordinate < parking_array[hangar_x_step][4].to_f
									end_coordinate = parking_array[hangar_x_step][4].to_f
									first_hangar = hangar_x_step
								end
							end
							if hangar_row_front_average_angle[step] >= 225 and hangar_row_front_average_angle[step] < 315
								if end_coordinate > parking_array[hangar_x_step][4].to_f
									end_coordinate = parking_array[hangar_x_step][4].to_f
									first_hangar = hangar_x_step
								end
							end
								if hangar_row_front_average_angle[step] >= 135 and hangar_row_front_average_angle[step] < 225
								if end_coordinate > parking_array[hangar_x_step][5].to_f
									end_coordinate = parking_array[hangar_x_step][5].to_f
									first_hangar = hangar_x_step
								end
							end
							if hangar_row_front_average_angle[step] < 45 and hangar_row_front_average_angle[step] >= 315
								if end_coordinate < parking_array[hangar_x_step][5].to_f
									end_coordinate = parking_array[hangar_x_step][5].to_f
									first_hangar = hangar_x_step
								end
							end

						end
						earth_radius = 6371
						lat1 = parking_array[hangar_x_step][4].to_f
						lat2 = parking_array[hangar_y_step][4].to_f
						long1 = parking_array[hangar_x_step][5].to_f
						long2 = parking_array[hangar_y_step][5].to_f
						dlat =(lat2-lat1).degrees
						dlong =(long2-long1).degrees
						rlat1 = lat1.degrees
						rlat2 = lat2.degrees
			
						calc_a = Math.sin(dlat/2) * Math.sin(dlat/2) + Math.sin(dlong/2) * Math.sin(dlong/2) * Math.cos(rlat1) * Math.cos(rlat2)
						calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
						distance =(earth_radius * calc_c * 100000)
	
						if distance > max_distance
							max_distance = distance
							max_distance_pair = hangar_y_step.to_s + ":" + hangar_x_step.to_s
						end
	
					
					end
				end
			end
			
			hangar_section = max_distance / (hangar_row_front_count[step] - 1)
			hangar_length = hangar_section * hangar_row_front_count[step]
			
			hangar_string = parking_array[first_hangar][7] + ":" + parking_array[first_hangar][4] + ":" + parking_array[first_hangar][5] + ":" + hangar_row_front_average_angle[step].to_s + ":" + hangar_section.to_s + ":" + hangar_row_front_count[step].to_s + ":" + parking_array[first_hangar][3].to_s
			hangar_row(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
		end

	end
	
	
	## Draw standalone hangars
	
	for step in 0..parking_array.length - 1
	#	if parking_array[step][7].count("(") > 0
			hangar_string = parking_array[step][7] + parking_array[step][3] + ":" + parking_array[step][4] + ":" + parking_array[step][5] + ":" + parking_array[step][6].to_s
			
			hangar_type = hangar_string[0..hangar_string.index(":") - 1]
			
			if hangar_type == "Standard"
				standalone_hangar(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
			end
			if hangar_type == "Arched"
				arched_hangar(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
			end
			if hangar_type == "THangar"
				detailed_t_hangar(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
			end
			if hangar_type == "Circular"
				circular_hangar(hangar_string,(max_latitude + min_latitude) / 2,(max_longitude + min_longitude) / 2)
			end

	#	end
	end
	
	
	
	# Draw parking spot
		
	if display_parking == "Yes"
		for step in 0..parking_array.length - 1
			parking_radius =  parking_array[step][3].to_f * 100 / $inch
			group = Sketchup.active_model.entities.add_group
			group_ents = group.entities
			pt1 = [0, 0, 0]
			vector = Geom::Vector3d.new 0,0,1
			vector2 = vector.normalize!				
			circle = group_ents.add_circle pt1, vector2, parking_radius
	
			#Calculate hangar's distance from midpoint
			earth_radius = 6371
			lat1 =(max_latitude + min_latitude) / 2
			lat2 = parking_array[step][4].to_f
			long1 =(max_longitude + min_longitude) / 2
			long2 = parking_array[step][5].to_f
			dlaty =(lat2-lat1).degrees
			dlongy =(0).degrees
			dlatx =(0).degrees
			dlongx =(long2-long1).degrees
			rlat1 = lat1.degrees
			rlat2 = lat2.degrees
			
			calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
			calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
			y_distance = (earth_radius * calc_c * 100000)
			calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
			calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
			x_distance = (earth_radius * calc_c * 100000)
	
			calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
			calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
			a_distance = (earth_radius * calc_c * 100000)
	
			if long1 > long2
				x_distance = 0 - x_distance
			end
			
			if lat1 > lat2
				y_distance = 0 - y_distance
			end
	
			hangar_x = x_distance / $inch
			hangar_y = y_distance / $inch
	
			point = Geom::Point3d.new(hangar_x, hangar_y, 0)
			t = Geom::Transformation.new point
			group.move! t
		end	
	end

	#Export file
	model = Sketchup.active_model
	
	if input[1] == ".kmz(Google Earth)"
#		export_string = "" + $local_folder + "\\" + input[0] + ".kmz"
		export_string = "" + $local_folder + "/" + input[0] + ".kmz"

		status = model.export export_string
	elsif input[1] == ".dae(COLLADA)"
#		export_string = "" + $local_folder + "\\" + input[0] + ".dae"
		export_string = "" + $local_folder + "/" + input[0] + ".dae"
		status = model.export export_string
	end
end





################################################################
###### Below is the code to calculate position of hangar	######
################################################################

def find_endpoints
	
	model = Sketchup.active_model
 	selection = model.selection
 	
 	number = selection.length
 	selected_edges = 0
 	selected_other = 0
	entity = []
	entity_type = []
	
 	if number != 2
 		if number > 2
 			for step in 0..number - 1
 				if selection[step].typename == "Edge"
 					selected_edges = selected_edges + 1
 				else
 					selected_other = selected_other + 1
 				end
 			end
 			
			if selected_edges != 2
	 			UI.messagebox "Please select two edges."
				return [0,0,0,0,0]
			end
 			if selected_edges == 2
 				current_entity = 0
 				for step in 0..number - 1
 					if selection[step].typename == "Edge"
 						entity [current_entity] =  selection[step]
 						current_entity = current_entity + 1
 					end
 				end
 				
 			end
 			
 		else 
 			if number != 0
 				UI.messagebox "Please select two edges."
 			end
			return [0,0,0,0,0]
 		end

	
	else
		entity[0] = selection[0]
		entity[1] = selection[1]
 	end
	
	entity_type[0] = entity[0].typename
	entity_type[1] = entity[1].typename
	
	if entity_type[0] != "Edge" or  entity_type[1] != "Edge"
		UI.messagebox "Please select edges only."
		return [0,0,0,0,0]
	end
	
	vertex_start = []
	vertex_end = []
	point_start = []
	point_end = []
	length = []
	vertex_start[0] = entity[0].start
	vertex_end[0] = entity[0].end
	vertex_start[1] = entity[1].start
	vertex_end[1] = entity[1].end
	point_start[0] = vertex_start[0].position
	point_end[0] = vertex_end[0].position
	point_start[1] = vertex_start[1].position
	point_end[1] = vertex_end[1].position
	length[0] = entity[0].length
	length[1] = entity[1].length
	
	if point_start[0][2].to_i != 0 or point_end[0][2].to_i != 0 or point_start[1][2].to_i != 0 or point_end[1][2].to_i != 0
		UI.messagebox "Edges must be on ground."
		return [0,0,0,0,0]
	
	end

	if point_start[0] != point_start[1]
		if point_start[0] == point_end[1]
			points = []
			points2 = []
			points = point_start[1]
			points2 = point_end[1]
			point_end[1] = points
			point_start[1] = points2
		elsif point_start[1] == point_end[0]
			points = []
			points2 = []
			points = point_start[0]
			points2 = point_end[0]
			point_end[0] = points
			point_start[0] = points2
		elsif point_end[0] == point_end[1]
			points = []
			points2 = []
			points = point_start[1]
			points2 = point_end[1]
			point_end[1] = points
			point_start[1] = points2
			points = []
			points2 = []
			points = point_start[0]
			points2 = point_end[0]
			point_end[0] = points
			point_start[0] = points2
		
		else
			UI.messagebox "Edges must be connected."
			return [0,0,0,0,0]
		end
		
	
	end

	
	shortwall = 0
	error_check = 0
	if point_start[0][1] == point_end[0][1]
		if point_start[0][0] < point_end[0][0]
			if point_start[1][1] > point_end[1][1]
				angle = 270
#				UI.messagebox "270 1-1"
			else
				angle = 0
				shortwall = 1
#				UI.messagebox "0 1-2"
			end
		else
			if point_start[1][1] < point_end[1][1]
				angle = 90
#				UI.messagebox "90 1-3"
			else
				angle = 180
				shortwall = 1
#				UI.messagebox "180 1-4"
			end
		end
	elsif point_start[0][0] == point_end[0][0]
		if point_start[0][1] < point_end[0][1]
			if point_start[1][0] < point_end[1][0]
				angle = 0
#				UI.messagebox "0 2-1"
			else
				angle = 90
				shortwall = 1
#				UI.messagebox "90 2-2"
			end
		else
			if point_start[1][0] < point_end[1][0]
				angle = 270
#				UI.messagebox "270 2-3"
				shortwall = 1
			else
				angle = 180
#				UI.messagebox "180 2-4"
			end
		end
	else
		adjacent =((point_start[0][0].to_f - point_end[0][0].to_f).abs)
		angle = Math::acos(adjacent / length[0]).radians
		if point_start[0][0] < point_end[0][0]   # X-axis start is less than X-axis end
			if point_start[0][1] < point_end[0][1] # Y-axis start is less than Y-axis end, first line running in direction XY
				if point_start[1][1] < point_end[1][1] #  Y-axis start is less than Y-axis end, second line running in direction -XY
					#corner pointing in -Y(angle 0-90 degrees)
					#angle correct from start - confirmed
#					UI.messagebox "-Y 1-1(angle 0-90 degrees) " + angle.to_s
#					UI.messagebox length[1] # Confirmed 
					shortwall = 1
				else  #  Y-axis start is more than Y-axis end, second line running in direction -XY
					#corner pointing in -X(angle 270-0 degrees)
					angle = 270 + angle # Confirmed
#					UI.messagebox "-X 1-2(angle 270-0 degrees) " + angle.to_s
#					UI.messagebox length[0] # Confirmed 
				end
			else # Y-axis start is more than Y-axis end, line running in direction X-Y
				if point_start[1][1] < point_end[1][1] #  Y-axis start is less than Y-axis end, second line running in direction XY
					#corner pointing in -X(angle 270-0 degrees)
					angle = 360 - angle # Confirmed
#					UI.messagebox "-X 1-3(angle 270-0 degrees) " + angle.to_s
#					UI.messagebox length[1] # Confirmed 
					shortwall = 1
				else  #  Y-axis start is more than Y-axis end, second line running in direction -X-Y
					#corner pointing in Y(angle 180-270 degrees)
					angle = 270 - angle # Confirmed
#					UI.messagebox "Y 1-4(angle 180-270 degrees) " + angle.to_s
#					UI.messagebox length[0] # Confirmed 
				end
			end
		else  # X-axis start is more than X-axis end
			if point_start[0][1] < point_end[0][1] # Y-axis start is less than Y-axis end, first line running in direction -XY
				if point_start[1][1] < point_end[1][1] #  Y-axis start is less than Y-axis end, second line running in direction XY
					#corner pointing in -Y(angle 0-90 degrees)
					angle = 90 - angle # Confirmed
#					UI.messagebox "-Y 2-1(angle 0-90 degrees) " + angle.to_s
#					UI.messagebox length[0] # Confirmed 
				else  #  Y-axis start is more than Y-axis end, second line running in direction -X-Y
					#corner pointing in X(angle 90-180 degrees)
					angle = 180 - angle # Confirmed
#					UI.messagebox "X 2-2(angle 90-180 degrees) " + angle.to_s
#					UI.messagebox length[1] # Confirmed 
					shortwall = 1
				end
			else # Y-axis start is more than Y-axis end, line running in direction -X-Y
				if point_start[1][1] < point_end[1][1] #     Y-axis less is more than Y-axis end, second line running in direction -XY
					#corner pointing in Y(angle 90-180 degrees)
					angle = 90 + angle # Confirmed
#					UI.messagebox "X 2-3(angle 90-180 degrees) " + angle.to_s
#					UI.messagebox length[0] # Confirmed
				else  # Y-axis start is more than Y-axis end, second line running in direction X-Y
					#corner pointing in X(angle 180-270 degrees)
					angle = 180 + angle # Confirmed
#					UI.messagebox "Y 2-4(angle 180-270 degrees) " + angle.to_s
#					UI.messagebox length[1] # Confirmed 
					shortwall = 1
				end
			end
		end
	end

	entity[0].erase!
	entity[1].erase!
	
	if shortwall == 0
		return [angle, point_start[0][0].to_f, point_start[0][1].to_f, length[0].to_f, length[1].to_f]	
	else
		return [angle, point_start[1][0].to_f, point_start[1][1].to_f, length[1].to_f, length[0].to_f]	
	end
end


##################################################
###### Below is the code for airport towers	######
##################################################

def atc_tower(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  logstring("Airport tower initiated", 1)
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "Tower"
	materials = Sketchup.active_model.materials
	if hangar_codes != ""
		afcad_read = 1
	else
		afcad_read = 0
		prompts = ["Tower total height?", "Tower sides?", "Tower radius?", "Tower levels?", "Angle?", "X-pos?", "Y-pos?"]
		lists = ["", "", "", "", "", "", ""]
		defaults = ["3200", "6", "400", "", "0", midlat, midlong]
    logstring("Display tower prompts", 1)
		input = UI.inputbox prompts, defaults, lists, "Control tower info"
    if (input === false)
      logstring("Cancel button clicked - Exiting", 1)
      return nil
    end
	 	#Texture prompts
	 	temp_wall_list = $wall_list.split("|")
    if temp_wall_list.length == 1
      default_wall_texture = temp_wall_list[0]
    else
      default_wall_texture = temp_wall_list[1]
    end
	 	texture_prompts = ["Wall texture?", "Level textures?"]
	 	texture_defaults = [default_wall_texture, ""]
	 	texture_list = [$wall_list, ""]
    logstring("Display texture prompts", 1)
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"
    if (texture_input === false)
      logstring("Cancel button clicked - Exiting", 1)
      return nil
    end

		wall_texture = texture_input[0]
    logstring("Wall texture '" + wall_texture + "' ", 2)
		level_texture = texture_input[1]
    logstring("Level texture '" + level_texture + "' ", 2)

    logstring("Unit type '" + $unit_type + "' ", 2)
    logstring("Tower height " + input[0] + " ", 2)
    logstring("Tower sides (minimum 3) " + input[1] + " ", 2)
    logstring("Tower radius " + input[2] + " ", 2)
    logstring("Tower levels " + input[3] + " ", 2)
    logstring("Tower angle " + input[4] + " ", 2)
    logstring("Tower x position " + input[5].to_s + " ", 2)
    logstring("Tower y position " + input[6].to_s + " ", 2)
		tower_total_height = input[0].to_f / $inch
		tower_sides = input[1].to_i
		if tower_sides < 3
			tower_sides = 3
		end
		tower_radius = input[2].to_f / $inch
		tower_levels = input[3]
		tower_angle = input[4].to_f
		tower_x_pos = input[5].to_f / $inch
		tower_y_pos = input[6].to_f / $inch
		tower_radians = tower_angle.degrees
	end

  ####	Add texture	####
	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step]
		  logstring("Adding texture '" + $texture_name[step] + "' as Wall Texture", 2)
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	
	if level_texture != ""
		level_textures = level_texture.split(";")
		level_texture_count = (level_texture.count(";") / 4).floor
		for step in 0..level_texture_count
			for step2 in 0..$texture_name.length
				if level_textures[(step * 4) + 1] == $texture_name[step2]
				  logstring("Adding texture '" + $texture_name[step2] + "' as Level Texture", 2)
					texture = materials.add $texture_name[step2]
          if $texture_filename[step].count(":") === 0
            texture.texture = $texture_files + $texture_filename[step2]
          else
            texture.texture = $texture_filename[step2]
          end
					texture.texture.size = [$texture_x_size[step2] / $inch, $texture_y_size[step2] / $inch]
				end
			end
		end
	end
	if wall_texture == "Untextured"
	  logstring("Using untextured Wall Texture", 2)
		texture = materials.add "Untextured"
		texture.color = "White"
	end


####	Add tower	####
	tower_section_angle = 360 / tower_sides
	tower_section_angle_offset = tower_section_angle / 2
	tower_x_coord = []
	tower_y_coord = []

	if tower_levels.count(";") > 0
	  logstring("Extracting Tower Levels - Total levels: " + (tower_levels.count(";") + 1).to_s, 2)
		tower_level_items = tower_levels.split(";")
		tower_levels_count = (tower_levels.count(";") / 2).floor
	end
	
	pts = []
	for step in 0..tower_sides - 1
		tower_x_coord[step] = Math.cos((tower_section_angle * step + tower_section_angle_offset).degrees) * tower_radius
		tower_y_coord[step] = Math.sin((tower_section_angle * step + tower_section_angle_offset).degrees) * tower_radius
		pts[step] = [tower_x_coord[step], tower_y_coord[step], 0]
		if tower_levels.count(";") == 0
			pts[step + tower_sides] = [tower_x_coord[step], tower_y_coord[step], tower_total_height]
		else
			for step2 in 1..tower_levels_count + 1
				current_x_coord = Math.cos((tower_section_angle * step + tower_section_angle_offset).degrees) * tower_level_items[step2 * 2 - 1].to_f / $inch
				current_y_coord = Math.sin((tower_section_angle * step + tower_section_angle_offset).degrees) * tower_level_items[step2 * 2 - 1].to_f / $inch
				pts[step + tower_sides * step2] = [current_x_coord, current_y_coord, tower_level_items[step2 * 2 - 2].to_f / $inch]
			end
		end
	end

	face_reversed = 0
	ptsx = []
	for step in 0..tower_sides - 1 
		if tower_levels.count(";") == 0
		  logstring("Drawing tower side " + (step + 1).to_s + " of " + tower_sides.to_s + " sides", 2)
			face = group_ents.add_face pts[step.modulo(tower_sides)], pts[(step + 1).modulo(tower_sides)], pts[(step + 1).modulo(tower_sides) + tower_sides], pts[step.modulo(tower_sides) + tower_sides]
			status = face.material = wall_texture
			ptsx[step] = pts[step + tower_sides]
		else
			for step2 in 0..tower_levels_count
			  logstring("Drawing tower side " + (step + 1).to_s + " of " + tower_sides.to_s + " sides at level " + (step2 + 1).to_s, 2)
				face = group_ents.add_face pts[(step + 1).modulo(tower_sides) + tower_sides * step2], pts[step.modulo(tower_sides) + tower_sides * step2], pts[step.modulo(tower_sides) + tower_sides * (step2 + 1)], pts[(step + 1).modulo(tower_sides) + tower_sides * (step2 + 1)]
				if face_reversed == 0
					status = face.reverse!
					face_reversed = 1
				end
				use_custom_texture = -1
				if level_texture != ""
					for step3 in 0..level_texture_count
						if level_textures[step3 * 4].to_i == step2 and (level_textures[step3 * 4 + 3] == "B" or level_textures[step3 * 4 + 3] == "W")
						  use_custom_texture = step3
						end
					end
				end
				if use_custom_texture == -1
					status = face.material = wall_texture
				else
					status = face.material = level_textures[(use_custom_texture * 4) + 1]
					status = face.material.alpha = level_textures[(use_custom_texture * 4) + 2].to_f / 100
					if level_textures[(use_custom_texture * 4) + 3] == "B"
						status = face.back_material = level_textures[(use_custom_texture * 4) + 1]
						status = face.back_material.alpha = level_textures[(use_custom_texture * 4) + 2].to_f / 100
					end
				end
			end
			ptsx[step] = pts[step + tower_sides * (tower_levels_count + 1)]
		end
	end
	
	logstring("Drawing tower roof", 2)
	face = group_ents.add_face ptsx
	status = face.material = wall_texture

	if level_texture != ""
		for step in 0..level_texture_count
			custom_floor_ceiling = -1
			if level_textures[step * 4 + 3] == "F" or level_textures[step * 4 + 3] == "C"
				custom_floor_ceiling = level_textures[step * 4].to_i
			end
			
			if custom_floor_ceiling != -1
				ptsx = []
				for step2 in 0..tower_sides - 1 
					ptsx[step2] = pts[step2 + tower_sides * custom_floor_ceiling]
				end
				face = group_ents.add_face ptsx
				status = face.material = level_textures[(step * 4) + 1]
				status = face.material.alpha = level_textures[(step * 4) + 2].to_f / 100
				if (face.normal[2] > 0 and level_textures[step * 4 + 3] == "C") or (face.normal[2] < 0 and level_textures[step * 4 + 3] == "F")
					status = face.reverse!
				end

			end
		end
	end


	##################
	#### Position ####
	##################

	logstring("Moving tower to position " + tower_x_pos.to_s + " , " + tower_y_pos.to_s, 2)
	trans = group.transformation
	point = Geom::Point3d.new(tower_x_pos + trans.origin[0], tower_y_pos + trans.origin[1], trans.origin[2])
	t = Geom::Transformation.new point
	group.move! t
	
	logstring("Rotating tower by " + tower_angle.to_s + " degrees", 2)
	t = Geom::Transformation.rotation [tower_x_pos, tower_y_pos, 0], Z_AXIS, tower_radians
	group.transform! t	
end


########################################################
########################################################
###### Below is the code for the Circular hangar  ######
########################################################
########################################################

def circular_hangar(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  
  
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "Circular"
	materials = Sketchup.active_model.materials

	if hangar_codes != ""
	
		afcad_read = 1
		hangar_info = hangar_codes.split(":")

		wall_texture = hangar_info[12]
		roof_texture = hangar_info[11]
		floor_texture = hangar_info[13]

		hangar_radius = hangar_info[1].to_f / $inch
		hangar_height = hangar_info[2].to_f / $inch
		wall_thickness = hangar_info[3].to_f / $inch
		roof_thickness = hangar_info[4].to_f / $inch
		door_width = hangar_info[5].to_f / $inch
		door_height = hangar_info[6].to_f / $inch
		extension_text = hangar_info[10]

		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[15].to_f
		long1 = midlong
		long2 = hangar_info[16].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)


		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		circumference = hangar_radius * 2 * Math::PI
		door_degrees = 360 -((door_width / circumference * 60).to_i * 6)
		door_segments = door_degrees / 6
		door_cutout_segments = 60 - door_segments
		
		hangar_angle = 270 +(door_cutout_segments * 3) - hangar_info[17].to_f
		hangar_x = x_distance / $inch
		hangar_y = y_distance / $inch
		radians = hangar_angle.degrees

		
	else
		afcad_read = 0
		prompts = ["Radius?", "Height?", "Wall thickness?", "Roof thickness?", "Door width?", "Door height?", "Door thickness?", "Door folding level?", "Central column sides?", "Central column radius?", "Side columns?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		lists = ["", "", "", "", "", "", "", "", "", "", "", "", "", "", ""]
		defaults = ["1200", "450", "20", "10", "1200", "420", "5", "10", "6", "20", "20", "0", "0", "0", ""]
		input = UI.inputbox prompts, defaults, lists, "Hangar info"


	 	#Texture prompts
	 	texture_prompts = ["Roof texture?", "Wall texture?", "Beam texture?", "Door texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Flat concrete wall", "Blue metal", "Vertical corrugated iron", "Concrete"]
	 	texture_list = [$roof_list, $wall_list, $metal_list, $door_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"

		roof_texture = texture_input[0]
    wall_texture = texture_input[1]
    beam_texture = texture_input[2]
		door_texture = texture_input[3]
		floor_texture = texture_input[4]

		hangar_radius = input[0].to_f / $inch
		hangar_height = input[1].to_f / $inch
		wall_thickness = input[2].to_f / $inch
		roof_thickness = input[3].to_f / $inch
		door_width = input[4].to_f / $inch
		door_height = input[5].to_f / $inch
		door_thickness = input[6].to_f / $inch
		door_folding_level = input[7].to_i
		central_column_sides = input[8].to_i
		central_column_radius = input[9].to_f
		side_columns = input[10].to_i
		
		if door_folding_level < 0
			door_folding_level = 0
		elsif door_folding_level > 90
			door_folding_level = 90
		end
		hangar_angle = input[11].to_f
		hangar_x = input[12].to_f / $inch
		hangar_y = input[13].to_f / $inch
		radians = hangar_angle.degrees

		extension_text = input[14]

		circumference = hangar_radius * 2 * Math::PI
	end

	
	#####################
	#### Add texture ####
	#####################

	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or door_texture == $texture_name[step] or beam_texture == $texture_name[step] or roof_texture == $texture_name[step] or floor_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or floor_texture == "Untextured"
		texture = materials.add "Untextured"
		texture.color = "White"
	end

	
	#####################
	#### Draw hangar ####
	#####################
	
	degree_length = circumference / 360
	door_degrees = ((door_width / degree_length).floor).to_i
	pts = []
	ptsx = []
	ptsc = []
	ptsr = []
	ptsf = []
	for step in 0..359
		x_coord = Math.cos(step.degrees) * hangar_radius
		y_coord = Math.sin(step.degrees) * hangar_radius
		xi_coord = Math.cos(step.degrees) * (hangar_radius - wall_thickness)
		yi_coord = Math.sin(step.degrees) * (hangar_radius - wall_thickness)
		if step < door_degrees
			pts[step] = [x_coord, y_coord, door_height]
			ptsx[step] = [xi_coord, yi_coord, door_height]
		else
			pts[step] = [x_coord, y_coord, 0]
			ptsx[step] = [xi_coord, yi_coord, 0]
		end
		pts[step + 360] = [x_coord, y_coord, hangar_height]
		ptsx[step + 360] = [xi_coord, yi_coord, hangar_height - roof_thickness]
		ptsc[step] = [xi_coord, yi_coord, hangar_height - roof_thickness]
		ptsr[step] = [x_coord, y_coord, hangar_height]
		ptsf[step] = [x_coord, y_coord, 0]
	end

	for step in 0..359
		face = group_ents.add_face pts[step.modulo(360)], pts[(step + 1).modulo(360)], pts[(step + 1).modulo(360) + 360], pts[step.modulo(360) + 360]
		status = face.material = wall_texture
	end
	for step in 0..door_degrees - 2
		face = group_ents.add_face pts[step], pts[step + 1], ptsx[step + 1], ptsx[step]
		status = face.material = wall_texture
	end
	face = group_ents.add_face pts[door_degrees - 1], pts[door_degrees], ptsx[door_degrees]
	status = face.material = wall_texture
	face = group_ents.add_face ptsx[door_degrees - 1], ptsx[door_degrees], pts[door_degrees - 1]
	status = face.material = wall_texture
	face = group_ents.add_face pts[0], pts[359], ptsx[0]
	status = face.material = wall_texture
	face = group_ents.add_face ptsx[0], ptsx[359], pts[359]
	status = face.material = wall_texture
	for step in 0..359
		face = group_ents.add_face ptsx[step.modulo(360)], ptsx[(step + 1).modulo(360)], ptsx[(step + 1).modulo(360) + 360], ptsx[step.modulo(360) + 360]
		status = face.material = wall_texture
	end
	face = group_ents.add_face ptsr
	status = face.material = roof_texture
	face = group_ents.add_face ptsc
	status = face.material = roof_texture
	face = group_ents.add_face ptsf
	status = face.reverse!
	status = face.material = floor_texture


	########################
	#### Central column ####
	########################
	
  beam_group = Sketchup.active_model.entities.add_group
  beam_group_ents = beam_group.entities
	
	if central_column_sides > 0
		if central_column_sides < 3
			central_column_sides = 3
		end
		central_column_degrees = 360 / central_column_sides
		pts = []
		ptsx = []
		for step in 0..central_column_sides - 1
			x_coord = Math.cos((step * central_column_degrees).degrees) * central_column_radius
			y_coord = Math.sin((step * central_column_degrees).degrees) * central_column_radius
			xi_coord = Math.cos((step * central_column_degrees).degrees) * central_column_radius * 2 
			yi_coord = Math.sin((step * central_column_degrees).degrees) * central_column_radius * 2
			pts[step] = [x_coord, y_coord, 0]
			pts[step + central_column_sides] = [x_coord, y_coord, hangar_height - roof_thickness * 3]
			pts[step + central_column_sides * 2] = [xi_coord, yi_coord, hangar_height - roof_thickness * 3]
			pts[step + central_column_sides * 3] = [xi_coord, yi_coord, hangar_height - roof_thickness * 2]
			ptsx[step] = [xi_coord, yi_coord, hangar_height - roof_thickness * 2]
		end
		for step in 0..central_column_sides - 1
			face = beam_group_ents.add_face pts[step.modulo(central_column_sides)], pts[(step + 1).modulo(central_column_sides)], pts[(step + 1).modulo(central_column_sides) + central_column_sides], pts[step.modulo(central_column_sides) + central_column_sides]
      status = face.material = beam_texture
			face = beam_group_ents.add_face pts[step.modulo(central_column_sides) + central_column_sides], pts[(step + 1).modulo(central_column_sides) + central_column_sides], pts[(step + 1).modulo(central_column_sides) + central_column_sides * 2], pts[step.modulo(central_column_sides) + central_column_sides * 2]
      status = face.material = beam_texture
			face = beam_group_ents.add_face pts[step.modulo(central_column_sides) + central_column_sides * 2], pts[(step + 1).modulo(central_column_sides) + central_column_sides * 2], pts[(step + 1).modulo(central_column_sides) + central_column_sides * 3], pts[step.modulo(central_column_sides) + central_column_sides * 3]
      status = face.material = beam_texture
		end
		face = beam_group_ents.add_face ptsx
    status = face.material = beam_texture
	end
	

	######################
	#### Side columns ####
	######################
 
	if side_columns > 0
		if side_columns < 3
			side_columns = 3
		end
	  side_column_degrees = 360.0 / (side_columns.to_f + 1)
  	side_column_degree = (((360 - door_degrees) / (side_columns - 1)).floor).to_i
		for step in 0..side_columns
      current_start_degree = door_degrees + 1 + step * side_column_degree
      current_end_degree = door_degrees + 2 + step * side_column_degree
      if current_start_degree < door_degrees * 1.5 + 1 or current_start_degree > 358 - door_degrees * 0.5
        current_start_radius = hangar_radius - wall_thickness - door_thickness - 2 / $inch
        current_end_radius = hangar_radius - wall_thickness - roof_thickness - door_thickness - 2 / $inch
      else
        current_start_radius = hangar_radius - wall_thickness
        current_end_radius = hangar_radius - wall_thickness - roof_thickness
      end
      xo1_coord = Math.cos((current_start_degree).degrees) * current_start_radius
      yo1_coord = Math.sin((current_start_degree).degrees) * current_start_radius
      xo2_coord = Math.cos((current_end_degree).degrees) * current_start_radius 
      yo2_coord = Math.sin((current_end_degree).degrees) * current_start_radius
      xi1_coord = Math.cos((current_start_degree).degrees) * current_end_radius
      yi1_coord = Math.sin((current_start_degree).degrees) * current_end_radius
      xi2_coord = Math.cos((current_end_degree).degrees) * current_end_radius 
      yi2_coord = Math.sin((current_end_degree).degrees) * current_end_radius
      xsc_coord = Math.cos((step * side_column_degrees).degrees) * (central_column_radius * 2 - roof_thickness * 2)
      ysc_coord = Math.sin((step * side_column_degrees).degrees) * (central_column_radius * 2 - roof_thickness * 2)
      xsci_coord = Math.cos(((step + 1) * side_column_degrees).degrees) * (central_column_radius * 2 - roof_thickness * 2)
      ysci_coord = Math.sin(((step + 1) * side_column_degrees).degrees) * (central_column_radius * 2 - roof_thickness * 2)
      pts = []
      pts[0] = [xo1_coord, yo1_coord, 0]
      pts[1] = [xo2_coord, yo2_coord, 0]
      pts[2] = [xi2_coord, yi2_coord, 0]
      pts[3] = [xi1_coord, yi1_coord, 0]
      pts[4] = [xo1_coord, yo1_coord, hangar_height - roof_thickness]
      pts[5] = [xo2_coord, yo2_coord, hangar_height - roof_thickness]
      pts[6] = [xi2_coord, yi2_coord, hangar_height - roof_thickness]
      pts[7] = [xi1_coord, yi1_coord, hangar_height - roof_thickness]
      pts[8] = [xi2_coord, yi2_coord, hangar_height - roof_thickness * 2]
      pts[9] = [xi1_coord, yi1_coord, hangar_height - roof_thickness * 2]
      pts[10] = [xsc_coord, ysc_coord, hangar_height - roof_thickness]
      pts[11] = [xsci_coord, ysci_coord, hangar_height - roof_thickness]
      pts[12] = [xsc_coord, ysc_coord, hangar_height - roof_thickness * 2]
      pts[13] = [xsci_coord, ysci_coord, hangar_height - roof_thickness * 2]
      if current_start_degree < door_degrees * 1.5 + 1 or current_start_degree > 358 - door_degrees * 0.5
        for step2 in 0..3
          face = beam_group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
          status = face.material = beam_texture
        end
      else
        for step2 in 1..3
          face = beam_group_ents.add_face pts[step2.modulo(4)], pts[(step2 + 1).modulo(4)], pts[(step2 + 1).modulo(4) + 4], pts[step2.modulo(4) + 4]
          status = face.material = beam_texture
        end
      end
      face = beam_group_ents.add_face pts[12], pts[13], pts[8], pts[9]
      status = face.material = beam_texture
      face = beam_group_ents.add_face pts[11], pts[13], pts[8], pts[6]
      status = face.material = beam_texture
      face = beam_group_ents.add_face pts[10], pts[12], pts[9], pts[7]
      status = face.material = beam_texture
       
      
 	  end
	end

  group_text = group.description
  group = group.parent.entities.add_group([group, beam_group])
  group.description = group_text


	####################
	#### Draw doors ####
	####################
	
	pts = []
	ptsx = []
	ptsl = []
	ptslx = []
	
	individual_door_degrees = ((door_degrees.to_f / 2).ceil + 1).to_i
	door_folding_degrees = ((door_degrees.to_f / 2).ceil + 1) - (((door_degrees.to_f / 2).ceil + 1) * (door_folding_level.to_f / 90).to_f)
	start_degree = door_folding_degrees + door_degrees.to_f / 2
	left_start_degree = -1 - door_folding_degrees
	for step in 0..individual_door_degrees - 1
		x_coord = Math.cos((step + start_degree).degrees) * (hangar_radius - wall_thickness - 1 / $inch)
		y_coord = Math.sin((step + start_degree).degrees) * (hangar_radius - wall_thickness - 1 / $inch)
		xi_coord = Math.cos((step + start_degree).degrees) * (hangar_radius - wall_thickness - door_thickness - 1 / $inch)
		yi_coord = Math.sin((step + start_degree).degrees) * (hangar_radius - wall_thickness - door_thickness - 1 / $inch)
		lx_coord = Math.cos((step + left_start_degree).degrees) * (hangar_radius - wall_thickness - 1 / $inch)
		ly_coord = Math.sin((step + left_start_degree).degrees) * (hangar_radius - wall_thickness - 1 / $inch)
		lxi_coord = Math.cos((step + left_start_degree).degrees) * (hangar_radius - wall_thickness - door_thickness - 1 / $inch)
		lyi_coord = Math.sin((step + left_start_degree).degrees) * (hangar_radius - wall_thickness - door_thickness - 1 / $inch)
		pts[step] = [x_coord, y_coord, 0]
		ptsx[step] = [xi_coord, yi_coord, 0]
		pts[step + individual_door_degrees] = [x_coord, y_coord, hangar_height - roof_thickness]
		ptsx[step + individual_door_degrees] = [xi_coord, yi_coord, hangar_height - roof_thickness]
		ptsl[step] = [lx_coord, ly_coord, 0]
		ptslx[step] = [lxi_coord, lyi_coord, 0]
		ptsl[step + individual_door_degrees] = [lx_coord, ly_coord, hangar_height - roof_thickness]
		ptslx[step + individual_door_degrees] = [lxi_coord, lyi_coord, hangar_height - roof_thickness]
	end

	for step in 0..individual_door_degrees - 2
		face = group_ents.add_face pts[step], pts[step + 1], pts[step + 1 + individual_door_degrees], pts[step + individual_door_degrees]
		status = face.material = door_texture
	end
	face = group_ents.add_face pts[0], pts[individual_door_degrees], ptsx[individual_door_degrees]
	status = face.material = door_texture
	face = group_ents.add_face ptsx[0], ptsx[individual_door_degrees], pts[0]
	status = face.material = door_texture
	face = group_ents.add_face pts[individual_door_degrees - 1], pts[individual_door_degrees * 2 - 1], ptsx[individual_door_degrees * 2 - 1]
	status = face.material = door_texture
	face = group_ents.add_face ptsx[individual_door_degrees - 1], ptsx[individual_door_degrees * 2 - 1], pts[individual_door_degrees - 1]
	status = face.material = door_texture
	for step in 0..individual_door_degrees - 2
		face = group_ents.add_face ptsx[step], ptsx[step + 1], ptsx[step + 1 + individual_door_degrees], ptsx[step + individual_door_degrees]
		status = face.material = door_texture
	end

	for step in 0..individual_door_degrees - 2
		face = group_ents.add_face ptsl[step], ptsl[step + 1], ptsl[step + 1 + individual_door_degrees], ptsl[step + individual_door_degrees]
		status = face.material = door_texture
	end
	face = group_ents.add_face ptsl[0], ptsl[individual_door_degrees], ptslx[individual_door_degrees]
	status = face.material = door_texture
	face = group_ents.add_face ptslx[0], ptslx[individual_door_degrees], ptsl[0]
	status = face.material = door_texture
	face = group_ents.add_face ptsl[individual_door_degrees - 1], ptsl[individual_door_degrees * 2 - 1], ptslx[individual_door_degrees * 2 - 1]
	status = face.material = door_texture
	face = group_ents.add_face ptslx[individual_door_degrees - 1], ptslx[individual_door_degrees * 2 - 1], ptsl[individual_door_degrees - 1]
	status = face.material = door_texture
	for step in 0..individual_door_degrees - 2
		face = group_ents.add_face ptslx[step], ptslx[step + 1], ptslx[step + 1 + individual_door_degrees], ptslx[step + individual_door_degrees]
		status = face.material = door_texture
	end

	
	##################
	#### Position ####
	##################

	trans = group.transformation
	point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
	t = Geom::Transformation.new point
	group.move! t
		
	if afcad_read == 0
		group.name = "Circular:" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":"
	end
	t = Geom::Transformation.rotation [hangar_x, hangar_y, 0], Z_AXIS, radians
	group.transform! t	

end


######################################################
######################################################
###### Below is the code for the Ramada shelter	######
######################################################
######################################################

def ramada_shelter(hangar_codes = "", midlat = 0.0, midlong = 0.0)
 

  
	#Define entity group and material
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "Ramada"
	materials = Sketchup.active_model.materials

	if hangar_codes != ""
	
		afcad_read = 1
	else
	
		prompts = ["Length?", "Depth?", "Height?", "Arc height?", "Roof thickness?", "Beam pairs?", "Beam thickness?", "Beam width?", "Beam support type?", "Beam support height?", "Beam support width/diameter?", "Angle?", "X-pos?", "Y-pos?"]
#		lists = ["", "", "", "", "", "", "", "", "Square", "", "", "", "", ""]
		lists = ["", "", "", "", "", "", "", "", "Square|Circular", "", "", "", "", ""]
		defaults = ["6000", "1400", "500", "50", "10", "5", "3", "15", "Square", "40", "40", "0", "0", "0"]
		input = UI.inputbox prompts, defaults, lists, "Hangar info"


	 	#Texture prompts
	 	texture_prompts = ["Roof texture?", "Beam texture?", "Base texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Blue metal", "Flat concrete wall", "Concrete"]
	 	texture_list = [$roof_list, $metal_list, $wall_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"
	
		#Get texture info
		wall_texture = texture_input[2]
		roof_texture = texture_input[0]
		floor_texture = texture_input[3]
		beam_texture = texture_input[1]
		
		hangar_length = input[0].to_f / $inch
		hangar_depth = input[1].to_f / $inch
		hangar_height = input[2].to_f / $inch
		hangar_arc = input[3].to_f / $inch
		roof_thickness = input[4].to_f / $inch
		beam_pairs = input[5].to_i
		beam_thickness = input[6].to_f / $inch
		beam_width = input[7].to_f / $inch
		beam_support_type = input[8]
		beam_support_height = input[9].to_f / $inch
		beam_support_width = input[10].to_f / $inch
		beam_distance =((hangar_length -(beam_support_width * beam_pairs)) /(beam_pairs - 1)) + beam_support_width
		hangar_angle = input[11].to_f
		hangar_x = input[12].to_f / $inch
		hangar_y = input[13].to_f / $inch
		radians = hangar_angle.degrees
		afcad_read = 0
		
	end

	#Add texture
		for step in 0..$texture_name.length
			if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step]
				texture = materials.add $texture_name[step]
        if $texture_filename[step].count(":") === 0
          texture.texture = $texture_files + $texture_filename[step]
        else
          texture.texture = $texture_filename[step]
        end
				texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
			end	
		end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured"
		texture = materials.add "Untextured"
		texture.color = "White"
	end


	#Draw beam
	
	for step in 0..beam_pairs - 1
		
			# Beam support
		for beam_side_step in 0..1
			if beam_support_type == "Square"
				pts = []
				pts[0] = [step * beam_distance,(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_support_width / 2), 0]
				pts[1] = [step * beam_distance,(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_support_width / 2), beam_support_height]
				pts[2] = [step * beam_distance,(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_support_width / 2), beam_support_height]
				pts[3] = [step * beam_distance,(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_support_width / 2), 0]
				face = group_ents.add_face pts
				status = face.material = wall_texture
				pts[4] = [step * beam_distance + beam_support_width,(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_support_width / 2), 0]
				pts[5] = [step * beam_distance + beam_support_width,(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_support_width / 2), beam_support_height]
				pts[6] = [step * beam_distance + beam_support_width,(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_support_width / 2), beam_support_height]
				pts[7] = [step * beam_distance + beam_support_width,(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_support_width / 2), 0]
				face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
				status = face.material = wall_texture
				face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
				status = face.material = wall_texture
				face = group_ents.add_face pts[6], pts[7], pts[0], pts[1]
				status = face.material = wall_texture
				face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
				status = face.material = wall_texture
			elsif beam_support_type == "Circular"
				centerpoint = Geom::Point3d.new(step * beam_distance + beam_support_width / 2,(hangar_depth *(1 +(beam_side_step * 2)) / 4), beam_support_height)
				 vector = Geom::Vector3d.new 0,0,-1
				 vector2 = vector.normalize!
				 model = Sketchup.active_model
				 circle = group_ents.add_circle centerpoint, vector2, beam_support_width / 2
				 face = group_ents.add_face circle
				 status = face.material = wall_texture
				 status = face.pushpull beam_support_height
			end
			#Vertical beams
			pts = []
			pts[0] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) - beam_thickness -(beam_width / 2), beam_support_height]
			pts[1] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) - beam_thickness -(beam_width / 2), beam_support_height]
			pts[2] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), beam_support_height]
			pts[3] = [step * beam_distance + (beam_support_width / 2) +(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), beam_support_height]
			pts[4] = [step * beam_distance + (beam_support_width / 2) +(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), beam_support_height]
			pts[5] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), beam_support_height]
			pts[6] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) + beam_thickness +(beam_width / 2), beam_support_height]
			pts[7] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) + beam_thickness +(beam_width / 2), beam_support_height]
			pts[8] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), beam_support_height]
			pts[9] = [step * beam_distance + (beam_support_width / 2) -(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), beam_support_height]
			pts[10] = [step * beam_distance + (beam_support_width / 2) -(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), beam_support_height]
			pts[11] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), beam_support_height]
		
			pts[12] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) - beam_thickness -(beam_width / 2), hangar_height]
			pts[13] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) - beam_thickness -(beam_width / 2), hangar_height]
			pts[14] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), hangar_height]
			pts[15] = [step * beam_distance + (beam_support_width / 2) +(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), hangar_height]
			pts[16] = [step * beam_distance + (beam_support_width / 2) +(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), hangar_height]
			pts[17] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), hangar_height]
			pts[18] = [step * beam_distance + (beam_support_width / 2) +(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) + beam_thickness +(beam_width / 2), hangar_height]
			pts[19] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) + beam_thickness +(beam_width / 2), hangar_height]
			pts[20] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), hangar_height]
			pts[21] = [step * beam_distance + (beam_support_width / 2) -(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) +(beam_width / 2), hangar_height]
			pts[22] = [step * beam_distance + (beam_support_width / 2) -(beam_thickness / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), hangar_height]
			pts[23] = [step * beam_distance + (beam_support_width / 2) -(beam_width / 2),(hangar_depth *(1 +(beam_side_step * 2)) / 4) -(beam_width / 2), hangar_height]
			for beam_step in 0..10
				face = group_ents.add_face pts[beam_step], pts[beam_step + 1], pts[beam_step + 13], pts[beam_step + 12]
				status = face.material = beam_texture
			end 
			face = group_ents.add_face pts[11], pts[0], pts[12], pts[23]
			status = face.material = beam_texture
				
			#Horizontal beams
			pts = []
			pts[0] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), 0, hangar_height]
			pts[1] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), 0, hangar_height]
			pts[2] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), 0, hangar_height + beam_thickness]
			pts[3] = [step * beam_distance +(beam_support_width / 2) +(beam_thickness / 2), 0, hangar_height + beam_thickness]
			pts[4] = [step * beam_distance +(beam_support_width / 2) +(beam_thickness / 2), 0, hangar_height + beam_width - beam_thickness]
			pts[5] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), 0, hangar_height + beam_width - beam_thickness]
			pts[6] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), 0, hangar_height + beam_width]
			pts[7] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), 0, hangar_height + beam_width]
			pts[8] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), 0, hangar_height + beam_width - beam_thickness]
			pts[9] = [step * beam_distance +(beam_support_width / 2) -(beam_thickness / 2), 0, hangar_height + beam_width - beam_thickness]
			pts[10] = [step * beam_distance +(beam_support_width / 2) -(beam_thickness / 2), 0, hangar_height + beam_thickness]
			pts[11] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), 0, hangar_height + beam_thickness]
			face = group_ents.add_face pts
			status = face.material = beam_texture
			pts[12] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), hangar_depth, hangar_height]
			pts[13] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), hangar_depth, hangar_height]
			pts[14] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), hangar_depth, hangar_height + beam_thickness]
			pts[15] = [step * beam_distance +(beam_support_width / 2) +(beam_thickness / 2), hangar_depth, hangar_height + beam_thickness]
			pts[16] = [step * beam_distance +(beam_support_width / 2) +(beam_thickness / 2), hangar_depth, hangar_height + beam_width - beam_thickness]
			pts[17] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), hangar_depth, hangar_height + beam_width - beam_thickness]
			pts[18] = [step * beam_distance +(beam_support_width / 2) +(beam_width / 2), hangar_depth, hangar_height + beam_width]
			pts[19] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), hangar_depth, hangar_height + beam_width]
			pts[20] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), hangar_depth, hangar_height + beam_width - beam_thickness]
			pts[21] = [step * beam_distance +(beam_support_width / 2) -(beam_thickness / 2), hangar_depth, hangar_height + beam_width - beam_thickness]
			pts[22] = [step * beam_distance +(beam_support_width / 2) -(beam_thickness / 2), hangar_depth, hangar_height + beam_thickness]
			pts[23] = [step * beam_distance +(beam_support_width / 2) -(beam_width / 2), hangar_depth, hangar_height + beam_thickness]
	
			for beam_step in 0..10
				face = group_ents.add_face pts[beam_step], pts[beam_step + 1], pts[beam_step + 13], pts[beam_step + 12]
				status = face.material = beam_texture
			end 
			face = group_ents.add_face pts[11], pts[0], pts[12], pts[23]
			status = face.material = beam_texture
			face = group_ents.add_face pts[12], pts[13], pts[14], pts[15], pts[16], pts[17], pts[18], pts[19], pts[20], pts[21], pts[22], pts[23]
			status = face.material = beam_texture
		end
		
			
		#Roof
		pts = []
		pts[0] = [0, 0, hangar_height + beam_width]
		pts[1] = [0, hangar_depth, hangar_height + beam_width]
		pts[2] = [hangar_length, hangar_depth, hangar_height + beam_width]
		pts[3] = [hangar_length, 0, hangar_height + beam_width]
		face = group_ents.add_face pts
		status = face.material = roof_texture
		pts[4] = [hangar_length, 0, hangar_height + beam_width + roof_thickness]
		pts[5] = [hangar_length, hangar_depth, hangar_height + beam_width + roof_thickness]
		pts[6] = [0, hangar_depth, hangar_height + beam_width + roof_thickness]
		pts[7] = [0, 0, hangar_height + beam_width + roof_thickness]
		face = group_ents.add_face pts[0], pts[1], pts[6], pts[7]
		status = face.material = roof_texture
		face = group_ents.add_face pts[1], pts[2], pts[5], pts[6]
		status = face.material = roof_texture
		face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = roof_texture
		face = group_ents.add_face pts[3], pts[0], pts[7], pts[4]
		status = face.material = roof_texture
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = roof_texture
		
		
	end
	
	point = Geom::Point3d.new(hangar_x, hangar_y, 0)
	t = Geom::Transformation.new point
	group.move! t

	t = Geom::Transformation.rotation [hangar_x,hangar_y,0], Z_AXIS, radians
	group.transform! t	
	if afcad_read == 0
		group.name = "Ramada():" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":"
	end
end			


##########################################################
##########################################################
######																              ######
###### Below is the code for the Detailed T-hangar	######
######																              ######
##########################################################
##########################################################

def detailed_t_hangar(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  

	#Define entity group and materials
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "T-hangar"
	materials = Sketchup.active_model.materials


	if hangar_codes != ""
		afcad_read = 1

		hangar_info = hangar_codes.split(":")

		wall_texture = hangar_info[20]
		roof_texture = hangar_info[21]
		floor_texture = hangar_info[25]
		door_texture = hangar_info[23]
		door_frame_texture = hangar_info[24]
		beam_texture = hangar_info[22]
		internal_wall_texture = wall_texture
		cross_beam_texture = ""

		wing_section_width = hangar_info[1].to_f / $inch
		wing_section_depth = hangar_info[2].to_f / $inch
		wing_section_height = hangar_info[3].to_f / $inch
		tail_section_width = hangar_info[4].to_f / $inch
		tail_section_depth = hangar_info[5].to_f / $inch
		tail_section_height = hangar_info[6].to_f / $inch
		door_height = hangar_info[7].to_f / $inch
		door_type = hangar_info[8]
		door_folding_level = hangar_info[9].to_i
		door_frame_thickness = hangar_info[10].to_f / $inch
		door_skin_thickness = hangar_info[11].to_f / $inch
		wall_thickness = hangar_info[12].to_f / $inch
		roof_arc = hangar_info[13].to_f / $inch
		roof_slope_over_wings = hangar_info[14]
		back_wall_slant = hangar_info[15].to_f / $inch
		
		extension_text = hangar_info[19]
		
		if wing_section_height > tail_section_height	
			tail_section_height = wing_section_height
		end
		if door_height > tail_section_height
			door_height = tail_section_height
		end

		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[27].to_f
		long1 = midlong
		long2 = hangar_info[28].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)

		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		
		hangar_angle = 360 - hangar_info[29].to_f
		hangar_x = x_distance / $inch
		hangar_y = y_distance / $inch
		radians = hangar_angle.degrees
		
		full_depth = wing_section_depth + tail_section_depth + wall_thickness
		full_width = wing_section_width * 2 + tail_section_width + wall_thickness * 2

	else
	
		angle = find_endpoints
		if angle[4] == 0
			angle[4] =((400 * 3) / $inch) +(20 / $inch / 3)
		end
		if angle[3] == 0
			angle[3] =((600 * 2) / $inch) +(10 / $inch / 2)
		end
		
		prompts = ["Wing section width?", "Wing section depth?", "Wing section height?", "Tail section width?", "Tail section depth?", "Tail section height", "Door height?", "Door type?", "Door folding level?", "Door frame thickness?", "Door skin thickness?", "Wall thickness?", "Roof arc?", "Roof slopes over wings?", "Back wall slant?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		lists = ["", "", "", "", "", "", "", "Sliding doors|Bi-folding doors|Encased sliding doors|Hydraulic swing door", "", "", "", "", "", $yn, "", "", "", "", ""]
		defaults = [(((angle[4] / 3) * $inch) -(20 / 3)).ceil,(((angle[3] / 2) * $inch) -(10 / 2)).ceil, "260",(((angle[4] / 3) * $inch) -(20 / 3)).ceil,(((angle[3] / 2) * $inch) -(10 / 2)).ceil, "400", "370", "Hydraulic swing door", "5", "5", "3", "2", "0", "No", "0", angle[0], angle[1] * $inch, angle[2] * $inch, ""]
		input = UI.inputbox prompts, defaults, lists, "Hangar info"

	 	#Texture prompts
	 	texture_prompts = ["Wall texture?", "Roof texture?", "Beam texture?", "Door texture?", "Door frame texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Vertical square iron", "Blue metal", "Vertical square iron", "Silver metal", "Concrete"]
	 	texture_list = [$wall_list, $roof_list, $metal_list, $door_list, $metal_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"
	
		#Get texture info
		wall_texture = texture_input[0]
		roof_texture = texture_input[1]
		floor_texture = texture_input[5]
		door_texture = texture_input[3]
		door_frame_texture = texture_input[4]
		beam_texture = texture_input[2]
		internal_wall_texture = wall_texture
		cross_beam_texture = ""

		wing_section_width = input[0].to_f / $inch
		wing_section_depth = input[1].to_f / $inch
		wing_section_height = input[2].to_f / $inch
		tail_section_width = input[3].to_f / $inch
		tail_section_depth = input[4].to_f / $inch
		tail_section_height = input[5].to_f / $inch
		door_height = input[6].to_f / $inch
		door_type = input[7]
		door_folding_level = input[8].to_i
		door_frame_thickness = input[9].to_f / $inch
		door_skin_thickness = input[10].to_f / $inch
		wall_thickness = input[11].to_f / $inch
		roof_arc = input[12].to_f / $inch
		roof_slope_over_wings = input[13]
		back_wall_slant = input[14].to_f / $inch
		
		hangar_angle = input[15].to_f
		hangar_x = input[16].to_f / $inch
		hangar_y = input[17].to_f / $inch
		radians = hangar_angle.degrees
		extension_text = input[18]
		
		if wing_section_height > tail_section_height	
			tail_section_height = wing_section_height
		end
		if door_height > tail_section_height
			door_height = tail_section_height
		end
		

	end

 	
	door_sections = 1



#############################
####	Get extension info ####
#############################

	extensions = []
	extension_types = []
	sign_count = 0
	sign_x_pos = []			# Position on wall from front or left
	sign_y_pos = []			# Position on wall from ground
	sign_x_size = []		# Sign width
	sign_y_size = []		# Sign height
	sign_thickness = []		# Sign thickness
	sign_double_sided = []		# 0 for logo on front only(only valid for textured signs), 1 for logo on both sides
	sign_wall = []			# Wall to put sign on, F, B, L, R
	sign_texture = []		# Sign texture name
	sign_type = []			# Sign type; L(logo), T(text)
	sign_filled = []		# 0 to not adjust texture size, 1 to adjust texture size.
	entrance_door_count = 0
	entrance_door_wall = []
	entrance_door_x_pos = []
	entrance_door_y_pos = []
	entrance_door_x_size = []
	entrance_door_y_size = []
	entrance_door_texture = []
	entrance_door_glass_texture = []
	entrance_door_glass_back_texture = []
	entrance_door_type = []
	window_count = 0
	window_x_pos = []
	window_y_pos = []
	window_x_size = []
	window_y_size = []
	window_texture = []
	window_back_texture = []
	window_transparency = []
	window_wall = []
	window_repeat = []
	window_gap = []
	uv_map_count = 0
	uv_map_x_pos = []
	uv_map_y_pos = []
	uv_map_x_size = []
	uv_map_y_size = []
	uv_map_texture = []
	uv_map_wall = []
	uv_map_rotation = []
	roof_window_count = 0
	roof_window_x_pos = []
	roof_window_y_pos = []
	roof_window_x_size = []
	roof_window_y_size = []
	roof_window_texture = []
	roof_window_back_texture = []
	roof_window_transparency = []
	roof_window_side = []
	roof_window_repeat = []
	roof_window_gap = []

	if extension_text.length > 0
		if extension_text.count('|') > 0
			extensions = extension_text.split("|")
		else
			extensions[0] = extension_text
		end
		door_set = 0
		for step in 0..extensions.length - 1
			door_found = 0
			if extensions[step].count('(') > 0 and extensions[step].count(')') > 0 
				extension_types[step] = extensions[step][0..extensions[step].index("(") - 1]
			else
				extension_types[step] = "ERR"
			end
	
			# Signage
			if extension_types[step] == "S"
				sign_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if sign_string.count(";") >= 3
					sign_failed = 0
					sign_details = sign_string.split(";")
					sign_type[sign_count] = sign_details[0]
					if sign_details[1].count("x") == 1
						sign_temp_string = sign_details[1].split("x")
						sign_x_pos[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_pos[sign_count] = sign_temp_string[1].to_f / $inch
					else
						sign_failed = 1
					end
					if sign_details[2].count("x") == 2
						sign_temp_string = sign_details[2].split("x")
						sign_x_size[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_size[sign_count] = sign_temp_string[1].to_f / $inch
						sign_thickness[sign_count] = sign_temp_string[2].to_f / $inch
					else
						sign_failed = 1
					end
					sign_wall[sign_count] = sign_details[3]
					if sign_type[sign_count] == "L" and sign_string.count(";") == 6
						sign_texture[sign_count] = sign_details[4]
						sign_filled[sign_count] = sign_details[5].to_i
						sign_double_sided[sign_count] = sign_details[6].to_i
					
					else
						sign_failed = 1
					end
					
					if sign_failed == 0 and(sign_wall[sign_count] == "F" or sign_wall[sign_count] == "B")  
						sign_count = sign_count + 1
					end
				end
			end

			# Entrance doors
			if extension_types[step] == "ED"
				entrance_door_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if entrance_door_string.count(";") == 4 or entrance_door_string.count(";") == 5
					entrance_door_failed = 0
					entrance_door_details = entrance_door_string.split(";")
					entrance_door_wall[entrance_door_count] = entrance_door_details[2]
					if entrance_door_details[0].count("x") == 1
						entrance_door_temp_string = entrance_door_details[0].split("x")
						entrance_door_x_pos[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_pos[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
						if entrance_door_y_pos[entrance_door_count] < 1 / $inch
							entrance_door_y_pos[entrance_door_count] = 1 / $inch
						end
					else
						entrance_door_failed = 1
					end
					if entrance_door_details[1].count("x") == 1
						entrance_door_temp_string = entrance_door_details[1].split("x")
						entrance_door_x_size[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_size[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
					else
						entrance_door_failed = 1
					end
					entrance_door_texture[entrance_door_count] = entrance_door_details[4]
					entrance_door_type[entrance_door_count] = entrance_door_details[3]
					
					if entrance_door_string.count(";") == 5

						entrance_door_glass_texture[entrance_door_count] = entrance_door_details[5]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_details[5]
					
					else
						entrance_door_glass_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
					end
					
					if entrance_door_failed == 0 and(entrance_door_wall[entrance_door_count] == "B" or entrance_door_wall[entrance_door_count] == "L" or entrance_door_wall[entrance_door_count] == "R") 
					#	cross_beam_exclusion_side[cross_beam_exclusion_count] = entrance_door_wall[entrance_door_count]
					#	cross_beam_exclusions[cross_beam_exclusion_count] = [entrance_door_x_pos[entrance_door_count], entrance_door_y_pos[entrance_door_count], entrance_door_x_pos[entrance_door_count] + entrance_door_x_size[entrance_door_count], entrance_door_y_pos[entrance_door_count] + entrance_door_y_size[entrance_door_count]]
					#	cross_beam_extras_drawn[cross_beam_exclusion_count] = 0
					#	cross_beam_exclusion_count = cross_beam_exclusion_count + 1
						entrance_door_count = entrance_door_count + 1
					end
				end
			end

			# Windows
			if extension_types[step] == "W"
				window_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if window_string.count(";") >= 4
					window_failed = 0
					window_items = window_string.split(";")
					if window_items[0].count("x") == 1
						window_details = window_items[0].split("x")
						window_x_pos[window_count] = window_details[0].to_f / $inch
						window_y_pos[window_count] = window_details[1].to_f / $inch
						if window_y_pos[window_count] >= arc_height
							window_failed = 1
						end
					else
						window_failed = 1
					end
					if window_items[1].count("x") == 1
						window_details = window_items[1].split("x")
						window_x_size[window_count] = window_details[0].to_f / $inch
						window_y_size[window_count] = window_details[1].to_f / $inch
						if window_y_size[window_count] + window_y_pos[window_count] > arc_height and window_failed == 0
							window_y_size[window_count] = arc_height - window_y_pos[window_count]
						end
					else
						window_failed = 1
					end
					if window_items[3].count("/") > 0
						window_details = window_items[3].split("/")
						window_texture[window_count] = window_details[0]
						window_back_texture[window_count] = window_details[1]
					else
						window_texture[window_count] = window_items[3]
						window_back_texture[window_count] = window_items[3]
					end	
					window_transparency[window_count] = window_items[4].to_i
					if window_transparency[window_count] < 0
						window_transparency[window_count] = 0
					elsif window_transparency[window_count] > 100
						window_transparency[window_count] = 100
					end
					
					window_wall[window_count] = window_items[2]
					if window_wall[window_count] != "F" and window_wall[window_count] != "B"
						window_failed = 1
					end
					if window_string.count(";") == 6
						window_repeat[window_count] = window_items[5].to_i
						window_gap[window_count] = window_items[6].to_f / $inch
					else
						window_repeat[window_count] = 1
						window_gap[window_count] = 0
					end
					if window_failed == 0
						window_count = window_count + 1
					end
				end
			end

			# UV Mapping
			if extension_types[step] == "UV"
				uv_map_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if uv_map_string.count(";") == 6
					uv_map_info = uv_map_string.split(";")
					uv_map_x_pos[uv_map_count] = uv_map_info[0].to_f / $inch
					uv_map_y_pos[uv_map_count] = uv_map_info[1].to_f / $inch
					uv_map_x_size[uv_map_count] = uv_map_info[2].to_f / $inch
					uv_map_y_size[uv_map_count] = uv_map_info[3].to_f / $inch
					uv_map_texture[uv_map_count] = uv_map_info[4]
					uv_map_wall[uv_map_count] = uv_map_info[5]
					uv_map_rotation[uv_map_count] = uv_map_info[6].to_f
					uv_map_count = uv_map_count + 1
				end
			end

			# Roof windows
			if extension_types[step] == "RW"
				roof_window_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if roof_window_string.count(";") >= 4
					roof_window_failed = 0
					roof_window_items = roof_window_string.split(";")
					if roof_window_items[0].count("x") == 1
						roof_window_details = roof_window_items[0].split("x")
						roof_window_x_pos[roof_window_count] = roof_window_details[0].to_f / $inch
						roof_window_y_pos[roof_window_count] = roof_window_details[1].to_f / $inch
					#	if roof_window_y_pos[roof_window_count] >= full_width / 2 - wall_thickness
					#		roof_window_failed = 1
					#	end
					else
						roof_window_failed = 1
					end
					if roof_window_items[1].count("x") == 1
						roof_window_details = roof_window_items[1].split("x")
						roof_window_x_size[roof_window_count] = roof_window_details[0].to_f / $inch
						roof_window_y_size[roof_window_count] = roof_window_details[1].to_f / $inch
				#		if roof_window_y_size[roof_window_count] + roof_window_y_pos[roof_window_count] > full_width / 2 - wall_thickness and roof_window_failed == 0
				#			roof_window_y_size[roof_window_count] = full_width / 2 - roof_window_y_pos[roof_window_count] - wall_thickness
				#		end
					else
						roof_window_failed = 1
					end
					if roof_window_items[3].count("/") > 0
						roof_window_details = roof_window_items[3].split("/")
						roof_window_texture[roof_window_count] = roof_window_details[0]
						roof_window_back_texture[roof_window_count] = roof_window_details[1]
					else
						roof_window_texture[roof_window_count] = roof_window_items[3]
						roof_window_back_texture[roof_window_count] = roof_window_items[3]
					end	
					roof_window_transparency[roof_window_count] = roof_window_items[4].to_i
					if roof_window_transparency[roof_window_count] < 0
						roof_window_transparency[roof_window_count] = 0
					elsif roof_window_transparency[roof_window_count] > 100
						roof_window_transparency[roof_window_count] = 100
					end
					
					roof_window_side[roof_window_count] = roof_window_items[2]
					if roof_window_side[roof_window_count] != "C" and roof_window_side[roof_window_count] != "W"
						roof_window_failed = 1
					end
					if roof_window_string.count(";") == 6
						roof_window_repeat[roof_window_count] = roof_window_items[5].to_i
						roof_window_gap[roof_window_count] = roof_window_items[6].to_f / $inch
					else
						roof_window_repeat[roof_window_count] = 1
						roof_window_gap[roof_window_count] = 0
					end
					if roof_window_failed == 0
						roof_window_count = roof_window_count + 1
					end
				end
			end
		end
	end


	
	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or door_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step] or door_frame_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or door_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured" or door_frame_texture == "Untextured" or internal_wall_texture == "Untextured" or cross_beam_texture == "Untextured"
			texture = materials.add "Untextured"
			texture.color = "White"
	end


	# draw beams
	beam_thickness = 2 / $inch
	beam_width = beam_thickness * 2

	back_internal_slant_gradient = back_wall_slant / tail_section_height
	internal_arc_slant = roof_arc * back_internal_slant_gradient

	external_arc_slant = internal_arc_slant
	roof_slope = roof_arc /((wall_thickness * 2 + tail_section_width) / 2)

	if roof_slope_over_wings == "No"
		tail_external_side_height = wing_section_height + wall_thickness
		wing_external_side_height = wing_section_height + wall_thickness
		tail_internal_side_height = wing_section_height
		wing_internal_side_height = wing_section_height
	else
		tail_external_side_height = tail_section_height + wall_thickness
		wing_external_side_height = tail_section_height + wall_thickness - wing_section_width * roof_slope
		tail_internal_side_height = tail_section_height
		wing_internal_side_height = tail_section_height - wing_section_width * roof_slope
		if door_height > wing_internal_side_height
			door_height = wing_internal_side_height
		end
	end
	
	wing_front_internal_position = wall_thickness
	wing_front_external_position = 0
	wing_left_internal_position = wall_thickness
	wing_left_external_position = 0
	wing_right_internal_position = wall_thickness +(wing_section_width * 2) + tail_section_width
	wing_right_external_position =(wall_thickness * 2) +(wing_section_width * 2) + tail_section_width
	
	wing_back_internal_position = wing_section_depth + wing_section_height * back_internal_slant_gradient
	wing_back_external_position = wing_section_depth + wall_thickness +(wing_section_height + wall_thickness) * back_internal_slant_gradient
	wing_back_internal_top_position = wing_section_depth
	wing_back_internal_top_end_position = wing_section_depth -(wing_internal_side_height - wing_section_height) * back_internal_slant_gradient
	wing_back_internal_top_inner_position = wing_section_depth -(tail_section_height - wing_section_height) * back_internal_slant_gradient
	wing_back_external_top_position = wing_section_depth + wall_thickness
	wing_back_external_top_end_position = wing_section_depth + wall_thickness -(wing_internal_side_height - wing_section_height) * back_internal_slant_gradient
	wing_back_external_top_inner_position = wing_section_depth + wall_thickness -(tail_section_height - wing_section_height) * back_internal_slant_gradient
	tail_back_internal_position = wing_section_depth + tail_section_depth + back_wall_slant
	tail_back_external_position = wing_section_depth + tail_section_depth + wall_thickness + tail_section_height * back_internal_slant_gradient
	tail_back_external_top_position = wing_section_depth + tail_section_depth + wall_thickness - wall_thickness * back_internal_slant_gradient
	tail_back_external_arc_top_position =(wing_section_depth + tail_section_depth + wall_thickness) -(roof_arc + wall_thickness) * back_internal_slant_gradient
	tail_back_internal_top_position = wing_section_depth + tail_section_depth 
	tail_left_external_position = wing_section_width
	tail_left_internal_position = wall_thickness + wing_section_width
	tail_right_external_position = wing_section_width + tail_section_width +(wall_thickness * 2)
	tail_right_internal_position = wing_section_width + tail_section_width + wall_thickness
	tail_external_height = tail_section_height + wall_thickness
	tail_external_arc_height = tail_section_height + wall_thickness + roof_arc


	wing_front_position = wall_thickness
	left_wing_wall_position = wall_thickness
	right_wing_wall_position = wall_thickness +(wing_section_width * 2) + tail_section_width
	wing_back_position = wing_section_depth
	left_tail_wall_position = wall_thickness + wing_section_width
	right_tail_wall_position = wall_thickness + wing_section_width + tail_section_width
	wing_beam_top_position = wing_section_height -(beam_width * 3)
	tail_beam_top_position = tail_section_height -(beam_width * 3)
	wing_beams =((wing_back_position - wall_thickness -(beam_width * 2)) /(150 / $inch)).to_i + 1
	wing_beam_distance =(wing_back_position - wing_front_position -(beam_width * 2)) /(wing_beams - 1)
	tail_beams =((tail_back_internal_top_position - wall_thickness -(beam_width * 2)) /(150 / $inch)).to_i + 1
	tail_beam_distance =(tail_back_internal_top_position - wing_front_position -(beam_width * 2)) /(tail_beams - 1)
	
	
	# create beam profiles
	beam_z_profile = []
	beam_z_profile[0] = [0, 0, 0]
	beam_z_profile[1] = [beam_thickness, 0, 0]
	beam_z_profile[2] = [beam_thickness,((beam_width - beam_thickness) / 2), 0]
	beam_z_profile[3] = [beam_thickness + beam_width,((beam_width - beam_thickness) / 2), 0]
	beam_z_profile[4] = [beam_thickness + beam_width, 0, 0]
	beam_z_profile[5] = [(beam_thickness * 2) + beam_width, 0, 0]
	beam_z_profile[6] = [(beam_thickness * 2) + beam_width, beam_width, 0]
	beam_z_profile[7] = [beam_thickness + beam_width, beam_width, 0]
	beam_z_profile[8] = [beam_thickness + beam_width, beam_width -((beam_width - beam_thickness) / 2), 0]
	beam_z_profile[9] = [beam_thickness, beam_width -((beam_width - beam_thickness) / 2), 0]
	beam_z_profile[10] = [beam_thickness, beam_width, 0]
	beam_z_profile[11] = [0, beam_width, 0]
	
	
	beam_info = []
	beam_info[0] = [left_wing_wall_position, wing_front_position, 0, wing_beam_top_position,"Z", 0]
	beam_info[1] = [left_wing_wall_position, wing_back_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[2] = [right_wing_wall_position -(beam_thickness * 2) - beam_width, wing_back_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[3] = [right_wing_wall_position -(beam_thickness * 2) - beam_width, wing_front_position, 0, wing_beam_top_position,"Z", 0]
	beam_info[4] = [left_tail_wall_position - beam_width, wing_back_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[5] = [right_tail_wall_position -(beam_thickness * 2), wing_back_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[6] = [left_tail_wall_position, tail_back_internal_top_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[7] = [right_tail_wall_position -(beam_thickness * 2) - beam_width, tail_back_internal_top_position - beam_width, 0, wing_beam_top_position,"Z", 0]
	beam_info[8] = [left_tail_wall_position, tail_back_internal_top_position - beam_width, wing_section_height - beam_width, tail_beam_top_position,"Z", 1]
	beam_info[9] = [right_tail_wall_position -(beam_thickness * 2) - beam_width, tail_back_internal_top_position - beam_width, wing_section_height - beam_width, tail_beam_top_position,"Z", 1]
	beam_info[10] = [left_tail_wall_position, wing_back_position, wing_section_height - beam_width, tail_beam_top_position,"Z-Turn", 3]
	beam_info[11] = [right_tail_wall_position - beam_width, wing_back_position, wing_section_height - beam_width, tail_beam_top_position,"Z-Turn", 3]
	beam_info[12] = [left_wing_wall_position, wing_front_position, wing_beam_top_position, wing_back_position,"Y-Turn", 0]
	beam_info[13] = [right_wing_wall_position - beam_width, wing_front_position, wing_beam_top_position, wing_back_position,"Y-Turn", 0]
	beam_info[14] = [left_tail_wall_position, wing_front_position, wing_beam_top_position, tail_back_internal_top_position,"Y-Turn", 0]
	beam_info[15] = [left_tail_wall_position, wing_front_position, tail_beam_top_position, tail_back_internal_top_position,"Y-Turn", 1]
	beam_info[16] = [right_tail_wall_position - beam_width, wing_front_position, wing_beam_top_position, tail_back_internal_top_position,"Y-Turn", 0]
	beam_info[17] = [right_tail_wall_position - beam_width, wing_front_position, tail_beam_top_position, tail_back_internal_top_position,"Y-Turn", 1]
	beam_info[18] = [left_tail_wall_position, wing_front_position, wing_section_height, tail_beam_top_position,"Z-Turn", 1]
	beam_info[19] = [right_tail_wall_position - beam_width, wing_front_position, wing_section_height, tail_beam_top_position,"Z-Turn", 1]
	beam_info[20] = [left_wing_wall_position + beam_width, wing_back_position - beam_width, wing_section_height -(beam_width * 3), left_tail_wall_position,"X", 0]
	beam_info[21] = [right_tail_wall_position, wing_back_position - beam_width, wing_section_height -(beam_width * 3), right_wing_wall_position - beam_width,"X", 0]
	beam_info[22] = [left_tail_wall_position + beam_width, tail_back_internal_top_position - beam_width, wing_section_height -(beam_width * 3), right_tail_wall_position - beam_width,"X", 0]
	beam_info[23] = [left_tail_wall_position + beam_width, tail_back_internal_top_position - beam_width, tail_section_height -(beam_width * 3), right_tail_wall_position - beam_width,"X", 1]

	for step in 0..wing_beams - 1
		beam_info[24 + step] = [left_wing_wall_position, wing_back_position -(beam_width * 2) -(step * wing_beam_distance), wing_section_height - beam_width, left_tail_wall_position + beam_width,"-X-Turn", 0]
	end
	for step in 0..wing_beams - 1
		beam_info[24 + wing_beams + step] = [right_tail_wall_position - beam_width, wing_back_position -(beam_width * 2) -(step * wing_beam_distance), wing_section_height - beam_width, right_wing_wall_position,"-X-Turn", 0]
	end
	for step in 0..tail_beams - 1
		beam_info[24 +(wing_beams * 2) + step] = [left_tail_wall_position, tail_back_internal_top_position -(beam_width * 2) -(step * tail_beam_distance), tail_section_height - beam_width, right_tail_wall_position,"-X-Turn", 0]
	end
	
	
	for beam_step in 0.. beam_info.length - 1

		if tail_section_height == wing_section_height and(beam_info[beam_step][5] == 1 or beam_info[beam_step][5] == 3)
		elsif tail_section_depth == 0 and(beam_info[beam_step][5] == 2 or beam_info[beam_step][5] == 3)
		else
	
			pts = []
			for step in 0..beam_z_profile.length - 1
		
				if beam_info[beam_step][4] == "Z"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][0]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][1]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][2]
				elsif beam_info[beam_step][4] == "Z-Turn"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][1]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][0]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][2]
				elsif beam_info[beam_step][4] == "Y"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][0]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][2]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][1]
				elsif beam_info[beam_step][4] == "Y-Turn"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][1]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][2]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][0]
				elsif beam_info[beam_step][4] == "X"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][2]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][1]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][0]
				elsif beam_info[beam_step][4] == "X-Turn" or beam_info[beam_step][4] == "-X-Turn"
					pts[step] = []
					pts[step][0] = beam_info[beam_step][0] + beam_z_profile[step][2]
					pts[step][1] = beam_info[beam_step][1] + beam_z_profile[step][0]
					pts[step][2] = beam_info[beam_step][2] + beam_z_profile[step][1]
				end
			end

			face = group_ents.add_face pts
			if beam_info[beam_step][2] == 0 or beam_info[beam_step][4] == "Z-Turn" or beam_info[beam_step][4] == "-X-Turn"
				status = face.reverse!
			end
			status = face.material = beam_texture
			if beam_info[beam_step][4] == "Y-Turn" or beam_info[beam_step][4] == "Y"
				status = face.pushpull beam_info[beam_step][3] - beam_info[beam_step][1]
			elsif beam_info[beam_step][4] == "X-Turn" or beam_info[beam_step][4] == "X" or beam_info[beam_step][4] == "-X-Turn"
				status = face.pushpull beam_info[beam_step][3] - beam_info[beam_step][0]
			else
				status = face.pushpull beam_info[beam_step][3] - beam_info[beam_step][2]
			end
			if beam_info[beam_step][4] == "-X-Turn" or beam_info[beam_step][4] == "X-Turn" or beam_info[beam_step][4] == "X"
				face = group_ents.add_face pts
				status = face.material = beam_texture
			end
		
		end
#		UI.messagebox beam_step
	end	

	
	# Add smaller frame elements
	# Back wall
	
	frame_item = []
	frame_item[0] = [left_tail_wall_position, right_tail_wall_position, tail_back_internal_top_position, tail_beam_top_position, wing_beam_top_position, 1]
	frame_item[1] = [left_tail_wall_position, right_tail_wall_position, tail_back_internal_top_position, wing_beam_top_position, 0, 1]
	frame_item[2] = [left_tail_wall_position, left_wing_wall_position, wing_back_position, wing_beam_top_position, 0, 1]
	frame_item[3] = [left_wing_wall_position, left_tail_wall_position, wing_back_position, wing_beam_top_position, 0, 1]
	frame_item[3] = [right_tail_wall_position, right_wing_wall_position, wing_back_position, wing_beam_top_position, 0, 1]
	frame_item[4] = [right_wing_wall_position, right_tail_wall_position, wing_back_position, wing_beam_top_position, 0, 1]
	
	if tail_section_height != wing_section_height
		start_step = 0
	else
		start_step = 1
	end
	
	for step in start_step..frame_item.length - 1
	
		pts = []
		pts[0] = [frame_item[step][0], frame_item[step][2], frame_item[step][3]]
		pts[1] = [frame_item[step][1], frame_item[step][2], frame_item[step][4]]
		pts[2] = [frame_item[step][1], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][4]]
		pts[3] = [frame_item[step][0], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][3]]
		face = group_ents.add_face pts
		if face.normal[2] > 0
			status = face.reverse!
		end
		status = face.material = beam_texture
		pts[4] = [frame_item[step][0], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][3] + beam_thickness]
		pts[5] = [frame_item[step][1], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][4] + beam_thickness]
		pts[6] = [frame_item[step][1], frame_item[step][2], frame_item[step][4] + beam_thickness]
		pts[7] = [frame_item[step][0], frame_item[step][2], frame_item[step][3] + beam_thickness]
	
		face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = beam_texture
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = beam_texture
	
		pts = []
		pts[0] = [frame_item[step][1], frame_item[step][2], frame_item[step][3]]
		pts[1] = [frame_item[step][0], frame_item[step][2], frame_item[step][4]]
		pts[2] = [frame_item[step][0], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][4]]
		pts[3] = [frame_item[step][1], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][3]]
		face = group_ents.add_face pts
		if face.normal[2] > 0
			status = face.reverse!
		end
		status = face.material = beam_texture
		pts[4] = [frame_item[step][1], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][3] + beam_thickness]
		pts[5] = [frame_item[step][0], frame_item[step][2] -(beam_thickness * frame_item[step][5]), frame_item[step][4] + beam_thickness]
		pts[6] = [frame_item[step][0], frame_item[step][2], frame_item[step][4] + beam_thickness]
		pts[7] = [frame_item[step][1], frame_item[step][2], frame_item[step][3] + beam_thickness]
	
		face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = beam_texture
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = beam_texture
	end

	# Add walls 
	# Internal walls

	if tail_section_height != wing_section_height
		pts = []
		pts[0] = [left_wing_wall_position, wing_front_position, wing_section_height -(beam_width * 3)]
		pts[1] = [left_wing_wall_position +(beam_width * 2), wing_front_position, wing_section_height -(beam_width * 3)]
		pts[2] = [left_wing_wall_position +(beam_width * 2), wing_front_position, wing_internal_side_height]
		pts[3] = [left_wing_wall_position, wing_front_position, wing_internal_side_height]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = wall_texture

		pts = []
		pts[0] = [right_wing_wall_position, wing_front_position, wing_section_height -(beam_width * 3)]
		pts[1] = [right_wing_wall_position -(beam_width * 2), wing_front_position, wing_section_height -(beam_width * 3)]
		pts[2] = [right_wing_wall_position -(beam_width * 2), wing_front_position, wing_internal_side_height]
		pts[3] = [right_wing_wall_position, wing_front_position, wing_internal_side_height]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = wall_texture
	end


	pts = []
	pts[0] = [wing_left_internal_position, wing_front_position + beam_width, 0]
	pts[1] = [wing_left_internal_position, wing_back_internal_position, 0]
	pts[2] = [wing_left_internal_position, wing_back_internal_top_end_position, wing_internal_side_height]
	pts[3] = [wing_left_internal_position, wing_front_position + beam_width, wing_internal_side_height]
	face = group_ents.add_face pts
	status = face.material = wall_texture

	pts[4] = [tail_left_internal_position, wing_back_internal_position, 0]
	pts[5] = [tail_left_internal_position, wing_back_internal_top_inner_position, tail_internal_side_height]
	pts[6] = [tail_left_internal_position, tail_back_internal_top_position, tail_section_height]
	pts[7] = [tail_left_internal_position, tail_back_internal_position, 0]
	pts[8] = [tail_left_internal_position, wing_front_internal_position, tail_section_height]
	pts[9] = [tail_left_internal_position, wing_front_internal_position, tail_internal_side_height]
	pts[10] = [tail_right_internal_position, tail_back_internal_position, 0]
	pts[11] = [tail_right_internal_position, tail_back_internal_top_position, tail_section_height]
	pts[12] = [tail_right_internal_position, wing_back_internal_position, 0]
	pts[13] = [tail_right_internal_position, wing_back_internal_top_inner_position, tail_internal_side_height]
	pts[14] = [tail_right_internal_position, wing_front_internal_position, tail_section_height]
	pts[15] = [tail_right_internal_position, wing_front_internal_position, tail_internal_side_height]

	face = group_ents.add_edges pts[1], pts[2], pts[5], pts[4]
	face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
	if face.normal[1] > 0
		status = face.reverse!
	end
	status = face.material = wall_texture
	
	if tail_section_height != wing_section_height and roof_slope_over_wings == "No"
		face = group_ents.add_face pts[4], pts[5], pts[9], pts[8], pts[6], pts[7]
		status = face.material = wall_texture
	else
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = wall_texture
	end
	face = group_ents.add_face pts[10], pts[11], pts[6], pts[7]
	if face.normal[1] > 0
		status = face.reverse!
	end
	status = face.material = wall_texture
	
	if back_wall_slant == 0
		ptsx = []
		ptsx[0] = [tail_right_internal_position, tail_back_internal_top_position, wing_section_height - beam_width]
		ptsx[1] = [tail_left_internal_position, tail_back_internal_top_position, wing_section_height - beam_width]
		ptsx[2] = [tail_left_internal_position, tail_back_internal_top_position, tail_section_height -(beam_width * 3)]
		ptsx[3] = [tail_right_internal_position, tail_back_internal_top_position, tail_section_height -(beam_width * 3)]
		face = group_ents.add_face ptsx
		status = face.reverse!
		status = face.material = wall_texture
	end

	if tail_section_height != wing_section_height and roof_slope_over_wings == "No" and tail_section_depth > 0
		face = group_ents.add_face pts[10], pts[12], pts[13], pts[15], pts[14], pts[11]
		status = face.material = wall_texture
	elsif tail_section_depth == 0
		face = group_ents.add_face pts[11], pts[14], pts[15], pts[13]
		status = face.reverse!
		status = face.material = wall_texture
	else
		face = group_ents.add_face pts[10], pts[12], pts[13], pts[11]
		status = face.material = wall_texture
	end

	
	pts = []
	pts[0] = [tail_right_internal_position, wing_back_internal_position, 0]
	pts[1] = [wing_right_internal_position, wing_back_internal_position, 0]
	pts[2] = [wing_right_internal_position, wing_back_internal_top_end_position, wing_internal_side_height]
	pts[3] = [tail_right_internal_position, wing_back_internal_top_inner_position, tail_internal_side_height]
	face = group_ents.add_face pts
	if face.normal[1] > 0
		status = face.reverse!
	end
	status = face.material = wall_texture

	pts = []
	pts[0] = [wing_right_internal_position, wing_front_internal_position, 0]
	pts[1] = [wing_right_internal_position, wing_back_internal_position, 0]
	pts[2] = [wing_right_internal_position, wing_back_internal_top_end_position, wing_internal_side_height]
	pts[3] = [wing_right_internal_position, wing_front_internal_position, wing_internal_side_height]
	face = group_ents.add_face pts
	if face.normal[2] > 0
		status = face.reverse!
	end
	status = face.material = wall_texture



	if tail_internal_side_height == wing_internal_side_height and wing_section_height != tail_section_height

		pts = []
		pts[0] = [right_tail_wall_position -(beam_width / 2), wing_front_internal_position, tail_section_height - beam_width]
		pts[1] = [left_tail_wall_position +(beam_width / 2), wing_front_internal_position, tail_section_height - beam_width]
		pts[2] = [left_tail_wall_position +(beam_width / 2), wing_front_internal_position, wing_section_height - beam_width]
		pts[3] = [left_tail_wall_position +(beam_width + door_frame_thickness), wing_front_internal_position, wing_section_height - beam_width]
		pts[4] = [left_tail_wall_position +(beam_width + door_frame_thickness), wing_front_internal_position, door_height]
		pts[5] = [right_tail_wall_position -(beam_width + door_frame_thickness), wing_front_internal_position, door_height]
		pts[6] = [right_tail_wall_position -(beam_width + door_frame_thickness), wing_front_internal_position, wing_section_height - beam_width]
		pts[7] = [right_tail_wall_position -(beam_width / 2), wing_front_internal_position, wing_section_height - beam_width]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = wall_texture
	elsif wing_section_height != tail_section_height
		pts = []
		pts[0] = [tail_left_internal_position, wing_front_internal_position, tail_internal_side_height]
		pts[1] = [tail_right_internal_position, wing_front_internal_position, tail_internal_side_height]
		pts[2] = [right_wing_wall_position, wing_front_internal_position, door_height]
		pts[3] = [wing_left_internal_position, wing_front_internal_position, door_height]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = wall_texture
	end
	if wing_internal_side_height > door_height
		pts = []
		pts[0] = [left_wing_wall_position, wing_front_position, wing_section_height]
		pts[1] = [left_wing_wall_position, wing_front_position, door_height]
		pts[2] = [right_wing_wall_position, wing_front_position, door_height]
		pts[3] = [right_wing_wall_position, wing_front_position, wing_section_height]
		face = group_ents.add_face pts
		if face.normal[1] < 0
			status = face.reverse!
		end
		status = face.material = wall_texture
	end


	# add skirting

	if wing_external_side_height != tail_external_side_height or wing_internal_side_height > door_height
		wing_skirt_height = door_height
	else
		wing_skirt_height = wing_internal_side_height - beam_width
	end
	pts = []
	pts[0] = [left_wing_wall_position +(beam_width * 2), 0, 0]
	pts[1] = [left_wing_wall_position +(beam_width * 2), wing_front_position, 0]
	pts[2] = [left_wing_wall_position +(beam_width * 2), wing_front_position, wing_skirt_height]
	pts[3] = [left_wing_wall_position +(beam_width * 2), 0, wing_skirt_height]
	face = group_ents.add_face pts
	status = face.material = wall_texture
	pts[4] = [left_tail_wall_position +(beam_width + door_frame_thickness), 0, wing_skirt_height]
	pts[5] = [left_tail_wall_position +(beam_width + door_frame_thickness), wing_front_position, wing_skirt_height]
	pts[6] = [left_tail_wall_position +(beam_width + door_frame_thickness), wing_front_position, door_height]
	pts[7] = [left_tail_wall_position +(beam_width + door_frame_thickness), 0, door_height]
	pts[8] = [right_tail_wall_position -(beam_width + door_frame_thickness), 0, door_height]
	pts[9] = [right_tail_wall_position -(beam_width + door_frame_thickness), wing_front_position, door_height]
	pts[10] = [right_tail_wall_position -(beam_width + door_frame_thickness), wing_front_position, wing_skirt_height]
	pts[11] = [right_tail_wall_position -(beam_width + door_frame_thickness), 0, wing_skirt_height]
	pts[12] = [right_wing_wall_position -(beam_width * 2), 0, wing_skirt_height]
	pts[13] = [right_wing_wall_position -(beam_width * 2), wing_front_position, wing_skirt_height]
	pts[14] = [right_wing_wall_position -(beam_width * 2), wing_front_position, 0]
	pts[15] = [right_wing_wall_position -(beam_width * 2), 0, 0]
	face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
	status = face.material = wall_texture

	if wing_section_height != tail_section_height and wing_internal_side_height < door_height
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = wall_texture
	end
	face = group_ents.add_face pts[6], pts[7], pts[8], pts[9]
	status = face.material = wall_texture
	if wing_section_height != tail_section_height and wing_internal_side_height < door_height
		face = group_ents.add_face pts[8], pts[9], pts[10], pts[11]
		status = face.material = wall_texture
	end
	face = group_ents.add_face pts[10], pts[11], pts[12], pts[13]
	status = face.material = wall_texture
	face = group_ents.add_face pts[12], pts[13], pts[14], pts[15]
	status = face.material = wall_texture

	# add ceiling
	pts = []
	pts[0] = [left_wing_wall_position, wing_back_internal_top_end_position, wing_internal_side_height]
	pts[1] = [left_wing_wall_position, wing_front_position, wing_internal_side_height]
	pts[2] = [left_tail_wall_position, wing_front_position, tail_internal_side_height]
	pts[3] = [left_tail_wall_position, wing_back_internal_top_inner_position, tail_internal_side_height]
	face = group_ents.add_face pts
	if face.normal[2] > 0
		status = face.reverse!
	end
	status = face.material = roof_texture

	pts = []
	pts[0] = [wing_right_internal_position, wing_back_internal_top_end_position, wing_internal_side_height]
	pts[1] = [wing_right_internal_position, wing_front_position, wing_internal_side_height]
	pts[2] = [tail_right_internal_position, wing_front_position, tail_internal_side_height]
	pts[3] = [tail_right_internal_position, wing_back_internal_top_inner_position, tail_internal_side_height]
	face = group_ents.add_face pts
	if face.normal[2] > 0
		status = face.reverse!
	end
	status = face.material = roof_texture

	if roof_arc == 0
		pts = []
		pts[0] = [tail_left_internal_position, tail_back_internal_top_position -(beam_width * 2), tail_section_height]
		pts[1] = [tail_left_internal_position, wing_front_position +(beam_width * 2), tail_section_height]
		pts[2] = [tail_right_internal_position, wing_front_position +(beam_width * 2), tail_section_height]
		pts[3] = [tail_right_internal_position, tail_back_internal_top_position -(beam_width * 2), tail_section_height]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = roof_texture
	else
		pts = []
		pts[0] = [left_tail_wall_position, tail_back_internal_top_position, tail_section_height]
		pts[1] = [left_tail_wall_position, wing_front_position, tail_section_height]
		pts[2] = [right_tail_wall_position, wing_front_position, tail_section_height]
		pts[3] = [right_tail_wall_position, tail_back_internal_top_position, tail_section_height]
		pts[4] = [left_tail_wall_position + tail_section_width / 2, tail_back_internal_top_position - internal_arc_slant, tail_section_height + roof_arc]
		pts[5] = [left_tail_wall_position + tail_section_width / 2, wing_front_position, tail_section_height + roof_arc]
		face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
		if face.normal[2] > 0
			status = face.reverse!
		end
		status = face.material = roof_texture
		face = group_ents.add_face pts[3], pts[2], pts[5], pts[4]
		status = face.material = roof_texture
		face = group_ents.add_face pts[0], pts[3], pts[4]
		if face.normal[1] > 0
			status = face.reverse!
		end
		status = face.material = wall_texture
		face = group_ents.add_face pts[1], pts[2], pts[5]
		status = face.reverse!
		status = face.material = wall_texture

	end
	# add external walls
	pts = []
	pts[0] = [wing_left_external_position, 0, wing_external_side_height]
	pts[1] = [wing_left_external_position, 0, 0]
	pts[2] = [wing_left_external_position, wing_back_external_position, 0]
	pts[3] = [wing_left_external_position, wing_back_external_top_end_position, wing_external_side_height]
	face = group_ents.add_face pts
	status = face.reverse!
	status = face.material = wall_texture
	pts[4] = [tail_left_external_position, wing_back_external_top_inner_position, tail_external_side_height]
	pts[5] = [tail_left_external_position, wing_back_external_position, 0]
	pts[6] = [tail_left_external_position, tail_back_external_position, 0]
	pts[7] = [tail_left_external_position, tail_back_external_top_position, tail_external_height]
	pts[8] = [tail_left_external_position, 0, tail_external_height]
	pts[9] = [tail_left_external_position, 0, tail_external_side_height]
	pts[10] = [tail_right_external_position, tail_back_external_top_position, tail_external_height]
	pts[11] = [tail_right_external_position, tail_back_external_position, 0]
	pts[12] = [tail_right_external_position, wing_back_external_position, 0]
	pts[13] = [tail_right_external_position, wing_back_external_top_inner_position, tail_external_side_height]
	pts[14] = [tail_right_external_position, 0, tail_external_side_height]
	pts[15] = [tail_right_external_position, 0, tail_external_height]
	pts[16] = [(wing_section_width * 2) + tail_section_width +(wall_thickness * 2), wing_back_external_top_end_position, wing_external_side_height]
	pts[17] = [(wing_section_width * 2) + tail_section_width +(wall_thickness * 2), wing_back_external_position, 0]
	pts[18] = [(wing_section_width * 2) + tail_section_width +(wall_thickness * 2), 0, 0]
	pts[19] = [(wing_section_width * 2) + tail_section_width +(wall_thickness * 2), 0, wing_external_side_height]

	pts[27] = [left_wing_wall_position +(beam_width * 2), 0, 0]
	pts[26] = [left_wing_wall_position +(beam_width * 2), 0, wing_skirt_height]
	pts[25] = [left_tail_wall_position +(beam_width + door_frame_thickness), 0, wing_skirt_height]
	pts[24] = [left_tail_wall_position +(beam_width + door_frame_thickness), 0, door_height]
	pts[23] = [right_tail_wall_position -(beam_width + door_frame_thickness), 0, door_height]
	pts[22] = [right_tail_wall_position -(beam_width + door_frame_thickness), 0, wing_skirt_height]
	pts[21] = [right_wing_wall_position -(beam_width * 2), 0, wing_skirt_height]
	pts[20] = [right_wing_wall_position -(beam_width * 2), 0, 0]


	face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
	status = face.material = wall_texture
	face = group_ents.add_face pts[0], pts[3], pts[4], pts[9]
	status = face.material = roof_texture
	if wing_section_height != tail_section_height and roof_slope_over_wings == "No" and tail_section_depth > 0
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7], pts[8], pts[9]
	elsif tail_section_depth == 0
		face = group_ents.add_face pts[4], pts[7], pts[8], pts[9]
	else
		face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
	end
	status = face.material = wall_texture
	face = group_ents.add_face pts[6], pts[7], pts[10], pts[11]
	status = face.material = wall_texture
	if wing_section_height != tail_section_height and roof_slope_over_wings == "No" and tail_section_depth > 0
		face = group_ents.add_face pts[10], pts[11], pts[12], pts[13], pts[14], pts[15]
	elsif tail_section_depth == 0
		face = group_ents.add_face pts[10], pts[13], pts[14], pts[15]
	else
		face = group_ents.add_face pts[10], pts[11], pts[12], pts[13]
	end
	status = face.material = wall_texture
	
	#Draw roof
	if roof_arc == 0
		face = group_ents.add_face pts[7], pts[8], pts[15], pts[10]
		status = face.material = roof_texture
	else
		pts[28] = [left_tail_wall_position + tail_section_width / 2, tail_back_external_arc_top_position, tail_external_arc_height]
		pts[29] = [left_tail_wall_position + tail_section_width / 2, 0, tail_external_arc_height]
		face = group_ents.add_face pts[7], pts[8], pts[29], pts[28]
		status = face.material = roof_texture
		face = group_ents.add_face pts[10], pts[15], pts[29], pts[28]
		status = face.material = roof_texture
		face = group_ents.add_face pts[8], pts[15], pts[29]
		status = face.material = wall_texture
		face = group_ents.add_face pts[10], pts[7], pts[28]
		status = face.material = wall_texture
	
	end
	face = group_ents.add_face pts[12], pts[13], pts[16], pts[17]
	status = face.material = wall_texture
	face = group_ents.add_face pts[16], pts[17], pts[18], pts[19]
	status = face.material = wall_texture
	face = group_ents.add_face pts[14], pts[13], pts[16], pts[19]
	status = face.material = roof_texture
	if wing_section_height != tail_section_height and wing_internal_side_height < door_height
		face = group_ents.add_face pts[1], pts[0], pts[9], pts[8], pts[15], pts[14], pts[19], pts[18], pts[20], pts[21], pts[22], pts[23], pts[24], pts[25], pts[26], pts[27]
	else
		face = group_ents.add_face pts[1], pts[0], pts[19], pts[18], pts[20], pts[21], pts[26], pts[27]
	end
	status = face.material = wall_texture

	if wing_section_height != tail_section_height and wing_section_height > door_height
		face = group_ents.add_face pts[14], pts[15], pts[8], pts[9]
		status = face.material = wall_texture
	end

	if roof_arc > 0 and roof_slope_over_wings == "Yes"
			pts = []
			pts[0] = [tail_left_external_position, wing_front_external_position, tail_external_side_height]
			pts[1] = [tail_right_external_position, wing_front_external_position, tail_external_side_height]
			pts[2] = [right_wing_wall_position + wall_thickness, wing_front_external_position, door_height + wall_thickness]
			pts[3] = [wing_left_external_position, wing_front_external_position, door_height + wall_thickness]
			face = group_ents.add_face pts
			status = face.material = wall_texture
	end



####################################################
####	Draw signs				####
####################################################
	
	if sign_count > 0
		for step in 0..sign_count - 1
		
			sign_group = draw_signs(sign_type[step], sign_texture[step], door_frame_texture, sign_wall[step], sign_x_pos[step], sign_y_pos[step], sign_x_size[step], sign_y_size[step], sign_thickness[step], sign_filled[step], sign_double_sided[step], full_length, 0, 0, 0)
			group = group.parent.entities.add_group([group, sign_group])
			
		end
	end


####################################################
####	Draw entrance doors			####
####################################################

	if entrance_door_count > 0
		for step in 0..entrance_door_count - 1
			if entrance_door_wall[step] == "L" or entrance_door_wall[step] == "R"   
				
				if entrance_door_x_pos[step] < wing_back_internal_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < wing_back_internal_position and entrance_door_wall[step] == "L"
					current_wall_position = 0
					current_wall_internal_position = wall_thickness
				elsif entrance_door_x_pos[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < tail_back_internal_position and entrance_door_wall[step] == "L"
					current_wall_position = tail_left_external_position
					current_wall_internal_position = tail_left_external_position + wall_thickness
				elsif entrance_door_x_pos[step] < wing_back_internal_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < wing_back_internal_position and entrance_door_wall[step] == "R"
					current_wall_position = wing_right_external_position
					current_wall_internal_position = wing_right_external_position - wall_thickness
				elsif entrance_door_x_pos[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < tail_back_internal_position and entrance_door_wall[step] == "R"
					current_wall_position = tail_right_external_position
					current_wall_internal_position = tail_right_external_position - wall_thickness
				end
				pts = []
				pts[0] = [current_wall_position, entrance_door_x_pos[step], entrance_door_y_pos[step]]
				pts[1] = [current_wall_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
				pts[2] = [current_wall_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				pts[3] = [current_wall_position, entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				
				face = group_ents.add_face pts
				status = face.erase!
				if wall_thickness > 0
					pts[4] = [current_wall_internal_position, entrance_door_x_pos[step], entrance_door_y_pos[step]]
					pts[5] = [current_wall_internal_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
					pts[6] = [current_wall_internal_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
					pts[7] = [current_wall_internal_position, entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
	
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					status = face.erase!
				end
			end
#wing_back_external_position

			if entrance_door_x_pos[step] < wing_back_internal_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < wing_back_internal_position and entrance_door_wall[step] == "L"
				entrance_door_wall[step] = "B"
				door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, 0, 0, wing_right_external_position, 0)
			elsif entrance_door_x_pos[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < tail_back_internal_position and entrance_door_wall[step] == "L"
				entrance_door_wall[step] = "B"
				door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, 0, 0, wing_right_external_position, tail_left_external_position)
			elsif entrance_door_x_pos[step] < wing_back_internal_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < wing_back_internal_position and entrance_door_wall[step] == "R"
				entrance_door_wall[step] = "F"
				door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, 0, 0, wing_right_external_position, 0)
			elsif entrance_door_x_pos[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] > wing_back_position and entrance_door_x_pos[step] + entrance_door_x_size[step] < tail_back_internal_position and entrance_door_wall[step] == "R"
				entrance_door_wall[step] = "F"
				door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, 0, 0, tail_right_external_position, 0)
			end
			group_text = group.description
			group = group.parent.entities.add_group([group, door_group])
			group.description = group_text
		end
	end


####################################################
####	Draw windows				####
####################################################

	if window_count > 0
		for step in 0..window_count - 1
			if window_wall[step] == "B"
				current_wall_position = 0
				current_wall_internal_position = wall_thickness
			else
				current_wall_position = full_length
				current_wall_internal_position = full_length - wall_thickness
			end

			for window_step in 0..window_repeat[step] - 1

				do_not_draw = 0
				if back_door == 1 and window_wall[step] == "B"
					if(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) > door_middle - door_width / 2 and window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) < door_middle + door_width / 2 and window_y_pos[step] < door_height) or(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > door_middle - door_width / 2 and window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] < door_middle + door_width / 2 and window_y_pos[step] < door_height) or window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > full_width
						do_not_draw = 1
					end
				elsif window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > full_width
					do_not_draw = 1
				end
				if do_not_draw == 0
					pts = []
					pts[0] = [current_wall_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
					pts[1] = [current_wall_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
					pts[2] = [current_wall_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
					pts[3] = [current_wall_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
					face = group_ents.add_face pts
						status = face.erase!
					if wall_thickness > 0
						pts[4] = [current_wall_internal_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
						pts[5] = [current_wall_internal_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
						pts[6] = [current_wall_internal_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
						pts[7] = [current_wall_internal_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
						face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
						status = face.erase!
						face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
						if face.normal[0] < 0
							status = face.reverse!
						end
						status = face.material = wall_texture
						face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
						status = face.material = wall_texture
						face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
						status = face.material = wall_texture
						face = group_ents.add_face pts[3], pts[0], pts[4], pts[7]
						status = face.material = wall_texture
					end
	
					window_group = draw_window(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step], window_x_size[step], window_y_size[step], window_wall[step], window_texture[step], window_back_texture[step], window_transparency[step], wall_thickness, 0, full_width, full_length, 0)
					group = group.parent.entities.add_group([group, window_group])
				end
			end
		end
	end



####################################################
####	Insert doors				####
####################################################

	door_middle = wall_thickness + wing_section_width +(tail_section_width / 2)
	if tail_section_height != wing_section_height and wing_internal_side_height < door_height
		door_width = tail_section_width - door_frame_thickness * 2 - beam_width * 2
		door_type = "Hydraulic swing door"
	else
		door_width = tail_section_width + wing_section_width * 2 - beam_width - door_frame_thickness * 2
	end

	if wing_section_height != tail_section_height and wing_internal_side_height < door_height
		door_hangar_type = "T-hangar"
	else
		door_hangar_type = "T-hangar one door"	
	end
	
	door_group = Sketchup.active_model.entities.add_group
	door_group_ents = door_group.entities
	if door_type == "Bi-folding doors"
		door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, door_width - door_frame_thickness * 6, door_middle, 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, tail_section_height + wall_thickness, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "T-Hangar", 0)
	else
		door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, door_width, door_middle, 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, tail_section_height + wall_thickness, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "T-Hangar", 0)
	end
	group_x = group

#	point = Geom::Point3d.new(0, tail_section_depth + wall_thickness, 0)
#	t = Geom::Transformation.new point
#	door_group.move! t
#	door_radians = 270.degrees
#	door_t = Geom::Transformation.rotation [0, tail_section_depth + wall_thickness, 0], Z_AXIS, door_radians
#	door_group.transform! door_t

	group_text = group.description + " with " + door_group.description
	group = group_x.parent.entities.add_group([group_x, door_group])
	group.description = group_text


	if wing_section_height != tail_section_height and wing_internal_side_height < door_height

		# add side doors
		# left
		left_door_start = wall_thickness +(beam_width * 2)
		left_door_middle = wall_thickness +(beam_width * 2) +((wing_section_width -(beam_width * 0.5)) / 2)
		left_door_end = wall_thickness + wing_section_width +(beam_width * 1.5)

		side_door_group1 = Sketchup.active_model.entities.add_group
		side_door_group_ents1 = side_door_group1.entities
		side_door_group2 = Sketchup.active_model.entities.add_group
		side_door_group_ents2 = side_door_group2.entities
		
		pts = []
		pts[0] = [left_door_start, 0, 0]
		pts[1] = [left_door_start, 0, wing_skirt_height]
		pts[2] = [left_door_middle, 0, wing_skirt_height]
		pts[3] = [left_door_middle, 0, 0]
		face = side_door_group_ents1.add_face pts
		status = face.reverse!
		status = face.material = door_texture
		pts[4] = [left_door_middle, wall_thickness, 0]
		pts[5] = [left_door_middle, wall_thickness, wing_skirt_height]
		pts[6] = [left_door_start, wall_thickness, wing_skirt_height]
		pts[7] = [left_door_start, wall_thickness, 0]
		face = side_door_group_ents1.add_face pts[0], pts[1], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[1], pts[2], pts[5], pts[6]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[0], pts[3], pts[4], pts[7]
		status = face.material = door_texture
		
		pts = []
		pts[0] = [left_door_middle, 0, 0]
		pts[1] = [left_door_middle, 0, wing_skirt_height]
		pts[2] = [left_door_end, 0, wing_skirt_height]
		pts[3] = [left_door_end, 0, 0]
		face = side_door_group_ents2.add_face pts
		status = face.reverse!
		status = face.material = door_texture

		pts[4] = [left_door_end, wall_thickness, 0]
		pts[5] = [left_door_end, wall_thickness, wing_skirt_height]
		pts[6] = [left_door_middle, wall_thickness, wing_skirt_height]
		pts[7] = [left_door_middle, wall_thickness, 0]
		face = side_door_group_ents2.add_face pts[0], pts[1], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[1], pts[2], pts[5], pts[6]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[0], pts[3], pts[4], pts[7]
		status = face.material = door_texture
		
		door_radians =(180 +(door_folding_level * 2)).degrees
		door_t = Geom::Transformation.rotation [left_door_middle, 0, 0], Z_AXIS, door_radians
		side_door_group2.transform! door_t

		left_door_group = side_door_group1.parent.entities.add_group([side_door_group1, side_door_group2])

		door_radians =(90 - door_folding_level).degrees
		door_t = Geom::Transformation.rotation [left_door_start, wall_thickness, 0], Z_AXIS, door_radians
		left_door_group.transform! door_t

		# right
		right_door_start = wall_thickness +(wing_section_width * 2) + tail_section_width -(beam_width * 2)
		right_door_middle = wall_thickness +(wing_section_width * 2) + tail_section_width -((beam_width * 2) +(wing_section_width +(beam_width * 0.5)) / 2)
		left_door_end = wall_thickness + wing_section_width + tail_section_width -(beam_width * 1.5)

		side_door_group1 = Sketchup.active_model.entities.add_group
		side_door_group_ents1 = side_door_group1.entities
		side_door_group2 = Sketchup.active_model.entities.add_group
		side_door_group_ents2 = side_door_group2.entities
		
		pts = []
		pts[0] = [right_door_start, 0, 0]
		pts[1] = [right_door_start, 0, wing_skirt_height]
		pts[2] = [right_door_middle, 0, wing_skirt_height]
		pts[3] = [right_door_middle, 0, 0]
		face = side_door_group_ents1.add_face pts
		status = face.material = door_texture

		pts[4] = [right_door_middle, wall_thickness, 0]
		pts[5] = [right_door_middle, wall_thickness, wing_skirt_height]
		pts[6] = [right_door_start, wall_thickness, wing_skirt_height]
		pts[7] = [right_door_start, wall_thickness, 0]
		face = side_door_group_ents1.add_face pts[0], pts[1], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[1], pts[2], pts[5], pts[6]
		status = face.material = door_texture
		face = side_door_group_ents1.add_face pts[0], pts[3], pts[4], pts[7]
		status = face.material = door_texture
		
		pts = []
		pts[0] = [right_door_middle, 0, 0]
		pts[1] = [right_door_middle, 0, wing_skirt_height]
		pts[2] = [left_door_end, 0, wing_skirt_height]
		pts[3] = [left_door_end, 0, 0]
		face = side_door_group_ents2.add_face pts
		status = face.material = door_texture

		pts[4] = [left_door_end, wall_thickness, 0]
		pts[5] = [left_door_end, wall_thickness, wing_skirt_height]
		pts[6] = [right_door_middle, wall_thickness, wing_skirt_height]
		pts[7] = [right_door_middle, wall_thickness, 0]
		face = side_door_group_ents2.add_face pts[0], pts[1], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[4], pts[5], pts[6], pts[7]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[2], pts[3], pts[4], pts[5]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[1], pts[2], pts[5], pts[6]
		status = face.material = door_texture
		face = side_door_group_ents2.add_face pts[0], pts[3], pts[4], pts[7]
		status = face.material = door_texture
		
		door_radians =(180 -(door_folding_level * 2)).degrees
		door_t = Geom::Transformation.rotation [right_door_middle, 0, 0], Z_AXIS, door_radians
		side_door_group2.transform! door_t

		right_door_group = side_door_group1.parent.entities.add_group([side_door_group1, side_door_group2])

		door_radians =(270 + door_folding_level).degrees
		door_t = Geom::Transformation.rotation [right_door_start, wall_thickness, 0], Z_AXIS, door_radians
		right_door_group.transform! door_t

		group_x = group
		group = group_x.parent.entities.add_group([group_x, left_door_group, right_door_group])
		
	end

	
####################################################
####	Add floor				####
####################################################

	if floor_texture != "None"
		if tail_section_depth > 0
			pts = []
			pts[0] = [0,0,0]
			pts[1] = [0, wing_back_internal_position, 0]
			pts[2] = [wing_section_width, wing_back_internal_position, 0]
			pts[3] = [wing_section_width, tail_back_internal_position, 0]
			pts[4] = [(wall_thickness * 2) + wing_section_width + tail_section_width, tail_back_internal_position, 0]
			pts[5] = [(wall_thickness * 2) + wing_section_width + tail_section_width, wing_back_internal_position, 0]
			pts[6] = [(wall_thickness * 2) +(wing_section_width * 2) + tail_section_width, wing_back_internal_position, 0]
			pts[7] = [(wall_thickness * 2) +(wing_section_width * 2) + tail_section_width, 0, 0]
			face = group_ents.add_face pts
			status = face.reverse!
			status = face.material = floor_texture
		else
			pts = []
			pts[0] = [0,0,0]
			pts[1] = [0, wing_back_internal_position, 0]
			pts[2] = [(wall_thickness * 2) +(wing_section_width * 2) + tail_section_width, wing_back_internal_position, 0]
			pts[3] = [(wall_thickness * 2) +(wing_section_width * 2) + tail_section_width, 0, 0]
			face = group_ents.add_face pts
			status = face.reverse!
			status = face.material = floor_texture
		end
	end

	if afcad_read == 1
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x -(full_width / 2) + trans.origin[0], hangar_y -(full_depth / 2) + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
	else	
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
		group.name = "THangar:" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + input[14].to_s + ":" + input[15].to_s + ":" + input[16].to_s + ":" + input[17].to_s + ":" + input[18].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":" + texture_input[4].to_s + ":" + texture_input[5].to_s + ":"
	end

	t = Geom::Transformation.rotation [hangar_x, hangar_y, 0], Z_AXIS, radians
	group.transform! t	
	
	
end


##################################################
##################################################
###### Below is the code for the hangar	row	######
##################################################
##################################################

def hangar_row(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  

	#Define entity group and materials
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "Hangar row"
	materials = Sketchup.active_model.materials

	afcad_read = 0
	
	if hangar_codes != ""
	
		afcad_read = 1

		hangar_info = hangar_codes.split(":")

		wall_texture = hangar_info[19]
		if hangar_info[20] != "Same as external"
			internal_wall_texture = hangar_info[20]
		else
			internal_wall_texture = wall_texture
		end
		roof_texture = hangar_info[21]
		door_texture = hangar_info[22]
		door_frame_texture = hangar_info[23]
		floor_texture = hangar_info[24]
		beam_texture = "Blue metal"
		cross_beam_texture = ""

		hangars = hangar_info[30].to_i
		wall_thickness = hangar_info[5].to_f / $inch
		internal_width = hangar_info[31].to_f / $inch


		internal_depth = hangar_info[3].to_f / $inch
		internal_height = hangar_info[4].to_f / $inch

		total_depth = internal_depth + (wall_thickness * 2)
		total_width =((internal_width + wall_thickness) * hangars) + wall_thickness
		external_height = internal_height + wall_thickness


		door_height = hangar_info[6].to_f / $inch
		if external_height - door_height < wall_thickness
			door_height = external_height - wall_thickness
		end
		door_width = hangar_info[7].to_f / $inch
		if internal_width - door_width < wall_thickness
			# UI.messagebox(internal_width - door_width)
			door_width = internal_width - wall_thickness
		end

		door_type = hangar_info[8]
		door_folding_level = hangar_info[9].to_i
		door_frame_thickness = hangar_info[10].to_f / $inch
		door_skin_thickness = hangar_info[11].to_f / $inch
		arc_height = hangar_info[12].to_f / $inch
		arc_position = hangar_info[13].to_f / $inch
		eaves_overhang = hangar_info[14].to_f / $inch
		if arc_position < 0
			arc_position = 0
		elsif arc_position > total_depth
			arc_position = total_depth
		end
		
		
		extension_text = hangar_info[18]

		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[26].to_f
		long1 = midlong
		long2 = hangar_info[27].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)
		
		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		hangar_angle = 360 - hangar_info[28].to_f
		hangar_x = x_distance / $inch -(internal_width / 2)
		hangar_y = y_distance / $inch -(internal_depth / 2)
		radians = hangar_angle.degrees

	else
		#Find preset angle and size
		angle = find_endpoints
		if angle[3] == 0
			angle[3] = (1620 / $inch)
		end	
	
	
		#Hangar prompts
		prompts = ["Spaces?", "Internal width?", "Internal depth?", "Internal height?", "Wall thickness?", "Door height?", "Door width?", "Door type?", "Door folding level?", "Door frame thickness?", "Door skin thickness?", "Arc height?", "Arc position?", "Eaves overhang?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		defaults = ["4", "1400",((angle[3] * $inch).ceil) - 20, "430", "5", "410", "1200", "Bi-folding doors", "5", "5", "3", "120", "240", "20", angle[0], angle[1] * $inch, angle[2] * $inch, ""]
		lists = ["", "", "", "", "", "", "", "Bi-folding doors|Hydraulic swing door", "", "", "", "", "", "", "", "", ""]
		input = UI.inputbox prompts, defaults, lists, "Hangar info"

	 	#Texture prompts
	 	texture_prompts = ["External wall texture?", "Internal wall texture?", "Roof texture?", "Door texture?", "Door frame texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Same as external", "Vertical square iron", "Vertical square iron", "Rusty metal", "Concrete"]
	 	texture_list = [$wall_list, $internal_wall_list, $roof_list, $door_list, $metal_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"

		#Get texture info
		wall_texture = texture_input[0]
		if texture_input[1] != "Same as external"
			internal_wall_texture = texture_input[1]
		else
			internal_wall_texture = wall_texture
		end
		roof_texture = texture_input[2]
		door_texture = texture_input[3]
		door_frame_texture = texture_input[4]
		floor_texture = texture_input[5]
		beam_texture = "Blue metal"
		cross_beam_texture = ""

		hangars = input[0].to_i
		wall_thickness = input[4].to_f / $inch
		if angle[4] == 0
			internal_width = input[1].to_f / $inch
		else
			internal_width = (angle[4] - (wall_thickness * (hangars + 1))) / hangars
		end

		internal_depth = input[2].to_f / $inch
		internal_height = input[3].to_f / $inch

		total_depth = internal_depth + (wall_thickness * 2)
		total_width = ((internal_width + wall_thickness) * hangars) + wall_thickness
		external_height = internal_height + wall_thickness

		door_height = input[5].to_f / $inch
		if external_height - door_height < wall_thickness
			door_height = external_height - wall_thickness
		end
		door_width = input[6].to_f / $inch
		if internal_width - door_width < wall_thickness
			# UI.messagebox(internal_width - door_width)
			door_width = internal_width - wall_thickness
		end

		door_type = input[7]
		door_folding_level = input[8].to_i
		door_folding_identical = 1
		door_frame_thickness = input[9].to_f / $inch
		door_skin_thickness = input[10].to_f / $inch
		arc_height = input[11].to_f / $inch
    arc_position = input[12].to_f / $inch
    eaves_overhang = input[13].to_f / $inch
		
		if arc_position < 0
			arc_position = 0
		elsif arc_position > total_depth
			arc_position = total_depth
		end
	
		hangar_angle = input[14].to_f
		hangar_x = input[15].to_f / $inch
		hangar_y = input[16].to_f / $inch
		radians = hangar_angle.degrees
		extension_text = input[17]
	end

	door_sections = 1
	
	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or door_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step] or internal_wall_texture == $texture_name[step] or door_frame_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or door_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured" or door_frame_texture == "Untextured" or internal_wall_texture == "Untextured" or cross_beam_texture == "Untextured"
			texture = materials.add "Untextured"
			texture.color = "White"
	end


#############################
####	Get extension info ####
#############################

	extensions = []
	extension_types = []
	hangar_wall_cut = []
	individual_door_folding_level = []
	individual_internal_width = []
	for step in 0..hangars
		hangar_wall_cut[step] = 0
		individual_door_folding_level[step] = door_folding_level
		individual_internal_width[step] = internal_width
	end

	if extension_text.length > 0
		if extension_text.count('|') > 0
			extensions = extension_text.split("|")
		else
			extensions[0] = extension_text
		end
		for step in 0..extensions.length - 1
			if extensions[step].count('(') > 0 and extensions[step].count(')') > 0 
				extension_types[step] = extensions[step][0..extensions[step].index("(") - 1]
			else
				extension_types[step] = "ERR"
			end

			# Hangar wall cut(between 1 and hangar sections)
			if extension_types[step] == "HW"
				hangar_wall_cut_id = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1].to_i
				hangar_wall_cut[hangar_wall_cut_id] = 1
			end
			# Individual door folding level(<id>;<door folding level>)
			if extension_types[step] == "DF"
				door_folding_level_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if door_folding_level_string.count(";") == 1
					temp_door_folding_level = door_folding_level_string.split(";")
					temp_door_folding_id = temp_door_folding_level[0].to_i
					individual_door_folding_level[temp_door_folding_id] = temp_door_folding_level[1].to_i
				end
				door_folding_identical = 0
			end
			# Individual door folding level(<id>;<door folding level>)
			if extension_types[step] == "HS"
				individual_internal_width_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if individual_internal_width_string.count(";") == 1
					temp_individual_internal_width = individual_internal_width_string.split(";")
					temp_hangar_id = temp_individual_internal_width[0].to_i
					individual_internal_width[temp_hangar_id] = temp_individual_internal_width[1].to_f / $inch
				end
			end
		end
		
	end



##############################
####	Draw external walls	####
##############################

	total_width = 0
	for step in 0..hangars - 1
		total_width = total_width + individual_internal_width[step]
	end
	total_width = total_width + wall_thickness * (hangars + 1)
	
	#Draw sidewalls and back walls
	pts = []
	pts[0] = [0, 0, 0]
	pts[1] = [0, 0, external_height]
	pts[2] = [0, arc_position, external_height + arc_height]
	pts[3] = [0, total_depth, external_height]
	pts[4] = [0, total_depth, 0]
	face = group_ents.add_face pts	
	status = face.material = wall_texture

	pts = []
	pts[0] = [0, total_depth, external_height]
	pts[1] = [total_width, total_depth, external_height]
	pts[2] = [total_width, total_depth, 0]
	pts[3] = [0, total_depth, 0]
	face = group_ents.add_face pts	
	status = face.material = wall_texture

	pts = []
	pts[0] = [total_width, 0, 0]
	pts[1] = [total_width, 0, external_height]
	pts[2] = [total_width, arc_position, external_height + arc_height]
	pts[3] = [total_width, total_depth, external_height]
	pts[4] = [total_width, total_depth, 0]
	face = group_ents.add_face pts	
	status = face.material = wall_texture

	for step in 0..1
		pts = []
		pts[0] = [0, total_depth * step, external_height]
		pts[1] = [0, arc_position, external_height + arc_height]
		pts[2] = [total_width, arc_position, external_height + arc_height]
		pts[3] = [total_width, total_depth * step, external_height]
		face = group_ents.add_face pts	
		if arc_position == total_depth * step
			status = face.material = wall_texture
		else
			status = face.material = roof_texture
		end
	end


##############################
####	Draw internal walls	####
##############################

	ptsx = []
	ptsx_step = 0
	
	if arc_position == 0
		internal_arc_pos = wall_thickness
	elsif arc_position == total_depth
		internal_arc_pos = total_depth - wall_thickness
	else
		internal_arc_pos = arc_position
	end	
	
	cursor_x = 0
	
	for hangar_step in 0..hangars - 1
	
		if hangar_step == 0 or hangar_wall_cut[hangar_step] == 0
			pts = []
			pts[0] = [cursor_x + wall_thickness, wall_thickness, 0]
			pts[1] = [cursor_x + wall_thickness, wall_thickness, internal_height]
			pts[2] = [cursor_x + wall_thickness, internal_arc_pos, internal_height + arc_height]
			pts[3] = [cursor_x + wall_thickness, total_depth - wall_thickness, internal_height]
			pts[4] = [cursor_x + wall_thickness, total_depth - wall_thickness, 0]
			face = group_ents.add_face pts	
			status = face.reverse!
			status = face.material = internal_wall_texture
		end
		
		pts = []
		pts[0] = [cursor_x + wall_thickness, total_depth - wall_thickness, internal_height]
		pts[1] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, total_depth - wall_thickness, internal_height]
		pts[2] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, total_depth - wall_thickness, 0]
		pts[3] = [cursor_x + wall_thickness, total_depth - wall_thickness, 0]
		face = group_ents.add_face pts	
		status = face.material = internal_wall_texture

		pts = []
		pts[0] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, wall_thickness, 0]
		pts[1] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, wall_thickness, internal_height]
		pts[2] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, internal_arc_pos, internal_height + arc_height]
		pts[3] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, total_depth - wall_thickness, internal_height]
		pts[4] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, total_depth - wall_thickness, 0]
	
		if hangar_step == hangars - 1 or hangar_wall_cut[hangar_step + 1] == 0
			face = group_ents.add_face pts	
			status = face.material = internal_wall_texture
		else
			pts[5] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness * 2, wall_thickness, 0]
			pts[6] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness * 2, wall_thickness, internal_height]
			pts[7] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness * 2, internal_arc_pos, internal_height + arc_height]
			pts[8] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness * 2, total_depth - wall_thickness, internal_height]
			pts[9] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness * 2, total_depth - wall_thickness, 0]
			face = group_ents.add_face pts[0], pts[1], pts[6], pts[5]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[1], pts[2], pts[7], pts[6]
			status = face.material = roof_texture
			face = group_ents.add_face pts[2], pts[3], pts[8], pts[7]
			status = face.material = roof_texture
			face = group_ents.add_face pts[3], pts[4], pts[9], pts[8]
			status = face.material = internal_wall_texture
		end

		for step in 0..1
			pts = []
			pts[0] = [cursor_x + wall_thickness,((total_depth - (wall_thickness * 2)) * step) + wall_thickness, internal_height]
			pts[1] = [cursor_x + wall_thickness, internal_arc_pos, internal_height + arc_height]
			pts[2] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, internal_arc_pos, internal_height + arc_height]
			pts[3] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness,((total_depth - (wall_thickness * 2)) * step) + wall_thickness, internal_height]
			face = group_ents.add_face pts	
			if arc_position == total_depth * step
				status = face.material = internal_wall_texture
			else
				status = face.material = roof_texture
			end
		end

		#door cutouts
		pts = []
		pts[0] = [cursor_x + wall_thickness, wall_thickness, 0]
		pts[1] = [cursor_x + wall_thickness, wall_thickness, internal_height]
		pts[2] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, wall_thickness, internal_height]
		pts[3] = [cursor_x + individual_internal_width[hangar_step] + wall_thickness, wall_thickness, 0]
		pts[4] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, 0]
		pts[5] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[6] = [(cursor_x + wall_thickness) +((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[7] = [(cursor_x + wall_thickness) +((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, 0]
		face = group_ents.add_face pts
		status = face.material = internal_wall_texture
		
		pts = []
		pts[0] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, 0]
		pts[1] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[2] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, door_height]
		pts[3] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, 0]
		face = group_ents.add_face pts
		status = face.material = wall_texture

		pts = []
		pts[0] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[1] = [(cursor_x + wall_thickness) +((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[2] = [(cursor_x + wall_thickness) +((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, door_height]
		pts[3] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) -((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, door_height]
		face = group_ents.add_face pts
		status = face.material = wall_texture

		pts = []
		pts[0] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, door_height]
		pts[1] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), wall_thickness, 0]
		pts[2] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, 0]
		pts[3] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, door_height]
		face = group_ents.add_face pts
		status = face.material = wall_texture
		
		
		ptsx[ptsx_step] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, 0]
		ptsx[ptsx_step + 1] = [(cursor_x + wall_thickness) + ((individual_internal_width[hangar_step] / 2) -(door_width / 2)), 0, door_height]
		ptsx[ptsx_step + 2] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) - ((individual_internal_width[hangar_step] / 2) - (door_width / 2)), 0, door_height]
		ptsx[ptsx_step + 3] = [(cursor_x + individual_internal_width[hangar_step] + wall_thickness) - ((individual_internal_width[hangar_step] / 2) - (door_width / 2)), 0, 0]
		ptsx_step = ptsx_step + 4
		
		cursor_x = cursor_x + individual_internal_width[hangar_step] + wall_thickness
	end

	ptsx[ptsx_step] = [total_width, 0, 0]
	ptsx[ptsx_step + 1] = [total_width, 0, external_height]
	ptsx[ptsx_step + 2] = [0, 0, external_height]
	ptsx[ptsx_step + 3] = [0, 0, 0]
	face = group_ents.add_face ptsx
	status = face.material = wall_texture

	door_group = []
	door_group_ents = []
	cursor_x = 0
	if door_type == "Detailed door" or door_type == "Bi-folding doors" or door_type == "Hydraulic swing door"
		if door_folding_identical == 1
      door_middle = []
      for step in 0..hangars - 1
        door_middle[step] = cursor_x + wall_thickness + (individual_internal_width[step] / 2)
        cursor_x = cursor_x + individual_internal_width[step] + wall_thickness
      end
      door_group = insert_door(door_type, door_height, individual_door_folding_level[hangar_step].to_i, door_sections, door_width, door_middle, total_depth, wall_thickness, door_frame_thickness, door_skin_thickness, external_height, arc_height, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "Hangar row", 0)
      point = Geom::Point3d.new(0, total_depth + wall_thickness, 0)
      t = Geom::Transformation.new point
      door_group.move! t
      door_radians = 270.degrees
      door_t = Geom::Transformation.rotation [0, total_depth + wall_thickness, 0], Z_AXIS, door_radians
      door_group.transform! door_t
      group = group.parent.entities.add_group(group, door_group)
		else
      for hangar_step in 0..hangars - 1
    # Create separate group for door entities
#        door_group[hangar_step] = Sketchup.active_model.entities.add_group
#        door_group_ents = door_group[hangar_step].entities
        door_group[hangar_step] = insert_door(door_type, door_height, individual_door_folding_level[hangar_step].to_i, door_sections, door_width, cursor_x + wall_thickness + (individual_internal_width[hangar_step] / 2), total_depth, wall_thickness, door_frame_thickness, door_skin_thickness, external_height, arc_height, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "Hangar row", 0)
        point = Geom::Point3d.new(0, total_depth + wall_thickness, 0)
        t = Geom::Transformation.new point
        door_group[hangar_step].move! t
        door_radians = 270.degrees
        door_t = Geom::Transformation.rotation [0, total_depth + wall_thickness, 0], Z_AXIS, door_radians
        door_group[hangar_step].transform! door_t
      
        step_number = hangar_step
        cursor_x = cursor_x + individual_internal_width[hangar_step] + wall_thickness
      end
      door_group[step_number + 1] = group
      group = door_group[step_number + 1].parent.entities.add_group(door_group)
		end
	end
	

	#draw floor
	if floor_texture != "None"
		pts = []
		pts[0] = [0,0,0]
		pts[1] = [total_width,0,0]
		pts[2] = [total_width,total_depth,0]
		pts[3] = [0,total_depth,0]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = floor_texture
	end


	if afcad_read == 1
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
		t = Geom::Transformation.rotation [hangar_x + (internal_width / 2), hangar_y + (internal_depth / 2), 0], Z_AXIS, radians
		group.transform! t
	else
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
		t = Geom::Transformation.rotation [hangar_x,hangar_y,0], Z_AXIS, radians
		group.transform! t
		group.name = "HRow(&ltid&gt):" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + input[14].to_s + ":" + input[15].to_s + ":" + input[16].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":" + texture_input[4].to_s + ":" + texture_input[5].to_s + ":"
	end
end


######################################################################################################################################
######################################################################################################################################
######																######
###### Below is the code for the arched hangar											######
######																######
######################################################################################################################################
######################################################################################################################################

def arched_hangar(hangar_codes = "", midlat = 0.0, midlong = 0.0)

	
  

	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	materials = Sketchup.active_model.materials


	if hangar_codes != ""
		afcad_read = 1

		hangar_info = hangar_codes.split(":")
#		hangar_dimensions = hangar_info[0][hangar_info[0].index("(") + 1..hangar_info[0].index(")") - 1].split("x")


#	 	texture_defaults = ["Blue metal", "Horizontal square iron", "Vertical corrugated iron", "Horizontal square iron", "Rusty metal", "Concrete"]
		beam_texture = hangar_info[22]
		roof_texture = hangar_info[23]
		wall_texture = hangar_info[24]
		door_texture = hangar_info[26]
		door_frame_texture = hangar_info[27]
		floor_texture = hangar_info[28]
		if hangar_info[25] == "Same as external"
			internal_wall_texture = wall_texture
		else
			internal_wall_texture = hangar_info[25] 
		end
		cross_beam_texture = ""
			
		beam_type = hangar_info[1]
		beams = hangar_info[2].to_i
		beam_width = hangar_info[4].to_f / $inch
		beam_thickness = hangar_info[5].to_f / $inch
		wall_thickness = hangar_info[6].to_f / $inch

		if beam_type == "T"
			full_beam_width =(beam_width * 2) + beam_thickness
			full_beam_depth = beam_thickness + beam_width
		elsif beam_type == "U"
			full_beam_width =(beam_width + beam_thickness) * 2
			full_beam_depth = beam_thickness + beam_width
		elsif beam_type == "H"
			full_beam_width = beam_width + beam_thickness
			full_beam_depth =(beam_thickness * 2) + beam_width
		end

		beam_spacing = hangar_info[3].to_f / $inch

		full_length =(full_beam_width * beams) +(beam_spacing *(beams - 1)) + wall_thickness * 2
		internal_width = hangar_info[7].to_f / $inch
		arc_height = hangar_info[8].to_f / $inch
		eaves_overhang = hangar_info[9].to_f / $inch
	
		full_width =(wall_thickness * 2) +  internal_width
		outside_height = arc_height + wall_thickness
	
		door_width = hangar_info[10].to_f / $inch
		door_height = hangar_info[11].to_f / $inch
		door_middle = full_width / 2
		door_type = hangar_info[12]
		door_folding_level = hangar_info[13].to_i
		door_sections = hangar_info[14].to_i
		door_frame_thickness = hangar_info[15].to_f / $inch
		door_skin_thickness = hangar_info[16].to_f / $inch
		arch_detail_level = hangar_info[17].to_i
		extension_text = hangar_info[21]
		
		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[30].to_f
		long1 = midlong
		long2 = hangar_info[31].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
	#	y_distance =(earth_radius * calc_c)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)
	#	x_distance =(earth_radius * calc_c)


		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		hangar_angle = 270 - hangar_info[32].to_f
		hangar_x = x_distance / $inch
		hangar_y = y_distance / $inch
		radians = hangar_angle.degrees			
	else

		set_length = 1
		angle = find_endpoints
		if angle[3] == 0
			angle[3] = 1520 / $inch
		end
		if angle[4] == 0
			angle[4] =((500 * 3) +(48 * 4) + 20) / $inch
			set_length = 0
		end
	
		prompts = ["Beam type?", "Number of beams?", "Space between beams?", "Beam width?", "Beam thickness?", "Wall thickness?", "Internal width?", "Arc height?", "Eaves overhang?", "Door width?", "Door height?", "Door type?", "Door folding level?", "Door sections?", "Door frame thickness?", "Door skin thickness?", "Arch detail level?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		defaults = ["T", "4",(((angle[4] * $inch) -(48 * 4) - 20) / 3).ceil, "20", "4", "5",((angle[3] * $inch) - 20).ceil , "450", "0", "1000", "280", "Sliding doors", "5", "2", "5", "3", "30", angle[0], angle[1] * $inch, angle[2] * $inch, ""]
		list = ["T|U|H", "", "", "", "", "", "", "", "", "", "", "Sliding doors|Bi-folding doors|Hydraulic swing door|Encased sliding doors", "", "", "", "", "", "", "", "", ""]
		input = UI.inputbox prompts, defaults, list, "Hangar info"
	
	 	texture_prompts = ["Beam texture?", "Roof texture?", "Wall texture?", "Internal wall texture?", "Door texture?", "Door frame texture?", "Floor texture?"]
	 	texture_defaults = ["Blue metal", "Horizontal square iron", "Vertical corrugated iron", "Same as external", "Horizontal square iron", "Rusty metal", "Concrete"]
	 	texture_list = [$metal_list, $roof_list, $wall_list, $internal_wall_list, $door_list, $metal_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"

		beam_texture = texture_input[0]
		roof_texture = texture_input[1]
		wall_texture = texture_input[2]
		door_texture = texture_input[4]
		door_frame_texture = texture_input[5]
		floor_texture = texture_input[6]
		if texture_input[3] == "Same as external"
			internal_wall_texture = wall_texture
		else
			internal_wall_texture = texture_input[3] 
		end
		cross_beam_texture = ""

		beam_type = input[0]
		beams = input[1].to_i
		beam_width = input[3].to_f / $inch
		beam_thickness = input[4].to_f / $inch
		wall_thickness = input[5].to_f / $inch

		if beam_type == "T"
			full_beam_width =(beam_width * 2) + beam_thickness
			full_beam_depth = beam_thickness + beam_width
		elsif beam_type == "U"
			full_beam_width =(beam_width + beam_thickness) * 2
			full_beam_depth = beam_thickness + beam_width
		elsif beam_type == "H"
			full_beam_width = beam_width + beam_thickness
			full_beam_depth =(beam_thickness * 2) + beam_width
		end

		if set_length == 1
			beam_spacing =(angle[4] -(full_beam_width * beams) -(wall_thickness * 2)) /(beams - 1)
		else
			beam_spacing = input[2].to_f / $inch
		end

		full_length =(full_beam_width * beams) +(beam_spacing *(beams - 1)) + wall_thickness * 2
		internal_width = input[6].to_f / $inch
		arc_height = input[7].to_f / $inch
		eaves_overhang = input[8].to_f / $inch
	
		full_width =(wall_thickness * 2) +  internal_width
		outside_height = arc_height + wall_thickness
	
		door_width = input[9].to_f / $inch
		door_height = input[10].to_f / $inch
		door_middle = full_width / 2
		door_type = input[11]
		door_folding_level = input[12].to_i
		door_sections = input[13].to_i
		door_frame_thickness = input[14].to_f / $inch
		door_skin_thickness = input[15].to_f / $inch
		arch_detail_level = input[16].to_i
		
		hangar_angle = input [17].to_f
		hangar_x = input [18].to_f / $inch
		hangar_y = input [19].to_f / $inch
		radians = hangar_angle.degrees

		extension_text = input[20]

		group.description = "Arched hangar:" + hangar_x.to_s + ":" + hangar_y.to_s + ":" + hangar_angle.to_s
	end

	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or door_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step] or door_frame_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or door_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured" or door_frame_texture == "Untextured" or internal_wall_texture == "Untextured" or cross_beam_texture == "Untextured"
			texture = materials.add "Untextured"
			texture.color = "White"
	end


####################################################
####	Get extension info			####
####################################################

	extensions = []
	extension_types = []
	door_offset = 0
	door_tail_height = 0
	door_tail_width = 0
	back_door = 0
	back_door_tail_height = 0
	back_door_tail_width = 0
	back_door_width = 0
	back_door_height = 0
	back_door_middle = 0
	sign_count = 0
	sign_x_pos = []			# Position on wall from front or left
	sign_y_pos = []			# Position on wall from ground
	sign_x_size = []		# Sign width
	sign_y_size = []		# Sign height
	sign_thickness = []		# Sign thickness
	sign_double_sided = []		# 0 for logo on front only(only valid for textured signs), 1 for logo on both sides
	sign_wall = []			# Wall to put sign on, F, B, L, R
	sign_texture = []		# Sign texture name
	sign_type = []			# Sign type; L(logo), T(text)
	sign_filled = []		# 0 to not adjust texture size, 1 to adjust texture size.
	entrance_door_wall = []
	entrance_door_count = 0
	entrance_door_x_pos = []
	entrance_door_y_pos = []
	entrance_door_x_size = []
	entrance_door_y_size = []
	entrance_door_texture = []
	entrance_door_glass_texture = []
	entrance_door_glass_back_texture = []
	entrance_door_type = []
	window_count = 0
	window_x_pos = []
	window_y_pos = []
	window_x_size = []
	window_y_size = []
	window_texture = []
	window_back_texture = []
	window_transparency = []
	window_wall = []
	window_repeat = []
	window_gap = []
	uv_map_count = 0
	uv_map_x_pos = []
	uv_map_y_pos = []
	uv_map_x_size = []
	uv_map_y_size = []
	uv_map_texture = []
	uv_map_wall = []
	uv_map_rotation = []
	roof_window_count = 0
	roof_window_x_pos = []
	roof_window_y_pos = []
	roof_window_x_size = []
	roof_window_y_size = []
	roof_window_texture = []
	roof_window_back_texture = []
	roof_window_transparency = []
	roof_window_side = []
	roof_window_repeat = []
	roof_window_gap = []
	hangar_side_count = 0
	hangar_side_start = []
	hangar_side_length = []
	hangar_side_depth = []
	hangar_side_width = []
	hangar_side_height = []
	hangar_side_texture = []
	hangar_side_fit = []
	hangar_side_opening = []
	hangar_side_type = []
	hangar_side_uv_set = []
	hangar_side_custom_opening_set = []
	hangar_side_attributes = []
	hangar_side_attribute_count = []
	hangar_side_attribute_type = []
	custom_beam_setup = 0
	custom_beam_position = []



	if extension_text.length > 0
		if extension_text.count('|') > 0
			extensions = extension_text.split("|")
		else
			extensions[0] = extension_text
		end
		door_set = 0
		for step in 0..extensions.length - 1
			door_found = 0
			if extensions[step].count('(') > 0 and extensions[step].count(')') > 0 
				extension_types[step] = extensions[step][0..extensions[step].index("(") - 1]
			else
				extension_types[step] = "ERR"
			end
			
			if extension_types[step] == "D" and door_offset == 0 and door_set == 0
				door_offset = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1].to_i
				door_middle = door_middle +(door_offset / $inch)
				door_found = 1
			end

			if extension_types[step] == "DB" and back_door == 0 #and doors_expanded != "Yes" and door_set == 0
				back_door = 1
				back_door_width = door_width
				back_door_height = door_height
				back_door_middle = door_middle
				back_door_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if back_door_string.count(";") > 0
					back_door_items = back_door_string.split(";")
					if back_door_items[0] == "Sliding"
						back_door_type = "Sliding doors"
					elsif back_door_items[0] == "Bi-fold"
						back_door_type = "Bi-folding doors"
					elsif back_door_items[0] == "Hydraulic"
						back_door_type = "Hydraulic swing door"
					elsif back_door_items[0] == "Encased"
						back_door_type = "Encased sliding doors"
					else
						back_door_type = door_type
					end
					
					back_door_folding_level = back_door_items[0].to_i
					
				else
					back_door_folding_level = back_door_string.to_i
					back_door_type = door_type
				end
			end

			# Signage
			if extension_types[step] == "S"
				sign_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if sign_string.count(";") >= 3
					sign_failed = 0
					sign_details = sign_string.split(";")
					sign_type[sign_count] = sign_details[0]
					if sign_details[1].count("x") == 1
						sign_temp_string = sign_details[1].split("x")
						sign_x_pos[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_pos[sign_count] = sign_temp_string[1].to_f / $inch
					else
						sign_failed = 1
					end
					if sign_details[2].count("x") == 2
						sign_temp_string = sign_details[2].split("x")
						sign_x_size[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_size[sign_count] = sign_temp_string[1].to_f / $inch
						sign_thickness[sign_count] = sign_temp_string[2].to_f / $inch
					else
						sign_failed = 1
					end
					sign_wall[sign_count] = sign_details[3]
					if sign_type[sign_count] == "L" and sign_string.count(";") == 6
						sign_texture[sign_count] = sign_details[4]
						sign_filled[sign_count] = sign_details[5].to_i
						sign_double_sided[sign_count] = sign_details[6].to_i
					
					else
						sign_failed = 1
					end
					
					if sign_failed == 0 and(sign_wall[sign_count] == "F" or sign_wall[sign_count] == "B")  
						sign_count = sign_count + 1
					end
				end
			end

			# Entrance doors
			if extension_types[step] == "ED"
				entrance_door_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if entrance_door_string.count(";") == 4 or entrance_door_string.count(";") == 5
					entrance_door_failed = 0
					entrance_door_details = entrance_door_string.split(";")
					entrance_door_wall[entrance_door_count] = entrance_door_details[2]
					if entrance_door_details[0].count("x") == 1
						entrance_door_temp_string = entrance_door_details[0].split("x")
						entrance_door_x_pos[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_pos[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
						if entrance_door_y_pos[entrance_door_count] < 1 / $inch
							entrance_door_y_pos[entrance_door_count] = 1 / $inch
						end
					else
						entrance_door_failed = 1
					end
					if entrance_door_details[1].count("x") == 1
						entrance_door_temp_string = entrance_door_details[1].split("x")
						entrance_door_x_size[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_size[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
					else
						entrance_door_failed = 1
					end
					entrance_door_texture[entrance_door_count] = entrance_door_details[4]
					entrance_door_type[entrance_door_count] = entrance_door_details[3]
					
					if entrance_door_string.count(";") == 5

						entrance_door_glass_texture[entrance_door_count] = entrance_door_details[5]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_details[5]
					
					else
						entrance_door_glass_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
					end
					
					if entrance_door_failed == 0 and(entrance_door_wall[entrance_door_count] == "F" or entrance_door_wall[entrance_door_count] == "B") 
					#	cross_beam_exclusion_side[cross_beam_exclusion_count] = entrance_door_wall[entrance_door_count]
					#	cross_beam_exclusions[cross_beam_exclusion_count] = [entrance_door_x_pos[entrance_door_count], entrance_door_y_pos[entrance_door_count], entrance_door_x_pos[entrance_door_count] + entrance_door_x_size[entrance_door_count], entrance_door_y_pos[entrance_door_count] + entrance_door_y_size[entrance_door_count]]
					#	cross_beam_extras_drawn[cross_beam_exclusion_count] = 0
					#	cross_beam_exclusion_count = cross_beam_exclusion_count + 1
						entrance_door_count = entrance_door_count + 1
					end
				end
			end

			# Windows
			if extension_types[step] == "W"
				window_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if window_string.count(";") >= 4
					window_failed = 0
					window_items = window_string.split(";")
					if window_items[0].count("x") == 1
						window_details = window_items[0].split("x")
						window_x_pos[window_count] = window_details[0].to_f / $inch
						window_y_pos[window_count] = window_details[1].to_f / $inch
						if window_y_pos[window_count] >= arc_height
							window_failed = 1
						end
					else
						window_failed = 1
					end
					if window_items[1].count("x") == 1
						window_details = window_items[1].split("x")
						window_x_size[window_count] = window_details[0].to_f / $inch
						window_y_size[window_count] = window_details[1].to_f / $inch
						if window_y_size[window_count] + window_y_pos[window_count] > arc_height and window_failed == 0
							window_y_size[window_count] = arc_height - window_y_pos[window_count]
						end
					else
						window_failed = 1
					end
					if window_items[3].count("/") > 0
						window_details = window_items[3].split("/")
						window_texture[window_count] = window_details[0]
						window_back_texture[window_count] = window_details[1]
					else
						window_texture[window_count] = window_items[3]
						window_back_texture[window_count] = window_items[3]
					end	
					window_transparency[window_count] = window_items[4].to_i
					if window_transparency[window_count] < 0
						window_transparency[window_count] = 0
					elsif window_transparency[window_count] > 100
						window_transparency[window_count] = 100
					end
					
					window_wall[window_count] = window_items[2]
					if window_wall[window_count] != "F" and window_wall[window_count] != "B"
						window_failed = 1
					end
					if window_string.count(";") == 6
						window_repeat[window_count] = window_items[5].to_i
						window_gap[window_count] = window_items[6].to_f / $inch
					else
						window_repeat[window_count] = 1
						window_gap[window_count] = 0
					end
					if window_failed == 0
						window_count = window_count + 1
					end
				end
			end

			# UV Mapping
			if extension_types[step] == "UV"
				uv_map_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if uv_map_string.count(";") == 6
					uv_map_info = uv_map_string.split(";")
					uv_map_x_pos[uv_map_count] = uv_map_info[0].to_f / $inch
					uv_map_y_pos[uv_map_count] = uv_map_info[1].to_f / $inch
					uv_map_x_size[uv_map_count] = uv_map_info[2].to_f / $inch
					uv_map_y_size[uv_map_count] = uv_map_info[3].to_f / $inch
					uv_map_texture[uv_map_count] = uv_map_info[4]
					uv_map_wall[uv_map_count] = uv_map_info[5]
					uv_map_rotation[uv_map_count] = uv_map_info[6].to_f
					uv_map_count = uv_map_count + 1
				end
			end


			# Roof windows
			if extension_types[step] == "RW"
				roof_window_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if roof_window_string.count(";") >= 4
					roof_window_failed = 0
					roof_window_items = roof_window_string.split(";")
					if roof_window_items[0].count("x") == 1
						roof_window_details = roof_window_items[0].split("x")
						roof_window_x_pos[roof_window_count] = roof_window_details[0].to_f / $inch
						roof_window_y_pos[roof_window_count] = roof_window_details[1].to_f / $inch
					#	if roof_window_y_pos[roof_window_count] >= full_width / 2 - wall_thickness
					#		roof_window_failed = 1
					#	end
					else
						roof_window_failed = 1
					end
					if roof_window_items[1].count("x") == 1
						roof_window_details = roof_window_items[1].split("x")
						roof_window_x_size[roof_window_count] = roof_window_details[0].to_f / $inch
						roof_window_y_size[roof_window_count] = roof_window_details[1].to_f / $inch
				#		if roof_window_y_size[roof_window_count] + roof_window_y_pos[roof_window_count] > full_width / 2 - wall_thickness and roof_window_failed == 0
				#			roof_window_y_size[roof_window_count] = full_width / 2 - roof_window_y_pos[roof_window_count] - wall_thickness
				#		end
					else
						roof_window_failed = 1
					end
					if roof_window_items[3].count("/") > 0
						roof_window_details = roof_window_items[3].split("/")
						roof_window_texture[roof_window_count] = roof_window_details[0]
						roof_window_back_texture[roof_window_count] = roof_window_details[1]
					else
						roof_window_texture[roof_window_count] = roof_window_items[3]
						roof_window_back_texture[roof_window_count] = roof_window_items[3]
					end	
					roof_window_transparency[roof_window_count] = roof_window_items[4].to_i
					if roof_window_transparency[roof_window_count] < 0
						roof_window_transparency[roof_window_count] = 0
					elsif roof_window_transparency[roof_window_count] > 100
						roof_window_transparency[roof_window_count] = 100
					end
					
					roof_window_side[roof_window_count] = roof_window_items[2]
					if roof_window_side[roof_window_count] != "C" and roof_window_side[roof_window_count] != "W"
						roof_window_failed = 1
					end
					if roof_window_string.count(";") == 6
						roof_window_repeat[roof_window_count] = roof_window_items[5].to_i
						roof_window_gap[roof_window_count] = roof_window_items[6].to_f / $inch
					else
						roof_window_repeat[roof_window_count] = 1
						roof_window_gap[roof_window_count] = 0
					end
					if roof_window_failed == 0
						roof_window_count = roof_window_count + 1
					end
				end
			end
	

			# Side wall extensions
			if extension_types[step] == "HL" or extension_types[step] == "HR" or extension_types[step] == "HS"
				hangar_side_extension_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if hangar_side_extension_string.count(";") == 7
					hangar_side_items = hangar_side_extension_string.split(";")
					hangar_side_start[hangar_side_count] = hangar_side_items[0].to_f / $inch
					hangar_side_length[hangar_side_count] = hangar_side_items[1].to_f / $inch
					hangar_side_depth[hangar_side_count] = hangar_side_items[2].to_f / $inch
					hangar_side_width[hangar_side_count] = hangar_side_items[3].to_f / $inch
					hangar_side_height[hangar_side_count] = hangar_side_items[4].to_f / $inch
					hangar_side_texture[hangar_side_count] = hangar_side_items[5]
					hangar_side_fit[hangar_side_count] = hangar_side_items[6]
					hangar_side_opening[hangar_side_count] = hangar_side_items[7]
					hangar_side_type[hangar_side_count] = extension_types[step]
					hangar_side_uv_set[hangar_side_count] = 0
					hangar_side_custom_opening_set[hangar_side_count] = 0
					hangar_side_attribute_count[hangar_side_count] = extensions[step].count(",")
					if extensions[step].count(",") > 0
						hangar_side_attribute_info = extensions[step].split(",")
						hangar_side_attribute_type[hangar_side_count] = []
						hangar_side_attributes[hangar_side_count] = []
						for hangar_side_attribute_step in 1..extensions[step].count(",")
						#	UI.messagebox hangar_side_attribute_info[hangar_side_attribute_step] 
							hangar_side_attribute_type[hangar_side_count][hangar_side_attribute_step - 1] = hangar_side_attribute_info[hangar_side_attribute_step][0..hangar_side_attribute_info[hangar_side_attribute_step].index("(") - 1]
							hangar_side_attributes[hangar_side_count][hangar_side_attribute_step - 1] = hangar_side_attribute_info[hangar_side_attribute_step][hangar_side_attribute_info[hangar_side_attribute_step].index("(") + 1..hangar_side_attribute_info[hangar_side_attribute_step].index(")") - 1]
							if hangar_side_attribute_type[hangar_side_count][hangar_side_attribute_step - 1] == "O"
								hangar_side_custom_opening_set[hangar_side_count] = hangar_side_custom_opening_set[hangar_side_count] + 1
							end
						end
					end
					hangar_side_count = hangar_side_count + 1
				end
			end
			
			if extension_types[step] == "BM" and custom_beam_setup == 0
				custom_beam_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				
				if custom_beam_string.count(";") > 0
					custom_beam_position = custom_beam_string.split(";")
					for cbp_step in 0..custom_beam_string.count(";")
						custom_beam_position[cbp_step] = custom_beam_position[cbp_step].to_f / $inch
					end
				else
					custom_beam_position[0] = custom_beam_string.to_f / $inch
				end
				custom_beam_setup = custom_beam_string.count(";") + 1
			end
			# Custom beams
			
			if door_found == 1 and door_set == 0
				current_extension = []
				current_extension_step = []
				if extensions[step].count(',') > 0
					current_extension = extensions[step].split(",")
					
					for ext_step in 0..current_extension.length - 1
						if current_extension[ext_step].count('(') > 0 and current_extension[ext_step].count(')') > 0
							current_extension_step[ext_step] = current_extension[ext_step][0..current_extension[ext_step].index("(") - 1]
						end
						if current_extension_step[ext_step] == "C"
							if current_extension[ext_step].count('x') > 0
								door_tail_height =(current_extension[ext_step][current_extension[ext_step].index("x") + 1..current_extension[ext_step].index(")") - 1].to_f) / $inch
								door_tail_width =(current_extension[ext_step][current_extension[ext_step].index("(") + 1..current_extension[ext_step].index("x") - 1].to_f) / $inch
								door_set = 1
							
							end
						end
					end
				end
				door_found = 0			
			end
		end
	end

	#find radius of curve
	
	edgearray = []
	first_edge = []
	arccurve = []

	for step in 0..1
		x1 = full_width -(wall_thickness * step)
		y1 = 0
		x2 = full_width / 2
		y2 = outside_height -(wall_thickness * step)
		x3 = 0 +(wall_thickness * step)
		y3 = 0
		
		mr =(y2 - y1)/(x2 - x1)
		mt =(y3 - y2)/(x3 - x2)
		
		x_coord =((mr * mt *(y3 - y1)) +(mr *(x2 + x3)) -(mt *(x1 + x2))) /(2 *(mr - mt))
		y_coord = 0 -((1 / mr) *(x_coord -((x1 + x2) / 2))) +((y1 + y2) / 2)
		
		radius = Math::sqrt(((x2 - x_coord) *(x2 - x_coord)) +((y2 - y_coord) *(y2 - y_coord)))
	
		start_angle =((Math::atan2(y1 - y_coord, x1 - x_coord) * 180 / Math::PI)).degrees
		end_angle =((Math::atan2(y3 - y_coord, x3 - x_coord) * 180 / Math::PI)).degrees
		if start_angle.radians < 0
			end_angle =(360 +(Math::atan2(y3 - y_coord, x3 - x_coord) * 180 / Math::PI)).degrees
		end
	
		#draw curve
	     	center_point = Geom::Point3d.new(0,x_coord,y_coord)
	     	normal_vector = Geom::Vector3d.new(1,0,0)
	     	zvector = Geom::Vector3d.new(0,1,0)

	     	if step == 0
	     		edgearray[step] = group_ents.add_arc center_point, zvector, normal_vector, radius, start_angle, end_angle, arch_detail_level
	     	else
	     		edgearray[step] = group_ents.add_arc center_point, zvector, normal_vector, radius, end_angle, start_angle, arch_detail_level
	     	end
	     	first_edge[step] = edgearray[step][0]
	     	arccurve[step] = first_edge[step].curve
	end
     
  	pts = []
  	ptsx = []
  	
  	inside_pts = []
  	
  	current_step = 0
  	current_stepx = 0
  	for step in 0..1
  		for stepx in 0..edgearray[step].length - 1
  			pts[current_step] = edgearray[step][stepx].start
  			if step == 1
  				inside_pts[current_stepx] = pts[current_step].position
  				inside_pts[current_stepx][0] = wall_thickness
  				current_stepx = current_stepx + 1
  			end
  			current_step = current_step + 1
  		end
	  	pts [current_step] = edgearray[step][edgearray[step].length - 1].end
  		if step == 1
  			inside_pts[current_stepx] = pts[current_step].position
			inside_pts[current_stepx][0] = wall_thickness
  			current_stepx = current_stepx + 1
  		end
	  	
		current_step = current_step + 1
  	end

	external_pts = []
	external_front_pts = []
	outside_pts = []
	outside_front_pts = []
	for step in 0..pts.length - 1
		external_pts[step] = pts[step].position 
		external_front_pts[step] = pts[step].position
		if step <= arch_detail_level
			outside_pts[step] = external_pts[step]
			outside_front_pts[step] = external_front_pts[step]
		end
		external_pts[step][0] = external_pts[step][0] - eaves_overhang
		external_front_pts[step][0] = external_front_pts[step][0] + full_length + eaves_overhang
	end

	outside_front_count = outside_front_pts.length
	inside_front_pts = []
	for step in 0..current_stepx - 1
		inside_front_pts[step] = []
		temp_pts1 = inside_pts[step][1]
		temp_pts2 = inside_pts[step][2]
		inside_front_pts[step][0] = full_length - wall_thickness
		inside_front_pts[step][1] = temp_pts1
		inside_front_pts[step][2] = temp_pts2
	end
	if door_tail_height == 0
		inside_front_pts[current_stepx] = [full_length - wall_thickness, door_middle +(door_width / 2), 0]
		inside_front_pts[current_stepx + 1] = [full_length - wall_thickness, door_middle +(door_width / 2), door_height]
		inside_front_pts[current_stepx + 2] = [full_length - wall_thickness, door_middle -(door_width / 2), door_height]
		inside_front_pts[current_stepx + 3] = [full_length - wall_thickness, door_middle -(door_width / 2), 0]
		outside_front_pts[outside_front_count] = [full_length, door_middle -(door_width / 2), 0]
		outside_front_pts[outside_front_count + 1] = [full_length, door_middle -(door_width / 2), door_height]
		outside_front_pts[outside_front_count + 2] = [full_length, door_middle +(door_width / 2), door_height]
		outside_front_pts[outside_front_count + 3] = [full_length, door_middle +(door_width / 2), 0]
	else
		inside_front_pts[current_stepx] = [full_length - wall_thickness, door_middle +(door_width / 2), 0]
		inside_front_pts[current_stepx + 1] = [full_length - wall_thickness, door_middle +(door_width / 2), door_height]
		inside_front_pts[current_stepx + 2] = [full_length - wall_thickness, door_middle +(door_tail_width / 2), door_height]
		inside_front_pts[current_stepx + 3] = [full_length - wall_thickness, door_middle +(door_tail_width / 2), door_height + door_tail_height]
		inside_front_pts[current_stepx + 4] = [full_length - wall_thickness, door_middle -(door_tail_width / 2), door_height + door_tail_height]
		inside_front_pts[current_stepx + 5] = [full_length - wall_thickness, door_middle -(door_tail_width / 2), door_height]
		inside_front_pts[current_stepx + 6] = [full_length - wall_thickness, door_middle -(door_width / 2), door_height]
		inside_front_pts[current_stepx + 7] = [full_length - wall_thickness, door_middle -(door_width / 2), 0]
		outside_front_pts[outside_front_count] = [full_length, door_middle -(door_width / 2), 0]
		outside_front_pts[outside_front_count + 1] = [full_length, door_middle -(door_width / 2), door_height]
		outside_front_pts[outside_front_count + 2] = [full_length, door_middle -(door_tail_width / 2), door_height]
		outside_front_pts[outside_front_count + 3] = [full_length, door_middle -(door_tail_width / 2), door_height + door_tail_height]
		outside_front_pts[outside_front_count + 4] = [full_length, door_middle +(door_tail_width / 2), door_height + door_tail_height]
		outside_front_pts[outside_front_count + 5] = [full_length, door_middle +(door_tail_width / 2), door_height]
		outside_front_pts[outside_front_count + 6] = [full_length, door_middle +(door_width / 2), door_height]
		outside_front_pts[outside_front_count + 7] = [full_length, door_middle +(door_width / 2), 0]
	end
	
	for step in 0..outside_front_pts.length - 1
		outside_front_pts[step][0] = full_length
	end

	
####################################################
####	Draw UV map walls			####
####################################################

	uv_map_left_wall = 0
	uv_map_right_wall = 0
	uv_map_back_wall = 0
	uv_map_back_inside_wall = 0
	uv_map_front_inside_wall = 0
	uv_map_front_wall = 0
	uv_map_floor = 0
	uv_map_roof = 0
	uv_map_ceiling = 0
	uv_map_group = []
	uv_map_group_ents = []
	
	section_step = 0
	external_section_length = external_pts[arch_detail_level - 1][1]
	external_section_height = external_pts[arch_detail_level - 1][2]
	external_section_length2 = external_pts[arch_detail_level - 2][1]
	external_section_height2 = external_pts[arch_detail_level - 2][2]
	external_section_size = Math.sqrt((external_section_length * external_section_length) +(external_section_height * external_section_height))
	initial_roof_angle = Math.atan(external_section_length / external_section_height).radians
	second_roof_angle = Math.atan(external_section_length2 / external_section_height2).radians - initial_roof_angle
	internal_section_length = external_pts[arch_detail_level + 2][1] - external_pts[arch_detail_level + 1][1]
	internal_section_height = external_pts[arch_detail_level + 2][2]
	internal_section_length2 = external_pts[arch_detail_level + 3][1] - external_pts[arch_detail_level + 2][1]
	internal_section_height2 = external_pts[arch_detail_level + 3][2] - external_pts[arch_detail_level + 2][2] 
	internal_section_size = Math.sqrt((internal_section_length * internal_section_length) +(internal_section_height * internal_section_height))
	internal_initial_roof_angle = Math.atan(internal_section_length / internal_section_height).radians
	internal_second_roof_angle = Math.atan(internal_section_length2 / internal_section_height2).radians - internal_initial_roof_angle
	arc_curve_ratio = internal_section_size / external_section_size
	
	if uv_map_count > 0
		for step in 0..uv_map_count - 1
			texture_id = ""
			if uv_map_wall[step] == "ER"
				texture_width = full_length
				texture_height = external_section_size * arch_detail_level
				
				for texture_section_step in 0..arch_detail_level - 1
					uv_map_group[section_step] = Sketchup.active_model.entities.add_group
					uv_map_group_ents[section_step] = uv_map_group[section_step].entities
					if texture_id == ""
						texture_id = uv_map_group[section_step].to_s
					end
					for texture_step in 0..$texture_name.length - 1
						if uv_map_texture[step] == $texture_name[texture_step]
							texture = materials.add $texture_name[texture_step] + texture_id
              if $texture_filename[texture_step].count(":") === 0
                texture.texture = $texture_files + $texture_filename[texture_step]
              else
                texture.texture = $texture_filename[texture_step]
              end
							uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
							uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
							texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
						end
					end
					roof_angle = initial_roof_angle + second_roof_angle * texture_section_step * 2
					pts = []
					pts[0] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
					pts[1] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + full_length, 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
					pts[2] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + full_length, 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
					pts[3] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
					face = uv_map_group_ents[section_step].add_face pts
					status = face.material = uv_map_texture[step] + texture_id

					if roof_window_count > 0
						for roof_window_step in 0..roof_window_count - 1
							if roof_window_y_pos[roof_window_step] < external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * texture_section_step + external_section_size
								pts = []
								pts[0] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
								pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
								pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
								pts[3] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!
							elsif roof_window_y_pos[roof_window_step] >= external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] < external_section_size * texture_section_step + external_section_size
								pts = []
								pts[0] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + roof_window_y_pos[roof_window_step]]
								pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + roof_window_y_pos[roof_window_step]]
								pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
								pts[3] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step + external_section_size]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!
							elsif roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] <= external_section_size * texture_section_step + external_section_size
								pts = []
								pts[0] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
								pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + external_section_size * texture_section_step]
								pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]]
								pts[3] = [roof_window_x_pos[roof_window_step], 0, uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!
							end
						end
					end


					point = Geom::Point3d.new(0 - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], external_pts[arch_detail_level - texture_section_step][1], external_pts[arch_detail_level - texture_section_step][2] - external_section_size * texture_section_step - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
					t = Geom::Transformation.new point
					uv_map_group[section_step].move! t
					t = Geom::Transformation.rotation [0, external_pts[arch_detail_level - texture_section_step][1], external_pts[arch_detail_level - texture_section_step][2]], X_AXIS, 0 - roof_angle.degrees
					uv_map_group[section_step].transform! t
					
					section_step = section_step + 1
					
				end
				uv_map_roof = 1
			elsif uv_map_wall[step] == "IR"
				texture_width = full_length - wall_thickness * 2
				texture_height = internal_section_size * arch_detail_level

				for texture_section_step in 0..arch_detail_level - 1
					uv_map_group[section_step] = Sketchup.active_model.entities.add_group
					uv_map_group_ents[section_step] = uv_map_group[section_step].entities
					if texture_id == ""
						texture_id = uv_map_group[section_step].to_s
					end
					for texture_step in 0..$texture_name.length - 1
						if uv_map_texture[step] == $texture_name[texture_step]
							texture = materials.add $texture_name[texture_step] + texture_id
              if $texture_filename[texture_step].count(":") === 0
                texture.texture = $texture_files + $texture_filename[texture_step]
              else
                texture.texture = $texture_filename[texture_step]
              end
							uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
							uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
							texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
						end
					end
					roof_angle = internal_initial_roof_angle + internal_second_roof_angle * texture_section_step
					pts = []
					pts[0] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
					pts[1] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + full_length - wall_thickness * 2, external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
					pts[2] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + full_length - wall_thickness * 2, external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
					pts[3] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
					face = uv_map_group_ents[section_step].add_face pts
					status = face.material = uv_map_texture[step] + texture_id

					if roof_window_count > 0
						for roof_window_step in 0..roof_window_count - 1
							if roof_window_y_pos[roof_window_step] * arc_curve_ratio < texture_height - internal_section_size * texture_section_step - internal_section_size and(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio > texture_height - internal_section_size * texture_section_step
								pts = []
								pts[0] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
								pts[1] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
								pts[2] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
								pts[3] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!	
							elsif roof_window_y_pos[roof_window_step] * arc_curve_ratio > texture_height - internal_section_size * texture_section_step - internal_section_size and roof_window_y_pos[roof_window_step] * arc_curve_ratio < texture_height - internal_section_size * texture_section_step
								pts = []
								pts[0] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
								pts[1] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + texture_height - roof_window_y_pos[roof_window_step] * arc_curve_ratio]
								pts[2] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + texture_height - roof_window_y_pos[roof_window_step] * arc_curve_ratio]
								pts[3] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!
							elsif(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio > texture_height - internal_section_size * texture_section_step - internal_section_size and(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio < texture_height - internal_section_size * texture_section_step
								pts = []
								pts[0] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
								pts[1] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + internal_section_size * texture_section_step + internal_section_size]
								pts[2] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + texture_height -(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio]
								pts[3] = [uv_map_x_pos[step] * texture_width / uv_map_x_size[step] + roof_window_x_pos[roof_window_step], external_pts[0][1], uv_map_y_pos[step] * texture_height / uv_map_y_size[step] + texture_height -(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio]
								face = uv_map_group_ents[section_step].add_face pts
								status = face.erase!
							end
						end
					end
					point = Geom::Point3d.new(wall_thickness - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], 0 - external_pts[arch_detail_level + 1 + texture_section_step][1], external_pts[external_pts.length - 1 - texture_section_step][2] - internal_section_size * texture_section_step - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
					t = Geom::Transformation.new point
					uv_map_group[section_step].move! t
					t = Geom::Transformation.rotation [0, external_pts[external_pts.length - 1 - texture_section_step][1], external_pts[external_pts.length - 1 - texture_section_step][2]], X_AXIS, roof_angle.degrees
					uv_map_group[section_step].transform! t
					
					section_step = section_step + 1
				end
				uv_map_ceiling = 1
			elsif uv_map_wall[step] == "B" and uv_map_back_wall == 0
				texture_width = x_coord * 2
				texture_height = outside_height
				uv_map_group[section_step] = Sketchup.active_model.entities.add_group
				uv_map_group_ents[section_step] = uv_map_group[section_step].entities
				texture_id = uv_map_group[section_step].to_s
				for texture_step in 0..$texture_name.length - 1
					if uv_map_texture[step] == $texture_name[texture_step]
						texture = materials.add $texture_name[texture_step] + texture_id
              if $texture_filename[texture_step].count(":") === 0
                texture.texture = $texture_files + $texture_filename[texture_step]
              else
                texture.texture = $texture_filename[texture_step]
              end
						uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
						uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
						texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
					end
				end
				uv_back_pts = []
				for pts_step in 0..outside_pts.length - 1
					temp_pts0 = outside_pts[pts_step][1] + uv_map_x_pos[step] * texture_width / uv_map_x_size[step]
					temp_pts1 = outside_pts[pts_step][0]
					temp_pts2 = outside_pts[pts_step][2] + uv_map_y_pos[step] * texture_height / uv_map_y_size[step]
					uv_back_pts[pts_step] = []
					uv_back_pts[pts_step][0] = temp_pts0
					uv_back_pts[pts_step][1] = temp_pts1
					uv_back_pts[pts_step][2] = temp_pts2
				end
				face = uv_map_group_ents[section_step].add_face uv_back_pts
				status = face.material = uv_map_texture[step] + texture_id
				point = Geom::Point3d.new(0 - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], texture_width, 0 - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
				t = Geom::Transformation.new point
				uv_map_group[section_step].move! t
				t = Geom::Transformation.rotation [0, texture_width, 0], Z_AXIS, -90.degrees
				uv_map_group[section_step].transform! t
				
				uv_map_back_wall = 1
				section_step = section_step + 1
			elsif uv_map_wall[step] == "IB" and uv_map_back_inside_wall == 0
				texture_width = internal_width
				texture_height = arc_height
				uv_map_group[section_step] = Sketchup.active_model.entities.add_group
				uv_map_group_ents[section_step] = uv_map_group[section_step].entities
				texture_id = uv_map_group[section_step].to_s
				for texture_step in 0..$texture_name.length - 1
					if uv_map_texture[step] == $texture_name[texture_step]
						texture = materials.add $texture_name[texture_step] + texture_id
            if $texture_filename[texture_step].count(":") === 0
              texture.texture = $texture_files + $texture_filename[texture_step]
            else
              texture.texture = $texture_filename[texture_step]
            end
						uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
						uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
						texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
					end
				end
				uv_back_pts = []
				step_counter = inside_pts.length - 1
				for pts_step in 0..inside_pts.length - 1
					temp_pts0 = inside_pts[pts_step][1] + uv_map_x_pos[step] * texture_width / uv_map_x_size[step]
					temp_pts1 = inside_pts[pts_step][0]
					temp_pts2 = inside_pts[pts_step][2] + uv_map_y_pos[step] * texture_height / uv_map_y_size[step]
					uv_back_pts[step_counter] = []
					uv_back_pts[step_counter][0] = temp_pts0
					uv_back_pts[step_counter][1] = 0
					uv_back_pts[step_counter][2] = temp_pts2
					step_counter = step_counter - 1
				end
				face = uv_map_group_ents[section_step].add_face uv_back_pts
				status = face.material = uv_map_texture[step] + texture_id
				point = Geom::Point3d.new(0 - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], uv_back_pts[uv_back_pts.length - 1][0], 0 - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
				t = Geom::Transformation.new point
				uv_map_group[section_step].move! t
				t = Geom::Transformation.rotation [wall_thickness, uv_back_pts[uv_back_pts.length - 1][0], 0], Z_AXIS, 90.degrees
				uv_map_group[section_step].transform! t
				
				uv_map_back_inside_wall = 1
				section_step = section_step + 1
			elsif uv_map_wall[step] == "F" and uv_map_front_wall == 0
				texture_width = x_coord * 2
				texture_height = outside_height
				uv_map_group[section_step] = Sketchup.active_model.entities.add_group
				uv_map_group_ents[section_step] = uv_map_group[section_step].entities
				texture_id = uv_map_group[section_step].to_s
				for texture_step in 0..$texture_name.length - 1
					if uv_map_texture[step] == $texture_name[texture_step]
						texture = materials.add $texture_name[texture_step] + texture_id
            if $texture_filename[texture_step].count(":") === 0
              texture.texture = $texture_files + $texture_filename[texture_step]
            else
              texture.texture = $texture_filename[texture_step]
            end
						uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
						uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
						texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
					end
				end
				uv_back_pts = []
				for pts_step in 0..outside_front_pts.length - 1
					temp_pts0 = outside_front_pts[pts_step][1] + uv_map_x_pos[step] * texture_width / uv_map_x_size[step]
					temp_pts1 = outside_front_pts[pts_step][0]
					temp_pts2 = outside_front_pts[pts_step][2] + uv_map_y_pos[step] * texture_height / uv_map_y_size[step]
					uv_back_pts[pts_step] = []
					uv_back_pts[pts_step][0] = temp_pts0
					uv_back_pts[pts_step][1] = 0
					uv_back_pts[pts_step][2] = temp_pts2
				end
				face = uv_map_group_ents[section_step].add_face uv_back_pts
				status = face.material = uv_map_texture[step] + texture_id
				point = Geom::Point3d.new(full_length - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], 0, 0 - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
				t = Geom::Transformation.new point
				uv_map_group[section_step].move! t
				t = Geom::Transformation.rotation [full_length, 0, 0], Z_AXIS, 90.degrees
				uv_map_group[section_step].transform! t
				
				uv_map_front_wall = 1
				section_step = section_step + 1
			elsif uv_map_wall[step] == "IF" and uv_map_front_inside_wall == 0
				texture_width = internal_width
				texture_height = arc_height
				uv_map_group[section_step] = Sketchup.active_model.entities.add_group
				uv_map_group_ents[section_step] = uv_map_group[section_step].entities
				texture_id = uv_map_group[section_step].to_s
				for texture_step in 0..$texture_name.length - 1
					if uv_map_texture[step] == $texture_name[texture_step]
						texture = materials.add $texture_name[texture_step] + texture_id
            if $texture_filename[texture_step].count(":") === 0
              texture.texture = $texture_files + $texture_filename[texture_step]
            else
              texture.texture = $texture_filename[texture_step]
            end
						uv_x_scale =($texture_x_size[texture_step] / $inch) / uv_map_x_size[step]
						uv_y_scale =($texture_y_size[texture_step] / $inch) / uv_map_y_size[step]
						texture.texture.size = [uv_map_x_size[step] * uv_x_scale /(uv_map_x_size[step] / texture_width), uv_map_y_size[step] * uv_y_scale /(uv_map_y_size[step] / texture_height)]
					end
				end
				uv_back_pts = []
				step_counter = inside_front_pts.length - 1
				for pts_step in 0..inside_front_pts.length - 1
					temp_pts0 = inside_front_pts[pts_step][1] + uv_map_x_pos[step] * texture_width / uv_map_x_size[step]
					temp_pts1 = inside_front_pts[pts_step][0]
					temp_pts2 = inside_front_pts[pts_step][2] + uv_map_y_pos[step] * texture_height / uv_map_y_size[step]
					uv_back_pts[step_counter] = []
					uv_back_pts[step_counter][0] = temp_pts0
					uv_back_pts[step_counter][1] = 0
					uv_back_pts[step_counter][2] = temp_pts2
					step_counter = step_counter - 1
				end
				face = uv_map_group_ents[section_step].add_face uv_back_pts
				status = face.material = uv_map_texture[step] + texture_id
				if door_tail_height == 0
					point = Geom::Point3d.new(full_length - wall_thickness * 2 - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], uv_back_pts[4][0], 0 - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
				else
					point = Geom::Point3d.new(full_length - wall_thickness * 2 - uv_map_x_pos[step] * texture_width / uv_map_x_size[step], uv_back_pts[8][0], 0 - uv_map_y_pos[step] * texture_height / uv_map_y_size[step])
				end
				t = Geom::Transformation.new point
				uv_map_group[section_step].move! t
				if door_tail_height == 0
					t = Geom::Transformation.rotation [full_length - wall_thickness, uv_back_pts[4][0], 0], Z_AXIS, -90.degrees
				else
					t = Geom::Transformation.rotation [full_length - wall_thickness, uv_back_pts[8][0], 0], Z_AXIS, -90.degrees
				end
				uv_map_group[section_step].transform! t
				
				uv_map_front_inside_wall = 1
				section_step = section_step + 1
			end
		end
		for step in 0..section_step - 1
			group_text = group.description
			group = group.parent.entities.add_group([group, uv_map_group[step]])
			group.description = group_text
		end

	end


####################################################
####	Draw walls and roof			####
####################################################

	if uv_map_roof == 0
		texture_width = full_length
		texture_height = external_section_size * arch_detail_level
		roof_group = []	
		roof_group_ents = []	
		for texture_section_step in 0..arch_detail_level - 1
			roof_group[texture_section_step] = Sketchup.active_model.entities.add_group
			roof_group_ents[texture_section_step] = roof_group[texture_section_step].entities
			roof_angle = initial_roof_angle + second_roof_angle * texture_section_step * 2
			pts = []
			pts[0] = [0, 0, external_section_size * texture_section_step]
			pts[1] = [full_length, 0, external_section_size * texture_section_step]
			pts[2] = [full_length, 0, external_section_size * texture_section_step + external_section_size]
			pts[3] = [0, 0, external_section_size * texture_section_step + external_section_size]
			face = roof_group_ents[texture_section_step].add_face pts
			status = face.material = roof_texture
			if roof_window_count > 0
				for roof_window_step in 0..roof_window_count - 1
					if roof_window_y_pos[roof_window_step] < external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * texture_section_step + external_section_size
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step], 0, external_section_size * texture_section_step]
						pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, external_section_size * texture_section_step]
						pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, external_section_size * texture_section_step + external_section_size]
						pts[3] = [roof_window_x_pos[roof_window_step], 0, external_section_size * texture_section_step + external_section_size]
						face = roof_group_ents[texture_section_step].add_face pts
						status = face.erase!
					elsif roof_window_y_pos[roof_window_step] >= external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] < external_section_size * texture_section_step + external_section_size
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step], 0, roof_window_y_pos[roof_window_step]]
						pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, roof_window_y_pos[roof_window_step]]
						pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, external_section_size * texture_section_step + external_section_size]
						pts[3] = [roof_window_x_pos[roof_window_step], 0, external_section_size * texture_section_step + external_section_size]
						face = roof_group_ents[texture_section_step].add_face pts
						status = face.erase!
					elsif roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * texture_section_step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] <= external_section_size * texture_section_step + external_section_size
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step], 0, external_section_size * texture_section_step]
						pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, external_section_size * texture_section_step]
						pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], 0, roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]]
						pts[3] = [roof_window_x_pos[roof_window_step], 0, roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]]
						face = roof_group_ents[texture_section_step].add_face pts
						status = face.erase!
					end
				end
			end
			point = Geom::Point3d.new(0, external_pts[arch_detail_level - texture_section_step][1], external_pts[arch_detail_level - texture_section_step][2] - external_section_size * texture_section_step)
			t = Geom::Transformation.new point
			roof_group[texture_section_step].move! t
			t = Geom::Transformation.rotation [0, external_pts[arch_detail_level - texture_section_step][1], external_pts[arch_detail_level - texture_section_step][2]], X_AXIS, 0 - roof_angle.degrees
			roof_group[texture_section_step].transform! t
		end
		for step in 0..arch_detail_level - 1
			group_text = group.description
			group = group.parent.entities.add_group([group, roof_group[step]])
			group.description = group_text
		end
	end
	if uv_map_ceiling == 0
		texture_width = full_length - wall_thickness * 2
		texture_height = internal_section_size * arch_detail_level
		ceiling_group = []	
		ceiling_group_ents = []	
		for texture_section_step in 0..arch_detail_level - 1
			ceiling_group[texture_section_step] = Sketchup.active_model.entities.add_group
			ceiling_group_ents[texture_section_step] = ceiling_group[texture_section_step].entities
			roof_angle = internal_initial_roof_angle + internal_second_roof_angle * texture_section_step
			pts = []
			pts[0] = [wall_thickness, external_pts[0][1], internal_section_size * texture_section_step]
			pts[1] = [full_length - wall_thickness, external_pts[0][1], internal_section_size * texture_section_step]
			pts[2] = [full_length - wall_thickness, external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
			pts[3] = [wall_thickness, external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
			face = ceiling_group_ents[texture_section_step].add_face pts
			status = face.material = roof_texture
			if roof_window_count > 0
				for roof_window_step in 0..roof_window_count - 1
					if roof_window_y_pos[roof_window_step] * arc_curve_ratio < texture_height - internal_section_size * texture_section_step - internal_section_size and(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio > texture_height - internal_section_size * texture_section_step
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step]
						pts[1] = [roof_window_x_pos[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
						pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
						pts[3] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step]
						face = ceiling_group_ents[texture_section_step].add_face pts
						status = face.erase!
					elsif roof_window_y_pos[roof_window_step] * arc_curve_ratio > texture_height - internal_section_size * texture_section_step - internal_section_size and roof_window_y_pos[roof_window_step] * arc_curve_ratio < texture_height - internal_section_size * texture_section_step
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step]
						pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], texture_height - roof_window_y_pos[roof_window_step] * arc_curve_ratio]
						pts[2] = [roof_window_x_pos[roof_window_step], external_pts[0][1], texture_height - roof_window_y_pos[roof_window_step] * arc_curve_ratio]
						pts[3] = [roof_window_x_pos[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step]
						face = ceiling_group_ents[texture_section_step].add_face pts
						status = face.erase!
					elsif(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio > texture_height - internal_section_size * texture_section_step - internal_section_size and(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio < texture_height - internal_section_size * texture_section_step
						pts = []
						pts[0] = [roof_window_x_pos[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
						pts[1] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], internal_section_size * texture_section_step + internal_section_size]
						pts[2] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[0][1], texture_height -(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio]
						pts[3] = [roof_window_x_pos[roof_window_step], external_pts[0][1], texture_height -(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step]) * arc_curve_ratio]
						face = ceiling_group_ents[texture_section_step].add_face pts
						status = face.erase!
					end
				end
			end
	
			point = Geom::Point3d.new(0, 0 - external_pts[arch_detail_level + 1 + texture_section_step][1], external_pts[external_pts.length - 1 - texture_section_step][2] - internal_section_size * texture_section_step)
			t = Geom::Transformation.new point
			ceiling_group[texture_section_step].move! t
			t = Geom::Transformation.rotation [0, external_pts[external_pts.length - 1 - texture_section_step][1], external_pts[external_pts.length - 1 - texture_section_step][2]], X_AXIS, roof_angle.degrees
			ceiling_group[texture_section_step].transform! t
						
		end
		for step in 0..arch_detail_level - 1
			group_text = group.description
			group = group.parent.entities.add_group([group, ceiling_group[step]])
			group.description = group_text
		end
	end
	if uv_map_back_inside_wall == 0
		face = group_ents.add_face inside_pts
		status = face.reverse!
		status = face.material = wall_texture
	end
	if uv_map_back_wall == 0
		face = group_ents.add_face outside_pts
		status = face.reverse!
		status = face.material = wall_texture
	end

	# Draw roof windows or cutout

	roof_window_number = 0
	roof_window_fit_location = []
	roof_window_group = []
	roof_window_group_ents = []
	if roof_window_count > 0
		for roof_window_step in 0..roof_window_count - 1
			texture_id = ""
			for step in 0..arch_detail_level - 1
				if roof_window_y_pos[roof_window_step] < external_section_size * step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * step + external_section_size
					pts = []
					pts[0] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step - 1][1], external_pts[arch_detail_level - step - 1][2]]
					pts[1] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step][1], external_pts[arch_detail_level - step][2]]
					pts[2] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 1][1], external_pts[arch_detail_level + step + 1][2]]
					pts[3] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 2][1], external_pts[arch_detail_level + step + 2][2]]
					face = group_ents.add_face pts
					status = face.material = roof_texture
					pts[4] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step - 1][1], external_pts[arch_detail_level - step - 1][2]]
					pts[5] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step][1], external_pts[arch_detail_level - step][2]]
					pts[6] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 1][1], external_pts[arch_detail_level + step + 1][2]]
					pts[7] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 2][1], external_pts[arch_detail_level + step + 2][2]]
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					if face.normal[0] > 0
						status = face.reverse!
					end
					status = face.material = roof_texture
					if roof_window_side[roof_window_step] == "W"
						roof_window_group[roof_window_number] = Sketchup.active_model.entities.add_group
						roof_window_group_ents[roof_window_number] = roof_window_group[roof_window_number].entities
						if texture_id == ""
							texture_id = roof_window_group[roof_window_number].to_s
							for texture_step in 0..$texture_name.length - 1
								if roof_window_texture[roof_window_step] == $texture_name[texture_step]
									texture = materials.add $texture_name[texture_step] + texture_id
                  if $texture_filename[texture_step].count(":") === 0
                    texture.texture = $texture_files + $texture_filename[texture_step]
                  else
                    texture.texture = $texture_filename[texture_step]
                  end
									texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
								end
							end
						end
						
						face = roof_window_group_ents[roof_window_number].add_face pts[0], pts[1], pts[5], pts[4]
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
						face = roof_window_group_ents[roof_window_number].add_face pts[2], pts[3], pts[7], pts[6]
						roof_window_number = roof_window_number + 1
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
					end
				elsif roof_window_y_pos[roof_window_step] >= external_section_size * step and roof_window_y_pos[roof_window_step] < external_section_size * step + external_section_size
					section_cut_ratio =((external_section_size * step + external_section_size) - roof_window_y_pos[roof_window_step]) / external_section_size
					section_external_height_difference =(external_pts[arch_detail_level - step - 1][2] - external_pts[arch_detail_level - step][2])
					section_external_depth_difference =(external_pts[arch_detail_level - step - 1][1] - external_pts[arch_detail_level - step][1])
					section_internal_height_difference =(external_pts[arch_detail_level + step + 1][2] - external_pts[arch_detail_level + step + 2][2])
					section_internal_depth_difference =(external_pts[arch_detail_level + step + 1][1] - external_pts[arch_detail_level + step + 2][1])
					pts = []
					pts[0] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step - 1][1], external_pts[arch_detail_level - step - 1][2]]
					pts[1] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step - 1][1] - section_external_depth_difference * section_cut_ratio, external_pts[arch_detail_level - step - 1][2] - section_external_height_difference * section_cut_ratio]
					pts[2] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 2][1] + section_internal_depth_difference * section_cut_ratio, external_pts[arch_detail_level + step + 2][2] + section_internal_height_difference * section_cut_ratio]
					pts[3] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 2][1], external_pts[arch_detail_level + step + 2][2]]
					face = group_ents.add_face pts
					status = face.material = roof_texture
					pts[4] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step - 1][1], external_pts[arch_detail_level - step - 1][2]]
					pts[5] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step - 1][1] - section_external_depth_difference * section_cut_ratio, external_pts[arch_detail_level - step - 1][2] - section_external_height_difference * section_cut_ratio]
					pts[6] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 2][1] + section_internal_depth_difference * section_cut_ratio, external_pts[arch_detail_level + step + 2][2] + section_internal_height_difference * section_cut_ratio]
					pts[7] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 2][1], external_pts[arch_detail_level + step + 2][2]]
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					if face.normal[0] > 0
						status = face.reverse!
					end
					status = face.material = roof_texture
					if external_section_size * step > 0
						face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
						status = face.material = roof_texture
					end
					if roof_window_side[roof_window_step] == "W"
						roof_window_group[roof_window_number] = Sketchup.active_model.entities.add_group
						roof_window_group_ents[roof_window_number] = roof_window_group[roof_window_number].entities
						if texture_id == ""
							texture_id = roof_window_group[roof_window_number].to_s
							for texture_step in 0..$texture_name.length - 1
								if roof_window_texture[roof_window_step] == $texture_name[texture_step]
									texture = materials.add $texture_name[texture_step] + texture_id
                  if $texture_filename[texture_step].count(":") === 0
                    texture.texture = $texture_files + $texture_filename[texture_step]
                  else
                    texture.texture = $texture_filename[texture_step]
                  end
									texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
								end
							end
						end
						face = roof_window_group_ents[roof_window_number].add_face pts[0], pts[1], pts[5], pts[4]
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
						face = roof_window_group_ents[roof_window_number].add_face pts[2], pts[3], pts[7], pts[6]
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
						roof_window_number = roof_window_number + 1
					end
				elsif roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] > external_section_size * step and roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step] <= external_section_size * step + external_section_size
					section_cut_ratio = 1 -((external_section_size * step + external_section_size) -(roof_window_y_pos[roof_window_step] + roof_window_y_size[roof_window_step])) / external_section_size
					section_external_height_difference =(external_pts[arch_detail_level - step - 1][2] - external_pts[arch_detail_level - step][2])
					section_external_depth_difference =(external_pts[arch_detail_level - step - 1][1] - external_pts[arch_detail_level - step][1])
					section_internal_height_difference =(external_pts[arch_detail_level + step + 1][2] - external_pts[arch_detail_level + step + 2][2])
					section_internal_depth_difference =(external_pts[arch_detail_level + step + 1][1] - external_pts[arch_detail_level + step + 2][1])
					roof_window_fit_location[roof_window_step] = external_pts[arch_detail_level - step][1] + section_external_depth_difference * section_cut_ratio
					pts = []
					pts[0] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step][1], external_pts[arch_detail_level - step][2]]
					pts[1] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level - step][1] + section_external_depth_difference * section_cut_ratio, external_pts[arch_detail_level - step][2] + section_external_height_difference * section_cut_ratio]
					pts[2] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 1][1] - section_internal_depth_difference * section_cut_ratio, external_pts[arch_detail_level + step + 1][2] - section_internal_height_difference * section_cut_ratio]
					pts[3] = [roof_window_x_pos[roof_window_step], external_pts[arch_detail_level + step + 1][1], external_pts[arch_detail_level + step + 1][2]]
					face = group_ents.add_face pts
					status = face.material = roof_texture
					pts[4] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step][1], external_pts[arch_detail_level - step][2]]
					pts[5] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level - step][1] + section_external_depth_difference * section_cut_ratio, external_pts[arch_detail_level - step][2] + section_external_height_difference * section_cut_ratio]
					pts[6] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 1][1] - section_internal_depth_difference * section_cut_ratio, external_pts[arch_detail_level + step + 1][2] - section_internal_height_difference * section_cut_ratio]
					pts[7] = [roof_window_x_pos[roof_window_step] + roof_window_x_size[roof_window_step], external_pts[arch_detail_level + step + 1][1], external_pts[arch_detail_level + step + 1][2]]
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					if face.normal[0] > 0
						status = face.reverse!
					end
					status = face.material = roof_texture
					face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
					status = face.material = roof_texture
					if roof_window_side[roof_window_step] == "W"
						roof_window_group[roof_window_number] = Sketchup.active_model.entities.add_group
						roof_window_group_ents[roof_window_number] = roof_window_group[roof_window_number].entities
						if texture_id == ""
							texture_id = roof_window_group[roof_window_number].to_s
							for texture_step in 0..$texture_name.length - 1
								if roof_window_texture[roof_window_step] == $texture_name[texture_step]
									texture = materials.add $texture_name[texture_step] + texture_id
                  if $texture_filename[texture_step].count(":") === 0
                    texture.texture = $texture_files + $texture_filename[texture_step]
                  else
                    texture.texture = $texture_filename[texture_step]
                  end
									texture.texture.size = [$texture_x_size[texture_step].to_f / $inch, $texture_y_size[texture_step].to_f / $inch]
								end
							end
						end
						face = roof_window_group_ents[roof_window_number].add_face pts[0], pts[1], pts[5], pts[4]
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
						face = roof_window_group_ents[roof_window_number].add_face pts[2], pts[3], pts[7], pts[6]
						status = face.material = roof_window_texture[roof_window_step] + texture_id
						status = face.back_material = roof_window_texture[roof_window_step] + texture_id
						if roof_window_texture[roof_window_step] == roof_window_back_texture[roof_window_step]
							status = face.material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						else
							status = face.back_material.alpha = roof_window_transparency[roof_window_step].to_f / 100
						end
						roof_window_number = roof_window_number + 1
					end
				end
			end
		end
	end

####################################################
####	Draw hangar side extensions		####
####################################################

	if hangar_side_count > 0
		for step in 0..hangar_side_count - 1
			if hangar_side_type[step] == "HL"
				max_extension_height = 0
				connect_height = 0
				fit_match_found = 0
				if hangar_side_fit[step] == "F" and roof_window_count == 0
					UI.messagebox "Nothing to fit to"
				elsif hangar_side_fit[step] == "F"
					for extension_fit_step in 0..roof_window_count - 1
						if hangar_side_start[step] == roof_window_x_pos[extension_fit_step]
							hangar_side_depth[step] = roof_window_fit_location[extension_fit_step]
							hangar_side_length[step] = roof_window_x_size[extension_fit_step]
							fit_match_found = 1
						end
					end
				end
				external_connect_depth = hangar_side_depth[step] + wall_thickness
				for hangar_arch_step in 0..arch_detail_level - 2
					if connect_height == 0 and external_pts[arch_detail_level - hangar_arch_step][2] > max_extension_height
						max_extension_height = external_pts[arch_detail_level - hangar_arch_step][2]
					end
					if external_pts[arch_detail_level - hangar_arch_step][1] < hangar_side_depth[step] and external_pts[arch_detail_level - hangar_arch_step - 1][1] > hangar_side_depth[step]
						connect_height_difference = external_pts[arch_detail_level - hangar_arch_step - 1][2] - external_pts[arch_detail_level - hangar_arch_step][2]
						connect_depth_difference = external_pts[arch_detail_level - hangar_arch_step - 1][1] - external_pts[arch_detail_level - hangar_arch_step][1]
						connect_ratio =(hangar_side_depth[step] - external_pts[arch_detail_level - hangar_arch_step][1]) / connect_depth_difference
						connect_height = external_pts[arch_detail_level - hangar_arch_step][2] + connect_height_difference * connect_ratio
						last_arch_step = hangar_arch_step
						if connect_height > hangar_side_height[step]
							hangar_side_height[step] = connect_height
						end
						if max_extension_height > hangar_side_height[step]
							hangar_side_height[step] = max_extension_height
						end
					end
					if external_pts[arch_detail_level - hangar_arch_step][1] < external_connect_depth and external_pts[arch_detail_level - hangar_arch_step - 1][1] > external_connect_depth
						external_connect_height_difference = external_pts[arch_detail_level - hangar_arch_step - 1][2] - external_pts[arch_detail_level - hangar_arch_step][2]
						external_connect_depth_difference = external_pts[arch_detail_level - hangar_arch_step - 1][1] - external_pts[arch_detail_level - hangar_arch_step][1]
						external_connect_ratio =(external_connect_depth - external_pts[arch_detail_level - hangar_arch_step][1]) / external_connect_depth_difference
						external_connect_height = external_pts[arch_detail_level - hangar_arch_step][2] + external_connect_height_difference * external_connect_ratio
						last_external_arch_step = hangar_arch_step
					end
				end
				if connect_height + wall_thickness < outside_height and connect_height == hangar_side_height[step]
					external_connect_height = connect_height + wall_thickness
					for hangar_arch_step in 0..arch_detail_level - 2
						if external_pts[arch_detail_level - hangar_arch_step][2] < external_connect_height and external_pts[arch_detail_level - hangar_arch_step - 1][2] > external_connect_height
							external_connect_height_difference = external_pts[arch_detail_level - hangar_arch_step - 1][2] - external_pts[arch_detail_level - hangar_arch_step][2]
							external_connect_depth_difference = external_pts[arch_detail_level - hangar_arch_step - 1][1] - external_pts[arch_detail_level - hangar_arch_step][1]
							external_connect_ratio =(external_connect_height - external_pts[arch_detail_level - hangar_arch_step][2]) / connect_height_difference
							external_connect_depth = external_pts[arch_detail_level - hangar_arch_step][1] + external_connect_depth_difference * external_connect_ratio
							last_external_arch_step = hangar_arch_step
						end
					end
					if hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
						hangar_side_external_width = hangar_side_width[step]
					else
						hangar_side_external_width = hangar_side_width[step] + wall_thickness
					end
					pts = []
					pts[0] = [hangar_side_start[step] - wall_thickness, external_connect_depth, external_connect_height]
					pts[1] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, external_connect_depth, external_connect_height]
					pts[2] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_external_width, external_connect_height]
					pts[3] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_external_width, external_connect_height]
					face = group_ents.add_face pts
					if face.normal[2] < 0
						status = face.reverse!
					end
				else
					if hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
						hangar_side_external_width = hangar_side_width[step]
					else
						hangar_side_external_width = hangar_side_width[step] + wall_thickness
					end
					pts = []
					pts[0] = [hangar_side_start[step] - wall_thickness, external_connect_depth, external_connect_height]
					pts[1] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, external_connect_depth, external_connect_height]
					pts[2] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, external_connect_depth, hangar_side_height[step] + wall_thickness]
					pts[3] = [hangar_side_start[step] - wall_thickness, external_connect_depth, hangar_side_height[step] + wall_thickness]
					face = group_ents.add_face pts
					if face.normal[1] < 0
						status = face.reverse!
					end
					pts[4] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_external_width, hangar_side_height[step] + wall_thickness]
					pts[5] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_external_width, hangar_side_height[step] + wall_thickness]
					face = group_ents.add_face pts[2], pts[3], pts[4], pts[5]
				end
				
				
				
				pts = []
				pts[0] = [hangar_side_start[step], 0 - hangar_side_width[step], 0]
				pts[1] = [hangar_side_start[step], 0 - hangar_side_width[step], hangar_side_height[step]]
				pts[2] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], hangar_side_height[step]]
				pts[3] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], 0]
				if hangar_side_opening[step] != "O"
					face = group_ents.add_face pts
					if face.normal[1] > 0
						status = face.reverse!
					end
				end
				pts[4] = [hangar_side_start[step] + hangar_side_length[step], hangar_side_depth[step], hangar_side_height[step]]
				pts[5] = [hangar_side_start[step], hangar_side_depth[step], hangar_side_height[step]]
				pts[6] = [hangar_side_start[step], hangar_side_depth[step], connect_height]
				pts[7] = [hangar_side_start[step] + hangar_side_length[step], hangar_side_depth[step], connect_height]
				face = group_ents.add_face pts[1], pts[2], pts[4], pts[5]
				if face.normal[2] > 0
					status = face.reverse!
				end
				if hangar_side_height[step] != connect_height
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					if face.normal[1] > 0
						status = face.reverse!
					end
				end
				
				depth_pts = []
				height_pts = []
				external_depth_pts = []
				external_height_pts = []
				if hangar_side_height[step] != connect_height
					depth_pts[0] = hangar_side_depth[step]
					depth_pts[1] = hangar_side_depth[step]
					depth_pts[2] = 0 - hangar_side_width[step]
					depth_pts[3] = 0 - hangar_side_width[step]
					height_pts[0] = connect_height
					height_pts[1] = hangar_side_height[step]
					height_pts[2] = hangar_side_height[step]
					height_pts[3] = 0
					external_depth_pts[0] = hangar_side_depth[step] + wall_thickness
					external_depth_pts[1] = hangar_side_depth[step] + wall_thickness
					if hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
						external_depth_pts[2] = 0 - hangar_side_width[step]
						external_depth_pts[3] = 0 - hangar_side_width[step]
					else
						external_depth_pts[2] = 0 - hangar_side_width[step] - wall_thickness
						external_depth_pts[3] = 0 - hangar_side_width[step] - wall_thickness
					end
					external_height_pts[0] = external_connect_height
					external_height_pts[1] = hangar_side_height[step] + wall_thickness
					external_height_pts[2] = hangar_side_height[step] + wall_thickness
					external_height_pts[3] = 0
					addon_pts = 4
					external_addon_pts = 4
				else
					depth_pts[0] = hangar_side_depth[step]
					depth_pts[1] = 0 - hangar_side_width[step]
					depth_pts[2] = 0 - hangar_side_width[step]
					height_pts[0] = hangar_side_height[step]
					height_pts[1] = hangar_side_height[step]
					height_pts[2] = 0
					external_depth_pts[0] = external_connect_depth
					if hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
						external_depth_pts[1] = 0 - hangar_side_width[step]
						external_depth_pts[2] = 0 - hangar_side_width[step]
					else
						external_depth_pts[1] = 0 - hangar_side_width[step] - wall_thickness
						external_depth_pts[2] = 0 - hangar_side_width[step] - wall_thickness
					end
					external_height_pts[0] = hangar_side_height[step] + wall_thickness
					external_height_pts[1] = hangar_side_height[step] + wall_thickness
					external_height_pts[2] = 0
					addon_pts = 3
					external_addon_pts = 3
				end
				if hangar_side_width[step] == 0
					addon_pts = addon_pts - 1
				end
				if hangar_side_width[step] == 0 and hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
					external_addon_pts = external_addon_pts - 1
				end
				for pts_step in 0..last_arch_step
					depth_pts[pts_step + addon_pts] = external_pts[arch_detail_level - pts_step][1]
					height_pts[pts_step + addon_pts] = external_pts[arch_detail_level - pts_step][2]
				end
				for pts_step in 0..last_external_arch_step
					external_depth_pts[pts_step + external_addon_pts] = external_pts[arch_detail_level - pts_step][1]
					external_height_pts[pts_step + external_addon_pts] = external_pts[arch_detail_level - pts_step][2]
				end
				
				side_start_wall_pts = []
				side_end_wall_pts = []
				external_side_start_wall_pts = []
				external_side_end_wall_pts = []
				for side_wall_step in 0..last_arch_step + addon_pts
					side_start_wall_pts[side_wall_step] = [hangar_side_start[step], depth_pts[side_wall_step], height_pts[side_wall_step]]
					side_end_wall_pts[side_wall_step] = [hangar_side_start[step] + hangar_side_length[step], depth_pts[side_wall_step], height_pts[side_wall_step]]
				end
				for side_wall_step in 0..last_external_arch_step + external_addon_pts
					external_side_start_wall_pts[side_wall_step] = [hangar_side_start[step] - wall_thickness, external_depth_pts[side_wall_step], external_height_pts[side_wall_step]]
					external_side_end_wall_pts[side_wall_step] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, external_depth_pts[side_wall_step], external_height_pts[side_wall_step]]
				end
				face = group_ents.add_face side_start_wall_pts
				if face.normal[0] < 0
					status = face.reverse!
				end
				face = group_ents.add_face side_end_wall_pts
				if face.normal[0] > 0
					status = face.reverse!
				end
				face = group_ents.add_face external_side_start_wall_pts
				face = group_ents.add_face external_side_end_wall_pts
				
				if hangar_side_opening[step] == "O" and hangar_side_custom_opening_set[step] == 0
					pts = []
					pts[0] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_width[step], 0]
					pts[1] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_width[step], hangar_side_height[step] + wall_thickness]
					pts[2] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_width[step], hangar_side_height[step] + wall_thickness]
					pts[3] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_width[step], 0]
					pts[4] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], 0]
					pts[5] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], hangar_side_height[step]]
					pts[6] = [hangar_side_start[step], 0 - hangar_side_width[step], hangar_side_height[step]]
					pts[7] = [hangar_side_start[step], 0 - hangar_side_width[step], 0]
					face = group_ents.add_face pts
				end
				
				if hangar_side_width[step] > 0
					pts = []
					pts[0] = [hangar_side_start[step], 0 - hangar_side_width[step], 0]
					pts[1] = [hangar_side_start[step], 0, 0]
					pts[2] = [hangar_side_start[step] + hangar_side_length[step], 0, 0]
					pts[3] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], 0]
					face = group_ents.add_face pts
					status = face.material = floor_texture
				end
				if hangar_side_attribute_count[step] > 0
					custom_opening_set = 0
					if hangar_side_custom_opening_set[step] > 0
						pts = []
						ptsx = []
						pts[0] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], 0]
						pts[1] = [hangar_side_start[step] + hangar_side_length[step], 0 - hangar_side_width[step], hangar_side_height[step]]
						pts[2] = [hangar_side_start[step], 0 - hangar_side_width[step], hangar_side_height[step]]
						pts[3] = [hangar_side_start[step], 0 - hangar_side_width[step], 0]
						ptsx[0] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_width[step] - wall_thickness, 0]
						ptsx[1] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - hangar_side_width[step] - wall_thickness, hangar_side_height[step] + wall_thickness]
						ptsx[2] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_width[step] - wall_thickness, hangar_side_height[step] + wall_thickness]
						ptsx[3] = [hangar_side_start[step] - wall_thickness, 0 - hangar_side_width[step] - wall_thickness, 0]
						minimum_opening_start = 0
						allowed_doors = 0
						allowed_door_info = []
						pts_start = 4
						for hangar_side_attribute_step in 0..hangar_side_attribute_count[step] - 1
							custom_opening_info = hangar_side_attributes[step][hangar_side_attribute_step].split(";")
							for coi_step in 0..2
								custom_opening_info[coi_step] = custom_opening_info[coi_step].to_f / $inch  
							end
							if custom_opening_info[0] > minimum_opening_start
								pts[pts_start] = [hangar_side_start[step] + custom_opening_info[0], 0 - hangar_side_width[step], 0]
								pts[pts_start + 1] = [hangar_side_start[step] + custom_opening_info[0], 0 - hangar_side_width[step], custom_opening_info[2]]
								pts[pts_start + 2] = [hangar_side_start[step] + custom_opening_info[0] + custom_opening_info[1], 0 - hangar_side_width[step], custom_opening_info[2]]
								pts[pts_start + 3] = [hangar_side_start[step] + custom_opening_info[0] + custom_opening_info[1], 0 - hangar_side_width[step], 0]
								ptsx[pts_start] = [hangar_side_start[step] + custom_opening_info[0], 0 - hangar_side_width[step] - wall_thickness, 0]
								ptsx[pts_start + 1] = [hangar_side_start[step] + custom_opening_info[0], 0 - hangar_side_width[step] - wall_thickness, custom_opening_info[2]]
								ptsx[pts_start + 2] = [hangar_side_start[step] + custom_opening_info[0] + custom_opening_info[1], 0 - hangar_side_width[step] - wall_thickness, custom_opening_info[2]]
								ptsx[pts_start + 3] = [hangar_side_start[step] + custom_opening_info[0] + custom_opening_info[1], 0 - hangar_side_width[step] - wall_thickness, 0]
								minimum_opening_start = custom_opening_info[0] + custom_opening_info[1]	
								pts_start = pts_start + 4
							#	allowed_door_info[allowed_doors] = []
								allowed_door_info[allowed_doors] = custom_opening_info
							#	allowed_door_info[allowed_doors][0] = custom_opening_info[0]
							#	allowed_door_info[allowed_doors][1] = custom_opening_info[1]
							#	allowed_door_info[allowed_doors][2] = custom_opening_info[2]
								allowed_doors = allowed_doors + 1
								custom_opening_set = 1
							end
						end
						
						face = group_ents.add_face pts
						face = group_ents.add_face ptsx
						
						if allowed_doors > 0
							for allowed_door_step in 0..allowed_doors - 1
								face = group_ents.add_face pts[allowed_door_step * 4 + 4], pts[allowed_door_step * 4 + 5], ptsx[allowed_door_step * 4 + 5], ptsx[allowed_door_step * 4 + 4]
								face = group_ents.add_face pts[allowed_door_step * 4 + 5], pts[allowed_door_step * 4 + 6], ptsx[allowed_door_step * 4 + 6], ptsx[allowed_door_step * 4 + 5]
								face = group_ents.add_face pts[allowed_door_step * 4 + 6], pts[allowed_door_step * 4 + 7], ptsx[allowed_door_step * 4 + 7], ptsx[allowed_door_step * 4 + 6]
							end
						end
						
						
					end
					for hangar_side_attribute_step in 0..hangar_side_attribute_count[step] - 1
						if hangar_side_attribute_type[step][hangar_side_attribute_step] == "HD"
							hangar_door_info = hangar_side_attributes[step][hangar_side_attribute_step].split(";")
							if custom_opening_set == 0
								door_group = insert_door(hangar_door_info[0], hangar_side_height[step], hangar_door_info[1].to_i, door_sections, hangar_side_length[step], hangar_side_start[step] + hangar_side_length[step] / 2, hangar_side_width[step] + wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, hangar_side_height[step] + wall_thickness, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "Arched", 0)
								group = group.parent.entities.add_group([group, door_group])
							else
								for allowed_door_step in 0..allowed_doors - 1
									door_group = insert_door(hangar_door_info[0], allowed_door_info[allowed_door_step][2], hangar_door_info[1].to_i, door_sections, allowed_door_info[allowed_door_step][1], hangar_side_start[step] + allowed_door_info[allowed_door_step][0] + allowed_door_info[allowed_door_step][1] / 2, hangar_side_width[step], wall_thickness, door_frame_thickness, door_skin_thickness, hangar_side_height[step] + wall_thickness, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "Arched", 0)
									group = group.parent.entities.add_group([group, door_group])
								end
							end
						end
						if hangar_side_attribute_type[step][hangar_side_attribute_step] == "W"
							side_window_info = hangar_side_attributes[step][hangar_side_attribute_step].split(";")
							for swi_step in 0..3
								side_window_info[swi_step] =side_window_info[swi_step].to_f / $inch 
							end
							side_window_info[6] = side_window_info[6].to_i
							if side_window_info[4] == "B"
								pts = []
								pts[0] = [hangar_side_start[step] - wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2]]
								pts[1] = [hangar_side_start[step] - wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2]]
								pts[2] = [hangar_side_start[step] - wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2] + side_window_info[3]]
								pts[3] = [hangar_side_start[step] - wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2] + side_window_info[3]]
								face = group_ents.add_face pts
								status = face.erase!
								pts[4] = [hangar_side_start[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2]]
								pts[5] = [hangar_side_start[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2]]
								pts[6] = [hangar_side_start[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2] + side_window_info[3]]
								pts[7] = [hangar_side_start[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2] + side_window_info[3]]
								face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
								status = face.erase!
								face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
								face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
								face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
								face = group_ents.add_face pts[3], pts[0], pts[4], pts[7]
								window_group = draw_window(side_window_info[0], side_window_info[2], side_window_info[1], side_window_info[3], side_window_info[4], side_window_info[5], side_window_info[5], side_window_info[6], wall_thickness, 0 - hangar_side_width[step] - wall_thickness, 0, 0, hangar_side_start[step] - wall_thickness)
								group = group.parent.entities.add_group([group, window_group])
							elsif side_window_info[4] == "F"
								pts = []
								pts[0] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2]]
								pts[1] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2]]
								pts[2] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2] + side_window_info[3]]
								pts[3] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness, 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2] + side_window_info[3]]
								face = group_ents.add_face pts
								status = face.erase!
								pts[4] = [hangar_side_start[step] + hangar_side_length[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2]]
								pts[5] = [hangar_side_start[step] + hangar_side_length[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2]]
								pts[6] = [hangar_side_start[step] + hangar_side_length[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0] + side_window_info[1], side_window_info[2] + side_window_info[3]]
								pts[7] = [hangar_side_start[step] + hangar_side_length[step], 0 - wall_thickness - hangar_side_width[step] + side_window_info[0], side_window_info[2] + side_window_info[3]]
								face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
								status = face.erase!
								face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
								face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
								face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
								face = group_ents.add_face pts[3], pts[0], pts[4], pts[7]
								window_group = draw_window(side_window_info[0], side_window_info[2], side_window_info[1], side_window_info[3], side_window_info[4], side_window_info[5], side_window_info[5], side_window_info[6], wall_thickness, 0 - hangar_side_width[step] - wall_thickness, 0, hangar_side_start[step]+ hangar_side_length[step] + wall_thickness, hangar_side_start[step] - wall_thickness)
								group = group.parent.entities.add_group([group, window_group])
							elsif side_window_info[4] == "L"
								allow_window = 1
								if allowed_doors > 0
									for door_step in 0..allowed_doors - 1
										if(hangar_side_length[step] + wall_thickness - side_window_info[0] < allowed_door_info[door_step][0] + allowed_door_info[door_step][1] and hangar_side_length[step] + wall_thickness - side_window_info[0] > allowed_door_info[door_step][0]) or(hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1] < allowed_door_info[door_step][0] + allowed_door_info[door_step][1] and hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1] > allowed_door_info[door_step][0]) 
											allow_window = 0
										end
									end
								end
								if allow_window == 1
									pts = []
									pts[0] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0], 0 - wall_thickness - hangar_side_width[step], side_window_info[2]]
									pts[1] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1], 0 - wall_thickness - hangar_side_width[step], side_window_info[2]]
									pts[2] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1], 0 - wall_thickness - hangar_side_width[step], side_window_info[2] + side_window_info[3]]
									pts[3] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0], 0 - wall_thickness - hangar_side_width[step], side_window_info[2] + side_window_info[3]]
									face = group_ents.add_face pts
									status = face.erase!
									pts[4] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0], 0 - hangar_side_width[step], side_window_info[2]]
									pts[5] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1], 0 - hangar_side_width[step], side_window_info[2]]
									pts[6] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0] - side_window_info[1], 0 - hangar_side_width[step], side_window_info[2] + side_window_info[3]]
									pts[7] = [hangar_side_start[step] + hangar_side_length[step] + wall_thickness - side_window_info[0], 0 - hangar_side_width[step], side_window_info[2] + side_window_info[3]]
									face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
									status = face.erase!
									face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
									face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
									face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
									face = group_ents.add_face pts[3], pts[0], pts[4], pts[7]
									window_group = draw_window(side_window_info[0], side_window_info[2], side_window_info[1], side_window_info[3], side_window_info[4], side_window_info[5], side_window_info[5], side_window_info[6], wall_thickness, 0 - hangar_side_width[step] - wall_thickness, 0, hangar_side_start[step]+ hangar_side_length[step] + wall_thickness, hangar_side_start[step] - wall_thickness)
									group = group.parent.entities.add_group([group, window_group])
								end
							end
						
						end
					end
				
				end
				
			end
		
		end

	end

	# create front wall and door

	if beam_type == "T"
		max_door_radius = radius -(wall_thickness + beam_thickness)
	else
		max_door_radius = radius -(wall_thickness + full_beam_depth)
	end

	current_door_radius =  Math::sqrt(((door_height - y_coord) *(door_height - y_coord)) +((((full_width / 2) +(door_width / 2)) - x_coord) *(((full_width / 2) +(door_width / 2)) - x_coord)))
	
	if current_door_radius > max_door_radius
#		UI.messagebox "Door too large!"
		door_height = door_height /((current_door_radius / max_door_radius) *(current_door_radius / max_door_radius))
		door_width = door_width /((current_door_radius / max_door_radius) *(current_door_radius / max_door_radius))
	end

	if uv_map_front_inside_wall == 0
		face = group_ents.add_face inside_front_pts
		if face.normal[0] > 0 #Check direction of door face and reverse if necessary
			status = face.reverse!
		end
		status = face.material = wall_texture
	end
	if uv_map_front_wall == 0
  		face = group_ents.add_face outside_front_pts
		if face.normal[0] < 0 #Check direction of door face and reverse if necessary
			status = face.reverse!
		end
		status = face.material = wall_texture
	end
     	
	if door_tail_height == 0
		for step in 0..2
			face = group_ents.add_face outside_front_pts[outside_front_pts.length - 1 - step], outside_front_pts[outside_front_pts.length - 2 - step], inside_front_pts[inside_front_pts.length - 3 + step], inside_front_pts[inside_front_pts.length - 4 + step]
			status = face.material = wall_texture
		end
	else
		for step in 0..6
			face = group_ents.add_face outside_front_pts[outside_front_pts.length - 1 - step], outside_front_pts[outside_front_pts.length - 2 - step], inside_front_pts[inside_front_pts.length - 7 + step], inside_front_pts[inside_front_pts.length - 8 + step]
			status = face.material = wall_texture
		end
	end
     	
     	correct_beam_position = []
     	if custom_beam_setup == 0
     		for step in 0..beams - 1
     			correct_beam_position[step] = wall_thickness +(step *(beam_spacing + full_beam_width))
     		end
     	else
     		correct_beam_position[0] = wall_thickness
     		for step in 0..custom_beam_setup - 1
     			correct_beam_position[step + 1] = custom_beam_position[step]
     		end
     		correct_beam_position[custom_beam_setup + 1] = wall_thickness +((beams - 1) *(beam_spacing + full_beam_width))
     		beams = custom_beam_setup + 2
     	end
     	
     	for step in 0..beams - 1
	     	arc_pts = []
	     	for stepx in 0..current_stepx - 1
	     			arc_pts[stepx] = inside_pts[stepx]
				arc_pts[stepx][0] = correct_beam_position[step]
		end
	     	edge = group_ents.add_edges arc_pts
	     	
	     	pts = []

		if beam_type == "T"
		     	pts[0] = [correct_beam_position[step], wall_thickness, 0]
			pts[1] = [correct_beam_position[step] +(beam_width * 2) + beam_thickness, wall_thickness, 0]
			pts[2] = [correct_beam_position[step] +(beam_width * 2) + beam_thickness, wall_thickness + beam_thickness, 0]
			pts[3] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness + beam_thickness, 0]
			pts[4] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness + beam_width + beam_thickness, 0]
			pts[5] = [correct_beam_position[step] + beam_width, wall_thickness + beam_width + beam_thickness, 0]
			pts[6] = [correct_beam_position[step] + beam_width, wall_thickness + beam_thickness, 0]
			pts[7] = [correct_beam_position[step], wall_thickness + beam_thickness, 0]
		elsif beam_type == "U"
			pts[0] = [correct_beam_position[step], wall_thickness, 0]
			pts[1] = [correct_beam_position[step] + full_beam_width, wall_thickness, 0]
			pts[2] = [correct_beam_position[step] +((beam_thickness + beam_width) * 2), wall_thickness + beam_thickness + beam_width, 0]
			pts[3] = [correct_beam_position[step] +((beam_width * 2) + beam_thickness), wall_thickness + beam_thickness + beam_width, 0]
			pts[4] = [correct_beam_position[step] +((beam_width * 2) + beam_thickness), wall_thickness + beam_thickness, 0]
			pts[5] = [correct_beam_position[step] + beam_thickness, wall_thickness + beam_thickness, 0]
			pts[6] = [correct_beam_position[step] + beam_thickness, wall_thickness + beam_thickness + beam_width, 0]
			pts[7] = [correct_beam_position[step], wall_thickness + beam_thickness + beam_width, 0]
		elsif beam_type == "H"
			pts[0] = [correct_beam_position[step], wall_thickness, 0]
			pts[1] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness, 0]
			pts[2] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness + beam_thickness, 0]
			pts[3] = [correct_beam_position[step] +(beam_width/2) + beam_thickness, wall_thickness + beam_thickness, 0]
			pts[4] = [correct_beam_position[step] +(beam_width/2) + beam_thickness, wall_thickness + beam_width + beam_thickness, 0]
			pts[5] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness + beam_width + beam_thickness, 0]
			pts[6] = [correct_beam_position[step] + beam_width + beam_thickness, wall_thickness + beam_width +(beam_thickness * 2), 0]
			pts[7] = [correct_beam_position[step], wall_thickness + beam_width +(beam_thickness * 2), 0]
			pts[8] = [correct_beam_position[step], wall_thickness + beam_width + beam_thickness, 0]
			pts[9] = [correct_beam_position[step] +(beam_width/2), wall_thickness + beam_width + beam_thickness, 0]
			pts[10] = [correct_beam_position[step] +(beam_width/2), wall_thickness + beam_thickness, 0]
			pts[11] = [correct_beam_position[step], wall_thickness + beam_thickness, 0]
		
		end
		face = group_ents.add_face pts

		status = face.material = beam_texture
		full_beam = face.followme edge
     	end
     	
####################################################
####	Draw additional door openings		####
####################################################
	
	foundation_wall_height = 0
	foundation_wall_thickness = wall_thickness
	
	if back_door_height != 0
		pts = []
		pts[0] = [0, back_door_middle - back_door_width / 2, foundation_wall_height]
		pts[1] = [0, back_door_middle - back_door_width / 2, back_door_height]
		pts[2] = [0, back_door_middle + back_door_width / 2, back_door_height]
		pts[3] = [0, back_door_middle + back_door_width / 2, foundation_wall_height]
		face = group_ents.add_face pts
		if wall_thickness > 0
			status = face.pushpull 0 - wall_thickness
		else
			status = face.erase!
		end
		if foundation_wall_height > 0
			pts = []
			pts[0] = [0, back_door_middle - back_door_width / 2, foundation_wall_height]
			pts[1] = [0, back_door_middle - back_door_width / 2, 0]
			pts[2] = [0, back_door_middle + back_door_width / 2, 0]
			pts[3] = [0, back_door_middle + back_door_width / 2, foundation_wall_height]
			face = group_ents.add_face pts
			if foundation_wall_thickness > 0
				status = face.pushpull 0 - foundation_wall_thickness
			else
				status = face.erase!
			end
		end
	end

####################################################
####	Draw signs				####
####################################################
	
	if sign_count > 0
		for step in 0..sign_count - 1
		
			sign_group = draw_signs(sign_type[step], sign_texture[step], door_frame_texture, sign_wall[step], sign_x_pos[step], sign_y_pos[step], sign_x_size[step], sign_y_size[step], sign_thickness[step], sign_filled[step], sign_double_sided[step], full_length, 0, 0, 0)
			group = group.parent.entities.add_group([group, sign_group])
			
		end
	end


####################################################
####	Draw entrance doors			####
####################################################

	if entrance_door_count > 0
		for step in 0..entrance_door_count - 1

			if entrance_door_wall[step] == "B" or entrance_door_wall[step] == "F"   

				if entrance_door_wall[step] == "B"
					current_wall_position = 0
					current_wall_internal_position = wall_thickness
				else
					current_wall_position = full_length
					current_wall_internal_position = full_length - wall_thickness
				end
				pts = []
				pts[0] = [current_wall_position, entrance_door_x_pos[step], entrance_door_y_pos[step]]
				pts[1] = [current_wall_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
				pts[2] = [current_wall_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				pts[3] = [current_wall_position, entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				
				face = group_ents.add_face pts
				status = face.erase!
				if wall_thickness > 0
					pts[4] = [current_wall_internal_position, entrance_door_x_pos[step], entrance_door_y_pos[step]]
					pts[5] = [current_wall_internal_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
					pts[6] = [current_wall_internal_position, entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
					pts[7] = [current_wall_internal_position, entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
	
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					status = face.erase!
				end
			end

			door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, 0, 0, full_length, 0)
	
			group_text = group.description
			group = group.parent.entities.add_group([group, door_group])
			group.description = group_text
		end
	end


####################################################
####	Draw windows				####
####################################################

	if window_count > 0
		for step in 0..window_count - 1
			if window_wall[step] == "B"
				current_wall_position = 0
				current_wall_internal_position = wall_thickness
			else
				current_wall_position = full_length
				current_wall_internal_position = full_length - wall_thickness
			end

			for window_step in 0..window_repeat[step] - 1

				do_not_draw = 0
				if back_door == 1 and window_wall[step] == "B"
					if(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) > door_middle - door_width / 2 and window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) < door_middle + door_width / 2 and window_y_pos[step] < door_height) or(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > door_middle - door_width / 2 and window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] < door_middle + door_width / 2 and window_y_pos[step] < door_height) or window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > full_width
						do_not_draw = 1
					end
				elsif window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step) + window_x_size[step] > full_width
					do_not_draw = 1
				end
				if do_not_draw == 0
					pts = []
					pts[0] = [current_wall_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
					pts[1] = [current_wall_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
					pts[2] = [current_wall_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
					pts[3] = [current_wall_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
					face = group_ents.add_face pts
						status = face.erase!
					if wall_thickness > 0
						pts[4] = [current_wall_internal_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
						pts[5] = [current_wall_internal_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step]]
						pts[6] = [current_wall_internal_position, window_x_pos[step] + window_x_size[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
						pts[7] = [current_wall_internal_position, window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step] + window_y_size[step]]
						face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
						status = face.erase!
						face = group_ents.add_face pts[0], pts[1], pts[5], pts[4]
						if face.normal[0] < 0
							status = face.reverse!
						end
						status = face.material = wall_texture
						face = group_ents.add_face pts[1], pts[2], pts[6], pts[5]
						status = face.material = wall_texture
						face = group_ents.add_face pts[2], pts[3], pts[7], pts[6]
						status = face.material = wall_texture
						face = group_ents.add_face pts[3], pts[0], pts[4], pts[7]
						status = face.material = wall_texture
					end
	
					window_group = draw_window(window_x_pos[step] +((window_x_size[step] + window_gap[step]) * window_step), window_y_pos[step], window_x_size[step], window_y_size[step], window_wall[step], window_texture[step], window_back_texture[step], window_transparency[step], wall_thickness, 0, full_width, full_length, 0)
					group = group.parent.entities.add_group([group, window_group])
				end
			end
		end
	end

 
####################################################
####	Extensions for arched hangar		####
####################################################
     	
  	if extensions.length > 0
	
		ext_group = []
		for step in 0..extensions.length - 1
			before_y_pos = group.local_bounds.center[1]
			extension_type = extensions[step][0..extensions[step].index("(") - 1]
	 		ext_group[step] = add_extensions(extensions[step], "Arched", full_length, full_width, outside_height, 0, wall_thickness, door_type, wall_texture, roof_texture, internal_wall_texture)
			group_text = group.description
			group = group.parent.entities.add_group([group, ext_group[step]])
			group.description = group_text
	 	end
	end

####################################################
####	Draw floor				####
####################################################
 
	if floor_texture != "None"
		pts = []
		pts[0] = [0, 0, 0]
		pts[1] = [full_length, 0, 0]
		pts[2] = [full_length, full_width, 0]
		pts[3] = [0, full_width, 0]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = floor_texture
	end     	
     	
     	
####################################################
####	Draw back door				####
####################################################

	if back_door_height != 0
		door_group = Sketchup.active_model.entities.add_group
		door_group_ents = door_group.entities
		door_group = insert_door(back_door_type, back_door_height, back_door_folding_level.to_i, back_door_sections, back_door_width, back_door_middle, 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "Y", "Arched", 0)
	   	point = Geom::Point3d.new(0, back_door_middle, 0)
		t = Geom::Transformation.scaling(point, -1, 1, 1)
		door_group.transform! t
		group_text = group.description
		group = group.parent.entities.add_group([group, door_group])
		group.description = group_text
	end		

	
####################################################
####	Draw hangar door			####
####################################################

	door_group = Sketchup.active_model.entities.add_group
	door_group_ents = door_group.entities
	door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, door_width, door_middle, full_length - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, 0, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "Y", "Arched", 0)
	group_text = group.description + " with " + door_group.description
	group_x = group
#	if afcad_read == 1
#		point = Geom::Point3d.new(hangar_x -(full_length / 2), hangar_y -(full_width / 2), 0)
#		t = Geom::Transformation.new point
#		door_group.move! t
#		group_x.description = "Arched hangar:" +(hangar_x -(full_length / 2)).to_s + ":" +(hangar_y -(full_width / 2)).to_s + ":" + hangar_angle.to_s + ":" + hangar_x.to_s + ":" + hangar_y.to_s
#	end
	group = group_x.parent.entities.add_group([group_x, door_group])
	group.description = group_text


####################################################
####	Position and rotate hangar		####
####################################################

	
	if afcad_read == 1
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x -(full_length / 2) + trans.origin[0], hangar_y -(full_width / 2) + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
	else
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t
		group.name = "Arched:" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + input[14].to_s + ":" + input[15].to_s + ":" + input[16].to_s + ":" + input[17].to_s + ":" + input[18].to_s + ":" + input[19].to_s + ":" + input[20].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":" + texture_input[4].to_s + ":" + texture_input[5].to_s + ":" + texture_input[6].to_s + ":"

	end


	t = Geom::Transformation.rotation [hangar_x,hangar_y,0], Z_AXIS, radians
	group.transform! t

     	
end


##################################################################
##################################################################
###### Below is the code for the double nested t hangar row	######
##################################################################
##################################################################

def d_nested_t_hangar_row(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  

	#Define entity group and materials
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "Double nested T-hangar row"
	materials = Sketchup.active_model.materials
	afcad_read = 0
	
	if hangar_codes != "" and hangar_codes != "Box"
		afcad_read = 1
		hangar_info = hangar_codes.split(":")

		wall_texture = hangar_info[18]
		internal_wall_texture = hangar_info[19]
		if internal_wall_texture == "Same as external"
			internal_wall_texture = wall_texture
		end
		roof_texture = hangar_info[20]
		ceiling_texture = hangar_info[21]
		door_texture = hangar_info[22]
		door_frame_texture = hangar_info[23]
		beam_texture = "Blue metal"
		floor_texture = hangar_info[24]
		cross_beam_texture = ""

		front_spaces = hangar_info[30].to_i
		back_spaces = hangar_info[31].to_i
		
		if front_spaces <= 1
			front_spaces = 1
			back_spaces = 1
		end
		
		if front_spaces != back_spaces
			if front_spaces == 1
				front_spaces = 2
				back_spaces = 1
			else
				back_spaces = front_spaces - 1
			end
		end

		section_depth = hangar_info[5].to_f / $inch
		wall_thickness = hangar_info[6].to_f / $inch
		door_height = hangar_info[7].to_f / $inch
		door_type = hangar_info[8]
		door_folding_level = hangar_info[9].to_i
		door_folding_identical = 1
		door_frame_thickness = hangar_info[10].to_f / $inch
		door_skin_thickness = hangar_info[11].to_f / $inch
		arc = hangar_info[12].to_f / $inch
		outside_height = door_height + wall_thickness
		total_depth = (section_depth * 3) + wall_thickness
		flat_ceiling = hangar_info[13]
		flip_hangar = hangar_info[3]

		wing_section_width =(hangar_info[29].to_f / $inch) / 4
	
		tail_section_width =(wing_section_width * 2) - wall_thickness
		section_width =(wing_section_width * 2) + tail_section_width + wall_thickness
		extension_text = hangar_info[17]

		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[26].to_f
		long1 = midlong
		long2 = hangar_info[27].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)


		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		
		hangar_angle = 360 - hangar_info[28].to_f
		hangar_x = x_distance / $inch -(section_width / 2)
		hangar_y = y_distance / $inch -(section_depth / 2)
		radians = hangar_angle.degrees

	else

		#Find preset angle and size
		angle = find_endpoints
		
		if hangar_codes == ""
  		if angle[3] == 0
	 		  angle[3] = ((500 * 3) / $inch) + 10
		  end
      default_hangar_height = 340
      default_hangar_arc_height = 120
		else
		  default_hangar_height = midlat * $inch
		  default_hangar_arc_height = midlong * $inch
		end    	
	
		#Hangar prompts
		prompts = ["Front spaces?", "Back spaces?", "Flip sides?", "Wing section width?", "Section depth?", "Wall thickness?", "Door height?", "Door type?", "Door folding level?", "Door frame thickness?", "Door skin thickness?", "Arc height?", "Flat ceiling?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		defaults = ["4", "4", "No", "500",(((angle[3] / 3) * $inch) - 10).ceil, "5", default_hangar_height, "Bi-folding doors", "5", "5", "3", default_hangar_arc_height, "Yes", angle[0], angle[1] * $inch, angle[2] * $inch, ""]
		list = ["", "", $yn, "", "", "", "", "Bi-folding doors|Hydraulic swing door", "", "", "", "", $yn, "", "", "", ""]
		input = UI.inputbox prompts, defaults, list, "Hangar info"
	 	#Texture prompts
	 	texture_prompts = ["Wall texture?", "Internal wall texture?", "Roof texture?", "Ceiling texture?", "Door texture?", "Door frame texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Same as external", "Vertical square iron", "Vertical square iron", "Vertical square iron", "Rusty metal", "Concrete"]
	 	texture_list = [$wall_list, $internal_wall_list, $roof_list, $roof_list, $door_list, $metal_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"
		#Get texture info
		wall_texture = texture_input[0]
		internal_wall_texture = texture_input[1]
		if internal_wall_texture == "Same as external"
			internal_wall_texture = wall_texture
		end
		roof_texture = texture_input[2]
		ceiling_texture = texture_input[3]
		door_texture = texture_input[4]
		door_frame_texture = texture_input[5]
		beam_texture = "Blue metal"
		floor_texture = texture_input[6]
		cross_beam_texture = ""

		front_spaces = input[0].to_i
		back_spaces = input[1].to_i
		
		if front_spaces <= 1
			front_spaces = 1
			back_spaces = 1
		end
		
		if front_spaces != back_spaces
			if front_spaces == 1
				front_spaces = 2
				back_spaces = 1
			else
				back_spaces = front_spaces - 1
			end
		end

		section_depth = input[4].to_f / $inch
		wall_thickness = input[5].to_f / $inch
		door_height = input[6].to_f / $inch
		door_type = input[7]
		door_folding_level = input[8].to_i
    door_folding_identical = 1
		door_frame_thickness = input[9].to_f / $inch
		door_skin_thickness = input[10].to_f / $inch
		arc = input[11].to_f / $inch
		outside_height = door_height + wall_thickness
		total_depth =(section_depth * 3) + wall_thickness
		flat_ceiling = input[12]
		flip_hangar = input[2]

		if angle[4] == 0
			wing_section_width = input[3].to_f / $inch
		else
			full_hangar_length = angle[4] * $inch
			if front_spaces == back_spaces
				wing_section_width =((full_hangar_length -(wall_thickness * $inch)) /((front_spaces * 4) + 2)) / $inch
			else
				wing_section_width =((full_hangar_length -(wall_thickness * $inch)) /(front_spaces * 4)) / $inch
			end
		end	
		tail_section_width =(wing_section_width * 2) - wall_thickness
		section_width =(wing_section_width * 2) + tail_section_width + wall_thickness
	
		hangar_angle = input[13].to_f
		hangar_x = input[14].to_f / $inch
		hangar_y = input[15].to_f / $inch
		radians = hangar_angle.degrees
	
		extension_text = input[16]
	end
 	
 	
 	door_sections = 1

	
	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or door_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step] or door_frame_texture == $texture_name[step] or internal_wall_texture == $texture_name[step] or ceiling_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end
	if wall_texture == "Untextured" or roof_texture == "Untextured" or door_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured" or door_frame_texture == "Untextured" or internal_wall_texture == "Untextured" or cross_beam_texture == "Untextured"
			texture = materials.add "Untextured"
			texture.color = "White"
	end

	if front_spaces == back_spaces
		total_width =((wing_section_width * 2) + tail_section_width + wall_thickness) * front_spaces +(wing_section_width * 2) + wall_thickness
	else
		total_width =((wing_section_width * 2) + tail_section_width + wall_thickness) * front_spaces + wall_thickness
	end

	cursor_x = 0
	cursor_y = 0


####################################################
####	Get extension info			####
####################################################
	
	extensions = []
	extension_types = []
	end_gaps_filled = 0
	fill_front_start_gap = 0
	fill_front_end_gap = 0
	fill_back_start_gap = 0
	fill_back_end_gap = 0
	entrance_door_wall = []
	entrance_door_count = 0
	entrance_door_x_pos = []
	entrance_door_y_pos = []
	entrance_door_x_size = []
	entrance_door_y_size = []
	entrance_door_texture = []
	entrance_door_glass_texture = []
	entrance_door_glass_back_texture = []
	entrance_door_type = []
	sign_count = 0
	sign_x_pos = []			# Position on wall from front or left
	sign_y_pos = []			# Position on wall from ground
	sign_x_size = []		# Sign width
	sign_y_size = []		# Sign height
	sign_thickness = []		# Sign thickness
	sign_double_sided = []		# 0 for logo on front only(only valid for textured signs), 1 for logo on both sides
	sign_wall = []			# Wall to put sign on, F, B, L, R
	sign_texture = []		# Sign texture name
	sign_type = []			# Sign type; L(logo), T(text)
	sign_filled = []		# 0 to not adjust texture size, 1 to adjust texture size.
  door_folding_ext_count = 0
  door_folding_ext_level = []
  

	if extension_text.length > 0
		if extension_text.count('|') > 0
			extensions = extension_text.split("|")
		else
			extensions[0] = extension_text
		end
		for step in 0..extensions.length - 1
			if extensions[step].count('(') > 0 and extensions[step].count(')') > 0 
				extension_types[step] = extensions[step][0..extensions[step].index("(") - 1]
			else
				extension_types[step] = "ERR"
			end
			
			# Fill end gap
			if extension_types[step] == "G" and end_gaps_filled == 0
				end_gap_info = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
			
				if end_gap_info.count(";") == 3
					gap_info = end_gap_info.split(";") 
					fill_front_start_gap = gap_info[0].to_i
					fill_front_end_gap = gap_info[1].to_i
					fill_back_start_gap = gap_info[2].to_i
					fill_back_end_gap = gap_info[3].to_i
				
					if fill_front_start_gap >= 2
						fill_front_start_gap = 2
						fill_back_start_gap = 0
					else
						if fill_back_start_gap >= 1
							fill_back_start_gap = 1
						else
							fill_back_start_gap = 0
						end
					end
	
					if front_spaces == back_spaces
						if fill_back_end_gap >= 2
							fill_front_end_gap = 0
							fill_back_end_gap = 2
						else
							if fill_front_end_gap >= 1
								fill_front_end_gap = 1
							else
								fill_front_end_gap = 0
							end
						end					
					else
						if fill_front_end_gap >= 2
							fill_front_end_gap = 2
							fill_back_end_gap = 0
						else
							if fill_back_end_gap >= 1
								fill_back_end_gap = 1
							else
								fill_back_end_gap = 0
							end
						end
					end
					
				end

			end

      # Custom door folding levels
      if extension_types[step] == "DF" and door_folding_ext_count == 0
        door_folding_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
        door_folding_ext_level = door_folding_string.split(";")
        door_folding_ext_count = 1
        door_folding_identical = 0
        if door_folding_ext_level.length < (front_spaces + back_spaces)
          for step2 in door_folding_ext_level.length..(front_spaces + back_spaces - 1)
            door_folding_ext_level[step2] = door_folding_level
          end
        end
        for step2 in 0..door_folding_ext_level.length - 1
          door_folding_ext_level[step2] = door_folding_ext_level[step2].to_i
          if door_folding_ext_level[step2] > 90
            door_folding_ext_level[step2] = 90
          end
        end
      end

			# Entrance doors
			if extension_types[step] == "ED"
				entrance_door_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if entrance_door_string.count(";") == 4 or entrance_door_string.count(";") == 5
					entrance_door_failed = 0
					entrance_door_details = entrance_door_string.split(";")
					entrance_door_wall[entrance_door_count] = entrance_door_details[2]
					if entrance_door_details[0].count("x") == 1
						entrance_door_temp_string = entrance_door_details[0].split("x")
						entrance_door_x_pos[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_pos[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
						if entrance_door_y_pos[entrance_door_count] < 1 / $inch
							entrance_door_y_pos[entrance_door_count] = 1 / $inch
						end
					else
						entrance_door_failed = 1
					end
					if entrance_door_details[1].count("x") == 1
						entrance_door_temp_string = entrance_door_details[1].split("x")
						entrance_door_x_size[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_size[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
					else
						entrance_door_failed = 1
					end
					entrance_door_texture[entrance_door_count] = entrance_door_details[4]
					entrance_door_type[entrance_door_count] = entrance_door_details[3]
					
					if entrance_door_string.count(";") == 5

						entrance_door_glass_texture[entrance_door_count] = entrance_door_details[5]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_details[5]
					
					else
						entrance_door_glass_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
					end
					
					if entrance_door_failed == 0
#						cross_beam_exclusion_side[cross_beam_exclusion_count] = entrance_door_wall[entrance_door_count]
#						cross_beam_exclusions[cross_beam_exclusion_count] = [entrance_door_x_pos[entrance_door_count], entrance_door_y_pos[entrance_door_count], entrance_door_x_pos[entrance_door_count] + entrance_door_x_size[entrance_door_count], entrance_door_y_pos[entrance_door_count] + entrance_door_y_size[entrance_door_count]]
#						cross_beam_extras_drawn[cross_beam_exclusion_count] = 0
#						cross_beam_exclusion_count = cross_beam_exclusion_count + 1
						entrance_door_count = entrance_door_count + 1
					end
				end
			end

			# Signage
			if extension_types[step] == "S"
				sign_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if sign_string.count(";") >= 3
					sign_failed = 0
					sign_details = sign_string.split(";")
					sign_type[sign_count] = sign_details[0]
					if sign_details[1].count("x") == 1
						sign_temp_string = sign_details[1].split("x")
						sign_x_pos[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_pos[sign_count] = sign_temp_string[1].to_f / $inch
					else
						sign_failed = 1
					end
					if sign_details[2].count("x") == 2
						sign_temp_string = sign_details[2].split("x")
						sign_x_size[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_size[sign_count] = sign_temp_string[1].to_f / $inch
						sign_thickness[sign_count] = sign_temp_string[2].to_f / $inch
					else
						sign_failed = 1
					end
					sign_wall[sign_count] = sign_details[3]
					if sign_type[sign_count] == "L" and sign_string.count(";") == 6
						sign_texture[sign_count] = sign_details[4]
						sign_filled[sign_count] = sign_details[5].to_i
						sign_double_sided[sign_count] = sign_details[6].to_i
					
					else
						sign_failed = 1
					end
					
					if sign_failed == 0
						sign_count = sign_count + 1
					end
				end
			end

		end
	end

  if door_folding_ext_count === 0
    for step in 0..(front_spaces + back_spaces - 1)
      door_folding_ext_level[step] = door_folding_level
    end  
  end


####################################################
####	Set bounds				####
####################################################

external_back_position = 0
external_front_position = total_width
external_left_position = 0
external_right_position = total_depth
internal_back_position = wall_thickness
internal_front_position = total_width - wall_thickness
internal_left_position = wall_thickness
internal_right_position = total_depth - wall_thickness
	

####################################################
####	Draw walls				####
####################################################

	#draw first end wall
	pts = []
	pts[0] = [0, 0, 0]	
	pts[1] = [0, 0, outside_height]	
	pts[2] = [0, total_depth/2, outside_height + arc]	
	pts[3] = [0, total_depth, outside_height]	
	pts[4] = [0, total_depth, 0]	
	face = group_ents.add_face pts
	status = face.material = wall_texture
	
	#draw front wall
	pts = []
	pts[0] = [total_width, 0, 0]
	pts[1] = [total_width, 0, outside_height]
	pts[2] = [0, 0, outside_height]
	pts[3] = [0, 0, 0]
	for step in 1..front_spaces
		pts [(step * 4)] = [((step - 1) * section_width) + wall_thickness, 0, 0]
		pts [(step * 4) + 1] = [((step - 1) * section_width) + wall_thickness, 0, door_height]
		pts [(step * 4) + 2] = [step * section_width, 0, door_height]
		pts [(step * 4) + 3] = [step * section_width, 0, 0]
	end
	face = group_ents.add_face pts
	status = face.material = wall_texture

	#draw second end wall
	pts = []
	pts[0] = [total_width, 0, 0]	
	pts[1] = [total_width, 0, outside_height]	
	pts[2] = [total_width, total_depth/2, outside_height + arc]	
	pts[3] = [total_width, total_depth, outside_height]	
	pts[4] = [total_width, total_depth, 0]	
	face = group_ents.add_face pts
	status = face.material = wall_texture

	#draw back wall
	pts = []
	pts[0] = [total_width, total_depth, 0]
	pts[1] = [total_width, total_depth, outside_height]
	pts[2] = [0, total_depth, outside_height]
	pts[3] = [0, total_depth, 0]
	for step in 1..back_spaces
		pts [(step * 4)] = [((step - 1) * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, 0]
		pts [(step * 4) + 1] = [((step - 1) * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, door_height]
		pts [(step * 4) + 2] = [((step - 1) * section_width) +(wing_section_width * 2) + wall_thickness +(wing_section_width * 2) + tail_section_width, total_depth, door_height]
		pts [(step * 4) + 3] = [((step - 1) * section_width) +(wing_section_width * 2) + wall_thickness +(wing_section_width * 2) + tail_section_width, total_depth, 0]
	end
	face = group_ents.add_face pts
	status = face.material = wall_texture

	#Draw roof
	for step in 0..1
		pts = []
		pts[0] = [0, total_depth * step, outside_height]
		pts[1] = [total_width, total_depth * step, outside_height]
		pts[2] = [total_width, total_depth / 2, outside_height + arc]
		pts[3] = [0, total_depth / 2, outside_height + arc]
		face = group_ents.add_face pts
		status = face.material = roof_texture
	end
	
	for step in 1..front_spaces
		pts = []
		pts[0] = [cursor_x, cursor_y, 0]
		pts[1] = [cursor_x + wall_thickness, cursor_y, 0]
		pts[2] = [cursor_x + wall_thickness, cursor_y + section_depth, 0]
		pts[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, 0]
		pts[4] = [cursor_x + wall_thickness + wing_section_width, cursor_y +(section_depth * 2), 0]
		pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +(section_depth * 2), 0]
		pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, 0]
		pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, 0]
		pts[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]
		
		ptsx = []

		if flat_ceiling == "Yes"
			ptsx[0] = [cursor_x + wall_thickness, cursor_y, door_height]
			ptsx[1] = [cursor_x + wall_thickness, cursor_y + section_depth, door_height]
			ptsx[2] = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, door_height]
			ptsx[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y +(section_depth * 2), door_height]
			ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +(section_depth * 2), door_height]
			ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, door_height]
			ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, door_height]
			ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			
			if fill_front_start_gap == 1 and step == 1
				pts[3] = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), 0]
				ptsx[2] = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), door_height]
			elsif fill_front_start_gap == 2 and step == 1
				pts[2] = [cursor_x + wall_thickness, cursor_y +(section_depth * 3), 0]
				pts[3] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 3), 0]
				pts[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 2), 0]
				ptsx[1] = [cursor_x + wall_thickness, cursor_y +(section_depth * 3), door_height]
				ptsx[2] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 3), door_height]
				ptsx[3] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 2), door_height]
			elsif fill_front_start_gap == 1 and step == front_spaces and front_spaces != back_spaces
				pts[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), 0]
				ptsx[5] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), door_height]
			elsif fill_front_end_gap == 2 and step == front_spaces and front_spaces != back_spaces
				pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y +(section_depth * 2), 0]
				pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y +(section_depth * 3), 0]
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 3), 0]
				ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y +(section_depth * 2), door_height]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y +(section_depth * 3), door_height]
				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 3), door_height]
			elsif fill_front_end_gap == 1 and step == front_spaces and front_spaces == back_spaces
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + section_depth, 0]
				pts[8] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + wall_thickness, 0]
				pts[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + wall_thickness, 0]
				pts[10] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]

				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + section_depth, door_height]
				ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + wall_thickness, door_height]
				ptsx[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + wall_thickness, door_height]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			end

			# inside walls and ceiling
			face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[3], pts[4], ptsx[3], ptsx[2]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[4], pts[5], ptsx[4], ptsx[3]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
			status = face.material = internal_wall_texture
			face = group_ents.add_face ptsx
			status = face.material = ceiling_texture
			
			if fill_front_end_gap == 1 and step == front_spaces and front_spaces == back_spaces
				face = group_ents.add_face pts[8], pts[9], ptsx[8], ptsx[7]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[9], pts[10], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
			end
		else
			ptsx[0] = [cursor_x + wall_thickness, cursor_y, door_height]
			ptsx[1] = [cursor_x + wall_thickness, cursor_y + section_depth, door_height +(arc * 2 / 3)]
			ptsx[2] = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
			ptsx[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
			ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
			ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
			ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
			ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]

			if fill_front_start_gap == 1 and step == 1
				pts[3] = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), 0]
				ptsx[2] = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[3] = [cursor_x + wall_thickness, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
				
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[3], ptsx[4], ptsx[5], ptsx[6], ptsx[7]
				status = face.material = ceiling_texture
			elsif fill_front_end_gap == 1 and step == front_spaces and front_spaces == back_spaces
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + section_depth, 0]
				pts[8] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + wall_thickness, 0]
				pts[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + wall_thickness, 0]
				pts[10] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]

				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
				ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y + wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				ptsx[10] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				ptsx[11] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[2], ptsx[3], ptsx[4], ptsx[5], ptsx[6], ptsx[7], ptsx[10], ptsx[11]
				status = face.material = ceiling_texture
			elsif fill_front_end_gap == 1 and step == front_spaces and front_spaces != back_spaces
				pts[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), 0]
			elsif fill_front_start_gap == 2 and step == 1
				pts[2] = [cursor_x + wall_thickness, cursor_y +(section_depth * 3), 0]
				pts[3] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 3), 0]
				pts[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 2), 0]
				ptsx[1] = [cursor_x + wall_thickness, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
				ptsx[2] = [cursor_x + wall_thickness, cursor_y +(section_depth * 3), door_height]
				ptsx[3] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 3), door_height]
				ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 - wall_thickness / 2, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
				ptsx[7] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
			elsif fill_front_end_gap == 2 and step == front_spaces and front_spaces != back_spaces
				pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2, cursor_y +(section_depth * 2), 0]
				pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2, cursor_y +(section_depth * 3), 0]
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 3), 0]

				ptsx[4] = [cursor_x + wall_thickness + wing_section_width, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[6] = [cursor_x + wall_thickness + wing_section_width * 2 + tail_section_width, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
			else
				face = group_ents.add_face ptsx
				status = face.material = ceiling_texture
			end

			ptsx[8] = [cursor_x + wall_thickness + wing_section_width, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
			ptsx[9] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]

			if fill_front_start_gap == 1 and step == 1
				ptsx[8] = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
			elsif fill_front_end_gap == 1 and step == front_spaces and front_spaces != back_spaces
				ptsx[4] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +((section_depth * 2) * 3 / 4), door_height + arc]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), door_height +(arc * 2 / 3)]
			elsif fill_front_start_gap == 2 and step == 1
				ptsx[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, door_height +(arc * 2 / 3)]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			elsif fill_front_end_gap == 2 and step == front_spaces and front_spaces != back_spaces
				ptsx[8] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2, cursor_y +(section_depth * 3), door_height]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 3), door_height]
			end

			if fill_front_start_gap == 2 and step == 1
				face = group_ents.add_face ptsx[1], ptsx[2], ptsx[3], ptsx[4], ptsx[5], ptsx[6]
				status = face.material = ceiling_texture
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[6], ptsx[7], ptsx[8], ptsx[9]
				status = face.material = ceiling_texture
			elsif fill_front_end_gap == 2 and step == front_spaces and front_spaces != back_spaces
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[2], ptsx[3], ptsx[6], ptsx[7]
				status = face.material = ceiling_texture
				face = group_ents.add_face ptsx[3], ptsx[4], ptsx[5], ptsx[8], ptsx[9], ptsx[6]
				status = face.material = ceiling_texture
			else
				face = group_ents.add_face ptsx[8], ptsx[9], ptsx[4], ptsx[3]
				status = face.material = ceiling_texture
			end

			if fill_front_start_gap == 1 and step == 1
				face = group_ents.add_face pts[1], pts[3], ptsx[2], ptsx[3], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[5], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
				status = face.material = internal_wall_texture
			elsif fill_front_end_gap == 1 and step == front_spaces and front_spaces != back_spaces
				face = group_ents.add_face ptsx[7], ptsx[4], ptsx[3], ptsx[2], ptsx[1], ptsx[0]
				status = face.material = ceiling_texture
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[6], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[8], ptsx[7], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
			elsif fill_front_start_gap == 2 and step == 1
				face = group_ents.add_face pts[1], pts[2], ptsx[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[4], ptsx[3]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[5], ptsx[4]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[7], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[8], ptsx[7]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
			elsif fill_front_end_gap == 2 and step == front_spaces and front_spaces != back_spaces
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[4], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[5], ptsx[4]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[8], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6], ptsx[9]
				status = face.material = internal_wall_texture
			else
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
				status = face.material = internal_wall_texture
			end
			if fill_front_end_gap == 1 and step == front_spaces and front_spaces == back_spaces
				face = group_ents.add_face pts[8], pts[9], ptsx[10], ptsx[7]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[9], pts[10], ptsx[11], ptsx[10]
				status = face.material = internal_wall_texture
			end

		end


		if floor_texture != "None"
			face = group_ents.add_face pts
			status = face.material = floor_texture
		end

		#line = group_ents.add_line pts


		cursor_x = cursor_x + section_width
	end
	
	cursor_y = total_depth
	cursor_x = wing_section_width * 2
	
	for step in 1..back_spaces
		pts = []
		pts[0] = [cursor_x, cursor_y, 0]
		pts[1] = [cursor_x + wall_thickness, cursor_y, 0]
		pts[2] = [cursor_x + wall_thickness, cursor_y - section_depth, 0]
		pts[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y - section_depth, 0]
		pts[4] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), 0]
		pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), 0]
		pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, 0]
		pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth, 0]
		pts[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]

		ptsx = []

		if flat_ceiling == "Yes"
			ptsx[0] = [cursor_x + wall_thickness, cursor_y, door_height]
			ptsx[1] = [cursor_x + wall_thickness, cursor_y - section_depth, door_height]
			ptsx[2] = [cursor_x + wall_thickness + wing_section_width, cursor_y - section_depth, door_height]
			ptsx[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), door_height]
			ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), door_height]
			ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, door_height]
			ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth, door_height]
			ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			
			if fill_back_start_gap == 1 and step == 1
				pts[2] = [cursor_x + wall_thickness, cursor_y - wall_thickness, 0]
				pts[3] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - wall_thickness, 0]
				pts[4] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - section_depth, 0]
				pts[5] = [cursor_x + wall_thickness + wing_section_width, cursor_y - section_depth, 0]
				pts[6] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), 0]
				pts[7] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), 0]
				pts[8] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, 0]
				pts[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth, 0]
				pts[10] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]

				ptsx[1] = [cursor_x + wall_thickness, cursor_y - wall_thickness, door_height]
				ptsx[2] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - wall_thickness, door_height]
				ptsx[3] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - section_depth, door_height]
				ptsx[4] = [cursor_x + wall_thickness + wing_section_width, cursor_y - section_depth, door_height]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), door_height]
				ptsx[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), door_height]
				ptsx[7] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, door_height]
				ptsx[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth, door_height]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			elsif fill_back_end_gap == 1 and step == back_spaces and back_spaces == front_spaces
				pts[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth * 2, 0]
				ptsx[5] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth * 2, door_height]
			elsif fill_back_end_gap == 2 and step == back_spaces and back_spaces == front_spaces
				pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 2), 0]
				pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 3), 0]
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y -(section_depth * 3), 0]
				ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 2), door_height]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 3), door_height]
				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y -(section_depth * 3), door_height]
			elsif fill_back_start_gap == 1 and step == back_spaces and back_spaces != front_spaces
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - section_depth, 0]
				pts[8] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - wall_thickness, 0]
				pts[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - wall_thickness, 0]
				pts[10] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, 0]
				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - section_depth, door_height]
				ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - wall_thickness, door_height]
				ptsx[8] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - wall_thickness, door_height]
				ptsx[9] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]
			
			end


			# inside walls and ceiling
			face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[3], pts[4], ptsx[3], ptsx[2]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[4], pts[5], ptsx[4], ptsx[3]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
			status = face.material = internal_wall_texture
			face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
			status = face.material = internal_wall_texture
			face = group_ents.add_face ptsx
			status = face.material = ceiling_texture

			if(fill_back_start_gap == 1 and step == 1) or(fill_back_end_gap == 1 and step == back_spaces and back_spaces != front_spaces)
				face = group_ents.add_face pts[8], pts[9], ptsx[8], ptsx[7]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[9], pts[10], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
			end


		else
			ptsx[0] = [cursor_x + wall_thickness, cursor_y, door_height]
			ptsx[1] = [cursor_x + wall_thickness, cursor_y - section_depth, door_height +(arc * 2 / 3)]
			ptsx[2] = [cursor_x + wall_thickness + wing_section_width, cursor_y - section_depth, door_height +(arc * 2 / 3)]
			ptsx[3] = [cursor_x + wall_thickness + wing_section_width, cursor_y -((section_depth * 2) * 3 / 4), door_height + arc]
			ptsx[4] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -((section_depth * 2) * 3 / 4), door_height + arc]
			ptsx[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, door_height +(arc * 2 / 3)]
			ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth, door_height +(arc * 2 / 3)]
			ptsx[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]

			if fill_back_start_gap == 1 and step == 1
				pts[2] = [cursor_x + wall_thickness, cursor_y - wall_thickness, 0]
				pts[9] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - wall_thickness, 0]
				pts[10] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - section_depth, 0]
				ptsx[1] = [cursor_x + wall_thickness, cursor_y - wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				ptsx[10] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				ptsx[11] = [cursor_x + wall_thickness -(wing_section_width * 2), cursor_y - section_depth, door_height +(arc * 2 / 3)]
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[10], ptsx[11], ptsx[2], ptsx[3], ptsx[4], ptsx[5], ptsx[6], ptsx[7]
				status = face.material = ceiling_texture
			elsif fill_back_end_gap == 1 and step == back_spaces and back_spaces == front_spaces
				pts[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - section_depth * 2, 0]
				ptsx[4] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y -((section_depth * 2) * 3 / 4), door_height + arc]
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[2], ptsx[3], ptsx[4], ptsx[6], ptsx[7]
				status = face.material = ceiling_texture
			elsif fill_back_end_gap == 2 and step == back_spaces and back_spaces == front_spaces
				pts[5] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 2), 0]
				pts[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 3), 0]
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y -(section_depth * 3), 0]

				ptsx[4] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y -((section_depth * 2) * 3 / 4), door_height + arc]
				ptsx[5] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), door_height +(arc * 2 / 3)]
				ptsx[6] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 2), door_height +(arc * 2 / 3)]
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[2], ptsx[3], ptsx[4], ptsx[7]
				status = face.material = ceiling_texture
			elsif fill_back_start_gap == 1 and step == back_spaces and back_spaces != front_spaces
				pts[7] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - wall_thickness, 0]
				pts[9] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - section_depth, 0]
				pts[10] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - wall_thickness, 0]
				ptsx[6] = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y - wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				ptsx[10] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - section_depth, door_height +(arc * 2 / 3)]
				ptsx[11] = [cursor_x + wall_thickness +(wing_section_width * 4) + tail_section_width, cursor_y - wall_thickness, door_height +(arc *(wall_thickness / section_depth) * 2 / 3)]
				face = group_ents.add_face ptsx[0], ptsx[1], ptsx[2], ptsx[3], ptsx[4], ptsx[5], ptsx[10], ptsx[11], ptsx[6], ptsx[7]
				status = face.material = ceiling_texture
			else
				face = group_ents.add_face ptsx
				status = face.material = ceiling_texture
			end



			ptsx[8] = [cursor_x + wall_thickness + wing_section_width, cursor_y -(section_depth * 2), door_height +(arc * 2 / 3)]
			ptsx[9] = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), door_height +(arc * 2 / 3)]
			if fill_back_end_gap == 1 and step == back_spaces and back_spaces == front_spaces
				ptsx[9] = [cursor_x + wall_thickness + wing_section_width * 2 + tail_section_width, cursor_y -(section_depth * 2), door_height +(arc * 2 / 3)]
			elsif fill_back_end_gap == 2 and step == back_spaces and back_spaces == front_spaces
				ptsx[8] = [cursor_x + wall_thickness + wing_section_width + tail_section_width / 2 + wall_thickness / 2, cursor_y -(section_depth * 3), door_height]
				ptsx[9] = [cursor_x + wall_thickness + wing_section_width * 2 + tail_section_width, cursor_y -(section_depth * 3), door_height]
			end
			
			if fill_back_end_gap == 2 and step == back_spaces and back_spaces == front_spaces
				face = group_ents.add_face ptsx[3], ptsx[5], ptsx[6], ptsx[8], ptsx[9], ptsx[4]
				status = face.material = ceiling_texture
			else
				face = group_ents.add_face ptsx[8], ptsx[9], ptsx[4], ptsx[3]
				status = face.material = ceiling_texture
			end
			
			if fill_back_start_gap == 1 and step == 1
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[9], ptsx[10], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[9], pts[10], ptsx[11], ptsx[10]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[10], pts[3], ptsx[2], ptsx[11]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
				status = face.material = internal_wall_texture
			elsif fill_back_end_gap == 1 and step == back_spaces and back_spaces == front_spaces
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[6], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[8], ptsx[7], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
			elsif fill_back_end_gap == 2 and step == back_spaces and back_spaces == front_spaces
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[5], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[8], ptsx[6]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
			elsif fill_back_start_gap == 1 and step == back_spaces and back_spaces != front_spaces
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[9], ptsx[10], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[9], pts[10], ptsx[11], ptsx[10]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[10], pts[7], ptsx[6], ptsx[11]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
				status = face.material = internal_wall_texture

			else		
				face = group_ents.add_face pts[1], pts[2], ptsx[1], ptsx[0]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[2], pts[3], ptsx[2], ptsx[1]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[3], pts[4], ptsx[8], ptsx[3], ptsx[2]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[4], pts[5], ptsx[9], ptsx[8]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[5], pts[6], ptsx[5], ptsx[4], ptsx[9]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[6], pts[7], ptsx[6], ptsx[5]
				status = face.material = internal_wall_texture
				face = group_ents.add_face pts[7], pts[8], ptsx[7], ptsx[6]
				status = face.material = internal_wall_texture
			end
	
		end

		if floor_texture != "None"
			if fill_back_start_gap == 1 and step == 1
				face = group_ents.add_face pts[1], pts[2], pts[9], pts[10], pts[3], pts[4], pts[5], pts[6], pts[7], pts[8]
				status = face.material = floor_texture
			elsif fill_back_start_gap == 1 and step == back_spaces and back_spaces != front_spaces
				face = group_ents.add_face pts[1], pts[2], pts[3], pts[4], pts[5], pts[6], pts[9], pts[10], pts[7], pts[8]
				status = face.material = floor_texture
			else
				face = group_ents.add_face pts
				status = face.material = floor_texture
			end
		end

		cursor_x = cursor_x + section_width
	end

	#draw doors
	# Create separate group for door entities
#	door_group = Sketchup.active_model.entities.add_group
#	door_group_ents = door_group.entities
  if door_folding_identical == 1
    multi_door_array = []
    multi_door_back_array = []
    for hangar_step in 0..front_spaces - 1
      if door_type == "Bi-folding doors"
        pts = []
        pts[0] = [(hangar_step * section_width) + wall_thickness, 0, 0]
        pts[1] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, 0, 0]
        pts[2] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, 0, door_height]
        pts[3] = [(hangar_step * section_width) + wall_thickness, 0, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, wall_thickness, 0]
        pts[5] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
    
        pts = []
        pts[0] = [((hangar_step + 1) * section_width), 0, 0]
        pts[1] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, 0, 0]
        pts[2] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, 0, door_height]
        pts[3] = [((hangar_step + 1) * section_width), 0, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, wall_thickness, 0]
        pts[5] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
        multi_door_array[hangar_step] = (hangar_step * section_width) + wall_thickness + ((section_width - wall_thickness) / 2)
      else
        multi_door_array[hangar_step] = (hangar_step * section_width) + wall_thickness + ((section_width - wall_thickness) / 2)
      end
    end
    for hangar_step in 0..back_spaces - 1
      if door_type == "Bi-folding doors"
        pts = []
        pts[0] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, 0]
        pts[1] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth, 0]
        pts[2] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth, door_height]
        pts[3] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth - wall_thickness, 0]
        pts[5] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth - wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
    
        pts = []
        pts[0] = [((hangar_step + 1) * section_width) +(wing_section_width * 2), total_depth, 0]
        pts[1] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth, 0]
        pts[2] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth, door_height]
        pts[3] = [((hangar_step + 1) * section_width) +(wing_section_width * 2), total_depth, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth - wall_thickness, 0]
        pts[5] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth - wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture
        multi_door_back_array[hangar_step] = (hangar_step * section_width) + (wing_section_width * 2) + wall_thickness + ((section_width - wall_thickness) / 2)
      else
        multi_door_back_array[hangar_step] = (hangar_step * section_width) + (wing_section_width * 2) + wall_thickness + ((section_width - wall_thickness) / 2)
      
      end
    end
    
    
    if door_type == "Bi-folding doors"
      door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, section_width - wall_thickness - door_frame_thickness * 6, multi_door_array, 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X")
      back_door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, section_width - wall_thickness - door_frame_thickness * 6, multi_door_back_array, total_depth - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "-X")
    else
      door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, section_width - wall_thickness, multi_door_array, 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X")
      back_door_group = insert_door(door_type, door_height, door_folding_level.to_i, door_sections, section_width - wall_thickness, multi_door_back_array, total_depth - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "-X")
    end
    
    group_x = group
    group = group_x.parent.entities.add_group([group_x, door_group, back_door_group])

  else
    for hangar_step in 0..front_spaces - 1
      if door_type == "Bi-folding doors"
        pts = []
        pts[0] = [(hangar_step * section_width) + wall_thickness, 0, 0]
        pts[1] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, 0, 0]
        pts[2] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, 0, door_height]
        pts[3] = [(hangar_step * section_width) + wall_thickness, 0, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, wall_thickness, 0]
        pts[5] = [(hangar_step * section_width) + wall_thickness + door_frame_thickness * 3, wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
    
        pts = []
        pts[0] = [((hangar_step + 1) * section_width), 0, 0]
        pts[1] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, 0, 0]
        pts[2] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, 0, door_height]
        pts[3] = [((hangar_step + 1) * section_width), 0, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, wall_thickness, 0]
        pts[5] = [((hangar_step + 1) * section_width) - door_frame_thickness * 3, wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
        door_group = insert_door(door_type, door_height, door_folding_ext_level[hangar_step].to_i, door_sections, section_width - wall_thickness - door_frame_thickness * 6, (hangar_step * section_width) + wall_thickness +((section_width - wall_thickness) / 2), 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X")
      else
        door_group = insert_door(door_type, door_height, door_folding_ext_level[hangar_step].to_i, door_sections, section_width - wall_thickness, (hangar_step * section_width) + wall_thickness + ((section_width - wall_thickness) / 2), 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X")
      end
      group_x = group
      group = group_x.parent.entities.add_group([group_x, door_group])
    end
  
    for hangar_step in 0..back_spaces - 1
      if door_type == "Bi-folding doors"
        pts = []
        pts[0] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, 0]
        pts[1] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth, 0]
        pts[2] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth, door_height]
        pts[3] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness, total_depth, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth - wall_thickness, 0]
        pts[5] = [(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness + door_frame_thickness * 3, total_depth - wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
    
        pts = []
        pts[0] = [((hangar_step + 1) * section_width) +(wing_section_width * 2), total_depth, 0]
        pts[1] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth, 0]
        pts[2] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth, door_height]
        pts[3] = [((hangar_step + 1) * section_width) +(wing_section_width * 2), total_depth, door_height]
        face = group_ents.add_face pts
        status = face.material = wall_texture 
        pts[4] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth - wall_thickness, 0]
        pts[5] = [((hangar_step + 1) * section_width) +(wing_section_width * 2) - door_frame_thickness * 3, total_depth - wall_thickness, door_height]
        face = group_ents.add_face pts[1], pts[2], pts[5], pts[4]
        status = face.material = wall_texture 
        door_group = insert_door(door_type, door_height, door_folding_ext_level[front_spaces + hangar_step].to_i, door_sections, section_width - wall_thickness - door_frame_thickness * 6,(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness +((section_width - wall_thickness) / 2), total_depth - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "-X")
      else
        door_group = insert_door(door_type, door_height, door_folding_ext_level[front_spaces + hangar_step].to_i, door_sections, section_width - wall_thickness,(hangar_step * section_width) +(wing_section_width * 2) + wall_thickness +((section_width - wall_thickness) / 2), total_depth - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "-X")
      end
      group_x = group
      group = group_x.parent.entities.add_group([group_x, door_group])
    end



  end
	


	if afcad_read == 1
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y - section_depth / 2 + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t		
		if flip_hangar == "Yes"
			point2 = Geom::Point3d.new(hangar_x +(total_width/2), hangar_y, 0)
			t = Geom::Transformation.scaling(point2, -1, 1, 1)
			group.transform! t
		end
		t = Geom::Transformation.rotation [hangar_x + section_width / 2, hangar_y + section_depth / 2, 0], Z_AXIS, radians
		group.transform! t
	else
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t		
		group.name = "TDRow(&ltid&gtx&ltF/B&gt):" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + input[14].to_s + ":" + input[15].to_s + ":" + input[16].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":" + texture_input[4].to_s + ":" + texture_input[5].to_s + ":" + texture_input[6].to_s + ":"
		if flip_hangar == "Yes"
			point2 = Geom::Point3d.new(hangar_x +(total_width/2), hangar_y, 0)
			t = Geom::Transformation.scaling(point2, -1, 1, 1)
			group.transform! t
		end
		t = Geom::Transformation.rotation [hangar_x, hangar_y, 0], Z_AXIS, radians
		group.transform! t
	end
end



###################################################
###################################################
###### Below is the code for the t-hangar row	#####
###################################################
###################################################


def t_hangar_row(hangar_codes = "", midlat = 0.0, midlong = 0.0)

  

	#Define entity group and materials
	group = Sketchup.active_model.entities.add_group
	group_ents = group.entities
	group.description = "T-hangar row"
	materials = Sketchup.active_model.materials
	afcad_read = 0

	if hangar_codes != "" and hangar_codes != "Box"

		afcad_read = 1	
		hangar_info = hangar_codes.split(":")


		#Get texture info
		wall_texture = hangar_info[22]
		internal_wall_texture = hangar_info[23]
		roof_texture = hangar_info[24]
		ceiling_texture = hangar_info[25]
		door_texture = hangar_info[26]
		door_frame_texture = hangar_info[27]
		floor_texture = hangar_info[28]
		beam_texture = "Blue metal"
		if internal_wall_texture == "Same as external"
			internal_wall_texture = wall_texture
		end
		cross_beam_texture = ""
		front_spaces = hangar_info[34].to_i
		back_spaces = hangar_info[35].to_i

		if front_spaces <= 1
			front_spaces = 1
			back_spaces = 1
		end
		
		if front_spaces != back_spaces
			back_spaces = front_spaces - 1
		end

		wall_thickness = hangar_info[7].to_f / $inch

		tail_section_width =(hangar_info[33].to_f  / $inch - wall_thickness) / 3
		wing_section_width =(hangar_info[33].to_f  / $inch - wall_thickness) / 3

		section_depth = hangar_info[6].to_f / $inch
		section_width =(wing_section_width * 2) + tail_section_width + wall_thickness
		door_height = hangar_info[8].to_f / $inch
		outside_height = hangar_info[9].to_f / $inch
		arc = hangar_info[10].to_f / $inch
		eaves_overhang = hangar_info[11].to_f / $inch

		door_type = hangar_info[12]
		door_folding_level = hangar_info[13].to_i
		door_sections = hangar_info[14].to_i
		door_frame_thickness = hangar_info[15].to_f / $inch
		door_skin_thickness = hangar_info[16].to_f / $inch
		flat_ceiling = hangar_info[17]
		extension_text = hangar_info[21]
		flip_hangar = hangar_info[3]



		#Calculate hangar's distance from midpoint
		earth_radius = 6371
		lat1 = midlat
		lat2 = hangar_info[30].to_f
		long1 = midlong
		long2 = hangar_info[31].to_f
		dlaty =(lat2-lat1).degrees
		dlongy =(0).degrees
		dlatx =(0).degrees
		dlongx =(long2-long1).degrees
		rlat1 = lat1.degrees
		rlat2 = lat2.degrees
		
		calc_a = Math.sin(dlaty/2) * Math.sin(dlaty/2) + Math.sin(dlongy/2) * Math.sin(dlongy/2) * Math.cos(rlat1) * Math.cos(rlat2)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		y_distance =(earth_radius * calc_c * 100000)
		calc_a = Math.sin(dlatx/2) * Math.sin(dlatx/2) + Math.sin(dlongx/2) * Math.sin(dlongx/2) * Math.cos(rlat1) * Math.cos(rlat1)
		calc_c = 2 * Math.atan2(Math.sqrt(calc_a), Math.sqrt(1-calc_a))
		x_distance =(earth_radius * calc_c * 100000)


		if long1 > long2
			x_distance = 0 - x_distance
		end
		
		if lat1 > lat2
			y_distance = 0 - y_distance
		end
		
		hangar_angle = 360 - hangar_info[32].to_f
		hangar_x = x_distance / $inch
		hangar_y = y_distance / $inch
		radians = hangar_angle.degrees
	
	else
	

    angle = find_endpoints
    
    if hangar_codes == ""
      if angle[3] == 0
        angle[3] =((600 * 2) + 10) / $inch
      end
      default_hangar_height = 340
      default_hangar_arc_height = 120
    else
      default_hangar_height = midlat * $inch
      default_hangar_arc_height = midlong * $inch
    end     


	
		prompts = ["Front spaces?", "Back spaces?", "Flip?", "Wing section width?", "Tail section width?", "Section depth?", "Wall thickness?", "Internal height?", "External height?", "Arc height?", "Eaves overhang?", "Door type?", "Door folding level?", "Door sections?", "Door frame thickness?", "Door skin thickness?", "Flat ceiling?", "Angle?", "X-pos?", "Y-pos?", "Extensions?"]
		defaults = ["4", "4", "No", "500", "600", (((angle[3] / 2) * $inch) - (10 / 2)).ceil, "5", default_hangar_height, default_hangar_height + 10, default_hangar_arc_height, "0", "Bi-folding doors", "5", "1", "5", "2", "Yes", angle[0], angle[1] * $inch, angle[2] * $inch, ""]
		list = ["", "", $yn, "", "", "", "", "", "", "", "", "Bi-folding doors|Hydraulic swing door|Sliding doors|Random mix", "", "", "", "", $yn, "", "", "", ""]
		input = UI.inputbox prompts, defaults, list, "Hangar info"


	 	#Texture prompts
	 	texture_prompts = ["Wall texture?", "Internal wall texture?", "Roof texture?", "Ceiling texture?", "Door texture?", "Door frame texture?", "Floor texture?"]
	 	texture_defaults = ["Vertical corrugated iron", "Same as external", "Vertical square iron", "Vertical square iron", "Vertical square iron", "Rusty metal", "Concrete"]
	 	texture_list = [$wall_list, $internal_wall_list, $roof_list, $roof_list, $door_list, $metal_list, $floor_list]
	 	texture_input = UI.inputbox texture_prompts, texture_defaults, texture_list, "Texture info"


		#Get texture info
		wall_texture = texture_input[0]
		internal_wall_texture = texture_input[1]
		roof_texture = texture_input[2]
		ceiling_texture = texture_input[3]
		door_texture = texture_input[4]
		door_frame_texture = texture_input[5]
		floor_texture = texture_input[6]
		beam_texture = "Blue metal"
		if internal_wall_texture == "Same as external"
			internal_wall_texture = wall_texture
		end
		cross_beam_texture = ""

		front_spaces = input[0].to_i
		back_spaces = input[1].to_i


		if front_spaces <= 1
			front_spaces = 1
			back_spaces = 1
		end
		
		if front_spaces != back_spaces
			back_spaces = front_spaces - 1
		end

		tail_section_width = input[4].to_f / $inch
		wall_thickness = input[6].to_f / $inch

		if angle[4] == 0
			wing_section_width = input[3].to_f / $inch
		else
			total_hangar_length = angle[4]
			if front_spaces == back_spaces
				wing_section_width =(total_hangar_length -((tail_section_width *(back_spaces - 1)) + tail_section_width) -((wall_thickness * 2 * back_spaces) + wall_thickness) -(tail_section_width * front_spaces)) /((front_spaces * 2) + 1)
				total_length =(((wing_section_width * 2) + tail_section_width) * front_spaces) +(tail_section_width *(back_spaces - 1)) +(tail_section_width + wing_section_width) +((wall_thickness * 2 * back_spaces ) + wall_thickness)
			else
				wing_section_width =(total_hangar_length -(tail_section_width * back_spaces) -((tail_section_width +(wall_thickness * 2)) * front_spaces)) /(front_spaces * 2)
				total_length =(((wing_section_width * 2) + tail_section_width +(wall_thickness * 2)) * front_spaces) +(tail_section_width * back_spaces)
			end
		end

		section_depth = input[5].to_f / $inch
		section_width =(wing_section_width * 2) + tail_section_width + wall_thickness
		door_height = input[7].to_f / $inch
		outside_height = input[8].to_f / $inch
		arc = input[9].to_f / $inch
		eaves_overhang = input[10].to_f / $inch
		
		door_type = input[11]
		door_folding_level = input[12].to_i
		door_sections = input[13].to_i
		door_frame_thickness = input[14].to_f / $inch
		door_skin_thickness = input[15].to_f / $inch
		flat_ceiling = input[16]
		hangar_angle = input[17].to_f
		hangar_x = input[18].to_f / $inch
		hangar_y = input[19].to_f / $inch
		extension_text = input[20]
		radians = hangar_angle.degrees
		flip_hangar = input[2]
	
	end
	

	if door_type == "Sliding doors"
		door_type = "Sliding doors - row"
	end	
	
	for step in 0..$texture_name.length
		if wall_texture == $texture_name[step] or roof_texture == $texture_name[step] or door_texture == $texture_name[step] or floor_texture == $texture_name[step] or beam_texture == $texture_name[step] or door_frame_texture == $texture_name[step] or ceiling_texture == $texture_name[step] or internal_wall_texture == $texture_name[step]
			texture = materials.add $texture_name[step]
      if $texture_filename[step].count(":") === 0
        texture.texture = $texture_files + $texture_filename[step]
      else
        texture.texture = $texture_filename[step]
      end
			texture.texture.size = [$texture_x_size[step] / $inch, $texture_y_size[step] / $inch]
		end	
	end	
	
	if wall_texture == "Untextured" or roof_texture == "Untextured" or door_texture == "Untextured" or floor_texture == "Untextured" or beam_texture == "Untextured" or door_frame_texture == "Untextured" or internal_wall_texture == "Untextured" or cross_beam_texture == "Untextured"
			texture = materials.add "Untextured"
			texture.color = "White"
	end
	
	cursor_x = 0
	cursor_y = 0
	total_depth = (section_depth * 2) + wall_thickness
	
	if front_spaces == back_spaces
		total_length =(((wing_section_width * 2) + tail_section_width) * front_spaces) +(tail_section_width *(back_spaces - 1)) +(tail_section_width + wing_section_width) +((wall_thickness * 2 * back_spaces ) + wall_thickness)
	else
		total_length =(((wing_section_width * 2) + tail_section_width +(wall_thickness * 2)) * front_spaces) +(tail_section_width * back_spaces)
	end


####################################################
####	Get extension info			####
####################################################
	
	extensions = []
	extension_types = []
	end_gaps_filled = 0
	fill_front_end_gap = 0
	fill_back_end_gap = 0
	entrance_door_wall = []
	entrance_door_count = 0
	entrance_door_x_pos = []
	entrance_door_y_pos = []
	entrance_door_x_size = []
	entrance_door_y_size = []
	entrance_door_texture = []
	entrance_door_glass_texture = []
	entrance_door_glass_back_texture = []
	entrance_door_type = []
	sign_count = 0
	sign_x_pos = []			# Position on wall from front or left
	sign_y_pos = []			# Position on wall from ground
	sign_x_size = []		# Sign width
	sign_y_size = []		# Sign height
	sign_thickness = []		# Sign thickness
	sign_double_sided = []		# 0 for logo on front only(only valid for textured signs), 1 for logo on both sides
	sign_wall = []			# Wall to put sign on, F, B, L, R
	sign_texture = []		# Sign texture name
	sign_type = []			# Sign type; L(logo), T(text)
	sign_filled = []		# 0 to not adjust texture size, 1 to adjust texture size.
  door_folding_ext_count = 0
  door_folding_ext_level = []
  
	if extension_text.length > 0
		if extension_text.count('|') > 0
			extensions = extension_text.split("|")
		else
			extensions[0] = extension_text
		end
		for step in 0..extensions.length - 1
			if extensions[step].count('(') > 0 and extensions[step].count(')') > 0 
				extension_types[step] = extensions[step][0..extensions[step].index("(") - 1]
			else
				extension_types[step] = "ERR"
			end
			
			# Fill end gap
			if extension_types[step] == "G" and end_gaps_filled == 0
				end_gap_info = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1].to_i
				
				if end_gap_info == 0
					fill_front_end_gap = 1
				elsif end_gap_info == 1
					fill_back_end_gap = 1
				else
					fill_front_end_gap = 1
					fill_back_end_gap = 1
				end
				end_gaps_filled == 1
			end

      # Custom door folding levels
      if extension_types[step] == "DF" and door_folding_ext_count == 0
        door_folding_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
        door_folding_ext_level = door_folding_string.split(";")
        door_folding_ext_count = 1
        if door_folding_ext_level.length < (front_spaces + back_spaces)
          for step2 in door_folding_ext_level.length..(front_spaces + back_spaces - 1)
            door_folding_ext_level[step2] = door_folding_level
          end
        end
        for step2 in 0..door_folding_ext_level.length - 1
          door_folding_ext_level[step2] = door_folding_ext_level[step2].to_i
          if door_folding_ext_level[step2] > 90
            door_folding_ext_level[step2] = 90
          end
        end
      end

			# Entrance doors
			if extension_types[step] == "ED"
				entrance_door_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if entrance_door_string.count(";") == 4 or entrance_door_string.count(";") == 5
					entrance_door_failed = 0
					entrance_door_details = entrance_door_string.split(";")
					entrance_door_wall[entrance_door_count] = entrance_door_details[2]
					if entrance_door_details[0].count("x") == 1
						entrance_door_temp_string = entrance_door_details[0].split("x")
						entrance_door_x_pos[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_pos[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
						if entrance_door_y_pos[entrance_door_count] < 1 / $inch
							entrance_door_y_pos[entrance_door_count] = 1 / $inch
						end
					else
						entrance_door_failed = 1
					end
					if entrance_door_details[1].count("x") == 1
						entrance_door_temp_string = entrance_door_details[1].split("x")
						entrance_door_x_size[entrance_door_count] = entrance_door_temp_string[0].to_f / $inch
						entrance_door_y_size[entrance_door_count] = entrance_door_temp_string[1].to_f / $inch
					else
						entrance_door_failed = 1
					end
					entrance_door_texture[entrance_door_count] = entrance_door_details[4]
					entrance_door_type[entrance_door_count] = entrance_door_details[3]
					
					if entrance_door_string.count(";") == 5

						entrance_door_glass_texture[entrance_door_count] = entrance_door_details[5]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_details[5]
					
					else
						entrance_door_glass_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
						entrance_door_glass_back_texture[entrance_door_count] = entrance_door_texture[entrance_door_count]
					end
					
					if entrance_door_failed == 0
#						cross_beam_exclusion_side[cross_beam_exclusion_count] = entrance_door_wall[entrance_door_count]
#						cross_beam_exclusions[cross_beam_exclusion_count] = [entrance_door_x_pos[entrance_door_count], entrance_door_y_pos[entrance_door_count], entrance_door_x_pos[entrance_door_count] + entrance_door_x_size[entrance_door_count], entrance_door_y_pos[entrance_door_count] + entrance_door_y_size[entrance_door_count]]
#						cross_beam_extras_drawn[cross_beam_exclusion_count] = 0
#						cross_beam_exclusion_count = cross_beam_exclusion_count + 1
						entrance_door_count = entrance_door_count + 1
					end
				end
			end

			# Signage
			if extension_types[step] == "S"
				sign_string = extensions[step][extensions[step].index("(") + 1..extensions[step].index(")") - 1]
				if sign_string.count(";") >= 3
					sign_failed = 0
					sign_details = sign_string.split(";")
					sign_type[sign_count] = sign_details[0]
					if sign_details[1].count("x") == 1
						sign_temp_string = sign_details[1].split("x")
						sign_x_pos[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_pos[sign_count] = sign_temp_string[1].to_f / $inch
					else
						sign_failed = 1
					end
					if sign_details[2].count("x") == 2
						sign_temp_string = sign_details[2].split("x")
						sign_x_size[sign_count] = sign_temp_string[0].to_f / $inch
						sign_y_size[sign_count] = sign_temp_string[1].to_f / $inch
						sign_thickness[sign_count] = sign_temp_string[2].to_f / $inch
					else
						sign_failed = 1
					end
					sign_wall[sign_count] = sign_details[3]
					if sign_type[sign_count] == "L" and sign_string.count(";") == 6
						sign_texture[sign_count] = sign_details[4]
						sign_filled[sign_count] = sign_details[5].to_i
						sign_double_sided[sign_count] = sign_details[6].to_i
					
					else
						sign_failed = 1
					end
					
					if sign_failed == 0
						sign_count = sign_count + 1
					end
				end
			end

		end
	end


  if door_folding_ext_count === 0
    for step in 0..(front_spaces + back_spaces - 1)
      door_folding_ext_level[step] = door_folding_level
    end  
  end
  

####################################################
####	Set bounds				####
####################################################

external_back_position = 0
external_front_position = total_length
external_left_position = 0
external_right_position = total_depth
internal_back_position = wall_thickness
internal_front_position = total_length - wall_thickness
internal_left_position = wall_thickness
internal_right_position = total_depth - wall_thickness
roof_back_position = external_back_position - eaves_overhang
roof_front_position = external_front_position + eaves_overhang

	
####################################################
####	Create walls				####
####################################################
	# Create end walls
	
	pts = []
	pts[0] = [0,0,0]
	pts[1] = [0,0, outside_height]
	pts[2] = [0, total_depth / 2, outside_height + arc]
	pts[3] = [0, total_depth, outside_height]
	pts[4] = [0, total_depth, 0]
	face = group_ents.add_face pts
	status = face.material = wall_texture
	
	pt1 = [total_length,0,0]
	pt2 = [total_length,0, outside_height]
	pt3 = [total_length, total_depth / 2, outside_height + arc]
	pt4 = [total_length, total_depth, outside_height]
	pt5 = [total_length, total_depth, 0]
	face = group_ents.add_face pt1, pt2, pt3, pt4, pt5
	status = face.reverse!
	status = face.material = wall_texture

	for step in 1..front_spaces
		pt1 = [cursor_x, cursor_y, 0]
		pt2 = [cursor_x + wall_thickness, cursor_y, 0]
		pt3 = [cursor_x + wall_thickness, cursor_y + section_depth, 0]
		pt4 = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, 0]
		pt5 = [cursor_x + wall_thickness + wing_section_width, cursor_y + (section_depth * 2), 0]
		pt6 = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + (section_depth * 2), 0]
		pt7 = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, 0]
		pt8 = [cursor_x + wall_thickness + (wing_section_width * 2) + tail_section_width, cursor_y + section_depth, 0]
		pt9 = [cursor_x + wall_thickness +  (wing_section_width * 2) + tail_section_width, cursor_y, 0]
		pt10 = [cursor_x + (wall_thickness * 2) +(wing_section_width * 2) +(tail_section_width * 2), cursor_y, 0]

		#create inner walls
		
		pt2x = [cursor_x + wall_thickness, cursor_y, door_height]
		pt5x = [cursor_x + wall_thickness + wing_section_width, cursor_y +(section_depth * 2), door_height]
		pt6x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y +(section_depth * 2), door_height]
		pt9x = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y, door_height]

		if flat_ceiling == "Yes"
			pt3x = [cursor_x + wall_thickness, cursor_y + section_depth, door_height]
			pt4x = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, door_height]
			pt7x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, door_height]
			pt8x = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, door_height]
		else
			pt3x = [cursor_x + wall_thickness, cursor_y + section_depth, door_height + arc]
			pt4x = [cursor_x + wall_thickness + wing_section_width, cursor_y + section_depth, door_height + arc]
			pt7x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y + section_depth, door_height + arc]
			pt8x = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y + section_depth, door_height + arc]
		end

		if step == 1 and fill_front_end_gap == 1
			pt4 = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), 0]
			pt4x = [cursor_x + wall_thickness, cursor_y +(section_depth * 2), door_height]
		end
		if step == front_spaces and fill_back_end_gap == 1 and front_spaces != back_spaces
			pt7 = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), 0]
			pt7x = [cursor_x + wall_thickness +(wing_section_width * 2) + tail_section_width, cursor_y +(section_depth * 2), door_height]
		end


		face = group_ents.add_face pt2, pt2x, pt3x, pt3
		status = face.reverse!
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt3, pt3x, pt4x, pt4
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt4, pt4x, pt5x, pt5
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt5, pt5x, pt6x, pt6
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt6, pt6x, pt7x, pt7
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt7, pt7x, pt8x, pt8
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt8, pt8x, pt9x, pt9
		status = face.material = internal_wall_texture

		if flat_ceiling == "Yes"
			face = group_ents.add_face pt2x, pt3x, pt4x, pt5x, pt6x, pt7x, pt8x, pt9x
			status = face.material = ceiling_texture
		else
			face = group_ents.add_face pt2x, pt3x, pt8x, pt9x
			status = face.material = ceiling_texture
			if step == 1 and fill_front_end_gap == 1
				face = group_ents.add_face pt3x, pt4x, pt6x, pt7x
				status = face.material = ceiling_texture
			elsif step == front_spaces and fill_back_end_gap == 1 and front_spaces != back_spaces
				face = group_ents.add_face pt4x, pt5x, pt7x, pt8x
				status = face.material = ceiling_texture
			else
				face = group_ents.add_face pt4x, pt5x, pt6x, pt7x
				status = face.material = ceiling_texture
			end
		end

		cursor_x = cursor_x +(wall_thickness * 2) +(wing_section_width * 2) +(tail_section_width * 2)
	end
	
	cursor_x = wall_thickness + wing_section_width
	cursor_y = total_depth

	for step in 1..back_spaces

		pt1 = [cursor_x, cursor_y, 0]
		pt2 = [cursor_x + wall_thickness + tail_section_width, cursor_y, 0]
		pt3 = [cursor_x + wall_thickness + tail_section_width, cursor_y - section_depth, 0]
		pt4 = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, 0]
		pt5 = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), 0]
		pt6 = [cursor_x + wall_thickness + wing_section_width +(tail_section_width * 2), cursor_y -(section_depth * 2), 0]
		pt7 = [cursor_x + wall_thickness + wing_section_width +(tail_section_width * 2), cursor_y - section_depth, 0]
		pt8 = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y - section_depth, 0]
		pt9 = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y, 0]
		pt10 = [cursor_x +(wall_thickness * 2) +(wing_section_width * 2) +(tail_section_width * 2), cursor_y, 0]

		#create inner walls
		pt2x = [cursor_x + wall_thickness + tail_section_width, cursor_y, door_height]
		pt5x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y -(section_depth * 2), door_height]
		pt6x = [cursor_x + wall_thickness + wing_section_width +(tail_section_width * 2), cursor_y -(section_depth * 2), door_height]
		pt9x = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y, door_height]

		if flat_ceiling == "Yes"
			pt3x = [cursor_x + wall_thickness + tail_section_width, cursor_y - section_depth, door_height]
			pt4x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, door_height]
			pt7x = [cursor_x + wall_thickness + wing_section_width +(tail_section_width * 2), cursor_y - section_depth, door_height]
			pt8x = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y - section_depth, door_height]
		else
			pt3x = [cursor_x + wall_thickness + tail_section_width, cursor_y - section_depth, door_height + arc]
			pt4x = [cursor_x + wall_thickness + wing_section_width + tail_section_width, cursor_y - section_depth, door_height + arc]
			pt7x = [cursor_x + wall_thickness + wing_section_width +(tail_section_width * 2), cursor_y - section_depth, door_height + arc]
			pt8x = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y - section_depth, door_height + arc]
		end

		if step == front_spaces and fill_back_end_gap == 1 and front_spaces == back_spaces
			pt7 = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y -(section_depth * 2), 0]
			pt7x = [cursor_x + wall_thickness +(wing_section_width * 2) +(tail_section_width * 2), cursor_y -(section_depth * 2), door_height]
		end


		face = group_ents.add_face pt2, pt2x, pt3x, pt3
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt3, pt3x, pt4x, pt4
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt4, pt4x, pt5x, pt5
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt5, pt5x, pt6x, pt6
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt6, pt6x, pt7x, pt7
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt7, pt7x, pt8x, pt8
		status = face.material = internal_wall_texture
		face = group_ents.add_face pt8, pt8x, pt9x, pt9
		status = face.material = internal_wall_texture

		if flat_ceiling == "Yes"
			face = group_ents.add_face pt2x, pt3x, pt4x, pt5x, pt6x, pt7x, pt8x, pt9x
			status = face.material = ceiling_texture
		else
			face = group_ents.add_face pt2x, pt3x, pt8x, pt9x
			status = face.material = ceiling_texture
			if step == front_spaces and fill_back_end_gap == 1 and front_spaces == back_spaces
				face = group_ents.add_face pt4x, pt5x, pt7x, pt8x
				status = face.material = ceiling_texture
			else
				face = group_ents.add_face pt4x, pt5x, pt6x, pt7x
				status = face.material = ceiling_texture
			end
		end

		cursor_x = cursor_x +(wall_thickness * 2) +(wing_section_width * 2) +(tail_section_width * 2)
	end


	#draw outer walls
	cursor_x = 0
	pts = []

	for step in 1..front_spaces
		if step == 1
			pts [(step - 1) * 5] = [cursor_x,0,0]
			pts [((step - 1) * 5) + 1] = [cursor_x + wall_thickness,0,0]
			pts [((step - 1) * 5) + 2] = [cursor_x + wall_thickness,0, door_height]
			pts [((step - 1) * 5) + 3] = [cursor_x + wall_thickness + (wing_section_width * 2) + tail_section_width, 0, door_height]
			pts [((step - 1) * 5) + 4] = [cursor_x + wall_thickness + (wing_section_width * 2) + tail_section_width, 0, 0]
			cursor_x = cursor_x + (wall_thickness * 3) + (wing_section_width * 2) + (tail_section_width * 2)
		else
			pts [((step - 1) * 4) + 1] = [cursor_x,0,0]
			pts [((step - 1) * 4) + 2] = [cursor_x ,0, door_height]
			pts [((step - 1) * 4) + 3] = [cursor_x + (wing_section_width * 2) + tail_section_width, 0, door_height]
			pts [((step - 1) * 4) + 4] = [cursor_x + (wing_section_width * 2) + tail_section_width, 0, 0]
			cursor_x = cursor_x + (wall_thickness * 2) + (wing_section_width * 2) + (tail_section_width * 2)
		end
	end
	
	pts [(front_spaces * 4) + 1] = [total_length,0,0]
	pts [(front_spaces * 4) + 2] = [total_length,0,outside_height]
	pts [(front_spaces * 4) + 3] = [0,0,outside_height]
	face = group_ents.add_face pts
	status = face.material = wall_texture
	
	cursor_x = 0
	pts = []
	
	for step in 1..back_spaces
		if step == 1
			pts [(step - 1) * 5] = [cursor_x,total_depth,0]
			pts [((step - 1) * 5) + 1] = [cursor_x +(wall_thickness * 2) + wing_section_width + tail_section_width, total_depth,0]
			pts [((step - 1) * 5) + 2] = [cursor_x +(wall_thickness * 2) + wing_section_width + tail_section_width, total_depth, door_height]
			pts [((step - 1) * 5) + 3] = [cursor_x +(wall_thickness * 2) +(wing_section_width * 3) +(tail_section_width * 2), total_depth, door_height]
			pts [((step - 1) * 5) + 4] = [cursor_x +(wall_thickness * 2) +(wing_section_width * 3) +(tail_section_width * 2), total_depth, 0]
			cursor_x = cursor_x +(wall_thickness * 4) +(wing_section_width * 3) +(tail_section_width * 3)
		else
			pts [((step - 1) * 4) + 1] = [cursor_x, total_depth,0]
			pts [((step - 1) * 4) + 2] = [cursor_x, total_depth, door_height]
			pts [((step - 1) * 4) + 3] = [cursor_x +(wing_section_width * 2) + tail_section_width, total_depth, door_height]
			pts [((step - 1) * 4) + 4] = [cursor_x +(wing_section_width * 2) + tail_section_width, total_depth, 0]
			cursor_x = cursor_x +(wall_thickness * 2) +(wing_section_width * 2) +(tail_section_width * 2)
		end
	end
	pts [(back_spaces * 4) + 1] = [total_length,total_depth,0]
	pts [(back_spaces * 4) + 2] = [total_length,total_depth,outside_height]
	pts [(back_spaces * 4) + 3] = [0,total_depth,outside_height]	
	face = group_ents.add_face pts
	status = face.material = wall_texture

	#add roof	
	for step in 0..1
		pts = []
		pts[0] = [roof_back_position, total_depth * step, outside_height]
		pts[1] = [roof_front_position, total_depth * step, outside_height]
		pts[2] = [roof_front_position, total_depth / 2, outside_height + arc]
		pts[3] = [roof_back_position, total_depth / 2, outside_height + arc]
		face = group_ents.add_face pts
		status = face.material = roof_texture
		if eaves_overhang > 0
			pts[4] = [roof_back_position, total_depth * step, outside_height - wall_thickness]
			pts[5] = [external_back_position, total_depth * step, outside_height - wall_thickness]
			pts[6] = [external_back_position, total_depth / 2, outside_height + arc - wall_thickness]
			pts[7] = [roof_back_position, total_depth / 2, outside_height + arc - wall_thickness]
			pts[8] = [external_back_position, total_depth * step, outside_height]
			face = group_ents.add_face pts[0], pts[3], pts[7], pts[4]
			status = face.material = roof_texture
			face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
			status = face.material = roof_texture
			face = group_ents.add_face pts[0], pts[8], pts[5], pts[4]
			status = face.material = roof_texture
			pts[9] = [roof_front_position, total_depth * step, outside_height - wall_thickness]
			pts[10] = [external_front_position, total_depth * step, outside_height - wall_thickness]
			pts[11] = [external_front_position, total_depth / 2, outside_height + arc - wall_thickness]
			pts[12] = [roof_front_position, total_depth / 2, outside_height + arc - wall_thickness]
			pts[13] = [external_front_position, total_depth * step, outside_height]
			face = group_ents.add_face pts[1], pts[2], pts[12], pts[9]
			status = face.material = roof_texture
			face = group_ents.add_face pts[9], pts[10], pts[11], pts[12]
			status = face.material = roof_texture
			face = group_ents.add_face pts[1], pts[13], pts[10], pts[9]
			status = face.material = roof_texture
		end
	end

	#Add floor
	if floor_texture != "None"
		pts = []
		pts[0] = [0,0,0]
		pts[1] = [total_length,0,0]
		pts[2] = [total_length,total_depth,0]
		pts[3] = [0,total_depth,0]
		face = group_ents.add_face pts
		status = face.reverse!
		status = face.material = floor_texture
	end

####################################################
####	Draw entrance doors			####
####################################################
	
	if entrance_door_count > 0
		for step in 0..entrance_door_count - 1
			if entrance_door_wall[step] == "L" or entrance_door_wall[step] == "R"   
				if entrance_door_wall[step] == "L"
					current_wall_position = external_left_position
					current_wall_internal_position = internal_left_position
					side_step_index = 1
				else
					current_wall_position = external_right_position
					current_wall_internal_position = internal_right_position
					side_step_index = -1
				end
				pts = []
				pts[0] = [external_front_position - entrance_door_x_pos[step], current_wall_position, entrance_door_y_pos[step]]
				pts[1] = [external_front_position - entrance_door_x_pos[step] - entrance_door_x_size[step], current_wall_position, entrance_door_y_pos[step]]
				pts[2] = [external_front_position - entrance_door_x_pos[step] - entrance_door_x_size[step], current_wall_position, entrance_door_y_pos[step] + entrance_door_y_size[step]]
				pts[3] = [external_front_position - entrance_door_x_pos[step], current_wall_position, entrance_door_y_pos[step] + entrance_door_y_size[step]]
						
				face = group_ents.add_face pts
				status = face.erase!
				if wall_thickness > 0
					pts[4] = [external_front_position - entrance_door_x_pos[step], current_wall_internal_position, entrance_door_y_pos[step]]
					pts[5] = [external_front_position - entrance_door_x_pos[step] - entrance_door_x_size[step], current_wall_internal_position, entrance_door_y_pos[step]]
					pts[6] = [external_front_position - entrance_door_x_pos[step] - entrance_door_x_size[step], current_wall_internal_position, entrance_door_y_pos[step] + entrance_door_y_size[step]]
					pts[7] = [external_front_position - entrance_door_x_pos[step], current_wall_internal_position, entrance_door_y_pos[step] + entrance_door_y_size[step]]
	
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					status = face.erase!
				end
			elsif entrance_door_wall[step] == "B" or entrance_door_wall[step] == "F"   

				if entrance_door_wall[step] == "B"
					current_wall_position = external_back_position
					current_wall_internal_position = internal_back_position
				else
					current_wall_position = external_front_position
					current_wall_internal_position = internal_front_position
				end
				pts = []
				pts[0] = [current_wall_position, external_left_position + entrance_door_x_pos[step], entrance_door_y_pos[step]]
				pts[1] = [current_wall_position, external_left_position + entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
				pts[2] = [current_wall_position, external_left_position + entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				pts[3] = [current_wall_position, external_left_position + entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
				
				face = group_ents.add_face pts
				status = face.erase!
				if wall_thickness > 0
					pts[4] = [current_wall_internal_position, external_left_position + entrance_door_x_pos[step], entrance_door_y_pos[step]]
					pts[5] = [current_wall_internal_position, external_left_position + entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step]]
					pts[6] = [current_wall_internal_position, external_left_position + entrance_door_x_pos[step] + entrance_door_x_size[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
					pts[7] = [current_wall_internal_position, external_left_position + entrance_door_x_pos[step], entrance_door_y_pos[step] + entrance_door_y_size[step]]
	
					face = group_ents.add_face pts[4], pts[5], pts[6], pts[7]
					status = face.erase!
				end
			end

			door_group = draw_entrance_door(entrance_door_x_pos[step], entrance_door_y_pos[step], entrance_door_x_size[step], entrance_door_y_size[step], entrance_door_wall[step], entrance_door_type[step], entrance_door_texture[step], entrance_door_glass_texture[step], entrance_door_glass_back_texture[step], wall_thickness, external_left_position, external_right_position, external_front_position, external_back_position)
	
			group_text = group.description
			group = group.parent.entities.add_group([group, door_group])
			group.description = group_text
		end
	end


####################################################
####	Draw signs				####
####################################################
	
	if sign_count > 0
		for step in 0..sign_count - 1
		
			sign_group = draw_signs(sign_type[step], sign_texture[step], door_frame_texture, sign_wall[step], sign_x_pos[step], sign_y_pos[step], sign_x_size[step], sign_y_size[step], sign_thickness[step], sign_filled[step], sign_double_sided[step], external_front_position, external_back_position, external_left_position, external_right_position)
			group = group.parent.entities.add_group([group, sign_group])
			
		end
	end




####################################################
####	Draw hangar doors			####
####################################################

	if door_type == "Detailed door" or door_type == "Bi-folding doors" or door_type == "Hydraulic swing door" or door_type == "Random mix" or door_type == "Sliding doors - row"
		# Create separate group for door entities
		door_group = Sketchup.active_model.entities.add_group
		door_group_ents = door_group.entities
		for step in 0..front_spaces - 1
			if door_type != "Random mix"
				new_door_type = door_type
			else
				door_random = rand(3)
				if door_random == 0
					new_door_type = "Hydraulic swing door"
				elsif door_random == 1
					new_door_type = "Bi-folding doors"
				elsif door_random == 2
					new_door_type = "Sliding doors - row"
				end
			end
			if new_door_type == "Sliding doors - row"
				if wing_section_width * 2 > tail_section_width and front_spaces > 1
					new_door_width = ((wing_section_width * 2) + (tail_section_width * 3)) / 2
				else
					new_door_width = section_width - wall_thickness
				end
			else
				new_door_width = section_width - wall_thickness
			end
			actual_door_width = (section_width - wall_thickness).to_s
			door_group = insert_door(new_door_type, door_height, door_folding_ext_level[step].to_i, door_sections, new_door_width, (step * (tail_section_width + wall_thickness)) + (step * section_width) + wall_thickness + ((section_width - wall_thickness)  / 2), 0 - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "X", "T-Hangar Row", actual_door_width)
			group_x = group
			group = group_x.parent.entities.add_group([group_x, door_group])
		end
		
		
		for step in 0..back_spaces - 1
			if door_type != "Random mix"
				new_door_type = door_type
			else
				door_random = rand(3)
				if door_random == 0
					new_door_type = "Hydraulic swing door"
				elsif door_random == 1
					new_door_type = "Bi-folding doors"
				elsif door_random == 2
					new_door_type = "Sliding doors - row"
				end
			end
			if new_door_type == "Sliding doors - row"
				if wing_section_width * 2 > tail_section_width and back_spaces > 1
					new_door_width =((wing_section_width * 2) +(tail_section_width * 3)) / 2
				else
					new_door_width = section_width - wall_thickness
				end
			else
				new_door_width = section_width - wall_thickness
			end
			actual_door_width = (section_width - wall_thickness).to_s
			door_group = insert_door(new_door_type, door_height, door_folding_ext_level[step + front_spaces].to_i, door_sections, new_door_width, wing_section_width + tail_section_width + wall_thickness +(step *(tail_section_width + wall_thickness)) +(step * section_width) + wall_thickness +((section_width - wall_thickness) / 2), total_depth - wall_thickness, wall_thickness, door_frame_thickness, door_skin_thickness, outside_height, arc, door_frame_texture, door_texture, wall_texture, roof_texture, beam_texture, floor_texture, "-X", "T-Hangar Row", actual_door_width)
			group_x = group
			group = group_x.parent.entities.add_group([group_x, door_group])
		end



	end

	#Flip hangar if necessary and move to correct location
	if afcad_read == 1
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x - wing_section_width - tail_section_width / 2 + trans.origin[0], hangar_y - section_depth + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t		
		if flip_hangar == "Yes"
			point2 = Geom::Point3d.new(hangar_x +(total_length/2), hangar_y, 0)
			t = Geom::Transformation.scaling(point2, -1, 1, 1)
			group.transform! t
		end		
		t = Geom::Transformation.rotation [hangar_x, hangar_y, 0], Z_AXIS, radians
		group.transform! t
	else
		trans = group.transformation
		point = Geom::Point3d.new(hangar_x + trans.origin[0], hangar_y + trans.origin[1], trans.origin[2])
		t = Geom::Transformation.new point
		group.move! t		
		if flip_hangar == "Yes"
			point2 = Geom::Point3d.new(hangar_x +(total_length/2), hangar_y, 0)
			t = Geom::Transformation.scaling(point2, -1, 1, 1)
			group.transform! t
		end		
		t = Geom::Transformation.rotation [hangar_x, hangar_y, 0], Z_AXIS, radians
		group.transform! t
		
		group.name = "TRow(&ltid&gtx&ltF/B&gt):" + input[0].to_s + ":" + input[1].to_s + ":" + input[2].to_s + ":" + input[3].to_s + ":" + input[4].to_s + ":" + input[5].to_s + ":" + input[6].to_s + ":" + input[7].to_s + ":" + input[8].to_s + ":" + input[9].to_s + ":" + input[10].to_s + ":" + input[11].to_s + ":" + input[12].to_s + ":" + input[13].to_s + ":" + input[14].to_s + ":" + input[15].to_s + ":" + input[16].to_s + ":" + input[17].to_s + ":" + input[18].to_s + ":" + input[19].to_s + ":" + input[20].to_s + ":" + texture_input[0].to_s + ":" + texture_input[1].to_s + ":" + texture_input[2].to_s + ":" + texture_input[3].to_s + ":" + texture_input[4].to_s + ":" + texture_input[5].to_s + ":" + texture_input[6].to_s + ":"

	end
	
end



####################################################
### Below is the code to convert a box to hangar ###
####################################################

def convert_to_hangar (hangar_type = "")

  model = Sketchup.active_model
  selection = model.selection


  if selection.count != 1 or selection[0].typename != "Face"
    echo "Please select the front side of the hangar only."
  else
    if (selection[0].normal[2] > 0 and (selection[0].normal[2] * 100000).floor != 0) or (selection[0].normal[2] < 0 and (selection[0].normal[2] * 100000).ceil != 0)
      echo "Please select a vertical face."
    else

      ##### Check hangar is on ground ####
      
      hangar_on_ground = 1
      for step in 0..1
        if selection[0].vertices[step].position[2] != 0
          hangar_on_ground = 0
        end
      end
      
      if hangar_on_ground == 0
        echo "Hangar has to be on the ground"
      else
        hangar_height = selection[0].vertices[2].position[2].to_f
        if hangar_height == selection[0].vertices[3].position[2].to_f
          arc_height = 0
        else
          arc_height = (selection[0].vertices[3].position[2]).to_f - hangar_height
        end
        
        front_vertices = selection[0].vertices
       
        bottom_left = selection[0].vertices[0].position
        top_left = selection[0].vertices[selection[0].vertices.count - 1].position
        bottom_right = selection[0].vertices[1].position
        top_right = selection[0].vertices[2].position
        
        
        #### Find left wall ####
        entities = model.entities
        
        for step in 0..entities.count - 1
          if entities[step].typename == "Face"
            if bottom_left == entities[step].vertices[1].position and top_left == entities[step].vertices[2].position
              bottom_back_left = entities[step].vertices[0].position
              top_back_left = entities[step].vertices[entities[step].vertices.count - 1].position
              if hangar_type == "Hangar row" or hangar_type == "T-hangar row" or hangar_type == "Double nested T-hangar row"
                hangar_roof_arc = entities[step].vertices[3].position
                if hangar_height == entities[step].vertices[3].position[2].to_f
                  arc_height = 0
                else
                  arc_height = (entities[step].vertices[3].position[2]).to_f - hangar_height
                end
              end
            end
          end
        end
        
        #### Find back wall ####
        
        for step in 0..entities.count - 1
          if entities[step].typename == "Face"
            if bottom_back_left == entities[step].vertices[1].position and top_back_left == entities[step].vertices[2].position
              bottom_back_right = entities[step].vertices[0].position
              top_back_right = entities[step].vertices[entities[step].vertices.count - 1].position
              back_vertices = entities[step].vertices
            end
          end
        end
        
#        status = selection[0].all_connected
 #       status.erase!
        selection.clear
        
        for step in 0..entities.count - 1
          selection_made = 0
          if entities[step].typename == "Face"
            for step2 in 0..entities[step].vertices.count - 1
              if selection_made == 0
                if entities[step].vertices[step2].position == bottom_left or entities[step].vertices[step2].position == top_left or entities[step].vertices[step2].position == bottom_back_left or entities[step].vertices[step2].position == top_back_left or entities[step].vertices[step2].position == bottom_back_right or entities[step].vertices[step2].position == top_back_right
                  status = selection.add entities[step]
                  selection_made = 1
                end  
              end
            end
          elsif entities[step].typename == "Edge"
            do_not_delete = 0
            if hangar_type == "Standard"
              if (entities[step].end.position == bottom_left and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_left and entities[step].end.position == bottom_back_left)
                hangar_length = entities[step].length
                do_not_delete = 1
              end
              if (entities[step].end.position == bottom_back_right and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_back_right and entities[step].end.position == bottom_back_left)
                hangar_width = entities[step].length
                do_not_delete = 1
              end
            elsif hangar_type == "Hangar row" or hangar_type == "T-hangar row" or hangar_type == "Double nested T-hangar row"
              if (entities[step].end.position == bottom_left and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_left and entities[step].end.position == bottom_back_left)
                hangar_length = entities[step].length
                do_not_delete = 1
              end
              if (entities[step].end.position == bottom_right and entities[step].start.position == bottom_left) or (entities[step].start.position == bottom_right and entities[step].end.position == bottom_left)
                hangar_width = entities[step].length
                do_not_delete = 1
              end
            end
            if do_not_delete == 0
              for step2 in 0..front_vertices.count - 1
                if selection_made == 0
                  if front_vertices[step2].position == entities[step].end.position or front_vertices[step2].position == entities[step].start.position
                    status = selection.add entities[step]
                    selection_made = 1
                  end
                end
              end
              for step2 in 0..back_vertices.count - 1
                if selection_made == 0
                  if back_vertices[step2].position == entities[step].end.position or back_vertices[step2].position == entities[step].start.position
                    status = selection.add entities[step]
                    selection_made = 1
                  end
                end
              end
              if hangar_type == "Hangar row" or hangar_type == "T-hangar row" or hangar_type == "Double nested T-hangar row"
                if entities[step].end.position == hangar_roof_arc or entities[step].start.position == hangar_roof_arc
                  status = selection.add entities[step]
                  selection_made = 1
                end
              end
            end
          end
        end
        
        while selection.count > 0        
          selection.each { |entity| entity.erase!}
        end
        
        #### Select left and back wall edges ####
        entities = model.entities
        
        if hangar_type == "Standard"
          for step in 0..entities.count - 1
            if entities[step].typename == "Edge"
              if (entities[step].end.position == bottom_left and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_left and entities[step].end.position == bottom_back_left)
                status = selection.add entities[step]
              end
              if (entities[step].end.position == bottom_back_right and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_back_right and entities[step].end.position == bottom_back_left)
                status = selection.add entities[step]
              end
            end
          end
        elsif hangar_type == "Hangar row" or hangar_type == "T-hangar row" or hangar_type == "Double nested T-hangar row"
          for step in 0..entities.count - 1
            if entities[step].typename == "Edge"
              if (entities[step].end.position == bottom_left and entities[step].start.position == bottom_back_left) or (entities[step].start.position == bottom_left and entities[step].end.position == bottom_back_left)
                status = selection.add entities[step]
              end
              if (entities[step].end.position == bottom_right and entities[step].start.position == bottom_left) or (entities[step].start.position == bottom_right and entities[step].end.position == bottom_left)
                status = selection.add entities[step]
              end
            end
          end
        end
        
        
        if hangar_type == "Standard"
          standalone_hangar("Box", hangar_height, arc_height)
        elsif hangar_type == "Double nested T-hangar row"
          d_nested_t_hangar_row("Box", hangar_height, arc_height)
        elsif hangar_type == "T-hangar row"
          t_hangar_row("Box", hangar_height, arc_height)
        elsif hangar_type == "Hangar row"
     #     hangar_row("Box", hangar_height, arc_height)
        end
      end
      
      
      
      
    end
  end

  
end




